<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:template name="content_categ">
  <xsl:param name="content" select="/page/content/topic"/>
  <xsl:call-template name="content_topic">
    <xsl:with-param name="content" select="$content"/>
  </xsl:call-template>
  <xsl:variable name="colCount">
    <xsl:choose>
      <xsl:when test="$content/columns/text() &gt; 3">3</xsl:when>
      <xsl:when test="$content/columns/text() &gt; 0"><xsl:value-of select="$content/columns"/></xsl:when>
      <xsl:otherwise>2</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:call-template name="content_categ_subtopics">
    <xsl:with-param name="topics" select="$content/subtopics/subtopic"/>
    <xsl:with-param name="colCount" select="$colCount"/>
  </xsl:call-template>
</xsl:template>

<xsl:template name="content_categ_subtopics">
  <xsl:param name="topics"/>
  <xsl:param name="colCount"/>
  <xsl:variable name="count" select="count($topics)" />
  <xsl:choose>
    <xsl:when test="$count &gt; 0 and $colCount &gt; 1">
      <div class="subTopics">
        <xsl:for-each select="$topics[(position() mod $colCount) = 1]">
          <xsl:variable name="indexmin" select="position() * $colCount - $colCount" />
          <xsl:variable name="indexmax" select="position() * $colCount" />
          <xsl:call-template name="content_categ_line">
            <xsl:with-param name="topics" select="$topics[((position() &gt; $indexmin) and (position() &lt;= $indexmax))]"/>
            <xsl:with-param name="colCount" select="$colCount"/>
          </xsl:call-template>
        </xsl:for-each>	
      </div>
    </xsl:when>
    <xsl:when test="$count &gt; 0">
      <div class="subTopics">
        <xsl:for-each select="$topics">
          <xsl:call-template name="content_categ_line">
            <xsl:with-param name="topics" select="."/>
            <xsl:with-param name="colCount" select="$colCount"/>
          </xsl:call-template>
        </xsl:for-each>	
      </div>
    </xsl:when>
  </xsl:choose>
</xsl:template>

<xsl:template name="content_categ_line">
  <xsl:param name="topics"/>
  <xsl:param name="colCount" />
  <xsl:variable name="count" select="count($topics)"/>
  <xsl:variable name="colWidth"><xsl:value-of select="floor(100 div $colCount)"/></xsl:variable>
  <div class="subTopicLine">
    <xsl:choose>
      <xsl:when test="$count &gt; 1">
        <table class="subTopicLine">
          <tr>
            <xsl:for-each select="$topics">
              <xsl:variable name="style">
                <xsl:choose>
                  <xsl:when test="position() = last()">width: <xsl:value-of select="100 - ($colWidth * (position() - 1))"/>%;</xsl:when>
                  <xsl:otherwise>width: <xsl:value-of select="$colWidth"/>%;</xsl:otherwise>
                </xsl:choose>
              </xsl:variable>
              <xsl:if test="position() &gt; 1">
                <td class="topicSpacer"><xsl:call-template name="space"/></td>
              </xsl:if>
              <td class="subTopic" style="{$style}">   
                <h2 class="contentSubTopicTitle"><xsl:value-of select="title" disable-output-escaping="yes"/></h2>
                <div class="contentTeaser">
                  <xsl:copy-of select="text/text()|text/*" />
                </div>
                <a href="{@href}" class="more"><xsl:call-template name="categ_morelink_caption"/></a>
              </td>
            </xsl:for-each>
          </tr>
        </table>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="topic" select="$topics[position() = 1]"/>
        <div class="subTopic">   
          <h2 class="contentSubTopicTitle"><xsl:value-of select="$topic/title" disable-output-escaping="yes"/></h2>
          <div class="contentTeaser"><xsl:copy-of select="$topic/text/text()|$topic/text/*" /></div>
          <a href="{$topic/@href}" class="more"><xsl:call-template name="categ_morelink_caption"/></a>
        </div>
      </xsl:otherwise>
    </xsl:choose>
  </div>
</xsl:template>

<xsl:template name="categ_morelink_caption">
  <xsl:call-template name="languageText">
    <xsl:with-param name="text">more</xsl:with-param>
  </xsl:call-template>
</xsl:template>

</xsl:stylesheet>