<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:template name="content_linkdb">
  <xsl:variable name="topic" select="/page/content/topic" />
  <xsl:if test="$topic/title/text() != ''">    
    <xsl:if test="$topic/backlink/text() != ''">
      <a class="back" href="{$topic/backlink}">Zur&#xFC;ck</a>
    </xsl:if>
    <xsl:if test="$topic/search/text() != ''">
      <a class="search" href="{$topic/search}">Suche</a>
    </xsl:if>
    <div class="contentLinkDb">
      <h1>
        <xsl:value-of select="$topic/title" />
        <xsl:if test="$topic/subtitle/text() != ''">
          <span class="subTitle"><xsl:value-of select="$topic/subtitle" /></span>
        </xsl:if>
      </h1>
      <xsl:if test="$topic/text/text() != ''">
        <div class="topicText">
          <xsl:value-of select="$topic/text" />
        </div>
      </xsl:if>
      <xsl:if test="$topic/pathnavi and (count($topic/pathnavi/categ) &gt; 1)">
        <xsl:call-template name="content_linkdb_navi" >
          <xsl:with-param name="navi" select="$topic/pathnavi" />
        </xsl:call-template>
      </xsl:if>
    </div>
  </xsl:if>
  <xsl:choose>
    <xsl:when test="$topic/categs or $topic/links or $topic/searchdlg" >
      <xsl:if test="$topic/searchdlg" >
        <xsl:call-template name="content_linkdb_search" >
          <xsl:with-param name="dialog" select="$topic/searchdlg/dialog"/>
        </xsl:call-template>
      </xsl:if>
      <xsl:if test="$topic/categs" >
        <xsl:call-template name="content_linkdb_categs" >
          <xsl:with-param name="topic" select="$topic"/>
       </xsl:call-template>
      </xsl:if>
      <xsl:if test="$topic/links">
        <xsl:call-template name="content_linkdb_links">
          <xsl:with-param name="topic" select="$topic" />
       </xsl:call-template>
      </xsl:if>
      <xsl:if test="$topic/linksnavi">
        <xsl:call-template name="content_linkdb_sites_navi">
          <xsl:with-param name="linksnavi" select="$topic/linksnavi" />
        </xsl:call-template>
      </xsl:if>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="content_no_elements" />
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<xsl:template name="content_linkdb_categs">
  <xsl:param name="topic" />
  <xsl:param name="categs" select="$topic/categs" />
  <xsl:variable name="categscount" select="count($categs/categ)" />
  <xsl:variable name="categslinecount">2</xsl:variable>
  <xsl:if test="$categscount &gt; 0">
    <div class="linkdbCategsChapter">
      Kategorien
    </div>  
    <xsl:for-each select="$categs/categ[(position() mod $categslinecount) = 1]">
      <xsl:variable name="indexmin" select="position() * $categslinecount - $categslinecount" />
      <xsl:variable name="indexmax" select="position() * $categslinecount" />
      <xsl:call-template name="linkdb_categs">
        <xsl:with-param name="categs" select="$categs/categ[((position() &gt; $indexmin) and (position() &lt;= $indexmax))]"/>
        <xsl:with-param name="categscount" select="$categscount" />
        <xsl:with-param name="categslinecount" select="$categslinecount"/>
      </xsl:call-template>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<xsl:template name="linkdb_categs">
  <xsl:param name="categs" />
  <xsl:param name="categscount"/>
  <xsl:param name="categslinecount"/>
  <xsl:param name="count" select="count($categs)" />
  <xsl:param name="width" select="round(100 div $categslinecount)"/>
  <xsl:choose>
      <xsl:when test="$count &gt; 1">
        <table class="categBlock" border="0">  
          <tr>
            <xsl:for-each select="$categs">   
              <xsl:variable name="swidth">
                <xsl:choose>
                  <xsl:when test="($categslinecount &gt; $count) and (position() = $count)">
                    <xsl:value-of select="0" />
                  </xsl:when>
                  <xsl:when test="position() != $categslinecount">
                    <xsl:value-of select="$width" />
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="( 100 - ( $width * ( $categslinecount - 1 ) ) )" />
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:variable>    
              <xsl:if test="position() &gt; 1">
                <td class="topicSpacer"><xsl:call-template name="space"/></td>
              </xsl:if>
              <td> 
                <xsl:if test="$swidth &gt; 0">
                  <xsl:attribute name="width">
                    <xsl:value-of select="$swidth"/>%
                  </xsl:attribute>
                </xsl:if>
                <h2>
                  <xsl:value-of select="@title" disable-output-escaping="yes"/>
                </h2>
                <div>
                  <xsl:value-of select="." disable-output-escaping="yes"/>
                </div>
                <a href="{@href}" class="more">mehr</a>
              </td>
            </xsl:for-each>
          </tr>
        </table>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="categ" select="$categs[position() = 1]"/>
        <div class="categsBlock">   
          <h2 class="contentSubTopicTitle">
            <xsl:value-of select="$categ/@title" disable-output-escaping="yes"/>
          </h2>
          <div><xsl:value-of select="$categ" disable-output-escaping="yes"/></div>
          <a href="{$categ/@href}" class="more">mehr</a>
        </div>
      </xsl:otherwise>
    </xsl:choose>
</xsl:template>

<xsl:template name="content_linkdb_navi">
  <xsl:param name="navi" />
  <div class="navi">
    <xsl:for-each select="$navi/categ">
        <xsl:if test="position() &gt; 1">
          &#xBB;
        </xsl:if> 
        <a href="{@href}"><xsl:value-of select="@title" /></a>
    </xsl:for-each>
  </div>
</xsl:template>


<xsl:template name="content_linkdb_links">
  <xsl:param name="topic" />
  <xsl:for-each select="$topic/links/link">
    <div class="linkdbLinkBox">
      <h2><a href="{@href}"><xsl:value-of select="@title" disable-output-escaping="yes"/></a></h2>
      <div>
        <xsl:value-of select="." disable-output-escaping="yes"/>
      </div>
    </div>
  </xsl:for-each>
</xsl:template>


<xsl:template name="content_no_elements">
  <div class="message">
    Weder Kategorien noch Links vorhanden. 
  </div>
</xsl:template>

<xsl:template name="content_linkdb_search">
  <xsl:param name="dialog" />
  <form action="{$dialog/@action}" method="post">
  <xsl:copy-of select="$dialog/input[@type='hidden']"/>
    <table class="linkDbSearchForm" border="0">
      <tr>   
        <td class="title">Suche:</td>
        <td class="inputField">
          <xsl:variable name="field" select="$dialog/lines/line[@fid='searchfor']/input"/>           
            <input type="{$dialog/lines/line[@fid='searchfor']/input/@type}" 
                   name="{$dialog/lines/line[@fid='searchfor']/input/@name}" 
                   value="{$dialog/lines/line[@fid='searchfor']/input/@value}" 
                   maxlength="{$dialog/lines/line[@fid='searchfor']/input/@maxlength}" 
                  class="searchform" 
                  fid="{$dialog/lines/line[@fid='searchfor']/input/@fid}"/>
        </td>   
        <td class="submitButton">     
          <input type="submit" value="Suchen" />       
        </td>
      </tr> 
    </table>
  </form> 
</xsl:template>

<xsl:template name="content_linkdb_sites_navi">
  <xsl:param name="linksnavi" />
  <div class="linkSiteNavi">
    <xsl:if test="$linksnavi/linksback">
      <a class="buttonPrev" href="{$linksnavi/linksback/@href}">
        <xsl:value-of select="$linksnavi/linksback/@text" />
      </a>
    </xsl:if>
    <xsl:if test="$linksnavi/linksnext">
      <a class="buttonNext" href="{$linksnavi/linksnext/@href}">
        <xsl:value-of select="$linksnavi/linksnext/@text" />
      </a>
    </xsl:if>  
    <div class="pageLink">
      <xsl:for-each select="$linksnavi/pages/page">
        <xsl:if test="position()>1"> - </xsl:if>
        <xsl:choose>
          <xsl:when test="@option = 'selected'">
            <xsl:value-of select="@num" />
          </xsl:when>
          <xsl:otherwise>
            <a href="{@href}"><xsl:value-of select="@num" /></a>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
    </div>
    <br clear="all" class="clear" />
  </div>
</xsl:template>

</xsl:stylesheet>