<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format">

<xsl:template name="content_whoiswho_viewer">
  <xsl:param name="content" select="/page/content/topic"/>
  <div class="contentTopic">
    <xsl:if test="$content/whoiswho/navigation/link[@type='back']">
      <a href="{$content/whoiswho/navigation/link[@type='back']/@href}" class="backButton">Zur&#xFC;ck</a>
    </xsl:if>
    <p>
      <b>
        <xsl:choose>
          <xsl:when test="$content/title/text() != ''"><xsl:value-of select="$content/title" disable-output-escaping="yes"/></xsl:when>
          <xsl:otherwise><xsl:value-of select="$PAGE_TITLE" disable-output-escaping="yes"/></xsl:otherwise>
        </xsl:choose>
        <xsl:if test="$content/subtitle/text() != ''">
          <div class="contentSubTitle"><xsl:value-of select="$content/subtitle" disable-output-escaping="yes"/></div>
        </xsl:if>   
      </b>
    </p>    
  	<xsl:if test="$content/text/text()">
    <div class="content">
      <p>
        <xsl:apply-templates select="$content/text"/>
      </p>
    </div>
  	</xsl:if>
    
  </div>
    
  <div class="whoiswho">
    <xsl:choose>
      <xsl:when test="$content/whoiswho/error">
        <xsl:if test="$content/whoiswho/navigation/dialog">
          <xsl:call-template name="whoiswho_search_dialog">
            <xsl:with-param name="dialog" select="$content/whoiswho/navigation/dialog"/>
          </xsl:call-template>
        </xsl:if>
        <xsl:choose>
          <xsl:when test="$content/whoiswho/error/@type = 'no_result'">
            <div class="contentData error">Nix gefunden.</div>
          </xsl:when>
          <xsl:when test="$content/whoiswho/error/@type = 'invalid_search'">
            <div class="contentData error">Ung&#xFC;ltiger Suchbegriff.</div>
          </xsl:when>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="$content/whoiswho/person">
        <xsl:if test="$content/whoiswho/groups">
          <xsl:call-template name="whoiswho_datagroup_list">
            <xsl:with-param name="datagroups" select="$content/whoiswho/groups"/>
          </xsl:call-template>
        </xsl:if> 
        <div>
          <xsl:call-template name="whoiswho_person_details">
            <xsl:with-param name="person" select="$content/whoiswho/person"/>
          </xsl:call-template>
        </div> 
      </xsl:when>
      <xsl:when test="$content/whoiswho/people">
        <xsl:if test="$content/whoiswho/navigation/dialog">
          <xsl:call-template name="whoiswho_search_dialog">
            <xsl:with-param name="dialog" select="$content/whoiswho/navigation/dialog"/>
          </xsl:call-template>
        </xsl:if>
        
        <xsl:if test="$content/whoiswho/navigation/chars">
          <div class="contentNavigation">
            <a href="{$content/whoiswho/navigation/@href}">Alle</a>
            <xsl:for-each select="$content/whoiswho/navigation/chars/char"> <!-- [@itemcount &gt; 0] //-->
              |
              <xsl:choose>
                <xsl:when test="@selected">
                  <xsl:value-of select="text()" disable-output-escaping="yes"/>
                </xsl:when>
                <xsl:otherwise>
                  <a href="{@href}"><xsl:value-of select="text()" disable-output-escaping="yes"/></a>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:for-each>
          </div>
        </xsl:if>
        <div>
          <xsl:call-template name="whoiswho_people_table">
            <xsl:with-param name="people" select="$content/whoiswho/people/person"/>
          </xsl:call-template>
        </div>
        <xsl:if test="$content/whoiswho/navigation/pages[@count &gt; 1]">
          <div class="contentNavigation">
            <xsl:for-each select="$content/whoiswho/navigation/pages/page">
              <xsl:if test="position() &gt; 1">
               -
              </xsl:if>
              <xsl:choose>
                <xsl:when test="@selected">
                  <xsl:value-of select="text()" disable-output-escaping="yes"/>
                </xsl:when>
                <xsl:otherwise>
                  <a href="{@href}">
                    <xsl:value-of select="text()" disable-output-escaping="yes"/>
                  </a>
                </xsl:otherwise>
              </xsl:choose>        
            </xsl:for-each>
          </div>
        </xsl:if>
      </xsl:when>
    </xsl:choose>
  </div>
</xsl:template>

<xsl:template name="whoiswho_skill_table">
  <div class="contentData">
    <div class="data">
      <table class="data">
        <xsl:for-each select="skill">
          <tr><xsl:call-template name="whoiswho_skill_table_line"/></tr>
        </xsl:for-each>
      </table>
    </div>
  </div>
</xsl:template>

<xsl:template name="whoiswho_skill_parameter">
  <xsl:choose>
    <xsl:when test="type='yesno'">
      <xsl:choose>
        <xsl:when test="value/@dec='1'">ja</xsl:when>
        <xsl:otherwise>nein</xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="value/parameters/parameter[@selected='selected']"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="whoiswho_skill_table_line">
  <th class="field"><xsl:value-of select="title"/>:</th>
  <td><xsl:call-template name="whoiswho_skill_parameter"/></td>
</xsl:template>

<xsl:template name="whoiswho_people_table">
  <xsl:param name="people"/>
  <xsl:variable name="linecount">2</xsl:variable>
  <table class="contentNavigation twoColumns">
  <xsl:for-each select="$people[(position() mod $linecount) = 1]">
    <xsl:variable name="indexmin" select="position() * $linecount - $linecount" />
    <xsl:variable name="indexmax" select="position() * $linecount" />
    <xsl:call-template name="whoiswho_people_line">
       <xsl:with-param name="people" select="$people[((position() &gt; $indexmin) and (position() &lt;= $indexmax))]"/>
       <xsl:with-param name="linecount" select="$linecount"/>
     </xsl:call-template>
   </xsl:for-each>
  </table>
</xsl:template>

<xsl:template name="whoiswho_people_line">
  <xsl:param name="people"/>
  <xsl:param name="topiccount"/>
    <tr valign="top">
    <xsl:choose>
      <xsl:when test="count($people) > 1"> 
        <td width="50%">
          <a href="{$people[position() = 1]/@href}"><xsl:value-of select="$people[position() = 1]" disable-output-escaping="yes"/></a>
        </td>
        <td width="50%">
          <a href="{$people[position() = 2]/@href}"><xsl:value-of select="$people[position() = 2]" disable-output-escaping="yes"/></a>
        </td>
      </xsl:when>
      <xsl:otherwise>
        <td>
          <a href="{$people[position() = 1]/@href}"><xsl:value-of select="$people[position() = 1]" disable-output-escaping="yes"/></a>
        </td>
        <td class="empty"></td>
      </xsl:otherwise>    
    </xsl:choose>
    </tr>
</xsl:template>

<xsl:template name="whoiswho_datagroup_table">
  <xsl:param name="datagroups"/>
  <xsl:variable name="linecount">2</xsl:variable>
  <ul>
  <xsl:for-each select="$datagroups">
    <li>
      <xsl:choose>
        <xsl:when test="@selected='selected'">
          <b><xsl:value-of select="."/></b>
        </xsl:when>
        <xsl:otherwise>
          <a href="{@href}"><xsl:value-of select="." disable-output-escaping="yes"/></a>
        </xsl:otherwise>
      </xsl:choose>
    </li>
  </xsl:for-each>
  </ul>
</xsl:template>


<xsl:template name="whoiswho_datagroup_list">
  <xsl:param name="datagroups"/>
  <xsl:variable name="linecount">3</xsl:variable>
  <div class="groupbar">
    <ul class="grouplist">
      <xsl:for-each select="$datagroups/group">
        <li>
          <xsl:choose>
            <xsl:when test="@selected">
              <xsl:value-of select="." disable-output-escaping="yes"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:choose>
                <xsl:when test="@href">
                  <a href="{@href}">
                    <xsl:value-of select="." disable-output-escaping="yes" />                
                  </a>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="." disable-output-escaping="yes" />
                </xsl:otherwise>
              </xsl:choose>
            </xsl:otherwise>
          </xsl:choose>
        </li>
      </xsl:for-each>
    </ul>
  </div>    
</xsl:template>

<xsl:template name="whoiswho_editor_buttons">
  <xsl:param name="buttons"/>
  <p><b class="header">Funktionen</b></p>
  <div class="groupbareditor">
    <ul class="grouplisteditor">
      <xsl:for-each select="$buttons/link">
        <li>
          <a href="{@href}">
            <xsl:value-of select="./text()" disable-output-escaping="yes"/>                
          </a>
          <xsl:if test="position() != last()"><xsl:call-template name="space"/><xsl:call-template name="space"/><xsl:call-template name="space"/>|</xsl:if>
        </li>
      </xsl:for-each>
    </ul>
  </div>
</xsl:template>

<xsl:template name="whoiswho_person_details">
  <xsl:param name="person"/>
  <div class="contentData">
    <p>
      
        <span class="name">
          <xsl:value-of select="$person/firstname" disable-output-escaping="yes"/>
          <xsl:call-template name="space"/>
          <xsl:value-of select="$person/lastname" disable-output-escaping="yes"/>
        </span>
        <span class="profession">
          -
          <xsl:value-of select="$person/data/group/@title" disable-output-escaping="yes"/>
        </span>
      
    </p>
    <xsl:if test="($person/data/group) and ($person/data/group/text() != '')">
      <p><xsl:value-of select="$person/data/group" disable-output-escaping="yes"/></p>
    </xsl:if>
  </div>
  <xsl:for-each select="$person/data/items/item">
    <xsl:choose>
      <xsl:when test="count(fieldgroup) &gt; 1">
        <xsl:for-each select="fieldgroup">
          <xsl:call-template name="whoiswho_data_details">
            <xsl:with-param name="fieldgroup" select="."/>
            <xsl:with-param name="grouptitle" select="@title"/>
          </xsl:call-template>
        </xsl:for-each>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="whoiswho_data_details">
          <xsl:with-param name="fieldgroup" select="fieldgroup[position() = 1]"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:for-each>
  
  <xsl:for-each select="$person/skills">
    <xsl:call-template name="whoiswho_skill_table"/>
  </xsl:for-each>
  
</xsl:template>

<xsl:template name="whoiswho_data_details">
  <xsl:param name="grouptitle"></xsl:param>
  <xsl:param name="fieldgroup"/>
  <xsl:variable name="colspan">
    <xsl:choose>
      <xsl:when test="count($fieldgroup/field[@type = 'image']) &gt; 0">1</xsl:when>
      <xsl:otherwise>2</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="textfieldCount"><xsl:value-of select="count($fieldgroup/field[(@type != 'image') and ((./value/text() != '') or (count(./value/*)))])"/></xsl:variable>
  <xsl:if test="($textfieldCount &gt; 0) or (count($fieldgroup/field[@type = 'image']) &gt; 0)">
    <div class="contentData">
      <xsl:if test="count($fieldgroup/field[@type = 'image']/value/*) &gt; 0">
        <div class="images">
          <xsl:for-each select="$fieldgroup/field[(@type = 'image') and (value/img/@src != '')]">
            <img src="{value/img/@src}" class="img" alt="{@alt}"/>
          </xsl:for-each>
        </div>
      </xsl:if> 
      <xsl:if test="$textfieldCount &gt; 0">
        <div class="data">
          <table class="data">
            <xsl:if test="$grouptitle != ''">
              <tr>
                <th colspan="2" class="group"><xsl:value-of select="$grouptitle" disable-output-escaping="yes"/></th>
              </tr>
            </xsl:if>
            <xsl:for-each select="$fieldgroup/field[(@type != 'image') and ((./value/text() != '') or (count(./value/*)))]">
              <tr>
                <th class="field"><xsl:value-of select="@title" disable-output-escaping="yes"/>:</th>
                <td><xsl:apply-templates select="value/*|value/text()" /></td>
              </tr>
            </xsl:for-each>
          </table>
        </div>
      </xsl:if>
    </div>
  </xsl:if>
</xsl:template>

<xsl:template name="whoiswho_search_dialog">
  <xsl:param name="dialog"/>
  <form action="{$dialog/@action}" method="POST">
    <xsl:apply-templates select="$dialog/input[@type='hidden']"/>
    <div class="searchfield">
      <label for="{$dialog/lines/line[@fid='searchfor']/input/@name}">Suche</label>
      <input type="text" class="searchText" name="{$dialog/lines/line[@fid='searchfor']/input/@name}"  value="{$dialog/lines/line[@fid='searchfor']/input/@value}" maxlength="{$dialog/lines/line[@fid='searchfor']/input/@maxlength}"/>
      <input type="submit" class="submitbutton" value="Suchen" />
    </div>
  </form>
</xsl:template>

<!-- Einsprungspunkt fuer den Editor -->
<xsl:template name="content_whoiswho_editor">
  <xsl:param name="content" select="/page/content/topic"/>  
  <div class="whoiswho">
    <xsl:call-template name="whoiswho_editor_buttons">
      <xsl:with-param name="buttons" select="$content/whoiswho/navigation"/>
    </xsl:call-template>
    <xsl:if test="$content//error">
      <div class="contentData error">
        <xsl:variable name="error" select="$content//error"/>
        <b><xsl:value-of select="$error" disable-output-escaping="yes"/></b>        
      </div>
    </xsl:if>
    <xsl:if test="$content/whoiswho/groups">
      <b class="header">Datengruppen</b>
      <xsl:call-template name="whoiswho_datagroup_table">
        <xsl:with-param name="datagroups" select="$content/whoiswho/groups/group"/>
      </xsl:call-template>
    </xsl:if> 
    <xsl:if test="$content/whoiswho//needed">
      <div class="contentData error">        
          <a name="neededfieldhint"></a>Pflichtfelder sind mit * gekennzeichnet.       
      </div>
    </xsl:if>
    
    <div id="editorArea">
      <xsl:apply-templates select="$content/whoiswho/form[fieldset/input]"/>
      <xsl:apply-templates select="$content/whoiswho/form[fieldset/skill]"/>
    </div>       
  </div>
</xsl:template>

<xsl:template match="fieldset[input]">
  <xsl:for-each select="label">
    <xsl:variable name="assoc"><xsl:value-of select="@for"/></xsl:variable>
    <xsl:variable name="assoc2"><xsl:value-of select="@for"/>[]</xsl:variable>
    <div class="line">
      <xsl:apply-templates select="."/>
      <div class="input">
        <xsl:apply-templates select="../input[@name=$assoc or @name=$assoc2]"/>
        <xsl:apply-templates select="../textarea[@name=$assoc]"/>
        <xsl:apply-templates select="../select[@name=$assoc]"/>
      </div>
    </div>
    <div class="clear"> </div>  
  </xsl:for-each>  
</xsl:template>

<xsl:template match="fieldset[skill != '']">
  <xsl:for-each select="label">
    <xsl:variable name="assoc"><xsl:value-of select="@for"/></xsl:variable>
    <div class="line">
      <xsl:apply-templates select="."/>
      <div class="input">
        <xsl:apply-templates select="../skill[@name=$assoc]"/>
      </div>
    </div>
    <div class="clear"> </div>  
  </xsl:for-each>  
</xsl:template>

<xsl:template match="whoiswho//input[(@type='radio') or (@type='checkbox')]">
    <input type="{@type}" name="{@name}" value="{@value}">
      <xsl:if test="@checked">
        <xsl:attribute name="checked">checked</xsl:attribute>
      </xsl:if>
    <xsl:value-of select="./text()"/>
    </input>
    <br />
</xsl:template>

<xsl:template match="whoiswho//skill[@type='radio']">
    <xsl:for-each select="parameters/parameter">
      <xsl:choose>
        <xsl:when test="@selected='selected'">
          <input type="radio" value="{@num}" name="{../../@name}" checked="checked">
            <xsl:value-of select="."/>
          </input>
        </xsl:when>
        <xsl:otherwise>
          <input type="radio" value="{@num}" name="{../../@name}">
            <xsl:value-of select="."/>
          </input>
        </xsl:otherwise>
      </xsl:choose>
    <br />
    </xsl:for-each>
</xsl:template>


<xsl:template match="whoiswho//skill[@type='yesno']">
  <div class="form_radio">
    <xsl:choose>
      <xsl:when test="@value='1'">
        <input type="radio" value="0" name="{@name}">nein</input>
        <input type="radio" value="1" name="{@name}" checked="checked">ja</input>        
      </xsl:when>
      <xsl:otherwise>
        <input type="radio" value="0" name="{@name}" checked="checked">nein</input>
        <input type="radio" value="1" name="{@name}">ja</input>        
      </xsl:otherwise>
    </xsl:choose>
  </div>
</xsl:template>

<xsl:template match="whoiswho//skill[@type='combo']">
  <div class="form_combo">
    <select name="{@name}" class="form_combobox">
      <xsl:for-each select="parameters/parameter">
        <xsl:choose>
          <xsl:when test="@selected='selected'">
            <option value="{@num}" selected="selected">
              <xsl:value-of select="."/>
            </option>
          </xsl:when>
          <xsl:otherwise>
            <option value="{@num}">
              <xsl:value-of select="."/>
            </option>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
    </select>
  </div>
</xsl:template>

<xsl:template match="whoiswho//label">
    <label for="{@for}">
      <xsl:if test="@class = 'error'">
        <xsl:attribute name="class">error</xsl:attribute>
      </xsl:if>
      <xsl:value-of select="." disable-output-escaping="yes"/>
      <xsl:if test="count(./needed) &gt; 0">
        <a href="#neededfieldhint" style="text-decoration: none;"><xsl:text>*</xsl:text></a>    
      </xsl:if>    
    </label>
</xsl:template>

<xsl:template match="whoiswho//input[@type = 'submit']">
  <div class="buttonline">
    <input type="{@type}" value="speichern" class="{@class}"/>      
  </div>
</xsl:template>

<xsl:template match="whoiswho//img">
  <div class="editpic">
    <img src="{@src}" class="img" alt="{@alt}"/>
  </div>
</xsl:template>

</xsl:stylesheet>