<?php
/**
* Community user management
*
* @package module_community
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_surfers.php,v 1.4 2005/12/05 13:49:26 weinert Exp $
*/

/**
* Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');
/**
* Basic class check conditions
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');
/**
* Community user management
*
* Administration of surfer and rights
*
* @package module_community
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class surfer_admin extends base_db {
  var $databaseURI = PAPAYA_DB_URI;
  var $tableSurfer = PAPAYA_DB_TBL_SURFER;
  var $tableTopics = PAPAYA_DB_TBL_TOPICS;
  var $tableLink = PAPAYA_DB_TBL_SURFERPERMLINK;
  var $tablePerm = PAPAYA_DB_TBL_SURFERPERM;
  var $tableGroups = PAPAYA_DB_TBL_SURFERGROUPS;
  var $tableTopic = PAPAYA_DB_TBL_TOPICS;
  var $tableAuthuser = PAPAYA_DB_TBL_AUTHUSER;

  var $paramName;
  var $params;
  var $baseLink;
  var $msgs;
  var $user;

  var $surfers;
  var $permissionsList;
  var $linkList;

  var $editSurfer;
  var $surferGroup;
  var $editPerm;

  var $displayThreshhold;
  var $listLength = 10;

  var $delFlag = FALSE;

  /**
  * Constuctor with parameter initalisation
  *
  * @param mixed &$msgs
  * @param string $paramName optional, default value "sadm"
  */
  function __construct(&$msgs, $paramName="sadm"){
    $this->paramName = $paramName;
    $this->initializeParams('PAPAYA_SESS_'.$paramName);
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);

    $this->baseLink = $this->getBaseLink();
    $this->msgs = &$msgs;
    $this->displayThreshhold = 100;
    $this->listLength = 10;

    $this->initializeSessionParam('offset');
    $this->initializeSessionParam('patt', array('offset'));
    $this->initializeSessionParam('mode', array('offset', 'patt'));

    $this->params['offset'] = @(int)$this->params['offset'];
    $this->params['patt'] = @(string)$this->params['patt'];
  }

  function surfer_admin(&$msgs, $paramName='cw') {
    $this->__construct($msgs, $paramName);
  }

  /**
  * Check if email exists
  *
  * @param string $email
  * @access public
  * @return boolean
  */
  function existEmail($email){
    $sql = "SELECT count(*) FROM %s WHERE surfer_email = '%s'";
    $params = array($this->tableSurfer, $email);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        return ((bool)$row[0] > 0);
      }
    }
    return FALSE;
  }

  /**
  * Exist Handle
  *
  * @param string $handle
  * @access public
  * @return boolean
  */
  function existHandle($handle){
    $sql = "SELECT count(*) FROM %s WHERE surfer_handle = '%s'";
    $params = array($this->tableSurfer, $handle);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        return ($row[0] > 0);
      }
    }
    return FALSE;
  }

  /**
  * Get ID by handle
  *
  * @param string $handle
  * @access public
  * @return integer
  */
  function getIdByHandle($handle) {
    $sql = "SELECT surfer_id FROM %s WHERE surfer_handle = '%s'";
    $params = array($this->tableSurfer, $handle);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        return $row[0];
      }
    }
    return 0;
  }

  /**
  * Load surfer ID
  *
  * @param $id
  * @access public
  * @return mixed NULL or array
  */
  function loadSurfer($id) {
    $sql = "SELECT s.surfer_id, s.surfer_handle,
                    s.surfer_givenname, s.surfer_surname,
                    s.surfer_email, s.surfer_email_alt,
                    s.surfer_phone, s.surfer_phone_alt,
                    s.surfer_valid, s.surfer_password,
                    s.surfergroup_id, s.surfer_lastlogin,
                    sg.surfergroup_title,
                    s.auth_user_id, u.user_id
              FROM %s AS s
              LEFT OUTER JOIN %s AS sg ON sg.surfergroup_id = s.surfergroup_id
              LEFT OUTER JOIN %s AS u ON u.user_id = s.auth_user_id
              WHERE surfer_id = '%d'";
    $params = array($this->tableSurfer, $this->tableGroups, $this->tableAuthuser, $id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->editSurfer = $row;
        return $this->editSurfer;
      }
    }
    return NULL;
  }

  /**
  * Save surfer
  *
  * @param mixed $params optional, default value NULL
  * @access public
  * @return boolean
  */
  function saveSurfer($params = NULL) {
    if (isset($params)) {
      $data = &$params;
    } else {
      $data = array(
        'surfer_handle' => $this->params['surfer_handle'],
        'surfergroup_id' => (int)$this->params['surfergroup_id'],
        'surfer_givenname' => $this->params['surfer_givenname'],
        'surfer_surname' => $this->params['surfer_surname'],
        'surfer_email' => $this->params['surfer_email'],
        'surfer_email_alt' => $this->params['surfer_email_alt'],
        'surfer_phone' => $this->params['surfer_phone'],
        'surfer_phone_alt' => $this->params['surfer_phone_alt'],
        'surfer_valid' => $this->params['surfer_valid']
      );
      if (isset($this->params['surfer_password']) && trim($this->params['surfer_password']) != '') {
        $data['surfer_password'] = md5($this->params['surfer_password']);
      }
    }
    if ($this->databaseUpdateRecord($this->tableSurfer, $data, 'surfer_id', $this->editSurfer['surfer_id']) !== FALSE) {
      $this->editSurfer = $this->loadSurfer($this->editSurfer['surfer_id']);
      if ($this->editSurfer['auth_user_id'] != '') {
        $data = array(
          'surname' => $this->editSurfer['surfer_surname'],
          'givenname' => $this->editSurfer['surfer_givenname'],
          'email' => $this->editSurfer['surfer_email'],
          'user_password' => $this->editSurfer['surfer_password'],
          'username' => $this->editSurfer['surfer_handle']
        );
        return ($this->databaseUpdateRecord($this->tableAuthuser, $data, 'user_id', $this->editSurfer['auth_user_id']) !== FALSE);
      } else {
        return TRUE;
      }
    } else {
      return FALSE;
    }
  }

  /**
  * Insert surfer
  *
  * @access public
  * @return boolean
  */
  function insertSurfer() {
    $data = array(
      'surfer_handle' => $this->params['surfer_handle'],
      'surfergroup_id' => (int)$this->params['surfergroup_id'],
      'surfer_givenname' => $this->params['surfer_givenname'],
      'surfer_surname' => $this->params['surfer_surname'],
      'surfer_email' => $this->params['surfer_email'],
      'surfer_password' => md5($this->params['surfer_password']),
      'surfer_email_alt' => $this->params['surfer_email_alt'],
      'surfer_phone' => $this->params['surfer_phone'],
      'surfer_phone_alt' => $this->params['surfer_phone_alt']
    );
    if (isset($this->params['surfer_password']) && trim($this->params['surfer_password']) != '') {
      $data['surfer_valid'] = $this->params['surfer_valid'];
    } else {
      $data['surfer_valid'] = FALSE;
    }
    return $this->databaseInsertRecord($this->tableSurfer, 'surfer_id', $data);
  }

  /**
  * Load list
  *
  * @access public
  * @return boolean
  */
  function loadList(){
    $success = TRUE;
    unset($this->surfers);
    if(isset($this->params['patt']) && $this->params['patt']){
      $replaceChars = array('%' => '\%', '_' => '\_', '*' => '%', '?' => '_');
      $patt = strtr($this->params['patt'], $replaceChars);
    } else {
      $patt = "%";
    }
    $sql = "SELECT surfer_id, surfer_handle, surfer_givenname, surfer_surname
              FROM %s
              WHERE surfer_handle LIKE '%s%%'
              ORDER BY surfer_handle";
    $params=array($this->tableSurfer, $patt);
    if ($res = $this->databaseQueryFmt($sql, $params, $this->listLength, @(int)$this->params['offset'])) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->surfers[(int)$row['surfer_id']] = $row;
      }
      $this->surferCount = $res->absCount();
      return $this->loadPermList();
    }
    return $success;
  }

  /**
  * Load permission list
  *
  * @access public
  * @return boolean
  */
  function loadPermList() {
    unset($this->permissionsList);
    $sql = "SELECT surferperm_id, surferperm_title, surferperm_active
              FROM %s
            ORDER BY surferperm_title ASC, surferperm_id DESC";
    if ($res = $this->databaseQueryFmt($sql, array($this->tablePerm))) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->permissionsList[$row['surferperm_id']] = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load permissions
  *
  * @param integer $id
  * @access public
  * @return mixed NULL or array
  */
  function loadPerm($id) {
    if (isset($this->permissionsList) && is_array($this->permissionsList) && isset($this->permissionsList[$id])) {
      return $this->permissionsList[$id];
    } else {
      $sql = "SELECT surferperm_id, surferperm_title, surferperm_active
                FROM %s
                WHERE surferperm_id = '%d'";
      if ($res = $this->databaseQueryFmt($sql, array($this->tablePerm, $id))) {
        if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          return $row;
        }
      }
    }
    return NULL;
  }


  /**
  * Save permissions
  *
  * @access public
  * @return mixed
  */
  function savePerm() {
    $data = array(
      'surferperm_title' => $this->params['surferperm_title'],
      'surferperm_active' => $this->params['surferperm_active']
    );
    return ($this->databaseUpdateRecord($this->tablePerm, $data, 'surferperm_id', $this->editPerm['surferperm_id']) !== FALSE);
  }

  /**
  * Permission exists ?
  *
  * @param $title
  * @param $id
  * @access public
  * @return
  */
  function permExists($title, $id) {
    $sql = "SELECT surferperm_id
              FROM %s
              WHERE surferperm_title LIKE '%s'";
    if ($res = $this->databaseQueryFmt($sql, array($this->tablePerm, $title))) {
      if ($row = $res->fetchRow()) {
        return ($row[0] != $id);
      }
    }
    return FALSE;
  }

  /**
  * Load group
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function loadGroup($id) {
    unset($this->surferGroup);
    if (isset($this->groupList[$id]) && is_array($this->groupList[$id])) {
      $this->surferGroup = $this->groupList[$id];
      return TRUE;
    } else {
      $sql = "SELECT surfergroup_id, surfergroup_title
                FROM %s
                WHERE surfergroup_id = '%d'";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableGroups, $id))) {
        if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->surferGroup = $row;
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  /**
  * Load groups
  *
  * @access public
  * @return boolean
  */
  function loadGroups() {
    unset($this->groupList);
    $sql = "SELECT surfergroup_id, surfergroup_title
              FROM %s
            ORDER BY surfergroup_title";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableGroups))) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->groupList[$row['surfergroup_id']] = $row;
      }
      $res->free();
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Add group
  *
  * @access public
  * @return mixed
  */
  function addGroup() {
    return $this->databaseInsertRecord($this->tableGroups, 'surfergroup_id', array('surfergroup_title'=>$this->_gt('New group')));
  }

  /**
  * Save group
  *
  * @access public
  * @return mixed
  */
  function saveGroup() {
    $data = array('surfergroup_title'=>$this->params['surfergroup_title']);
    return ($this->databaseUpdateRecord($this->tableGroups, $data, 'surfergroup_id', $this->surferGroup['surfergroup_id']) !== FALSE);
  }

  /**
  * Delete group
  *
  * @param integer $groupId
  * @access public
  * @return boolean
  */
  function deleteGroup($groupId) {
    if ($this->databaseDeleteRecord($this->tableLink, 'surfergroup_id', $groupId) !== FALSE) {
      if ($this->databaseDeleteRecord($this->tableGroups, 'surfergroup_id', $groupId) !== FALSE) {
        $this->addMsg(MSG_INFO, $this->_gt('Permission links and group deleted.'));
        return TRUE;
      } else {
        $this->addMsg(MSG_ERROR, $this->_gt('Permission links deleted! Could not delete group.'));
      }
    }
    return FALSE;
  }

  /**
  * Delete permissions
  *
  * @param integer $permId
  * @access public
  * @return boolean
  */
  function deletePerm($permId) {
    if ($this->databaseDeleteRecord($this->tableLink, 'surfer_permid', $permId) !== FALSE) {
      if ($this->databaseDeleteRecord($this->tablePerm, 'surferperm_id', $permId) !== FALSE) {
        return TRUE;
      }
    }
  }

  /**
  * Delete surfer
  *
  * @param integer $surferId
  * @access public
  * @return boolean
  */
  function deleteSurfer($surferId) {
    return ($this->databaseDeleteRecord($this->tableSurfer, 'surfer_id', $surferId) !== FALSE);
  }

  /**
  * Load links
  *
  * @param integer $groupId
  * @access public
  */
  function loadLinks($groupId){
    unset($this->linkList);
    $sql = "SELECT surfer_permid
              FROM %s
              WHERE surfergroup_id = '%d'";
    $params = array($this->tableLink, $groupId);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->linkList[$row['surfer_permid']] = TRUE;
      }
    }
  }

  /**
  * Basic function for handing parameters
  *
  * @access public
  */
  function execute(){
    $this->loadList();
    if (isset($this->params['cmd'])){
      switch (trim($this->params['cmd'])){
        case 'export':
          if ($this->module->hasPerm(3, TRUE)) {
            $this->exportSurfers();
          }
        case 'edit':
          if (isset($this->params['id']) && ($this->params['id'] > 0)) {
            $this->editSurfer = $this->loadSurfer($this->params['id']);
            if (isset($this->editSurfer) && is_array($this->editSurfer)) {
              $this->initializeSurferForm($this->editSurfer);
              if ($this->surferDialog->modified()) {
                if ($this->surferDialog->checkDialogInput() && $this->checkInputs()) {
                  if ($this->saveSurfer()) {
                    $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('surfer')));
                  }
                }
              }
            }
          }
          break;
        case 'add':
          $this->initializeSurferForm(NULL, 'add');
          if($this->surferDialog->checkDialogInput() && $this->checkInputs()){
            if($newId = $this->insertSurfer()) {
              $this->params['id'] = $newId;
              $this->editSurfer = $this->loadSurfer($newId);
              $this->initializeSurferForm($this->editSurfer, 'edit', TRUE);
              $this->addMsg(MSG_INFO, $this->_gt("New surfer created."));
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt("Database error!"));
            }
          }
          break;
        case 'delete' :
            if (isset($this->params['id']) && $this->params['id'] > 0 &&
              isset($this->params['confirm_delete'])) {
            $this->editSurfer = $this->loadSurfer($this->params['id']);
            if (isset($this->editSurfer) && $this->editSurfer['surfer_id']) {
              if ($this->deleteSurfer($this->params['id'])) {
                $this->addMsg(MSG_INFO, $this->_gt('Surfer deleted.'));
                unset($this->editSurfer);
                unset($this->params['id']);
              } else {
                $this->addMsg(MSG_ERROR, $this->_gt('Database error!'));
              }
            }
          }
          break;
        case 'add_group' :
          if ($newId = $this->addGroup()) {
            $this->params['group_id'] = $newId;
            $this->addMsg(MSG_INFO, $this->_gt("New group created."));
          }
          break;
        case 'chg_group' :
          if (isset($this->params['group_id']) && ($this->params['group_id'] > 0)) {
            $this->loadGroup($this->params['group_id']);
            if (isset($this->surferGroup) && is_array($this->surferGroup)) {
              $this->initializeGroupDialog($this->surferGroup);
              if ($this->groupDialog->modified()) {
                if($this->groupDialog->checkDialogInput()) {
                  if ($this->saveGroup()) {
                    $this->addMsg(MSG_INFO, sprintf($this->_gt("%s modified."), $this->_gt('Group')));
                  } else {
                    $this->addMsg(MSG_ERROR, $this->_gt("Database error!"));
                  }
                }
              }
            }
          }
          break;
        case 'del_group' :
            if (isset($this->params['group_id']) && $this->params['group_id'] > 0 &&
              isset($this->params['confirm_delete'])) {
            if ($this->deleteGroup($this->params['group_id'])) {
              $this->addMsg(MSG_INFO, $this->_gt('Group deleted.'));
              unset($this->params['group_id']);
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error!'));
            }
          }
          break;
        case 'chg_perm':
          if (isset($this->params['perm_id']) && ($this->params['perm_id'] > 0)) {
            $this->editPerm = $this->loadPerm($this->params['perm_id']);
            if (isset($this->editPerm) && is_array($this->editPerm)) {
              $this->initializePermDialog($this->editPerm);
              if ($this->permDialog->modified()) {
                if ($this->permDialog->checkDialogInput() && $this->checkPermInputs()) {
                  if ($this->savePerm()) {
                    $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Permission')));
                  }
                }
              }
            }
          }
          break;
        case 'addlink' :
          //Recht verknpfen
          if (isset($this->params['group_id']) && $this->params['group_id'] > 0 &&
              isset($this->params['perm_id']) && $this->params['perm_id'] > 0) {
            $sql = "REPLACE INTO %s (surfer_permid, surfergroup_id) VALUES ('%d', '%d')";
            $params = array($this->tableLink, $this->params['perm_id'], $this->params['group_id']);
            if ($this->databaseQueryFmt($sql, $params) !== FALSE)  {
              $this->addMsg(MSG_INFO, $this->_gt("Permission linked!"));
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error!'));
            }
          }
          break;
        case 'dellink' :
          //Verknpfung lsen
          if (isset($this->params['group_id']) && $this->params['group_id'] > 0 &&
              isset($this->params['perm_id']) && $this->params['perm_id'] > 0) {
            $filter = array('surfer_permid' => (int)$this->params['perm_id'], 'surfergroup_id' => (int)$this->params['group_id']);
            if ($this->databaseDeleteRecord($this->tableLink, $filter) !== FALSE)  {
              $this->addMsg(MSG_INFO, $this->_gt("Link deleted!"));
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error!'));
            }
          }
          break;
        case 'perm_add':
          if ($newId = $this->databaseInsertRecord($this->tablePerm, 'surferperm_id',
            array('surferperm_title' => $this->_gt('New permission'), 'surferperm_active' => FALSE))) {
            $this->addMsg(MSG_INFO, $this->_gt("Permission added!"));
            $this->params['perm_id'] = $newId;
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error!'));
          }
          break;
        case 'perm_del' :
            if (isset($this->params['perm_id']) && $this->params['perm_id'] > 0 &&
              isset($this->params['confirm_delete'])) {
            if ($this->deletePerm($this->params['perm_id'])) {
              $this->addMsg(MSG_INFO, $this->_gt('Permission deleted.'));
              unset($this->params['perm_id']);
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error!'));
            }
          }
          break;
      }//switch
    }//if ($this->params['cmd'])

    $this->loadList();
    $this->loadGroups();
    if (isset($this->params['group_id']) && $this->params['group_id'] > 0) {
      $this->loadGroup($this->params['group_id']);
    }
    if (isset($this->params['id']) && $this->params['id'] > 0) {
      $this->editSurfer = $this->loadSurfer($this->params['id']);
      if ($this->editSurfer['auth_user_id'] && (!$this->editSurfer['user_id'])) {
        $this->params['cmd'] = 'delete';
      }
    }
    if (isset($this->params['perm_id']) && $this->params['perm_id'] > 0) {
      $this->editPerm = $this->loadPerm($this->params['perm_id']);
    }
    if($this->params['mode'] == 0) {
      $this->loadList();
    }
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  /**
  * Check permission inputs
  *
  * @access public
  * @return boolean
  */
  function checkPermInputs(){
    $result = TRUE;
    if ($this->permExists($this->params['surferperm_title'], $this->editPerm['surferperm_id'])) {
      $result = FALSE;
      $this->addMsg(MSG_ERROR, "Permission title already in use.");
    }
    return $result;
  }

  /**
  * Check inputs
  *
  * @access public
  * @return boolean
  */
  function checkInputs() {
    $result = TRUE;
    if ($this->params['surfer_password'] != $this->params['surfer_password2']) {
      $result = FALSE;
      $this->addMsg(MSG_ERROR, $this->_gt('Password inputs didn\'t matched.'));
    }
    if ($this->editSurfer['surfer_handle'] != $this->params['surfer_handle'] &&
        ($this->getIdByHandle($this->params['surfer_handle']) > 0) ) {
      $result = FALSE;
      $this->addMsg(MSG_ERROR, $this->_gt('Login name already in use.'));
    }
    return $result;
  }

  /**
  * Get
  *
  * @param object xsl_layout &$layout
  * @access public
  */
  function get(&$layout){
    if ($this->params['mode'] == 1) {
      $layout->addLeft($this->getGroupList());
      if (@$this->params['cmd'] == 'del_group') {
        $this->getDelGroupForm($layout);
      } elseif (@$this->params['cmd'] == 'perm_del') {
        $this->getDelPermForm($layout);
      }

      if(isset($this->surferGroup) && is_array($this->surferGroup)) {
        $this->layout->add($this->getGroupEdit());
      } elseif (isset($this->params['perm_id']) && ($this->params['perm_id'] > 0)){
        if (!(isset($this->editPerm) && is_array($this->editPerm))) {
          $this->editPerm = $this->permissionsList[$this->params['perm_id']];
        }
        $layout->add($this->editPerm($this->editPerm));
      }
      $layout->addRight($this->permList());
    } else {
      if (@$this->params['cmd'] == 'delete') {
        $this->getDelSurferForm($layout);
      } elseif (@$this->params['cmd'] == 'perm_del') {
        $this->getDelPermForm($layout);
      }

      if (isset($this->params['id']) && (!is_array($this->editSurfer)) && ($this->params['id'] == -1)){
        $this->editSurfer = $this->getNewSurferData();
      } else if (isset($this->params['id']) && !is_array($this->editSurfer) && ($this->params['id'] > 0)){
        $this->editSurfer = $this->loadSurfer($this->params['id']);
      }
      $layout->addLeft($this->surferList());

      if (isset($this->editSurfer) && is_array($this->editSurfer)){
        $layout->add($this->editSurfer($this->editSurfer));
      } elseif (isset($this->params['perm_id']) && ($this->params['perm_id'] > 0)){
        if (!(isset($this->editPerm) && is_array($this->editPerm))) {
          $this->editPerm = $this->permissionsList[$this->params['perm_id']];
        }
        $layout->add($this->editPerm($this->editPerm));
      }
      $layout->addRight($this->permList());
    }
  }

  /**
  * Get group list
  *
  * @access public
  * @return string
  */
  function getGroupList() {
    $result = '';
    if (isset($this->groupList) && is_array($this->groupList)) {
      $result .= '<listview title="'.$this->_gt('Groups').'" width="100%">'.LF;
      $result .= '<items>'.LF;
      foreach($this->groupList as $groupId=>$group) {
        $selected = ($groupId == @$this->params['group_id']) ? ' selected="selected"' : '';
        $href = sprintf('%s?%s[group_id]=%d&amp;%s[offset]=%s', $this->baseLink,
          $this->paramName, @(int)$groupId,
          $this->paramName, @(int)$this->params['offset']);
        $result .= sprintf('<listitem title="%s" href="%s" image="%s" %s/>',
          $group['surfergroup_title'], $href, $this->images[23], $selected);
      }
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
    }
    return $result;
  }

  /**
  * Initialize group dialog
  *
  * @param array $group
  * @access public
  */
  function initializeGroupDialog($group) {
    if (!(isset($this->groupDialog) && is_object($this->groupDialog))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array(
        'save'=> 1,
        'cmd'=> 'chg_group',
        'group_id' => $group['surfergroup_id'],
        'offset'=> $this->params['offset']);
      $data = array(
        'surfergroup_title' => $group['surfergroup_title'],
      );
      $fields = array(
        'surfergroup_title' => array('Title', 'isNoHTML', TRUE, 'input', 100, '', $this->_gt('New')),
      );
      $this->groupDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->groupDialog->dialogTitle = $this->_gt('Change group');
      $this->groupDialog->baseLink = $this->baseLink;
      $this->groupDialog->msgs = &$this->msgs;
      $this->groupDialog->loadParams();
    }
  }

  /**
  * Get group edit formular
  *
  * @access public
  * @return string
  */
  function getGroupEdit() {
    if (isset($this->surferGroup) && is_array($this->surferGroup))
    {
      $this->initializeGroupDialog($this->surferGroup);
      return $this->groupDialog->getDialogXML();
    }
    return '';
  }

  /**
  * Get delete group formular
  *
  * @param object xsl_layout &$layout
  * @access public
  */
  function getDelGroupForm(&$layout) {
    if ((!isset($this->params['confirm_delete'])) && @is_array($this->surferGroup)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'del_group',
        'group_id' => $this->surferGroup['surfergroup_id'],
        'confirm_delete'=>1,
      );
      $msg = sprintf($this->_gt('Delete group "%s" (%s)?'),
        $this->surferGroup['surfergroup_title'], $this->surferGroup['surfergroup_id']);
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Delete';
      $dialog->baseLink = $this->baseLink;
      $layout->add($dialog->getMsgDialog());
    }
  }

  /**
  * Get delete surfer formular
  *
  * @param object xsl_layout &$layout
  * @access public
  */
  function getDelSurferForm(&$layout) {
    if ((!isset($this->params['confirm_delete'])) && @is_array($this->editSurfer)) {
      if ($this->editSurfer['auth_user_id'] != '') {
        $this->addMsg(MSG_WARNING, $this->_gt('You can not delete system users.'));
      } else {
        include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
        $hidden = array(
          'cmd' => 'delete',
          'id' => $this->editSurfer['surfer_id'],
          'confirm_delete'=>1,
        );
        $msg = sprintf($this->_gt('Delete surfer "%s %s" (%s)?'),
          htmlspecialchars($this->editSurfer['surfer_givenname']),
          htmlspecialchars($this->editSurfer['surfer_surname']),
          (int)$this->editSurfer['surfer_id']);
        $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
        $dialog->msgs = &$this->msgs;
        $dialog->buttonTitle = 'Delete';
        $dialog->baseLink = $this->baseLink;
        $layout->add($dialog->getMsgDialog());
      }
    }
  }

  /**
  * Get delete permission formular
  *
  * @param object xsl_layout &$layout
  * @access public
  */
  function getDelPermForm(&$layout) {
    if ((!isset($this->params['confirm_delete'])) && @is_array($this->editPerm)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'perm_del',
        'perm_id' => $this->editPerm['surferperm_id'],
        'confirm_delete'=>1,
      );
      $msg = sprintf($this->_gt('Delete permission "%s" (%s)?'),
        $this->editPerm['surferperm_title'], $this->editPerm['surferperm_id']);
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Delete';
      $dialog->baseLink = $this->baseLink;
      $layout->add($dialog->getMsgDialog());
    }
  }

  /**
  * Initialize permission dialog
  *
  * @param array $perm
  * @access public
  */
  function initializePermDialog($perm) {
    if (!(isset($this->permDialog) && is_object($this->permDialog))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array(
        'save'=> 1,
        'cmd'=> 'chg_perm',
        'perm_id' => $perm['surferperm_id']
      );
      $data = array(
        'surferperm_title' => $perm['surferperm_title'],
        'surferperm_active' => $perm['surferperm_active'],
      );
      $fields = array(
        'surferperm_title' => array('Title', 'isNoHTML', TRUE, 'input', 100, '', $this->_gt('New Permission')),
        'surferperm_active' => array('Active', 'isNum', TRUE, 'yesno',NULL, '', 0, 'center')
      );
      $this->permDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->permDialog->dialogTitle = $this->_gt('Change permission');
      $this->permDialog->baseLink = $this->baseLink;
      $this->permDialog->msgs = &$this->msgs;
      $this->permDialog->loadParams();
    }
  }

  /**
  * Edit permissions
  *
  * @param array $perm
  * @access public
  * @return string
  */
  function editPerm($perm) {
    $this->initializePermDialog($perm);
    return $this->permDialog->getDialogXML();
  }

  /**
  * Initialize surfer formular
  *
  * @param array $surfer
  * @param string $mode optional, default value 'edit'
  * @param boolean $force optional, default value FALSE
  * @access public
  */
  function initializeSurferForm($surfer, $mode = 'edit', $force = FALSE) {
    if ($force && isset($this->surferDialog)) unset($this->surferDialog);
    if (!(isset($this->surferDialog) && is_object($this->surferDialog))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array(
        'save'=> 1,
        'cmd'=> $mode,
        'id' => $surfer['surfer_id'],
        'patt'=> $this->params['patt'],
        'offset'=> $this->params['offset']);
      if (isset($surfer) && $mode == 'edit') {
        $data = array(
          'surfer_handle'    => @$surfer['surfer_handle'],
          'surfergroup_id'   => @$surfer['surfergroup_id'],
          'surfer_givenname' => @$surfer['surfer_givenname'],
          'surfer_surname'   => @$surfer['surfer_surname'],
          'surfer_email'     => @$surfer['surfer_email'],
          'surfer_email_alt' => @$surfer['surfer_email_alt'],
          'surfer_phone'     => @$surfer['surfer_phone'],
          'surfer_phone_alt' => @$surfer['surfer_phone_alt'],
          'surfer_valid'     => @$surfer['surfer_valid'],
          'surfer_lastlogin' => date('Y-m-d H:i:s', @$surfer['surfer_lastlogin'])
        );
      } else {
        $data = array();
      }
      $fields = array(
        'surfer_handle' => array('Login', 'isNoHTML', TRUE, 'input', 200, '', $this->_gt('New')),
        'surfergroup_id' => array('Group', 'isNum', TRUE, 'function', 'getGroupCombo', '', 0),
        'surfer_valid' => array('Active', 'isNum', TRUE, 'yesno', NULL, '', 0, 'left'),
        'surfer_givenname' => array('Givenname', 'isNoHTML', TRUE, 'input', 200, ''),
        'surfer_surname' => array('Surname', 'isNoHTML', TRUE, 'input', 200, ''),
        'surfer_email' => array('Email', 'isEMail', TRUE, 'input', 200, ''),
        'Password',
        'surfer_password' => array('Password', 'isNoHTML', FALSE, 'password', 20),
        'surfer_password2' => array('Repetition', 'isNoHTML', FALSE, 'password', 20),
        'Additional data',
        'surfer_email_alt' => array('Additional Email', 'isEMail', FALSE, 'input', 30),
        'surfer_phone' => array('Phone', 'isPhone', FALSE, 'input', 30),
        'surfer_phone_alt' => array('Additional Phone', 'isPhone', FALSE, 'input', 30),
        'surfer_lastlogin' => array('Last Login', 'isISODateTime', FALSE, 'disabled_input', 30)
      );
      $this->surferDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->surferDialog->baseLink = $this->baseLink;
      $this->surferDialog->msgs = &$this->msgs;
      $this->surferDialog->loadParams();
    }
  }

  /**
  * Get group combo
  *
  * @param string $name
  * @param $field
  * @param $data
  * @access public
  * @return string
  */
  function getGroupCombo($name, $field, $data) {
    $result = sprintf('<select name="%s[%s]" class="%s">'.LF,
      $this->paramName, $name, 'input');
    if (isset($this->groupList) && is_array($this->groupList)) {
      foreach($this->groupList as $groupId=>$group) {
        $selected = ($groupId == @$this->editSurfer['surfergroup_id']) ? ' selected="selected"' : '';
        $result .= sprintf('<option value="%d"%s>%s</option>', $groupId, $selected,
          htmlspecialchars($group['surfergroup_title']));
      }
    }
    $result .= '</select>'.LF;
    return $result;
  }

  /**
  * Edit surfer
  *
  * @param array $surfer
  * @access public
  * @return string
  */
  function editSurfer($surfer){
    $result = "";
    if ($surfer['surfer_id'] < 0 && (!$this->delFlag)) {
      $mode = 'add';
    } elseif(!$this->delFlag) {
      $mode = 'edit';
    } else {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'reallydel',
        'id' => $surfer['surfer_id'],
        'patt'=> $this->params['patt'],
        'offset'=> $this->params['offset']
      );
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $this->_gtf('Delete surfer "%s"?', array($serf['surfer_handle'])), 'question');
      $dialog->msgs = &$this->msgs;
      $dialog->baseLink = $this->baseLink;
      $dialog->buttonTitle = 'Delete';
      return $dialog->getMsgDialog();
    }
    $this->initializeSurferForm($surfer, $mode);
    $this->surferDialog->buttonTitle = ($mode == 'add') ? 'Add' : 'Save';
    $this->surferDialog->dialogTitle = $this->_gt(($mode == 'add') ? 'Add' : 'Change');
    return $this->surferDialog->getDialogXML();
  }

  /**
  * Get new surfer data
  *
  * @access public
  * @return array
  */
  function getNewSurferData()
  {
      return array("surfer_id" => '-1',
                  "surfer_handle" => $this->_gt('New surfer'),
                  "surfer_password" => '',
                  "surfer_givenname" => '',
                  "surfer_surname" => '',
                  "surfer_email" => '',
                  "surfer_valid" => 0);
  }

  /**
  * Permission list
  *
  * @access public
  * @return string
  */
  function permList(){
    $result = "";
    if (isset($this->permissionsList) && is_array($this->permissionsList)){
      if ((isset($this->editSurfer) && is_array($this->editSurfer))) {
        if (!is_array($this->linkList)){
          $this->loadLinks(@$this->editSurfer['surfergroup_id']);
        }
        $mode = 'show';
      } else if ((isset($this->surferGroup) && is_array($this->surferGroup)) && (!isset($this->editSurfer))) {
        if (!is_array($this->linkList)){
          $this->loadLinks($this->surferGroup['surfergroup_id']);
        }
        $mode = 'link';
      } else {
        $mode = 'edit';
      }
      $result .= '<listview title="'.$this->_gt('Permissions').'" width="300">';
      $result .= '<cols>';
      $result .= '<col>'.$this->_gt('Name').'</col>';
      $result .= '<col align="center">'.$this->_gt('Activated').'</col>';
      if ($mode != 'edit') {
        $result .= '<col align="center">'.$this->_gt('Linked').'</col>';
      }
      $result .= '</cols>';
      $result .= '<items>';
      foreach($this->permissionsList as $permId=>$perm){
        $selected = ($permId == @$this->params['perm_id']) ? ' selected="selected"' : '';
        $href = sprintf('%s?%s[offset]=%s&amp;%s[perm_id]=%s', $this->baseLink,
          $this->paramName, @(int)$this->params['offset'],
          $this->paramName, @(int)$perm['surferperm_id']);
        $result .= sprintf('<listitem title="%s" href="%s" image="%s" %s>',
          $perm['surferperm_title'], $href, $this->images[26], $selected);
        switch ($mode) {
          case 'show' :
            $result .= '<subitem align="center">';
            if ($perm['surferperm_active']) {
              $result .= sprintf('<glyph src="%s" />', $this->images[29]);
            } else {
              $result .= sprintf('<glyph src="%s" />', $this->images[30]);
            }
            $result .= '</subitem>';
            $result .= '<subitem align="center">';
            if (isset($this->linkList[$perm['surferperm_id']])) {
              $result .= sprintf('<glyph src="%s" />', $this->images[29]);
            } else {
              $result .= sprintf('<glyph src="%s" />', $this->images[30]);
            }
            $result .= '</subitem>';
            break;
          case 'link' :
            $result .= '<subitem align="center">';
            if ($perm['surferperm_active']) {
              $result .= sprintf('<glyph src="%s" />', $this->images[29]);
            } else {
              $result .= sprintf('<glyph src="%s" />', $this->images[30]);
            }
            $result .= '</subitem>';
            $result .= '<subitem align="center">';
            if (isset($this->linkList[$perm['surferperm_id']])) {
              $href .= sprintf('&amp;%s[cmd]=%s&amp;%s[group_id]=%s',
                $this->paramName, 'dellink', $this->paramName, $this->surferGroup['surfergroup_id']);
              $result .= sprintf('<a href="%s"><glyph src="%s" /></a>', $href, $this->images[27]);
            } else {
              $href .= sprintf('&amp;%s[cmd]=%s&amp;%s[group_id]=%s',
                $this->paramName, 'addlink', $this->paramName, $this->surferGroup['surfergroup_id']);
              $result .= sprintf('<a href="%s"><glyph src="%s" /></a>', $href, $this->images[28]);
            }
            $result .= '</subitem>';
            break;
          case 'edit' :
            $result .= '<subitem align="center">';
            if ($perm['surferperm_active']) {
              $result .= sprintf('<glyph src="%s" />', $this->images[29]);
            } else {
              $result .= sprintf('<glyph src="%s" />', $this->images[30]);
            }
            $result .= '</subitem>';
            break;
        }
        $result .= '</listitem>';
      }
      $result .= '</items>';
      $result .= '</listview>';
    }
    return $result;
  }

  /**
  * Get pages
  *
  * @param array $list
  * @param mixed $numIncs optional, default value NULL
  * @param integer $numEls optional, default value 7
  * @access public
  * @return array
  */
  function getPages($list, $numIncs=NULL, $numEls=7){
    unset($incs);
    $sorted = $list;
    sort($sorted);
    $num = count($sorted);
    if(!$numIncs){
      $numIncs = ceil($num / $numEls);
    }
    for ($i = 0; $i < $numIncs; $i++){
      $incs[] = array('start' => $sorted[$i * floor($num / $numIncs)],
                      'end' => $sorted[(($i+1) * floor($num / $numIncs))-1]);
    }
    return $incs;
  }

  /**
  * Get character buttons
  *
  * @access public
  * @return string
  */
  function getCharBtns() {
    $result="";
    $chars = 'abcdefghijklmnopqrstuvwxyz';
    $charCount = strlen($chars);
    for ($i = 0; $i < $charCount; $i++) {
      $result .= sprintf('<a href="%s?%s[patt]=%s*">%s</a> '.LF,
        $this->baseLink, $this->paramName, $chars[$i], $chars[$i]);
      if ($i == 13) $result .= '<br />';
    }
    $result .= sprintf('<br /><a href="%s?%s[patt]=">%s</a> '.LF,
        $this->baseLink, $this->paramName, $this->_gt('All'));
    return $result;
  }

  /**
  * Surfer list
  *
  * @access public
  * @return string
  */
  function surferList(){
    $result = "";
    $result .= sprintf('<dialog title="%s" action="%s" width="200">'.LF, $this->_gt('Search'), $this->baseLink);
    $result .= '<lines>'.LF;
    $result .= '<line align="center">'.$this->getCharBtns().'</line>'.LF;
    $result .= '<line align="center">';
    $result .= sprintf('<input type="text" class="smallinput" name="%s[patt]" value="%s" /></line>'.LF,
      $this->paramName, htmlspecialchars(@$this->params['patt']));
    $result .= '</lines>'.LF;
    $result .= sprintf('<dlgbutton value="%s" />'.LF, $this->_gt('Search'));
    $result .= '</dialog>'.LF;
    if (isset($this->surfers) && is_array($this->surfers)){
      $shortSurf = $this->surfers;
      $listLen = $this->listLength;
      $numSurf = count($this->surfers);
      if ($numSurf > $listLen){
        $shortSurf = array_slice($this->surfers, @(int)$this->params['offset'], $listLen);
        $result .= '<panel title="'.$this->_gt('Pages').'" width="200">'.$this->getPageBtns().'</panel>'.LF;
      }
      $result .= '<listview title="'.$this->_gt('Surfer').'" width="200">'.LF;
      $result .= '<items>'.LF;
      foreach($shortSurf as $surfer){
        $selected = ($surfer['surfer_id'] == @$this->editSurfer['surfer_id']) ? ' selected="selected"' : '';
        $href = sprintf('%s?%s[patt]=%s&amp;%s[offset]=%s&amp;%s[id]=%d',
          $this->baseLink, $this->paramName, @$this->params['patt'],
          $this->paramName, @(int)$this->params['offset'], $this->paramName, (int)$surfer['surfer_id']);
        $result .= sprintf('<listitem href="%s" title="%s" hint="%s %s" image="%s" %s/>',
          $href, htmlspecialchars($surfer['surfer_handle']),
          htmlspecialchars($surfer['surfer_givenname']), htmlspecialchars($surfer['surfer_surname']),
          $this->images[24], $selected);
      }
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
    }
    return $result;
  }

  /**
  * Get pages navigation
  *
  * @param object base_btnbuilder &$toolbar
  * @param integer $offset
  * @param integer $step
  * @param integer $max
  * @access public
  */
  function getPagesNav(&$toolbar, $offset, $step, $max) {
    if (($max > $step) && is_object($toolbar)) {
      if ($offset > 0) {
        $i = (($offset - $step) > 0) ? ($offset - $step) : 0;
        $toolbar->addButton('Prior',
          sprintf('%s?%s[cmd]=%s&amp;%s[patt]=%s&amp;%s[id]=0&amp;%s[offset]=%d',
          $this->baseLink, $this->paramName, 'show', $this->paramName, $this->params['patt'],
          $this->paramName, $this->paramName, $i), 8, '');
      }
      for($i=0,$x=1;$i<$max;$i+=$step,$x++) {
        $pages[$i] = $x;
      }
      $toolbar->addCombo('Pages',
          sprintf('%s?%s[cmd]=%s&amp;%s[patt]=%s&amp;%s[id]=0',
          $this->baseLink, $this->paramName, 'show', $this->paramName, $this->params['patt'],
          $this->paramName),
        $this->paramName.'[offset]', $offset, $pages);
      if ($offset < ($max-$step)) {
        $i = (($offset + $step) < $max) ? ($offset + $step) : ($max-$step);
        $toolbar->addButton('Next',
          sprintf('%s?%s[cmd]=%s&amp;%s[patt]=%s&amp;%s[id]=0&amp;%s[offset]=%d',
          $this->baseLink, $this->paramName, 'show', $this->paramName, $this->params['patt'],
          $this->paramName, $this->paramName, $i), 9, '');
      }
    }
  }

  /**
  * Get buttons
  *
  * @access public
  */
  function getButtons() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $toolbar = &new base_btnbuilder;
    $toolbar->images = &$this->images;

    if ($this->params['mode'] == 1) {
      //Gruppen ndern
      $toolbar->addButton('Groups',
        $this->baseLink.'?'.$this->paramName.'[mode]=1',
        23, '', TRUE);
      $toolbar->addButton('Surfer',
        $this->baseLink.'?'.$this->paramName.'[mode]=0',
        24, '', FALSE);
      $toolbar->addSeperator();
      $toolbar->addButton('New Group',
        $this->baseLink.'?'.$this->paramName.'[cmd]=add_group',
        66, '', FALSE);
      if (isset($this->surferGroup) && is_array($this->surferGroup)) {
        $toolbar->addButton('Delete Group',
          $this->baseLink.'?'.$this->paramName.'[cmd]=del_group&amp;'.
            $this->paramName.'[group_id]='.(int)$this->surferGroup['surfergroup_id'],
          10, '', FALSE);
      }
      $toolbar->addSeperator();
    } else {
      $toolbar->addButton('Groups',
        $this->baseLink.'?'.$this->paramName.'[mode]=1',
        23, '', FALSE);
      $toolbar->addButton('Surfer',
        $this->baseLink.'?'.$this->paramName.'[mode]=0',
        24, '', TRUE);
      $toolbar->addSeperator();

      $toolbar->addButton('New surfer',
        $this->baseLink.'?'.$this->paramName.'[id]=-1',
        36, '', FALSE);

      if (isset($this->editSurfer['surfer_id']) && ($this->editSurfer['surfer_id'] > 0)) {
        $toolbar->addButton('Delete surfer',
          sprintf('%s?%s[cmd]=%s&amp;%s[patt]=%s&amp;%s[offset]=%s&amp;%s[id]=%d',
            $this->baseLink, $this->paramName, 'delete', $this->paramName, $this->params['patt'],
            $this->paramName, (int)$this->params['offset'], $this->paramName, $this->editSurfer['surfer_id']),
          10, '', FALSE);
      }
      $toolbar->addSeperator();
      $this->getPagesNav($toolbar, $this->params['offset'], $this->listLength, $this->surferCount);
      if ($this->module->hasPerm(3)) {
        $toolbar->addSeperator();
        $toolbar->addButton('Export surfers', sprintf('%s?%s[cmd]=export', $this->baseLink, $this->paramName),
          96, '', FALSE);
      }
    }
    $toolbar->addSeperator();
    $toolbar->addButton('New permission',
      $this->baseLink.'?'.$this->paramName.'[cmd]=perm_add', 37, '', FALSE);

    if (($this->editPerm['surferperm_id'] > -1)) {
      $toolbar->addButton('Delete permission',
        sprintf('%s?%s[cmd]=%s&amp;%s[perm_id]=%d',
          $this->baseLink, $this->paramName, 'perm_del', $this->paramName, $this->editPerm['surferperm_id']),
        10, '', FALSE);
    }
    if ($str = $toolbar->getXML()) {
      $this->layout->addMenu(sprintf('<menu ident="%s">%s</menu>'.LF, 'edit', $str));
    }
  }

  function exportSurfers() {
    $this->loadGroups();
    $sql = "SELECT surfer_id, surfer_handle,
                    surfer_givenname, surfer_surname,
                    surfer_email,
                    surfer_phone,
                    surfer_valid,
                    surfergroup_id,
                    surfer_lastlogin,
                    auth_user_id
              FROM %s
          ORDER BY surfer_surname, surfer_givenname, surfer_handle";
    if ($res = $this->databaseQueryFmt($sql, $this->tableSurfer)) {
      $agentString = strtolower(@$_SERVER["HTTP_USER_AGENT"]);
      if (strpos($agentString, 'opera') !== FALSE) {
        $agent =  'OPERA';
      } elseif (strpos($agentString, 'msie') !== FALSE) {
        $agent =  'IE';
      } else {
        $agent =  'STD';
      }
      $mimeType = ($agent == 'IE' || $agent == 'OPERA')
        ? 'application/octetstream'
        : 'application/octet-stream';
      $fileName = 'surfers_'.date('Y-m-d').'.csv';
      if ($agent == 'IE') {
        header('Content-Disposition: inline; filename="'.$fileName.'"');
      } else {
        header('Content-Disposition: attachment; filename="'.$fileName.'"');
      }
      header('Content-type: ' . $mimeType);
      echo 'Name,EMail,Phone,Login,Group,Active,LastLoginTime,Editor'."\r\n";
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        echo $this->escapeForCSV($row['surfer_givenname'].' '.$row['surfer_surname']).',';
        echo $this->escapeForCSV($row['surfer_email']).',';
        echo $this->escapeForCSV($row['surfer_phone']).',';
        echo $this->escapeForCSV($row['surfer_handle']).',';
        echo $this->escapeForCSV($this->groupList[$row['surfergroup_id']]['surfergroup_title']).',';
        echo (($row['surfer_valid'] != 0) ? 1 : 0).',';
        if ($row['surfer_lastlogin'] > 0) {
          echo $this->escapeForCSV(date('Y-m-d H:i:s', $row['surfer_lastlogin'])).',';
        } else {
          echo $this->escapeForCSV(' ').',';
        }
        echo (($row['auth_user_id'] != '') ? 1 : 0)."\r\n";
      }
      exit;
    }
  }

  function escapeForCSV($str) {
    if (strpos($str, ' ') !== FALSE || strpos($str, '"') !== FALSE || strpos($str, ',') !== FALSE) {
      return '"'.str_replace('"', '""', $str).'"';
    } else {
      return $str;
    }
  }
}

?>