<?php
/**
* Page module - user profiles
*
* @package module_community
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: content_profile.php,v 1.4 2006/01/12 17:23:10 weinert Exp $
*/

/**
* Basic class page module
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_content.php');

/**
* Page module - user profiles
*
* Surfer profile data change formular
*
* @package module_community
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class content_profile extends base_content {

  var $tableSurfer = PAPAYA_DB_TBL_SURFER;
  var $tableLink = PAPAYA_DB_TBL_SURFERPERMLINK;

  var $baseSurfers = NULL;

  var $editFields = array(
    'title'=>array ('Title', 'isNoHTML', TRUE, 'input', 200, ''),
    'Messages',
    'Saved'=>array ('Saved', 'isNoHTML', TRUE, 'input', 200, ''),
    'Error_No_User'=>array ('No user', 'isNoHTML', TRUE, 'input', 200, ''),
    'Error_Input'=>array ('Input error', 'isNoHTML', TRUE, 'input', 200, ''),
    'Error_Password'=>array ('Wrong password', 'isNoHTML', TRUE, 'input', 200, ''),
    'Error_Database'=>array ('Database error', 'isNoHTML', TRUE, 'input', 200, '')
  );

  /**
  * content profile
  *
  * @param object base_topic &$topic
  * @access public
  */
  function __construct(&$topic) {
    $this->paramName = 'srg';
    parent::__construct($topic, $this->paramName);
    if(!is_object($this->baseSurfers)) {
      include_once(dirname(__FILE__).'/base_surfers.php');
      $this->baseSurfers = &new surfer_admin($this->msgs);
    }
  }

  /**
  * Get parsed data
  *
  * @access public
  * @return string
  */
  function getParsedData() {

    $this->initializeParams();
    $this->baseLink = $this->getBaseLink();

    $result = sprintf('<title>%s</title>'.LF, htmlspecialchars(@$this->data['title']));
    $result .= '<userdata>';
    if (is_object($GLOBALS['PAPAYA_SURFER']) && $GLOBALS['PAPAYA_SURFER']->isValid==TRUE) {
      $this->initializeOutputForm();
      if (isset($this->params['save']) && $this->params['save'] > 0 ) {
        if ($this->outputDialog->modified()) {
          if ($this->outputDialog->checkDialogInput()) {
            if ($msg = $this->checkProfileInput()) {
              $result .= '<message>'.htmlspecialchars($msg).'</message>';
            } else {
              if (md5($this->params['surfer_old_password']) == $GLOBALS['PAPAYA_SURFER']->surfer['surfer_password']) {
                if ($this->saveProfileData()) {
                  $result .= '<message>'.htmlspecialchars(@$this->data['Saved']).'</message>';
                } else {
                  $result .= '<message>'.htmlspecialchars(@$this->data['Error_Database']).'</message>';
                }
              } else {
                $this->outputDialog->inputErrors['surfer_old_password'] = 1;
                $result .= '<message>'.htmlspecialchars(@$this->data['Error_Password']).'</message>';
              }
            }
          } else {
            $result .= '<message>'.htmlspecialchars(@$this->data['Error_Input']).'</message>';
          }
        }
      }
      $result.= $this->getOutputForm();
    } else {
      $result .= '<message>'.htmlspecialchars(@$this->data['Error_No_User']).'</message>';
    }
    $result .='</userdata>';
    return $result;
  }

  /**
  * Get output formular - Create XML for user profile formular
  *
  * @access public
  * @return string
  */
  function getOutputForm() {
    $this->initializeOutputForm();
    $this->outputDialog->baseLink = $this->baseLink;
    $this->outputDialog->dialogTitle = '';
    $this->outputDialog->dialogDoubleButtons = FALSE;
    return $this->outputDialog->getDialogXML();
  }

  /**
  * Initialize user profile formular
  *
  * @access public
  */
  function initializeOutputForm() {
    if (!(isset($this->outputDialog) && is_object($this->outputDialog))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array( 'save'=>1 );
      $fields = array(
        'surfer_handle' => array('Nickname', 'isNoHTML', TRUE, 'disabled_input', 20),
        'surfergroup_title' => array('Group', 'isNoHTML', TRUE, 'disabled_input', 50),
        'surfer_givenname' => array('Givenname', 'isNoHTML', TRUE, 'input', 100),
        'surfer_surname' => array('Surname', 'isNoHTML', TRUE, 'input', 100),
        'surfer_email_1' => array('E-mail', 'isEmail', TRUE, 'input', 100),
        'surfer_email_2' => array('E-mail alternative', 'isEmail', FALSE, 'input', 100),
        'surfer_phone_1' => array('Phone', 'isPhone', FALSE, 'input', 100),
        'surfer_phone_2' => array('Phone alternative', 'isPhone', FALSE, 'input', 100),
        'surfer_password_1' => array('Password', 'isPassword', FALSE, 'password', 200),
        'surfer_password_2' => array('Verification', 'isPassword', FALSE, 'password', 200),
        'surfer_old_password' => array('Old Password', 'isPassword', TRUE, 'password', 200)
      );
      $data = array(
        'surfer_handle' => @$GLOBALS['PAPAYA_SURFER']->surfer['surfer_handle'],
        'surfergroup_title' => @$GLOBALS['PAPAYA_SURFER']->surfer['surfergroup_title'],
        'surfer_surname' => @$GLOBALS['PAPAYA_SURFER']->surfer['surfer_surname'],
        'surfer_givenname' => @$GLOBALS['PAPAYA_SURFER']->surfer['surfer_givenname'],
        'surfer_email_1' => @$GLOBALS['PAPAYA_SURFER']->surfer['surfer_email'],
        'surfer_email_2' => @$GLOBALS['PAPAYA_SURFER']->surfer['surfer_email_alt'],
        'surfer_phone_1' => @$GLOBALS['PAPAYA_SURFER']->surfer['surfer_phone'],
        'surfer_phone_2' => @$GLOBALS['PAPAYA_SURFER']->surfer['surfer_phone_alt']
      );
      $this->outputDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->outputDialog->msgs = &$this->msgs;
      $this->outputDialog->loadParams();
      $this->outputDialog->params['surfer_handle'] = @$GLOBALS['PAPAYA_SURFER']->surfer['surfer_handle'];
      $this->outputDialog->params['surfergroup_title'] = @$GLOBALS['PAPAYA_SURFER']->surfer['surfergroup_title'];
    }
  }

  /**
  * Check profile input
  *
  * @return mixed False or error message
  */
  function checkProfileInput() {
    $result = FALSE;
    if (isset($this->params['surfer_password_1']) && trim($this->params['surfer_password_1']) != '') {
      if (!(isset($this->params['surfer_password_2']) && $this->params['surfer_password_1'] == $this->params['surfer_password_2'])) {
        $this->outputDialog->inputErrors['surfer_password_1'] = 1;
        $this->outputDialog->inputErrors['surfer_password_2'] = 1;
        return @$this->data['Error_Input'];
      }
    }
    return $result;
  }

  /**
  * Save changed profile
  *
  * @return boolean
  */
  function saveProfileData() {
    include_once(dirname(__FILE__).'/base_surfers.php');
    $surferAdmin = &new surfer_admin($this->msgs);
    $data = array(
      'surfer_surname' => $this->params['surfer_surname'],
      'surfer_givenname' => $this->params['surfer_givenname'],
      'surfer_email' => $this->params['surfer_email_1'],
      'surfer_email_alt' => $this->params['surfer_email_2'],
      'surfer_phone' => $this->params['surfer_phone_1'],
      'surfer_phone_alt' => $this->params['surfer_phone_2']
    );
    if (isset($this->params['surfer_password_1']) && trim($this->params['surfer_password_1']) != '') {
      $data['surfer_password'] = md5($this->params['surfer_password_1']);
    }
    $surferAdmin->editSurfer['surfer_id'] = $GLOBALS['PAPAYA_SURFER']->surferId;
    if ($surferAdmin->saveSurfer($data)) {
      //surfer-objekt aktualisieren...
      if (isset($data['surfer_password'])) {
        $GLOBALS['PAPAYA_SURFER']->login($GLOBALS['PAPAYA_SURFER']->surferHandle, $this->params['surfer_password_1']);
      }
      return TRUE;
    } else {
      return FALSE;
    }
    return $result;
  }
}

?>
