<?php
/**
* Diary
*
* @package module_calendar
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_calendar.php,v 1.8 2006/04/26 10:28:59 weinert Exp $
*/

/**
* Basic class calendar
*/
include_once(dirname(__FILE__)."/base_calendar.php");

/**
  * Diary
  *
  * @author Thomas Weinert
  * @package module_calendar
  */
class papaya_calendar extends base_calendar {

  var $images;

  var $cronModules;
  var $regulars;
  var $regular;

  var $inputFieldSize = 'medium';

  var $dateFields = array(
    'topic_title'=>array ('Title', 'isSomeText', TRUE, 'input', 400),
    'topic_date'=>array ('Date text', 'isSomeText', FALSE, 'input', 400),
    'topic_startf'=>array ('From', 'isIsoDate', FALSE, 'input', 14,
      'Start date in ISO format (2002-12-31)'),
    'topic_endf'=>array ('To', 'isIsoDate', FALSE, 'input', 14,
      'End date in ISO format (2003-01-01)'),
    'topic_module_guid'=>array ('Content module', 'isSomeText', TRUE, 'function', 'getComboContentModules'),
    'topic_state'=>array ('State', 'isSomeText', TRUE, 'combo', array(1=>'Created',2=>'Published'), '', 1)
  );

  var $regularFields = array(
    'calreg_title'=>array ('Title', 'isSomeText', TRUE, 'input', 400),
    'calreg_datestr'=>array ('Date text', 'isSomeText', FALSE, 'input', 400),
    'calreg_days'=>array ('Duration (Days)', 'isSomeText', FALSE, 'input', 4, '', 1),
    'Module selection',
    'calreg_content_guid'=>array ('Content', 'isSomeText', TRUE, 'function', 'getComboContentModules',
      'Content module'),
    'calreg_module_guid'=>array ('Time', 'isSomeText', TRUE, 'function', 'getComboTimeModules',
      'Module for calculating the dates'),
    'Planning horizon and max. count',
    'calreg_start_iso'=>array ('From', 'isIsoDate', FALSE, 'input', 14,
      'Earliest date in ISO format (2002-12-31)'),
    'calreg_end_iso'=>array ('To', 'isIsoDate', FALSE, 'input', 14,
      'Latest date in ISO format (2003-12-31)'),
    'calreg_max'=>array ('Maximum', 'isNum', FALSE, 'input', 2,
      'Maximum count'),
  );

  /**
  * Initialize seassion parameters
  *
  * @param string $paramName optional, default value 'cal'
  * @param string $userId optional, default value ''
  * @access public
  */
  function initialize($paramName = 'cal',$userId='') {
    $this->paramName = $paramName;
    $this->authorId = $userId;
    $this->sessionParamName = 'PAPAYA_SESS_calendar'.$this->paramName;
    $this->initializeParams();

    $this->sessionParams = $this->getSessionValue($this->sessionParamName);
    $this->initializeSessionParam('regtopic', array('topic'));
    $this->initializeSessionParam('topic', array('regtopic'));
    //$this->initializeSessionParam('cmd');
    $this->initializeSessionParam('mode');
    $this->initializeSessionParam('date');

    $this->setSessionValue($this->sessionParamName, $this->sessionParams);

    if (isset($this->params['date']) && $this->params['date'] > 0) {
      $this->selectedDate = $this->parseDateToArray($this->params['date']);
    } else {
      $this->selectedDate = $this->parseDateToArray(time());
    }
  }

  /**
  * Execute - basic function for handling parameters
  *
  * @access public
  */
  function execute() {
    switch (@$this->params['cmd']) {
      case 'day' :
        $this->mcalClickDay($this->params['newdate']);
        break;
      case 'wday' :
        $this->mcalClickWeekDay($this->params['wday']);
        break;
      case 'week' :
        $this->mcalClickWeek($this->params['week']);
        break;
      case 'month' :
        $this->mcalClickMonth($this->params['month']);
        break;
      case 'topic' :
        $this->mode = 'topic';
        $this->selectedTopic = (int)$this->params['topic'];
        break;
    }
    if ($this->showEdit) {
      $this->loadModulesList();
      $this->regularMode = (int)$this->params['mode'];
      if (isset($this->params['save'])  && $this->regularMode < 1) {
        if (isset($this->params['regtopic']) && $this->loadRegular((int)$this->params['regtopic'])) {
          if ($changes = $this->checkRegularInput()) {
            if ($this->saveRegular()) {
              $this->loadRegular((int)$this->params['regtopic']);
              if ($changes['replace']) {
                //Termine lschen und erzeugen
                if (FALSE !== ($count = $this->recreateRegularCache())) {
                  $this->addMsg(MSG_INFO, sprintf($this->_gt('Regulary date saved and %d dates created.'), $count));
                } else {
                  $this->addMsg(MSG_ERROR, $this->_gt('Regulary date saved but no dates created.'));
                }
              } elseif ($changes['update']) {
                //Termindaten ndern.
                if ($this->updateRegularCache()) {
                  $this->addMsg(MSG_INFO, $this->_gt('Regulary date saved and dates changed.'));
                } else {
                  $this->addMsg(MSG_ERROR, $this->_gt('Regulary date saved but dates not changed.'));
                }
              }
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
            }
          }
        } elseif (isset($this->params['topic'])) {
          $this->mcalClickEdit($this->params['topic']);
          $row = $this->prepareInput($this->loadTopic($this->params['topic']));
          if ($this->checkInput($row)) {
            $this->save($row);
          }
        }
      }
      switch (@$this->params['cmd']) {
        case 'del' :
          $this->mcalClickDelete($this->params['topic']);
          break;
        case 'createcache' :
          $this->loadRegular((int)$this->params['regtopic']);
          if (FALSE !== ($count = $this->recreateRegularCache())) {
            $this->addMsg(MSG_INFO, sprintf($this->_gt('%d single date(s) created!'), $count));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error! No dates created.'));
          }
          $this->params['cmd'] = 'regtopic';
          $this->params['mode'] = 2;
          $this->regularMode = 2;
          //$this->initializeSessionParam('cmd');
          $this->initializeSessionParam('mode');
          $this->loadRegular((int)$this->params['regtopic']);
          break;
        case 'addreg' :
          if ($newId = $this->addRegular()) {
            $this->loadRegular((int)$this->params['regtopic']);
            $this->params['mode'] = 0;
            $this->params['regtopic'] = $newId;
            $this->initializeSessionParam('regtopic', array('topic'));
            $this->initializeSessionParam('mode');
            $this->regularMode = 0;
            $this->loadRegular($newId);
          }
          break;
        case 'delreg' :
          if (isset($this->params['delconfirm'])) {
            if ($this->deleteRegular((int)$this->params['regtopic'])) {
              $this->addMsg(MSG_INFO, $this->_gt('Regulary date deleted.'));
              $this->params['cmd'] = '';
              $this->params['regtopic'] = 0;
              $this->params['mode'] = 0;
              $this->initializeSessionParam('regtopic', array('topic'));
              $this->initializeSessionParam('mode');
              $this->initializeSessionParam('cmd');
              $this->regularMode = 0;
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
            }
          } else {
            $this->loadRegular((int)$this->params['regtopic']);
          }
          $this->mode = 'delreg';
          break;
        case 'separate' :
          $this->loadRegular((int)$this->params['regtopic']);
          if (@is_array($this->regular)) {
            if ($topicId = $this->addIgnoreDate((int)$this->params['ignore_id'], TRUE)) {
              $this->addMsg(MSG_INFO, sprintf($this->_gt('Date from regulary date detached.'), @(int)$count));
              $this->params['cmd'] = 'topic';
              $this->params['topic'] = $topicId;
              $this->params['mode'] = 0;
              $this->initializeSessionParam('topic', array('regtopic', 'mode'));
              //$this->initializeSessionParam('cmd');
              $this->mode = 'topic';
              $this->selectedTopic = $topicId;
              unset($this->regular);
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
            }
          }
          break;
        case 'ignore' :
          $this->loadRegular((int)$this->params['regtopic']);
          if (@is_array($this->regular)) {
            if ($this->addIgnoreDate((int)$this->params['ignore_id'])) {
              $this->addMsg(MSG_INFO, $this->_gt('Added date to ignore list.'));
              $this->params['cmd'] = 'regtopic';
              $this->params['mode'] = 2;
              $this->regularMode = 2;
              //$this->initializeSessionParam('cmd');
              $this->initializeSessionParam('mode');
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
            }
          }
          $this->loadRegular((int)$this->params['regtopic']);
          break;
        case 'delignore' :
          $this->loadRegular((int)$this->params['regtopic']);
          if (@is_array($this->regular)) {
            if ($this->delIgnoreDate((int)$this->params['ignore_id'])) {
              $this->addMsg(MSG_INFO, $this->_gt('Removed date from ignore list. Please recreate cached dates.'));
              $this->params['cmd'] = 'regtopic';
              $this->params['mode'] = 2;
              $this->regularMode = 2;
              //$this->initializeSessionParam('cmd');
              $this->initializeSessionParam('mode');
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
            }
          }
        case 'regtopic' :
          $this->loadRegular((int)$this->params['regtopic']);
          break;
        case 'add' :
          $this->params['topic'] = $this->add();
          $this->mcalClickEdit($this->params['topic']);
          $row = $this->prepareInput($this->loadTopic($this->selectedTopic));
          break;
        case 'copy' :
        case 'topic' :
        case 'edit' :
        default :
          if (isset($this->params['topic'])) {
            switch (@$this->params['cmd']) {
              case 'copy' : $this->params['topic'] = $this->copy(); break;
            }
            $this->mcalClickEdit($this->params['topic']);
            $row = $this->prepareInput($this->loadTopic($this->selectedTopic));
            if (isset($row['regular_id']) && $row['regular_id'] > 0) {
              $this->loadRegular($row['regular_id']);
              $this->params['regtopic'] = $row['regular_id'];
              $this->initializeSessionParam('regtopic', array('topic'));
              $this->mode = 'regtopic';
            }
          } elseif (isset($this->params['regtopic'])) {
            $this->loadRegular((int)$this->params['regtopic']);
          }
          break;
      }
      $this->loadRegulars();
      $this->loadModulesList();
    }
  }


  /**
  * Load regular
  *
  * @param integer $regularId
  * @access public
  * @return boolean
  */
  function loadRegular($regularId) {
    unset($this->regular);
    $sql = "SELECT cr.calreg_id, cr.calreg_title, cr.calreg_text, cr.calreg_datestr, cr.calreg_days,
                    cr.calreg_start, cr.calreg_end, cr.calreg_max,
                    cr.calreg_content_guid,
                    mc.module_class AS class_content,
                    mc.module_path AS module_path_content,
                    mc.module_file AS module_file_content,
                    cr.calreg_module_guid, cr.calreg_moduledata,
                    mt.module_class AS class_time,
                    mt.module_path AS module_path_time,
                    mt.module_file AS module_file_time
            FROM %s AS cr
              LEFT OUTER JOIN %s AS mc ON mc.module_guid = cr.calreg_content_guid
              LEFT OUTER JOIN %s AS mt ON mt.module_guid = cr.calreg_module_guid
            WHERE calreg_id = '%d'";
    $params = array($this->tableRegulars, $this->tableModules, $this->tableModules, $regularId);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $row['file_content'] = $row['module_path_content'].$row['module_file_content'];
        $row['file_time'] = $row['module_path_time'].$row['module_file_time'];
        $row['calreg_start_iso'] = date('Y-m-d', (int)$row['calreg_start']);
        $row['calreg_end_iso'] = date('Y-m-d', (int)$row['calreg_end']);
        $this->regular = $row;
        $this->mode = 'regtopic';
        $this->loadRegularCache();
        $this->loadRegularIgnores();
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Load regular
  *
  * @access public
  */
  function loadRegulars() {
    unset($this->regulars);
    $sql = "SELECT cr.calreg_id, cr.calreg_title, cr.calreg_text, cr.calreg_start, cr.calreg_end,
                    cr.calreg_max, cr.calreg_module_guid, cr.calreg_moduledata,
                    count(c.topic_id) AS calreg_count
            FROM %s AS cr
            LEFT OUTER JOIN %s AS c ON c.regular_id = cr.calreg_id
            GROUP BY cr.calreg_id, cr.calreg_title, cr.calreg_text, cr.calreg_start, cr.calreg_end,
                     cr.calreg_max, cr.calreg_module_guid, cr.calreg_moduledata
            ORDER BY cr.calreg_title, cr.calreg_id";
    $params = array($this->tableRegulars, $this->tableTopics);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->regulars[$row['calreg_id']] = $row;
      }
    }
  }

  /**
  * Save regular
  *
  * @access public
  * @return mixed db_updateRecord
  */
  function saveRegular() {
    $data = array(
      'calreg_title' => $this->params['calreg_title'],
      'calreg_datestr'  => $this->params['calreg_datestr'],
      'calreg_days'  => (int)$this->params['calreg_days'],
      'calreg_start' => (int)$this->params['calreg_start'],
      'calreg_end'   => (int)$this->params['calreg_end'],
      'calreg_max'   => (int)$this->params['calreg_max'],
      'calreg_content_guid'   => $this->params['calreg_content_guid'],
      'calreg_module_guid' => $this->params['calreg_module_guid']);
    return (FALSE !== $this->databaseUpdateRecord($this->tableRegulars, $data, 'calreg_id', (int)$this->params['regtopic']));
  }

  /**
  * Add regular
  *
  * @access public
  * @return mixed
  */
  function addRegular() {
    $start = time();
    $end = $start+31536000;
    $data = array(
      'calreg_title' => 'Neuer Regeltermin',
      'calreg_start' => $start,
      'calreg_end'   => $end,
      'calreg_max'   => 10);
    return $this->databaseInsertRecord($this->tableRegulars, 'calreg_id', $data);
  }

  /**
  * delete regular
  *
  * @param integer $calregId
  * @access public
  * @return boolean
  */
  function deleteRegular($calregId) {
    if (FALSE !== $this->databaseDeleteRecord($this->tableTopics, 'regular_id', (int)$calregId)) {
      if (FALSE !== $this->databaseDeleteRecord($this->tableRegignore, 'calreg_id', (int)$calregId)) {
        return (FALSE !== $this->databaseDeleteRecord($this->tableRegulars, 'calreg_id', (int)$calregId));
      }
    }
    return FALSE;
  }

  /**
  * Save content data
  *
  * @param integer $data
  * @access public
  * @return mixed db_updateRecord
  */
  function saveCronData($data) {
    $info = array(
      'calreg_moduledata' => $data
    );
    return $this->databaseUpdateRecord($this->tableRegulars, $info, "calreg_id", (int)$this->params['regtopic']);
  }

  /**
  * Save topic content data
  *
  * @param string $data
  * @access public
  * @return mixed db_updateRecord
  */
  function saveTopicContentData($data) {
    $info = array(
      'topic_data' => $data
    );
    return $this->databaseUpdateRecord($this->tableTopics, $info, "topic_id", (int)$this->params['topic']);
  }

  /**
  * Save regular content data
  *
  * @param string $data
  * @access public
  * @return boolean
  */
  function saveRegularContentData($data) {
    $info = array(
      'calreg_text' => $data
    );
    if ( $this->databaseUpdateRecord($this->tableRegulars, $info, "calreg_id", (int)$this->params['regtopic']) ) {
      if ($this->updateRegularCache()) {
        $this->addMsg(MSG_INFO, $this->_gt('Regulary date saved and dates changed.'));
      } else {
        $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load regular cache
  *
  * @access public
  */
  function loadRegularCache() {
    unset($this->cacheTopics);
    $sql = "SELECT %s
              FROM %s
              WHERE regular_id = %d
              ORDER BY topic_start ASC, topic_end ASC, topic_title ASC";
    $params = array($this->fieldsDetail, $this->tableTopics,(int)$this->regular['calreg_id']);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->cacheTopics[$row['topic_id']] = $row;
      }
    }
  }

  /**
  * Load regular ignores
  *
  * @access public
  */
  function loadRegularIgnores() {
    unset($this->regularIgnores);
    $sql = "SELECT calregignore_id, calreg_ignoredate
            FROM %s
            WHERE calreg_id = %d
            ORDER BY calreg_ignoredate ASC";
    $params = array($this->tableRegignore,(int)$this->regular['calreg_id']);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->regularIgnores[$row['calregignore_id']] = $row;
      }
    }
  }

  /**
  * Recreate regular cache
  *
  * @access public
  * @return mixed db_query 0 or FALSE
  */
  function recreateRegularCache() {
    if (isset($this->regular) && is_array($this->regular)) {
      if (FALSE !== $this->databaseDeleteRecord($this->tableTopics, 'regular_id', (int)$this->regular['calreg_id'])) {
        $module = &$this->createPluginObject($this->regular['file_time'], $this->regular['class_time'],
            $this->regular['calreg_module_guid'], $this, $this->regular['calreg_moduledata']);
        $counter = 0;
        unset($days);
        if (isset($module) && is_object($module)) {
          if (@is_array($this->regularIgnores)) {
            foreach($this->regularIgnores as $row) {
              $ignores[] = $this->dayOnly($row['calreg_ignoredate']);
            }
          }
          $date = $this->getNextExecute($module, $this->regular['calreg_start']);
          while ($date > 0) {
            $day = $this->dayOnly($date);
            if (!@in_array($day, $ignores)) {
              if ((++$counter > $this->regular['calreg_max']) || $counter > 60)  break;
              $days[] = $day;
            }
            $date = $this->getNextExecute($module, $date);
          }
        }
        if (@is_array($days) && (count($days) > 0)) {
          $sql = "INSERT INTO {$this->tableTopics} \n
            (regular_id, topic_title, topic_data, topic_module_guid, topic_state, topic_start, topic_end, topic_date) VALUES \n";
          $addsqlPattern = sprintf("('%d', '%s', '%s', '%s', '2', '%%s', '%%s', '%%s'), ",
            (int)$this->regular['calreg_id'], addslashes($this->regular['calreg_title']),
              addslashes($this->regular['calreg_text']),
              addslashes($this->regular['calreg_content_guid']));
          foreach ($days as $day) {
            if ($this->regular['calreg_days'] <= 1) {
              $dayEnd = $day;
              $dateString = date('Y-m-d', $day);
            } else {
              $dayEnd = $this->dayOnly((($this->regular['calreg_days']-1) * $this->secOfDay)+$day);
            }
            if (strlen(trim($this->regular['calreg_datestr'])) > 0) {
              $dateString = $this->regular['calreg_datestr'];
            } elseif ($day < $dayEnd) {
              $dateString = date('Y-m-d', $day).' - '.date('Y-m-d', $dayEnd);
            }
            $sql .= sprintf($addsqlPattern, $day, $dayEnd, addslashes($dateString));
          }
          $sql = substr($sql,0,-2);
          return $this->databaseQuery($sql);
        }
        return 0;
      }
    }
    return FALSE;
  }

  /**
  * Add ignore date
  *
  * @param integer $ignoreId
  * @param boolean $createTopic optional, default value FALSE
  * @access public
  * @return boolean
  */
  function addIgnoreDate($ignoreId, $createTopic = FALSE) {
    if (@is_array($this->regular)) {
      $sql = "SELECT topic_start
              FROM %s
              WHERE topic_id = %d ";
      $params = array($this->tableTopics,(int)$ignoreId);
      if ($res = $this->databaseQueryFmt($sql,$params)) {
        if ($row = $res->fetchRow($sql)) {
          $date = $this->dayOnly($row[0]);
          if ($row[0] > 0) {
            if ($createTopic) {
              $data = array('regular_id'=>0, 'topic_state' => 1);
              if ($this->databaseUpdateRecord($this->tableTopics, $data, 'topic_id', (int)$ignoreId)) {
                $result = $ignoreId;
              }
            } else {
              $result = $this->databaseDeleteRecord($this->tableTopics, 'topic_id', (int)$ignoreId);
            }
            if ($result) {
              $data = array('calreg_ignoredate'=>$date, 'calreg_id'=>(int)$this->regular['calreg_id']);
              if ($this->databaseInsertRecord($this->tableRegignore, 'calregignore_id', $data)) {
                return $result;
              }
            }
          }
        }
      }
    }
    return FALSE;
  }

  /**
  * Delete ignore date
  *
  * @param integer $ignoreId
  * @access public
  * @return mixed FALSE or db_deleteRecord
  */
  function delIgnoreDate($ignoreId) {
    if (@is_array($this->regular)) {
      return $this->databaseDeleteRecord($this->tableRegignore, 'calregignore_id', (int)$ignoreId);
    }
    return FALSE;
  }

  /**
  * Day only
  *
  * @param string $date
  * @access public
  * @return integer
  */
  function dayOnly($date) {
    $dayArr = explode('-',date('Y-m-d', $date));
    if (isset($dayArr) && is_array($dayArr)) {
      return mktime(0,0,0,$dayArr[1],$dayArr[2],$dayArr[0]);
    }
    return 0;
  }

  /**
  * Get next execute
  *
  * @param object base_cronjob &$cronModule
  * @param integer $minTime
  * @access public
  * @return integer
  */
  function getNextExecute(&$cronModule, $minTime) {
    if (@is_object($cronModule)) {
      $nextExecutionTime = $cronModule->getNextDateTime($minTime);
      if (-1 == $this->checkTimeFrame($nextExecutionTime) && 0 < $nextExecutionTime) {
        if ($nextExecutionTime == ($i = $cronModule->getNextDateTime($nextExecutionTime))) {
          break;
        } else {
          $nextExecutionTime = $i;
        }
      }
      if ($nextExecutionTime > 0) {
        switch ($this->checkTimeFrame($nextExecutionTime)) {
          case -1 : return -1;
          case  0 : return $nextExecutionTime;
        }
      }
    }
    return 0;
  }


  /**
  * Check time frame
  *
  * @param string $dateTime
  * @access public
  * @return integer
  */
  function checkTimeFrame($dateTime) {
    if (@is_array($this->regular)) {
      if ($dateTime >= $this->regular['calreg_start'] && $dateTime <= $this->regular['calreg_end']) {
        return 0;
      } elseif ($dateTime <= $this->regular['calreg_start']) {
        return -1;
      }
    }
    return 1;
  }

  /**
  * Update regular cache
  *
  * @access public
  * @return mixed return value db_updateRecord
  */
  function updateRegularCache() {
    $data = array(
      'topic_title' => $this->regular['calreg_title'],
      'topic_data' => $this->regular['calreg_text'],
      'topic_module_guid' => $this->regular['calreg_content_guid'],
      'topic_state' => 2
    );
    return $this->databaseUpdateRecord($this->tableTopics, $data, "regular_id", (int)$this->regular['calreg_id']);
  }

  /**
  * Show topics
  *
  * @param string $headString
  * @param boolean $detail optional, default value TRUE
  * @access public
  * @return string
  */
  function showTopics($headString, $detail = TRUE) {
    $result = '';
    if (isset($this->topics) && is_array($this->topics)) {
      if (!$detail) {
        $result .= sprintf('<listview title="%s" width="100%%">'.LF, $headString);
        $result .= '<items>';
        foreach($this->topics as $row) {
          $imageIdx = ($row['topic_state'] == 2) ? 34 : 35;
          $selected = (isset($row['topic_id']) && isset($this->params['topic_id']) && ($row['topic_id'] == $this->params['topic'] && $this->params['topic'] > 0) ||
              ((int)@$this->params['regtopic'] > 0 && $row['regular_id'] == (int)@$this->params['regtopic']))
              ? ' selected="selected"' : '';
          if ($row['topic_state'] == 2) {
            $imageIdx = 34;
          } else {
            $imageIdx = 35;
          }
          $result .= sprintf('<listitem title="%s" href="%s" image="%s" %s>'."\r\n",
            $row['topic_title'],
            $this->buildLink('', '', array('cmd'=>'topic','topic'=>$row['topic_id']), '', 'mcaldtitle', '', FALSE),
            $this->images[$imageIdx], $selected);
          $result .= '</listitem>'.LF;
        }
        $result .= '</items>';
        $result .= '</listview>'.LF;
      } else {
        $result = base_calendar::showTopics($headString, $detail);
      }
    }
    return $result;
  }

  /**
  * Show regular topics
  *
  * @access public
  * @return string
  */
  function showRegularTopics() {
    $result = '';
    if (isset($this->regulars) && is_array($this->regulars)) {
      $result .= sprintf('<listview title="%s" width="100%%">'.LF, $this->_gt('Regulary dates'));
      $result .= '<cols>';
      $result .= sprintf('<col>%s</col>', $this->_gt('Title'));
      $result .= sprintf('<col align="center">%s</col>', $this->_gt('Dates'));
      $result .= '</cols>';
      $result .= '<items>';
      foreach($this->regulars as $row) {
        $imageIdx = ($row['calreg_count'] > 0) ? 34 : 35;
        $selected = ($row['calreg_id'] == @$this->params['regtopic']) ? ' selected="selected"' : '';
        $result .= sprintf('<listitem title="%s" href="%s" image="%s" %s>'."\r\n",
          $row['calreg_title'],
          $this->buildLink('', '', array('cmd'=>'regtopic','regtopic'=>$row['calreg_id']), '', 'mcaldtitle', '', FALSE),
          $this->images[$imageIdx], $selected);
        $result .= sprintf('<subitem align="center">%s</subitem>', $row['calreg_count']);
        $result .= '</listitem>'.LF;
      }
      $result .= '</items>';
      $result .= '</listview>'.LF;
    }
    return $result;
  }

  /**
  * Show list
  *
  * @access public
  * @return string
  */
  function showList() {
    $result = $this->showMonth(FALSE);
    return $result;
  }

  /**
  * Show data
  *
  * @access public
  * @return string
  */
  function showData() {
    $result = '';
    switch ($this->mode) {
      case 'topic' :
        $result = $this->showTopic();
        break;
      case 'del'  :
        if (!(isset($this->params['delete']) && $this->params['delete'])) {
          $result = $this->showDelConfirm();
        }
        break;
      case 'edit' :
        if ($this->selectedTopic) {
          switch ($this->regularMode) {
            case 3 :
              $result .=  $this->getContentModuleEdit(FALSE);
              break;
            default :
              if ($row = $this->loadTopic($this->selectedTopic)) {
                $result = $this->showEdit($this->prepareInput($row));
              }
          }
        }
        break;
      case 'delreg' :
        $result = $this->getDeleteRegularDialog();
        break;
      case 'regtopic' :
        switch ($this->regularMode) {
          case 1 :
            $result .=  $this->getCronModuleEdit();
            break;
          case 3 :
            $result .=  $this->getContentModuleEdit(TRUE);
            break;
          case 2 :
            $result .= $this->getRegularCacheList();
            $result .= $this->getRegularIgnoreList();
            break;
          default :
            $result .= $this->showRegularEdit();
        }
        break;
    }
    return $result;
  }

  /**
  * Get buttons
  *
  * @access public
  */
  function getButtons() {

    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $toolbar = &new base_btnbuilder;
    $toolbar->images = &$this->images;

    $toolbar->addButton($this->tplAddlink,
      $this->buildLink('','',array('cmd'=>'add'), '', '', '', FALSE),
      34, '', FALSE);
    $toolbar->addButton($this->tplAddlinkReg,
      $this->buildLink('','',array('cmd'=>'addreg'), '', '', '', FALSE),
      34, '', FALSE);

    if (isset($this->dateId) && ($this->dateId > 0) && (!@is_array($this->regular))) {
      $toolbar->addSeperator();
      $toolbar->addButton($this->tplCopylink,
        $this->buildLink('','',array('cmd'=>'copy','topic'=>$this->dateId), '', '', '', FALSE),
        18, '', FALSE);

      $toolbar->addSeperator();
      $toolbar->addButton($this->tplDellink,
        $this->buildLink('','',array('cmd'=>'del','topic'=>$this->dateId), '', '', '', FALSE),
        10, '', FALSE);
    }
    if (@is_array($this->regular)) {
      $toolbar->addSeperator();
      $toolbar->addButton('Create dates',
        $this->buildLink('','',array('cmd'=>'createcache','regtopic'=>$this->regular['calreg_id']), '', '', '', FALSE),
        34, 'Create single dates for a regulary date', FALSE);
      $toolbar->addSeperator();
      $toolbar->addButton($this->tplDellink,
        $this->buildLink('','',array('cmd'=>'delreg','regtopic'=>$this->regular['calreg_id']), '', '', '', FALSE),
        10, '', FALSE);
    }

    if ($str = $toolbar->getXML()) {
      $this->layout->addMenu(sprintf('<menu ident="%s">%s</menu>'.LF, 'edit', $str));
    }
  }

  /**
  * Get toolbar
  *
  * @access public
  */
  function getToolbar() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $toolbar = &new base_btnbuilder;
    $toolbar->images = &$this->images;
    if (@is_array($this->regular)) {

      $toolbar->addButton('Properties',
        $this->buildLink('','',array('mode'=>0,'cmd'=>'regtopic','regtopic'=>(int)$this->regular['calreg_id']),
          '', '', '', FALSE),
        44, '', ($this->regularMode==0));
      $toolbar->addButton('Content',
        $this->buildLink('','',array('mode'=>3,'cmd'=>'regtopic','regtopic'=>(int)$this->regular['calreg_id']),
          '', '', '', FALSE),
        44, '', ($this->regularMode==3));
      $toolbar->addButton('Time',
        $this->buildLink('','',array('mode'=>1,'cmd'=>'regtopic','regtopic'=>(int)$this->regular['calreg_id']),
          '', '', '', FALSE),
        72, 'Time configuration', ($this->regularMode==1));
      $toolbar->addButton('Date list',
        $this->buildLink('','',array('mode'=>2,'cmd'=>'regtopic','regtopic'=>(int)$this->regular['calreg_id']),
          '', '', '', FALSE),
        44, '', ($this->regularMode==2));
    } elseif (@is_array($this->loadedTopic)) {
      $toolbar->addButton('Properties',
        $this->buildLink('','',array('mode'=>0,'cmd'=>'topic','topic'=>(int)$this->loadedTopic['topic_id']),
          '', '', '', FALSE),
        44, '', ($this->regularMode==0));
      $toolbar->addButton('Content',
        $this->buildLink('','',array('mode'=>3,'cmd'=>'topic','topic'=>(int)$this->loadedTopic['topic_id']),
          '', '', '', FALSE),
        44, '', ($this->regularMode==3));
    }
    if ($str = $toolbar->getXML()) {
      $this->layout->add(sprintf('<toolbar>%s</toolbar>'.LF, $str));
    }
  }

  /**
  * Show edit
  *
  * @param array $data
  * @access public
  * @return string
  */
  function showEdit($data) {
    if (isset($this->msg) && $this->msg) {
      $this->addMsg($this->msgType, $this->msg);
    }

    include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
    $hidden = array('cmd'=>'edit', 'save'=>1, 'topic'=>$data['topic_id']);
    $this->dialog = &new base_dialog($this, $this->paramName, $this->dateFields, $data, $hidden);
    $this->dialog->msgs = &$this->msgs;
    $this->dialog->loadParams();
    $this->dialog->inputFieldSize = $this->inputFieldSize;
    $this->dialog->baseLink = $this->baseLink;

    $this->dialog->dialogTitle = htmlspecialchars($this->_gt('Content'));
    $this->dialog->dialogDoubleButtons = TRUE;
    return $this->dialog->getDialogXML();
  }

  /**
  * Get combo time modules
  *
  * @param string $paramName
  * @param $field
  * @param integer $value
  * @access public
  * @return string
  */
  function getComboTimeModules($paramName, $field, $value) {
    $result = sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">', $this->paramName, $paramName);
    if (isset($this->cronModules) && is_array($this->cronModules)) {
      $result .= sprintf('<option value="0">[%s]</option>', $this->_gt('Please select'));
      foreach($this->cronModules as $moduleGuid=>$module) {
        $selected = ($moduleGuid == $value) ? ' selected="selected"' : '';
        $result .= sprintf('<option value="%s" %s>%s</option>',
          htmlspecialchars($moduleGuid), $selected, htmlspecialchars($module['module_title']));
      }
    }
    $result .= '</select>';
    return $result;
  }

  /**
  * Get combo content modules
  *
  * @param string $paramName
  * @param $field
  * @param integer $value
  * @access public
  * @return string
  */
  function getComboContentModules($paramName, $field, $value) {
    $result = sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">', $this->paramName, $paramName);
    if (isset($this->contentModules) && is_array($this->contentModules)) {
      $result .= sprintf('<option value="0">[%s]</option>', $this->_gt('Please select'));
      foreach($this->contentModules as $moduleGuid=>$module) {
        $selected = ($moduleGuid == $value) ? ' selected="selected"' : '';
        $result .= sprintf('<option value="%s" %s>%s</option>',
          htmlspecialchars($moduleGuid), $selected, htmlspecialchars($module['module_title']));
      }
    }
    $result .= '</select>';
    return $result;
  }

  /**
  * Show regular edit
  *
  * @access public
  * @return string
  */
  function showRegularEdit() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
    $hidden = array('cmd'=>'regtopic', 'save'=>1, 'regtopic'=>$this->regular['calreg_id']);
    $this->dialog = &new base_dialog($this, $this->paramName, $this->regularFields, $this->regular, $hidden);
    $this->dialog->msgs = &$this->msgs;
    $this->dialog->loadParams();
    $this->dialog->inputFieldSize = $this->inputFieldSize;
    $this->dialog->baseLink = $this->baseLink;

    $this->dialog->dialogTitle = htmlspecialchars($this->_gt('Properties'));
    $this->dialog->dialogDoubleButtons = TRUE;
    return $this->dialog->getDialogXML();
  }

  /**
  * Get delete regular dialog
  *
  * @access public
  * @return string
  */
  function getDeleteRegularDialog() {
    if (@is_array($this->regular)) {
      $result = sprintf('<msgdialog width="100%%" action="%s" type="question">', $this->baseLink);
      $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />'.LF,
        $this->paramName, 'cmd', 'delreg');
      $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />'.LF,
        $this->paramName, 'delconfirm', 1);
      $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />'.LF,
        $this->paramName, 'regtopic', $this->regular['calreg_id']);
      $result .= '<message>';
      $result .= sprintf('Soll der Eintrag "%s (%s)" wirklich gelscht werden?',
        htmlspecialchars($this->regular['calreg_title']), (int)$this->regular['calreg_id']);
      $result .= '</message>';
      $result .= '<dlgbutton value="Lschen" />';
      $result .= '</msgdialog>';
      return $result;
    }
    return '';
  }

  /**
  * Prepare input
  *
  * @param array $row
  * @access public
  * @return array
  */
  function prepareInput($row) {
    $data = $row;
    if (isset($this->params) && is_array($this->params)) {
      if (isset($this->params['topic_title'])) {
        $data['topic_title'] = $this->prepareMagicQuotes($this->params['topic_title']);
      }
      if (isset($this->params['topic_date'])) {
        $data['topic_date'] = $this->prepareMagicQuotes($this->params['topic_date']);
      }
      if (isset($this->params['topic_state'])) {
        $data['topic_state'] = (int)$this->params['topic_state'];
      }
      if (isset($this->params['topic_module_guid'])) {
        $data['topic_module_guid'] = $this->params['topic_module_guid'];
      }
      if (isset($this->params['topic_startf'])) {
        if ($this->params['topic_startf'] == '') {
          $data['topic_start'] = time();
        } else {
          $date = explode('-',$this->params['topic_startf']);
          $data['topic_start'] = mktime(0,0,0,$date[1],$date[2],$date[0]);
        }
      }
      $data['topic_startf'] = date('Y-m-d',$row['topic_start']);
      if (isset($this->params['topic_endf'])) {
        $date = explode('-',$this->params['topic_endf']);
        $data['topic_end'] = mktime(0,0,0,$date[1],$date[2],$date[0]);
        if ($data['topic_end'] < $data['topic_start']) {
          $data['topic_end'] = $data['topic_start'];
        }
      }
      $data['topic_endf'] = date('Y-m-d',@$data['topic_end']);
      return $data;
    } else {
      return $row;
    }
  }

  /**
  * Check input
  *
  * @param array $data
  * @access public
  * @return boolean
  */
  function checkInput($data) {
    $result = TRUE;
    if (!checkit::isNoHTML($data['topic_title'],TRUE)) {
      $this->addMsg(MSG_ERROR,
        sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Title')));
      $result = FALSE;
    }
    if (!checkit::isNoHTML($data['topic_date'],TRUE)) {
      $this->addMsg(MSG_ERROR,
        sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Date (text)')));
      $result = FALSE;
    }
    if (!checkit::check($data['topic_startf'],'/^\d{4}\-\d{2}\-\d{2}$/',TRUE)) {
      $this->addMsg(MSG_ERROR,
        sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('From')));
      $result = FALSE;
    }
    if (!checkit::check($data['topic_endf'],'/^\d{4}\-\d{2}\-\d{2}$/',TRUE)) {
      $this->addMsg(MSG_ERROR,
        sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('To')));
      $result = FALSE;
    }
    if (!checkit::check($data['topic_module_guid'],'/^[a-fA-F\d]{32}$/',TRUE)) {
      $this->addMsg(MSG_ERROR,
        sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Module')));
      $result = FALSE;
    }
    $data['topic_state'] = (int)$data['topic_state'];
    return $result;
  }

  /**
  * Check regular input
  *
  * @access public
  * @return array or FALSE
  */
  function checkRegularInput() {
    $result = TRUE;
    $caclDates = FALSE;
    $updateDates = FALSE;
    if (!checkit::isNoHTML($this->params['calreg_title'], TRUE)) {
      $this->addMsg(MSG_ERROR,
        sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Title')));
      $result = FALSE;
    } else {
      if ($this->params['calreg_title'] != $this->regular['calreg_title']) {
        $updateDates = TRUE;
      }
    }
    if (!checkit::isSomeText($this->params['calreg_datestr'], FALSE)) {
      $this->addMsg(MSG_ERROR,
        sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Date (text)')));
      $result = FALSE;
    } elseif (!checkit::filled($this->params['calreg_datestr'])) {
      $this->addMsg(MSG_WARNING,
        sprintf($this->_gt('No input in field "%s". Using default values.'), $this->_gt('Date (text)')));
      $updateDates = TRUE;
    } else {
      if ($this->params['calreg_datestr'] != $this->regular['calreg_datestr']) {
        $caclDates = TRUE;
      }
    }
    if (!checkit::isNum($this->params['calreg_days'], TRUE)) {
      $this->addMsg(MSG_ERROR,
        sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Days')));
      $result = FALSE;
    } else {
      if ($this->params['calreg_days'] != $this->regular['calreg_days']) {
        $caclDates = TRUE;
      }
    }
    if (!checkit::isISODate($this->params['calreg_start_iso'], TRUE)) {
      $this->addMsg(MSG_ERROR,
        sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('From')));
      $result = FALSE;
    } else {
      $this->params['calreg_start'] = $this->decodeISODatetimeStr($this->params['calreg_start_iso']);
      if ($this->params['calreg_start_iso'] != $this->regular['calreg_start_iso']) {
        $caclDates = TRUE;
      }
    }
    if (!checkit::isISODate($this->params['calreg_end_iso'], TRUE)) {
      $this->addMsg(MSG_ERROR,
        sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('To')));
      $result = FALSE;
    } elseif ($this->params['calreg_start'] >=
        ($this->params['calreg_end'] = $this->decodeISODatetimeStr($this->params['calreg_end_iso']))) {
      $this->params['calreg_end'] = $this->params['calreg_start']+31536000;
      $this->params['calreg_end_iso'] = date('Y-m-d', $this->params['calreg_end']);
      $caclDates = TRUE;
    } else {
      if ($this->params['calreg_end_iso'] != $this->regular['calreg_end_iso']) {
        $caclDates = TRUE;
      }
    }
    if (!checkit::isGUID($this->params['calreg_module_guid'], TRUE)) {
      $this->addMsg(MSG_ERROR,
        sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Time module')));
      $result = FALSE;
    } else {
      if ($this->params['calreg_module_guid'] != $this->regular['calreg_module_guid']) {
        $caclDates = TRUE;
      }
    }
    if (!checkit::isNum($this->params['calreg_max'], TRUE)) {
      $this->addMsg(MSG_ERROR,
        sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Maximum')));
      $result = FALSE;
    } elseif ($this->params['calreg_max'] > 180) {
      $this->addMsg(MSG_ERROR,
        sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Maximum')));
      $result = FALSE;
    } elseif ($this->params['calreg_max'] < 1) {
      $this->addMsg(MSG_ERROR,
        sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Maximum')));
      $result = FALSE;
    } else {
      if ($this->params['calreg_max'] != $this->regular['calreg_max']) {
        $caclDates = TRUE;
      }
    }
    return (($result) ? array('update'=>$updateDates,'replace'=>$caclDates) : FALSE);
  }

  /**
  * Get regular cache list
  *
  * @access public
  * @return string
  */
  function getRegularCacheList() {
    $result = '';
    if (@is_array($this->cacheTopics)) {
      $result = sprintf('<listview title="%s" width="100%%">', $this->_gt('Date list'));
      $result .= '<cols>';
      $result .= '<col>Zeit</col>';
      $result .= sprintf('<col align="center">%s</col>', $this->_gt('Detach'));
      $result .= sprintf('<col align="center">%s</col>', $this->_gt('Delete'));
      $result .= '</cols>';
      $result .= '<items>';
      foreach($this->cacheTopics as $id=>$row) {
        $titleDate = ($row['topic_start'] < $row['topic_end']) ?
          date('Y-m-d', $row['topic_start']).'-'.date('Y-m-d', $row['topic_end']) :
          $this->wDay2Str($this->getWDay($row['topic_start'])).', '.date('Y-m-d', $row['topic_start']);
        $result .= sprintf('<listitem title="%s">', $titleDate);
        $result .= sprintf('<subitem align="center"><a href="%s"><glyph src="%s"/></a></subitem>',
          $this->buildLink('','',array('cmd'=>'separate','ignore_id'=>(int)$id),
          '', '', '', FALSE), $this->images[34]);
        $result .= sprintf('<subitem align="center"><a href="%s"><glyph src="%s"/></a></subitem>',
          $this->buildLink('','',array('cmd'=>'ignore','ignore_id'=>(int)$id),
          '', '', '', FALSE), $this->images[10]);
        $result .= '</listitem>';
      }
      $result .= '</items>';
      $result .= '</listview>';
    }
    return $result;
  }


  /**
  * Get regular Ignore list
  *
  * @access public
  * @return string
  */
  function getRegularIgnoreList() {
    $result = '';
    if (@is_array($this->regularIgnores)) {
      $result = sprintf('<listview title="%s" width="100%%">', $this->_gt('Ignored dates'));
      $result .= '<cols>';
      $result .= sprintf('<col>%s</col>', $this->_gt('Date'));
      $result .= sprintf('<col align="center">%s</col>', $this->_gt('Delete'));
      $result .= '</cols>';
      $result .= '<items>';
      foreach($this->regularIgnores as $id=>$row) {
        $result .= sprintf('<listitem title="%s, %s">',
          $this->wDay2Str($this->getWDay($row['calreg_ignoredate'])),date('Y-m-d', $row['calreg_ignoredate']));
        $result .= sprintf('<subitem align="center"><a href="%s"><glyph src="%s"/></a></subitem>',
          $this->buildLink('','',array('cmd'=>'delignore','ignore_id'=>(int)$id),
          '', '', '', FALSE), $this->images[10]);
        $result .= '</listitem>';
      }
      $result .= '</items>';
      $result .= '</listview>';
    }
    return $result;
  }

  /**
  * Edit area for time module
  *
  * @access public
  * @return mixed string or FALSE
  */
  function getCronModuleEdit() {
    if (@is_array($this->regular)) {
      return $this->getModuleEdit($this->regular['file_time'], $this->regular['class_time'],
        $this->regular['calreg_module_guid'], $this->regular['calreg_moduledata'], 'saveCronData');
    }
    return FALSE;
  }

  /**
  * Edit area for Content modules
  *
  * @param boolean $isRegular optional, default value FALSE
  * @access public
  * @return string
  */
  function getContentModuleEdit($isRegular = FALSE) {
    if ($isRegular && @is_array($this->regular)) {
      $content = (strpos($this->regular['calreg_text'], '<data>') === 0) ?
        $this->regular['calreg_text'] :
        sprintf('<data><data-element name="text"><![CDATA[%s]]></data-element></data>',
          $this->regular['calreg_text']);
      return $this->getModuleEdit($this->regular['file_content'], $this->regular['class_content'],
        $this->regular['calreg_content_guid'], $content, 'saveRegularContentData');
    } elseif ((!$isRegular) && @is_array($this->loadedTopic) && @$this->loadedTopic['topic_module_guid'] &&
      isset($this->contentModules[$this->loadedTopic['topic_module_guid']])) {
      $module = $this->contentModules[$this->loadedTopic['topic_module_guid']];
      $content = (strpos($this->loadedTopic['topic_data'], '<data>') === 0) ?
        $this->loadedTopic['topic_data'] :
        sprintf('<data><data-element name="text"><![CDATA[%s]]></data-element></data>',
          $this->loadedTopic['topic_data']);
      return $this->getModuleEdit($module['module_path'].$module['module_file'], $module['module_class'],
        $module['module_guid'], $content, 'saveTopicContentData');
    }
    return FALSE;
  }

  /**
  * Get module Edit
  *
  * @param string $fileName
  * @param string $className
  * @param $data
  * @param string $saveFunc
  * @access public
  * @return string
  */
  function getModuleEdit($fileName, $className, $guid, $data, $saveFunc) {
    $moduleObj = &$this->createPluginObject($fileName, $className, $guid, $this, $data);
    if (isset($moduleObj) && is_object($moduleObj)) {
      $moduleObj->initializeDialog();
      if ($moduleObj->modified()) {
        if ($moduleObj->checkData()) {
          if ($this->$saveFunc($moduleObj->getData())) {
            $this->addMsg(MSG_INFO, $this->_gt('Changes saved.'));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
          }
        }
      }
      $result = $moduleObj->getForm();
      return $result;
    }
    return '';
  }

  /**
  * Load next dates
  *
  * @param integer $min
  * @param integer $addDays optional, default value 0
  * @param integer $maxCount optional, default value 1
  * @access public
  * @return boolean
  */
  function loadNextDates($min, $addDays = 0, $maxCount = 1) {
    unset($this->topics);
    $minArr = $this->parseDateToArray($min);
    $minDate = mktime(0,0,0,$minArr['month'],$minArr['day'],$minArr['year']) + (86400 * $addDays);
    $sql = "SELECT %s
              FROM %s
              WHERE topic_start >= %d
                AND topic_state = 2
          ORDER BY topic_start, topic_title, topic_id";
    $params = array($this->fieldsDetail,$this->tableTopics,$minDate);
    if ($res = $this->databaseQueryFmt($sql, $params, $maxCount)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->topics[$row['topic_id']] = $row;
      }
      return TRUE;
    }
    return FALSE;

  }

}
?>