<?php
/**
* Cronjob-module send updates on subscribed categories
*
* @package module_catalog
* @author David Rekowski <rekowski@papaya-cms.com>
* @version $Id: cronjob_catalog_subscriptions.php,v 1.16 2006/06/02 14:36:16 kelm Exp $
*/

/**
* Basic class Cronjobs
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_cronjob.php');
require_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');

/**
* Cronjob-module send updates on subscribed categories
*
* @package module_catalog
* @author David Rekowski <rekowski@papaya-cms.com>
*/
class cronjob_catalog_subscriptions extends base_cronjob {

  /**
  * Handler for modification status
  * @var boolean 
  */
  var $modified = FALSE;

  /**
  * Class constructor
  *
  * @param array $param
  * @access public
  */
  function __construct($param) {
    parent::__construct($paramName);
    $this->layout = &$param->layout;
    $this->dialog = &$param->dialog;
    $this->paramName = 'cs';
    $this->sessionParamName = 'PAPAYA_SESS_'.$this->paramName;
    $this->initializeParams();
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
    $this->tableTopicVersions    = PAPAYA_DB_TABLEPREFIX."_topic_versions";
    $this->tableTopicPublic      = PAPAYA_DB_TABLEPREFIX."_topic_public";
    $this->tableTopicPublic      = PAPAYA_DB_TABLEPREFIX."_topic_public";
    $this->tableTopicPublicTrans = PAPAYA_DB_TABLEPREFIX."_topic_public_trans";
    $this->tableCatalog          = PAPAYA_DB_TABLEPREFIX."_catalog";
    $this->tableCatalogSubscriptions = PAPAYA_DB_TABLEPREFIX."_catalog_subscriptions";
    $this->tableCatalogSubscriptionsMeta = PAPAYA_DB_TABLEPREFIX."_catalog_subscriptions_meta";
  }

  /**
  * PHP4 class constructor
  * 
  * @param array $param
  * @access public
  */
  function cronjob_catalog_subscriptions($param) {
    $this->__construct(&$param);
  }

  /**
  * callback function to generate input fields for change levels
  *
  * @param string $name
  * @param array $field
  * @param array $data
  * @access public
  * @return string $result XML
  */
  function callbackChangelevels($name, $field, $data) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_statictables.php');
    $changeLevels = base_statictables::getChangeLevels();
    $result = '<div class="dialogGroup dialogScale">';
    foreach($changeLevels as $i => $v) {
      $selected = (isset($data[$i])) ? 'checked="checked"' : '';
      $result .= sprintf('<input type="checkbox" name="%s[%s][%d]" value="1" %s></input>%s<br />'.LF,
        $this->paramName, $name, $i, $selected, $v);
    }
    $result .= '</div>';
    return $result;
  }

  /**
  * initialize dialog
  *
  * @see cronjob_catalog_subscriptions::getContentForm()
  * @see base_cronjob::initializeDialog()
  *
  * @access public
  * @return string
  */
  function initializeDialog() {
    $this->loadLngSelect();
    $this->layout->addLeft($this->lngSelect->getComboFormXML());
    if (isset($this->params['cmd'])) {
      switch ($this->params['cmd']) {
        case 'addlng':
          return $this->getContentForm();
          break;
        case 'store':
          $this->data['changelevels'] = $this->params['changelevels'];
          $this->data['bcc']          = $this->params['bcc'];
          $this->data['content'][$this->lngSelect->currentLanguageId] = array(
            'subject'      => $this->params['subject'],
            'body'         => $this->params['body'],
          );
          $this->modified = TRUE;
          return $this->getContentForm($this->data, 
            $this->lngSelect->currentLanguageId);
        default:
          break;
      }
    }
    return parent::initializeDialog();
  }

  /**
  * get formular
  *
  * @see cronjob_catalog_subscriptions::getContentForm()
  * @see cronjob_catalog_subscriptions::getLngAddForm()
  *
  * @access public
  * @return string
  */
  function getForm() {
    if (isset($this->data['content']) && is_array($this->data['content']) && 
        count($this->data['content']) > 0 && 
        isset($this->data['content'][$this->lngSelect->currentLanguageId]) || 
        isset($this->params['cmd']) && $this->params['cmd'] = 'addlng') {
      return $this->getContentForm($this->data, 
        $this->lngSelect->currentLanguageId);
    } else {
      return $this->getLngAddForm();
    }
  }

  /**
  * get formular to add content for language
  *
  * @see base_msgdialog::getMsgDialog()
  *
  * @access public
  * @return string
  */
  function getLngAddForm() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'addlng'
    );
    $msg = sprintf($this->_gt('Add content for language "%s" (%s)?'),
      $this->lngSelect->currentLanguage['lng_title'], 
      $this->lngSelect->currentLanguage['lng_short']);
    $this->dialog = &new base_msgdialog($this, $this->paramName, 
      $hidden, $msg, 'question');
    $this->dialog->buttonTitle = $this->_gt('Add');
    return $this->dialog->getMsgDialog();
  }

  /**
  * get formular for content 
  *
  * @see base_dialog::getDialogXML()
  *
  * @param mixed $data optional, default value NULL
  * @param mixed $lngId optional, default value NULL
  * @access public
  * @return string
  */
  function getContentForm($data = NULL, $lngId = NULL) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
    $hidden = array(
      'cmd' => 'store');
    $fields = array(
      'subject' => array('Subject', 'isNoHTML', TRUE, 'input', 200),
      'body' => array(
        'Body', 
        'isNoHTML', 
        TRUE, 
        'textarea', 
        10, 
        '', 
        "The following topics have been updated:\n\n{%links%}\n\n"
      ),
      'language independent',
      'changelevels' => array(
        'Change Levels', 
        '', 
        TRUE, 
        'function', 
        'callbackChangelevels'
      ),
      'bcc' => array('BCC Recipient', 'isEmail', FALSE, 'input', 200));
    $changeLevels = $data['changelevels'];
    $bcc = $data['bcc'];
    $data = $data['content'][$lngId];
    $data['changelevels'] = $changeLevels;
    $data['bcc'] = $bcc;
    $this->dialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
    $this->dialog->dialogTitle = 
      $this->_gt('Edit Content').': '.$this->lngSelect->currentLanguage['lng_title'];
    $this->dialog->msgs = &$this->msgs;
    $this->dialog->loadParams();
    $this->dialog->buttonTitle = $this->_gt('Save');
    return $this->dialog->getDialogXML();
  }

  /**
  * function for execution
  *
  * @access public
  */
  function execute() {
    $this->loadLngSelect();
    $time = $this->cron->cronjob['cronjob_lastexec'];
    include_once(PAPAYA_INCLUDE_PATH.'modules/free/catalog/base_catalog.php');
    $this->catalog = &new base_catalog;
    $changeLevelCondition = ' AND '.$this->catalog->databaseGetSQLCondition('topic_change_level',
      array_keys($this->data['changelevels']));
    $sqlMax = "SELECT MAX(version_time) as max_time, topic_id
                 FROM %s
                WHERE version_time >= '%d'
                 $changeLevelCondition
             GROUP BY topic_id";
    $paramsMax = array($this->tableTopicVersions, $time);
    if ($resMax = $this->catalog->databaseQueryFmt($sqlMax, $paramsMax)) {
      while ($row = $resMax->fetchRow(DB_FETCHMODE_ASSOC)) {
        $conditions[] = sprintf("tv.topic_id = '%d' AND tv.version_time = '%d'", 
          $row['topic_id'], $row['max_time']);
      }
      if (isset($conditions) && is_array($conditions) && count($conditions) > 0) {
        $maxTimeCondition = sprintf(' AND (%s) ', implode(' OR ', $conditions));
      }
    }
    $sql = "SELECT cl.catalog_id, tv.topic_id, tt.topic_title, c.catalog_parent_path, tt.lng_id
              FROM %s AS cl
              LEFT OUTER JOIN %s AS tv ON (tv.topic_id = cl.topic_id)
              LEFT OUTER JOIN %s AS tt ON (tv.topic_id = tt.topic_id)
              LEFT OUTER JOIN %s AS c ON (c.catalog_id = cl.catalog_id)
             WHERE tv.version_time >= '%d'
              $maxTimeCondition
          GROUP BY tv.version_time, tt.lng_id ASC";
    $params = array(
      $this->catalog->tableCatalogLinks, 
      $this->tableTopicVersions,
      $this->tableTopicPublicTrans, 
      $this->tableCatalog,
      $time
    );
    if ($res = $this->catalog->databaseQueryFmt($sql, $params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $catalogs[$row['catalog_id']][$row['topic_id']] = 1;
        @$catalogCount[$row['catalog_id']][] = $row['topic_id'];
        $topics[$row['catalog_id']][] = $row;
        $path = explode(';', $row['catalog_parent_path']);
        foreach ($path as $catalogId) {
          if ($catalogId != '' && $catalogId != 0) {
            @$catalogCount[$catalogId][] = $row['topic_id'];
            $topics[$catalogId][] = $row;
          }
        }
      }
    }
    // get all subscriptions in categories that have a modified topic
    if (isset($catalogCount) && is_array($catalogCount) && 
        count($catalogCount) > 0) {
      $catalogCondition = 
        'WHERE '.$this->catalog->databaseGetSQLCondition('s.catalog_id', 
          array_keys($catalogCount));
      $sql = "SELECT m.catalog_subscription_email, s.catalog_id, m.lng_id
                FROM %s AS s
                LEFT OUTER JOIN %s AS m 
                  ON (m.catalog_subscription_id = s.catalog_subscription_id)
              $catalogCondition";
      $params = array(
        $this->tableCatalogSubscriptions, 
        $this->tableCatalogSubscriptionsMeta
      );
      if ($res = $this->catalog->databaseQueryFmt($sql, $params)) {
        while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $subscriptions[$row['catalog_subscription_email']][] = $row['catalog_id'];
          $lng[$row['catalog_subscription_email']] = $row['lng_id'];
        }
      }
    } else {
      echo 'No modified topics found.';
      return 0;
    }

    // send emails
    include_once(PAPAYA_INCLUDE_PATH.'system/sys_email.php');
    $emailObj = &new email;

    $mailsCount = 0;
    if (isset($subscriptions) && is_array($subscriptions) && count($subscriptions) > 0) {
      foreach ($subscriptions as $email => $catalogIds) {
        $links = '';
        $lngId = $lng[$email];
        foreach ($catalogIds as $cid) {
          if (isset($topics[$cid]) && is_array($topics[$cid]) && count($topics[$cid]) > 0) {
            foreach ($topics[$cid] as $topic) {
              if ($topic['lng_id'] == $lngId) {
                $link  = $this->getAbsoluteURL($this->getWebLink($topic['topic_id'], $this->lngSelect->languages[$lngId]['lng_ident']));
                $title = $topic['topic_title'];
                $links .= sprintf('%s: %s'.LF, $title, $link);
              }
            }
          }
        }
        if (trim($links) != '') {
          $bodyData = array('links' => $links);
          $emailObj->setTemplate('body', $this->data['content'][$lngId]['body'], $bodyData);
          if ($this->data['bcc'] != '') {
            $emailObj->setHeader('bcc', $this->data['bcc']);
          }
          if ($emailObj->send($email, $this->data['content'][$lngId]['subject'])) {
            $mailsCount++;
          }
        }
      }
      printf('%d emails sent.'.LF, $mailsCount);
    }
    return 0;
  }

  /**
  * Check dialog input
  * 
  * @see base_dialog::checkDialogInput()
  *
  * @access public
  * @return boolean
  */
  function checkDialogInput() {
    $result = FALSE;
    if (isset($this->dialog) && is_object($this->dialog)) {
      if ($result = $this->dialog->checkDialogInput()) {
//      $this->data = $this->dialog->data;
      }
    }
    $this->fieldErrors = $this->dialog->inputErrors;
    return $result;
  }

  /**
  * return modification status
  *
  * @access public
  * @return boolean
  */
  function modified() {
    return $this->modified;
  }

  /**
  * Check execution parameters
  *
  * @access public
  * @return boolean Execution possible?
  */
  function checkExecParams() {
    if (isset($this->data['content']) && is_array($this->data['content']) &&
        isset($this->data['changelevels']) && is_array($this->data['changelevels'])) {
      return TRUE;
    }
    return FALSE;
  }

  /**
  * load language selection
  *
  * @access public
  */
  function loadLngSelect() {
    if (!(isset($this->lngSelect) && is_object($this->lngSelect))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_language_select.php');
      $this->lngSelect = &base_language_select::getInstance();
    }
  }
}

?>