<?php
/**
* Actionbox featuring (random) linkDB links
*
* @author David Rekowski <david.rekowski@eternium.de>
* @version $Id:$
* @package module_linkdb
*/

/**
* Basic class page module
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_actionbox.php');
/**
* Basic db object
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

require_once(PAPAYA_INCLUDE_PATH.'system/papaya_strings.php');

/**
* Actionbox featuring (random) linkDB links
*
* @author David Rekowski <david.rekowski@eternium.de>
* @package module_linkdb
*/
class actbox_linkdb extends base_actionbox {

  var $editFields = array(
    'link_category_id' => array('Category of Links', 'isNum', TRUE, 'function', 'getLinkDBCombo'),
    'number' => array('Number', 'isNum', TRUE, 'input', 3),
    'mode' => array('Display Mode', 'isNum', TRUE, 'combo', array(0 => 'random', 1 => 'latest')),
    'page_id' => array('Exitpage For Links', 'isNum', TRUE, 'pageid'),
  );

  var $linksPerPage = NULL;

  function __construct(&$topic, $paramName = 'ldb') {
    parent::__construct($topic, $paramName);
    $this->dbObj = new base_db();
    $this->tableLinkdb = PAPAYA_DB_TABLEPREFIX.'_linkdb';
    $this->tableLinkdbCateg = PAPAYA_DB_TABLEPREFIX.'_linkdb_categ';
    $this->tableLinkdbClicks = PAPAYA_DB_TABLEPREFIX.'_linkdb_clicks';
  }

  function actbox_linkdb(&$topic, $paramName = 'ldb') {
    $this->__construct($topic, $paramName);
  }

  /**
  * Get parsed data
  *
  * @access public
  * @return string
  */
  function getParsedData() {
    $result = '';
    $this->linksPerPage = $this->data['number'];
    switch($this->data['mode']) {
      case 0: // random
        $this->loadLinks($this->data['link_category_id'], TRUE, 'RAND()');
      break;
      case 1: // latest
        $this->loadLinks($this->data['link_category_id'], TRUE, 'link_created', 'DESC');
      break;
    }
    $result .= '<links>'.LF;
    if (isset($this->links) && is_array($this->links) && count($this->links) > 0) {
      foreach ($this->links as $link) {
        $result .= sprintf('<link url="%s" title="%s"><description>%s</description></link>'.LF,
          $this->getWebLink($this->data['page_id'], NULL, NULL, array('link_id' => $link['link_id']), $this->paramName),
          papaya_strings::escapeHTMLChars($link['link_title']),
          $this->getXHTMLString($link['link_description']));
      }
    }
    $result .= '</links>'.LF;
    return $result;
  }

  function getLinkDbCombo($name, $field, $data) {
    $result ="";
    $this->loadCategs();
    $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">'.LF,
    $this->paramName, $name);
    $selected = ($data == 0) ? ' selected="selected"' : '';
    $result .= sprintf('<option value="%d"%s><![CDATA[%s]]></option>'.LF,
      0,
      $selected,
      htmlspecialchars($this->_gt('Base'))
    );
    if (isset($this->categs) && is_array($this->categs)) {
      $result .= $this->getCategSubTree(0,0,$data);
    }
    $result .= '</select>'.LF;
    return $result;
  }

  function getCategSubTree($parent, $indent,$data) {
    $result = '';
    if (isset($this->categTree[$parent]) &&
      is_array($this->categTree[$parent])) {
      foreach($this->categTree[$parent] as $id) {
        $result .= $this->getChildCategs($id, $indent,$data);
      }
    }
    return $result;
  }

  /**
  * Element of category tree
  * @param integer $id ID
  * @param integer $indent shifting
  * @return string $result String
  */
  function getChildCategs($id, $indent,$data) {
    $result = '';
    if (isset($this->categs[$id]) && is_array($this->categs[$id])) {

      $title = "'".str_repeat('-', $indent*4)."->";
      $title .= htmlspecialchars($this->categs[$id]['linkcateg_title']);

      $selected = ($data == $this->categs[$id]['linkcateg_id']) ? ' selected="selected"' : '';
      $result .= sprintf('<option value="%d"%s><![CDATA[%s]]></option>'.LF,
        $this->categs[$id]['linkcateg_id'],
        $selected,
        htmlspecialchars($title)
      );
      $result .= $this->getCategSubTree($id, $indent+1, $data);
    }
    return $result;
  }

  function loadCategs($id = NULL, $repair = NULL) {
    unset($this->categs);
    unset($this->categTree);
    if (isset($id)) {
      $sql = "SELECT linkcateg_id,
                      linkcateg_parent_id,
                      linkcateg_title,
                      linkcateg_description
                FROM %s
                WHERE linkcateg_parent_id = %d
            ORDER BY linkcateg_title";

      $params = array($this->tableLinkdbCateg, $id);
    } elseif (isset($repair) && $repair == TRUE ) {
      $sql = "SELECT linkcateg_id,
                      linkcateg_parent_id,
                      linkcateg_title,
                      linkcateg_path
                FROM %s
                WHERE linkcateg_parent_id > -1
            ORDER BY linkcateg_title";
      $params = array($this->tableLinkdbCateg);
    } else {
      $sql = "SELECT linkcateg_id,
                      linkcateg_parent_id,
                      linkcateg_title,
                      linkcateg_path
                FROM %s
            ORDER BY linkcateg_title";
      $params = array($this->tableLinkdbCateg);
    }

    if ($res = $this->dbObj->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->categs[(int)$row['linkcateg_id']] = $row;
        $this->categTree[(int)$row['linkcateg_parent_id']][] = $row['linkcateg_id'];
      }
      return TRUE;
    }
    return FALSE;
  }

  function loadLinks($id, $full = FALSE, $orderBy = 'link_title', $orderMode = 'ASC') {
    unset($this->links);
    if ($full) {
      $sql = "SELECT link_id,
                      linkcateg_id,
                      link_title,
                      link_url,
                      link_description,
                      link_status
                FROM %s AS links
                WHERE linkcateg_id = %d
                  AND link_status = 1
            ORDER BY %s %s";
      $params = array($this->tableLinkdb, (int)$id, (string)$orderBy, (string)$orderMode);
    } else {
      $sql = "SELECT link_id,
                      linkcateg_id,
                      link_title,
                      link_url,
                      link_description,
                      link_status
                FROM %s AS links
                WHERE linkcateg_id = %d
            ORDER BY %s %s";
      $params = array($this->tableLinkdb, (int)$id, (string)$orderBy, (string)$orderMode);
    }

    $ids = array();
    if ($res = $this->dbObj->databaseQueryFmt($sql, $params,
        $this->linksPerPage, @(int)$this->params['offset'])) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $row['counted'] = 0;
        $this->links[(int)$row['link_id']] = $row;
        $ids[] = (int)$row['link_id'];
      }
      $this->linkCount = $res->absCount();
      if (count($ids) > 0 && (!$full)) {
        if (count($ids) > 1) {
          $filter = ' IN ('.implode(',',$ids).')';
        } else {
          $filter = ' = '.(int)$ids[0];
        }
        $sql = "SELECT link_id, SUM(clicks_count) AS counted
                  FROM %s
                  WHERE link_id $filter
                  GROUP BY link_id";
        if ($res = $this->dbObj->databaseQueryFmt($sql, $this->tableLinkdbClicks)) {
          while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
            $this->links[(int)$row['link_id']]['counted'] = $row['counted'];
          }
        }
      }
      return TRUE;
    }
    return FALSE;
  }

}

?>