<?php
/**
* Action box email formular
*
* @package module_feedback
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: actbox_form.php,v 1.5 2006/01/23 09:38:50 weinert Exp $
*/

/**
* Basic class action box
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_actionbox.php');

define('PAPAYA_BOXXMLFILES_PATH', PAPAYA_PATH_DATA.'xml/');

/**
* Action box email formular
*
* @package module_feedback
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class actionbox_form extends base_actionbox {

  /**
  * Content edit fields
  * @var array $editFields
  */
  var $editFields = array(
    'xmlfile' => array('XML-Formular', 'isFile', TRUE, 'filecombo', array(PAPAYA_XMLFILES_PATH, '/^\w+\.xml$/i'), ''),
    'mailto' => array('Empfnger', 'isEMail', TRUE, 'input', 60, '', 'webmaster@localhost'),
    'subject' => array('Betreff', 'isNoHTML', TRUE, 'input', 60, '', 'Feeback von der Webseite'),
    'mailtext' => array('E-Mail Text', 'isSomeText', FALSE, 'textarea', 6, '',
      'Blitzanfrage von der Webseite'),
    'msg_hello' => array ('Text Aufforderung', 'isSomeText', FALSE, 'textarea', 6, '', ''),
    'msg_error' => array ('Text Eingabefehler', 'isSomeText', FALSE, 'textarea', 6, '',
      'Bitte prfen Sie die folgenden Felder.'),
    'msg_send' => array ('Text Besttigung', 'isSomeText', FALSE, 'textarea', 6, '',
      'Vielen Dank. Die Nachricht wurde versendet.'),
    'msg_notsend' => array ('Text Sendefehler', 'isSomeText', FALSE, 'textarea', 6, '',
      'Beim Senden ist ein Fehler aufgetreten. Bitte versuchen Sie es noch einmal.')
  );

  /**
  * On load
  *
  * @access public
  */
  function onLoad() {
    $this->initializeLookups(basename($this->data['xmlfile']));
  }

  /**
  * Get parsed data
  *
  * @access public
  * @return string
  */
  function getParsedData() {
    $xmlData = '<box>'.LF;
    if ($this->params['send']) {
      if ($error = $this->checkBoxInput()) {
        $xmlData .= $this->getBoxForm();
        $xmlData .= sprintf('<text>%s <ul>%s</ul></text>'.LF, $this->getXHTMLString($this->data['msg_error'], TRUE),
          '<li class="boxtext">'.implode('</li><li class="boxtext">', $error).'</li>');
      } else {
        if ($this->sendEmail()) {
          $xmlData .= sprintf('<text>%s</text>'.LF, $this->getXHTMLString($this->data['msg_send'], TRUE));
        } else {
          $xmlData .= sprintf('<text>%s</text>'.LF, $this->getXHTMLString($this->data['msg_notsend'], TRUE));
        }
      }
    } else {
      $xmlData .= $this->getBoxForm();
      $xmlData .= sprintf('<text>%s</text>'.LF, $this->getXHTMLString($this->data['msg_hello'], TRUE));
    }
    $xmlData .= '</box>'.LF;
    return $xmlData;
  }

  /**
  * Get parameter name
  *
  * @param string $paramName
  * @access public
  * @return string
  */
  function getParamName($paramName) {
    return $paramName.'_abf';
  }

  /**
  * Send mail
  *
  * @access public
  * @return integer
  */
  function sendEmail() {
    $msg = $this->data['mailtext']."\n\n";
    foreach ($this->inputFields as $key => $val) {
      $msg .= $val[0].": \t\t".$this->params[$key].LF;
    }
    $result = mail($this->data['mailto'], $this->data['subject'], $msg);
    return $result;
  }

  /**
  * initialize Lookups
  * @param string $fileName filename
  */
  function initializeLookups($fileName) {
    $this->loadInputFields(basename($fileName));
    if (isset($this->inputFields) && is_array($this->inputFields)) {
      $this->editFields[] = 'LookUps';
        foreach($this->inputFields as $key => $val) {
        if (preg_match('/^lookup_/', $val[3])) {
          $name = 'lookup_'.$val[4];
          $element = array ($val[0], 'isSomeText', FALSE, 'textarea', 6);
          $this->editFields[$name] = $element;
        }
      }
    }
  }

  /**
  * Initialize box formular
  */
  function initializeBoxForm() {
    if (!@is_object($this->dialog)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array('send' => 1);
      $data = NULL;
      $this->boxForm = &new base_dialog($this, $this->paramName, $this->inputFields, $data, $hidden);
      $this->boxForm->loadParams();
      $this->boxForm->inputFieldSize = $this->inputFieldSize;
      $this->boxForm->baseLink = $this->baseLink;
    }
  }

  /**
  * Return box formular
  * @return string XML des Dialoges
  */
  function getBoxForm() {
    $this->initializeBoxForm();
    return $this->boxForm->getDialogXML();
  }

  /**
  * Check box input
  * @return array error
  */
  function checkBoxInput() {
    $this->initializeBoxForm();
    if ($result = $this->boxForm->checkDialogInput()) {
      $this->boxData = $this->boxForm->data;
      return FALSE;
    } else {
      return $this->boxForm->inputErrors;
    }
  }

  /**
  * Load input fields
  * @param string $fileName filename
  */
  function loadInputFields($fileName) {
    unset($this->inputFields);
    if (file_exists(PAPAYA_BOXXMLFILES_PATH.$fileName) && ($fileName != '')) {
      if ($xmlTree = $this->createXMLTree(@implode('', file(PAPAYA_BOXXMLFILES_PATH.$fileName)))) {
        $this->readInputFields($xmlTree);
      }
    }
  }

  /**
  * Read input fields
  * @param reference &$xmlTree XML-tree
  */
  function readInputFields(&$xmlTree) {
    $parent = NULL;
    $template = &new simple_xmlnode($parent, 'dialog');
    $xmlTree->extract($elements, $template);
    if (isset($elements["nodes"]) && is_array($elements["nodes"])) {
      foreach($elements["nodes"] as $node) {
        if (is_object($node)) {
          $this->inputName = $node->attributes["name"];
        }
      }
    }
    $template = &new simple_xmlnode($parent, 'element');
    $xmlTree->extract($elements, $template);
    if (isset($elements["nodes"]) && is_array($elements["nodes"])) {
      foreach($elements["nodes"] as $node) {
        if (is_object($node)) {
          $xId = FALSE;
          if (isset($node->attributes["name"])) {
            $xId = $node->attributes["name"];
            $xData = array($node->attributes["caption"], $node->attributes["check"], $node->attributes["needed"],
              $node->attributes["type"], $node->attributes["typeparam"], $node->attributes["hint"],
              $node->attributes["default"], $node->attributes["align"]);
            $this->inputFields[$xId] = $xData;
          }
        }
      }
    }
  }
}
?>