<?php
/**
* Module Feedback
*
* @author David Rekowski <rekowski@papaya-cms.com>
* @version $Id:$
* @package module_feedback
*/

/**
* Basic class for database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* Module Feedback
*
* @author David Rekowski <rekowski@papaya-cms.com>
* @package module_feedback
*/

class admin_feedback_store extends base_db {

  var $entriesAbsCount = 0;
  var $pagingCount     = 10;
  var $limitList       = array(5, 10,20,50,100);

  /**
  * @param string $paramName optional, default value 'stat'
  */
  function __construct($paramName = 'fbs') {
    $this->paramName = $paramName;
    $this->tableFeedback = PAPAYA_DB_TABLEPREFIX.'_feedback';
  }

  /**
  * pipes to __construct()
  */
  function admin_feedback_store($paramName = 'fbs') {
    $this->__construct($paramName);
  }

  /**
  * Initialize parameters
  */
  function initialize() {
    $this->sessionParamName = 'PAPAYA_SESS_'.$this->paramName;
    $this->initializeParams();
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);

    $this->initializeSessionParam('limit', array('offset'));
    $this->initializeSessionParam('offset');

    if (!isset($this->params['limit'])) {
      $this->params['limit'] = 10;
    }

    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  /**
  * executes commands sent by user
  */
  function execute() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $this->menubar = &new base_btnbuilder;
    $this->menubar->images = &$this->images;

    switch(@$this->params['cmd']) {
      case 'export':
        $this->exportFeedbackList();
      break;
      default:
        $xml = $this->getFeedbackList();
        $this->layout->add($this->getMultiPagingBarXML());
        $this->layout->add($xml);
      break;
    }
  }

  /**
  * generates XML for admin page
  */
  function getXML() {
    if ($this->module->hasPerm(1, TRUE)) {
      $this->menubar->addButton('Export', $this->getLink(array('cmd' => 'export')), 49, $this->_gt('Export'));
    }
    if ($str = $this->menubar->getXML()) {
      $this->layout->addMenu(sprintf('<menu>%s</menu>'.LF, $str));
    }
  }

  function getMultiPagingBarXML($canChangeLimit = TRUE) {
    $result = '';
    $result .= '<pagingbar>';
    $result.= sprintf('<paging first="%s" last="%s" total="%d">',
      $this->getLink(array('offset' => 0)),
      $this->getLink(array('offset' => (floor($this->entriesAbsCount/$this->params['limit']))*$this->params['limit'])),
      ceil($this->entriesAbsCount/$this->params['limit'])
    );
    for($i=0; $i<=$this->entriesAbsCount/$this->params['limit']; $i++) {
      $diff = (int)$this->params['offset']/(int)$this->params['limit']-$i;
      if ($diff > 0 && abs($diff) >= $this->pagingCount && !$previousSet) {
        $link = $this->getLink(array('offset' => $this->params['offset']-$this->params['limit']));
        $result .= sprintf('<pagingelement href="%s" num="%s"/>'.LF, $link, '&#171;');
        $previousSet = TRUE;
      } elseif (abs($diff) < $this->pagingCount) {
        $selected = ($this->params['offset'] == $i*$this->params['limit']) ? 'selected="selected"' : '';
        $link = $this->getLink(array('offset' => $i*$this->params['limit']));
        $result .= sprintf('<pagingelement href="%s" num="%d" %s/>', $link, $i+1, $selected);
      } elseif ($diff < 0 && abs($diff) >= $this->pagingCount && !$followingSet) {
        $link = $this->getLink(array('offset' => $this->params['offset']+$this->params['limit']));
        $result .= sprintf('<pagingelement href="%s" num="%s"/>'.LF, $link, '&#187;');
        $followingSet = TRUE;
      }
    }
    $result .= '</paging>';
    if ($canChangeLimit) {
      $result .= '<limitlist>';
      foreach($this->limitList as $limit) {
        $selected = ($limit == $this->params['limit']) ? 'selected="selected"' : '' ;
        $result .= sprintf('<limitelement href="%s" num="%s" %s/>', $this->getLink(array('limit' => $limit)), $limit, $selected);
      }
      $result .= '</limitlist>';
    }
    $result .= '</pagingbar>';
    return $result;
  }

  function loadFeedbackData($limit = 0, $offset = 0) {
    $sql = "SELECT feedback_time, feedback_name, feedback_email, feedback_subject, feedback_message
              FROM %s
            ORDER BY feedback_time DESC
           ";
    $params = array($this->tableFeedback);
    if ($limit > 0) {
      $res = $this->databaseQueryFmt($sql, $params, $limit, $offset);
    } else {
      $res = $this->databaseQueryFmt($sql, $params);
    }

    if ($res) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->feedbackData[] = $row;
      }
      $this->entriesAbsCount = $res->absCount();
    }
  }

  function getFeedbackList() {
    $this->loadFeedbackData((int)$this->params['limit'], (int)@$this->params['offset']);
    $result = '';
    $result .= sprintf('<listview title="%s">'.LF, $this->_gt('Stored Feedback submissions'));
    $result .= sprintf('<cols><col>%s</col><col>%s</col><col>%s</col><col>%s</col><col>%s</col></cols>'.LF,
      $this->_gt('Date'), $this->_gt('Name'), $this->_gt('Email'), $this->_gt('Subject'), $this->_gt('Message'));
    $result .= '<items>';
    if (isset($this->feedbackData) && is_array($this->feedbackData) && count($this->feedbackData) > 0) {
      foreach($this->feedbackData as $row) {
        $result .= sprintf('<listitem title="%s">'.LF, date('d.m.Y H:i', $row['feedback_time']));
        $result .= sprintf('<subitem>%s</subitem>'.LF, trim($row['feedback_name']));
        $result .= sprintf('<subitem>%s</subitem>'.LF, trim($row['feedback_email']));
        $result .= sprintf('<subitem>%s</subitem>'.LF, trim($row['feedback_subject']));
        $result .= sprintf('<subitem>%s</subitem>'.LF, trim($row['feedback_message']));
        $result .= '</listitem>';
      }
    }
    $result .= '</items>';
    $result .= '</listview>';
    return $result;
  }

  function exportFeedbackList() {
    $this->loadFeedbackData();
    $agentStr = strtolower(@$_SERVER["HTTP_USER_AGENT"]);
    if (strpos($agentStr, 'opera') !== FALSE) {
      $agent =  'OPERA';
    } elseif (strpos($agentStr, 'msie') !== FALSE) {
      $agent =  'IE';
    } else {
      $agent =  'STD';
    }
    $mimeType = ($agent == 'IE' || $agent == 'OPERA')
      ? 'application/octetstream'
      : 'application/octet-stream';
    $fileName = 'feedback_'.date('Y-m-d').'.csv';
    if ($agent == 'IE') {
      header('Content-Disposition: inline; filename="'.$fileName.'"');
    } else {
      header('Content-Disposition: attachment; filename="'.$fileName.'"');
    }
    header('Content-type: ' . $mimeType);
    echo 'Date,Name,Email,Subject,Message'."\r\n";
    foreach ($this->feedbackData as $entry) {
      echo '"'.$this->escapeForCSV(date('d.m.Y H:i', $entry['feedback_time'])).'",';
      echo '"'.$this->escapeForCSV($entry['feedback_name']).'",';
      echo '"'.$this->escapeForCSV($entry['feedback_email']).'",';
      echo '"'.$this->escapeForCSV($entry['feedback_subject']).'",';
      echo '"'.$this->escapeForCSV($entry['feedback_message']).'"';
      echo "\r\n";
    }
    exit;
  }

  function escapeForCSV($str) {
    if (strpos($str, '"') !== FALSE ||
      strpos($str, ',') !== FALSE) {
      return '"'.str_replace('"', '""', $str).'"';
    } else {
      return $str;
    }
  }

}
?>