<?php
/**
* Page module - Comment to page site
*
* @package module_feedback
* @author Bodo Grassmann <grassmann@papaya-cms.com>
* @version $Id: content_feedback.php,v 1.21 2006/05/26 12:28:42 kelm Exp $
*/

/**
* Basic class page module
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_content.php');

/**
* Check library for string validation
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');

/**
* Page to send comments
*
* @package module_feedback
* @author Bodo Grassmann <grassmann@papaya-cms.com>
*/
class content_feedback extends base_content {

  /**
  * Linked topic
  * @var
  */
  var $linkedTopic = NULL;

  /**
  * cacheable ?
  * @var boolean
  */
  var $cacheable = FALSE;

  /**
  * Parameter name
  * @var string
  */
  var $paramName = 'mail';

  /**
  * Content edit fields
  * @var array
  */
  var $editFields = array(
    'title' => array('Page Title', 'isSomeText', FALSE, 'input', 200, '', ''),
    'msg_mailto' => array('Recipient', 'isEmail', FALSE, 'input', 200, '', ''),
    'nl2br'=>array ('Automatic linebreak', 'isNum', FALSE, 'combo', array(0=>'Yes', 1=>'No'),
     'Apply linebreaks from input to the HTML output.'),
    'msg_hello'=> array('Hello/General', 'isSomeText', FALSE, 'textarea', 6, '', ''),
    'msg_subject'=> array('Template Subject', 'isSomeText', FALSE, 'input', 200, '{%SUBJECT%}', '[Comment] {%SUBJECT%}'),
    'msg_body'=> array('Template Body', 'isSomeText', FALSE, 'textarea', 6, '{%FROM%}, {%NAME%}, {%TEXT%}',
                       "Name: {%NAME%}\nEmail: {%FROM%}\n\nComment:\n{%TEXT%}"),
    'Captions',
    'cap_sender' => array('Sender', 'isNoHTML', FALSE, 'input', 200, '', 'Your name'),
    'cap_email' => array('E-Mail', 'isNoHTML', FALSE, 'input', 200, '', 'Your email address'),
    'cap_subject' => array('Subject', 'isNoHTML', FALSE, 'input', 200, '', 'Subject'),
    'cap_message' => array('Message', 'isNoHTML', FALSE, 'input', 200, '', 'Your message'),
    'Confirmation',
    'confirm_subject' => array('Confirmation Email Subject', 'isNoHTML', FALSE, 'input', 200,
      'Email subject sent to surfer. Leave empty for not sending any email to surfer.', ''),
    'confirm_body' => array('Confirmation Email Body', 'isNoHTML', FALSE, 'textarea', 5, 'Email body sent to surfer.',
      'Thank you for your comment. We will handle it as soon as possible.'),
    'Messages',
    'msg_hello'=> array('Hello', 'isSomeText', FALSE, 'textarea', 3, '',
      ''),
    'msg_error'=> array('Input Error', 'isSomeText', FALSE, 'textarea', 3, '',
      'Please check your input.'),
    'msg_send'=> array('Confirmation', 'isSomeText', FALSE, 'textarea', 3, '',
      'Message sent. Thank You.'),
    'msg_notsend'=> array('Send Error', 'isSomeText', FALSE, 'textarea', 3, '',
      'Send Error. Please try again.'),
    'msg_privacy' => array('Privacy', 'isSomeText', FALSE, 'textarea', 10, '', ''),
  );

  /**
  * Get parsed data
  *
  * @access public
  * @return string
  */
  function getParsedData() {
    $this->initializeParams();
    include_once(PAPAYA_INCLUDE_PATH.'system/sys_email.php');
    $this->emailObj = new email();

    $result = '';

    if (isset($this->data['title'])) {
      $result .= sprintf('<title>%s</title>', @papaya_strings::escapeHTMLChars($this->data['title']));
      $result .= sprintf("<text>%s</text>", @$this->getXHTMLString($this->data['msg_hello'], !((bool)@$this->data['nl2br']) ));
    }
    $result .= '<mail>'.LF;

    if (isset($this->params['submit']) &&  $this->params['submit']) {
      $errors = $this->checkMailDialogInput();
      if (!$errors) {
        $result .= $this->sendEmail();
      } else {
        $result .= '<message type="error">'.LF;
        $result .= @$this->getXHTMLString($this->data['msg_error']);
        if (is_array($errors)) {
          $result .= '<ul>'.LF;
          foreach ($errors as $fieldCaption) {
            $result .= sprintf('<li>%s</li>'.LF, papaya_strings::escapeHTMLChars($fieldCaption));
          }
          $result .= '</ul>'.LF;
        }
        $result .= '</message>'.LF;
        $result .= $this->getMailDialog();
      }
    } else {
      $result .= $this->getMailDialog();
    }
    $result .= sprintf('<privacy>%s</privacy>', @$this->getXHTMLString($this->data['msg_privacy']));
    $result .= '</mail>'.LF;
    return $result;
  }

  function sendEmail() {
    $result = '';
    $content['NAME']    = $this->params['mail_name'];
    $content['FROM']    = $this->params['mail_from'];
    $content['SUBJECT'] = $this->params['mail_subject'];
    $content['TEXT']    = $this->params['mail_message'];

    if (isset($this->params['mail_from'])) {
      $this->emailObj->setHeader('From', @$this->params['mail_from']);
    }
    $this->emailObj->setTemplate('subject', @$this->data['msg_subject'], $content);
    $this->emailObj->setTemplate('body', @$this->data['msg_body'], $content);

    if ($this->emailObj->send(@$this->data['msg_mailto'])) {
      $result .= sprintf('<message type="normal">%s</message>'.LF, @$this->getXHTMLString($this->data['msg_send']));
      if (!empty($this->data['confirm_subject'])) {
        $emailConfirmObj = new email();
        $emailConfirmObj->setHeader('From', $this->data['msg_mailto']);
        $emailConfirmObj->send(@$this->params['mail_from'], @(string)$this->data['confirm_subject'], @(string)$this->data['confirm_body']);
      }
    } else {
      $result .= sprintf('<message type="warning">%s</message>'.LF, @$this->getXHTMLString($this->data['msg_notsend']));
    }
    return $result;
  }

  function getMailDialog() {
    $result = sprintf('<form action="%s" method="post">'.LF, $this->baseLink);
    $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />'.LF,
      $this->paramName, 'submit', 1);
    $result .= sprintf('<label for="%s[%s]">%s</label>'.LF,
      $this->paramName, 'mail_name', papaya_strings::escapeHTMLChars(@$this->data['cap_sender']));
    $result .= sprintf('<input type="text" name="%s[%s]" value="%s" class="text" />'.LF,
      $this->paramName, 'mail_name', papaya_strings::escapeHTMLChars(@$this->params['mail_name']));

    $result .= sprintf('<label for="%s[%s]">%s</label>'.LF,
      $this->paramName, 'mail_from', papaya_strings::escapeHTMLChars(@$this->data['cap_email']));
    $result .= sprintf('<input type="text" name="%s[%s]" value="%s" class="text" />'.LF,
      $this->paramName, 'mail_from', papaya_strings::escapeHTMLChars(@$this->params['mail_from']));

    $result .= sprintf('<label for="%s[%s]">%s</label>'.LF,
      $this->paramName, 'mail_subject', papaya_strings::escapeHTMLChars(@$this->data['cap_subject']));
    $result .= sprintf('<input type="text" name="%s[%s]" value="%s" class="text" />'.LF,
      $this->paramName, 'mail_subject', papaya_strings::escapeHTMLChars(@$this->params['mail_subject']));

    $result .= sprintf('<label for="%s[%s]">%s</label>'.LF,
      $this->paramName, 'mail_message', papaya_strings::escapeHTMLChars(@$this->data['cap_message']));
    $result .= sprintf('<textarea name="%s[%s]" class="text">%s</textarea>'.LF,
      $this->paramName, 'mail_message', papaya_strings::escapeHTMLChars(@$this->params['mail_message']));
    $result .= '</form>'.LF;
    return $result;
  }

  function checkMailDialogInput() {
    $result = FALSE;
    if (!checkit::isSomeText(@$this->params['mail_name'], TRUE)) {
      $result['mail_name'] = @$this->data['cap_sender'];
    }
    if (!checkit::isEmail(@$this->params['mail_from'], FALSE)) {
      $result['mail_from'] = @$this->data['cap_email'];
    }
    if (!checkit::isSomeText(@$this->params['mail_subject'], TRUE)) {
      $result['mail_subject'] = @$this->data['cap_subject'];
    }
    if (!checkit::isSomeText(@$this->params['mail_message'], TRUE)) {
      $result['mail_message'] = @$this->data['cap_message'];
    }
    return $result;
  }
}

?>