<?php 
/**
* class for a pdf table row
* 
* @module papaya_pdf
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_pdf_table_row.php,v 1.1 2006/04/27 15:37:16 weinert Exp $
*/

/**
* table row
*/
class papaya_pdf_table_row {
  /**
  * table (parent object)
  * @var object papaya_pdf_table
  */
  var $table = NULL;
  
  /**
  * array of papaya_pdf_table_cell
  * @var array $cells
  */
  var $_cells = array();
  
  /**
  * row index
  * @var integer
  */
  var $index = 0;
  
  /**
  * needed height (first possible line break)
  * @var float
  */
  var $minHeight = 0;
  
  /**
  * top left of this column
  * @var array
  */
  var $leftTop = NULL;
  /**
  * bottom left of this column
  * @var array
  */
  var $leftBottom = NULL;
  
  /**
  * attrbutes
  * @var array
  */
  var $attr = array();
  
  /**
  * table row
  *
  * @param object papaya_pdf_table &$table
  * @access public
  * @return
  */
  function __construct(&$table, $attr = NULL) {
    $this->table = &$table;
    if (isset($attr)) {
      $this->attributes = $attr;
    }
  }
  
  /**
  * PHP 4 constructor redirect
  *
  * @param object papaya_pdf_table &$table
  * @access public
  * @return
  */
  function papaya_pdf_table_row(&$table, $attr = NULL) {
    $this->__construct($table, $attr);
  }
  
  /**
  * add a table cell object to row
  *
  * @param object papaya_pdf_table_cell &$cell
  * @access public
  */
  function addCell(&$cell) {
    $idx = count($this->_cells);
    $this->_cells[$idx] = &$cell;
    return $idx;
  }
  
  /**
  * minimum height needed for (breakable) content
  *
  * @param float $height
  * @access public
  */
  function updateMinHeight($height) {
    $padInc = $this->table->padding * 2;
    if (($height + $padInc) > $this->minHeight) {
      $this->minHeight = $height + $padInc;
    }
  }
  
  function setLeftTop($page, $column, $x, $y) {
    $this->leftTop = array(
      'page' => $page,
      'col' => $column,
      'x' => $x,
      'y' => $y,
    );
  }
  
  function getLeftTop() {
    if (isset($this->leftTop)) {
      return $this->leftTop;
    } elseif ($this->index > 0) {
      $row = &$this->table->getRowByIndex($this->index - 1);
      $this->leftTop = $row->getLeftBottom();
      return $this->leftTop;
    } else {
      return NULL;
    }
  }
  
  function getAttributes() {
    $parentAttr = $this->table->getAttributes();
    if (isset($this->attributes)) {
      return array_merge($parentAttr, $this->attributes);
    } else {
      return $parentAttr;
    }
  }
  
  function setLeftBottom($page, $column, $x, $y) {
    if ((!isset($this->leftBottom)) ||
        ($this->leftBottom['page'] < $page) ||
        ($this->leftBottom['page'] == $page && $this->leftBottom['col'] < $column) ||
        ($this->leftBottom['page'] == $page && $this->leftBottom['col'] == $column && $this->leftBottom['y'] < $y)) {
      $this->leftBottom = array(
        'page' => $page,
        'col' => $column,
        'x' => $x,
        'y' => $y + ($this->table->padding * 2),
      );   
    }
  }
  
  function getLeftBottom() {
    return $this->leftBottom;
  }
  
  function cellCount() {
    return count($this->_cells);
  }
  
  function &getCellByIndex($cellIdx) {
    if (isset($this->_cells[$cellIdx])) {
      return $this->_cells[$cellIdx];
    }
    $null = NULL;
    return $null;  
  }
}
?>