<?php
/**
* User autentification
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_auth.php,v 1.12 2006/05/31 13:53:32 weinert Exp $
*/

/**
* Base class data access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* Function class input verifying
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');

/**
  * User autentification
  *
  * Verifying and change user data for admin interface
  *
  * @package papaya
  * @author Thomas Weinert
  */
class base_auth extends base_db {
  var $tableOptions = PAPAYA_DB_TBL_AUTHOPTIONS;

  var $tableAuthUser = PAPAYA_DB_TBL_AUTHUSER;
  var $tableAuthGroups = PAPAYA_DB_TBL_AUTHGROUPS;
  var $tableAuthUserLinks = PAPAYA_DB_TBL_AUTHLINK;
  
  var $tableAuthPermissions = PAPAYA_DB_TBL_AUTHPERM;
  var $tableModulePermissions = PAPAYA_DB_TBL_AUTHMODPERMS;
  var $tableModulePermissionsLinks = PAPAYA_DB_TBL_AUTHMODPERMLINKS;

  var $tableCommunitySurfers = PAPAYA_DB_TBL_SURFER;

  var $paramName = 'usr';
  var $params = NULL;
  var $sessionParamName = 'PAPAYA_SESS_usr';
  var $sessionParams = NULL;

  var $user = NULL;
  var $userId = '';
  var $userPerms = NULL;
  var $startNode = 0;
  var $subLevel = 0;

  var $isValid = FALSE;

  var $users = NULL;
  var $groups = NULL;
  var $perms = NULL;
  var $groupTree = NULL;

  var $layout = NULL;
  var $images = NULL;
  var $authUser = NULL;
  var $msgs = NULL;
  
  var $permDefs = array(
      1=>array('Create pages', 2),
      2=>array('Edit pages from others', 2),
      4=>array('User management', 3),
      6=>array('Configure views', 3),
      7=>array('Define metatags', 2),
      8=>array('Upload files', 4),
      9=>array('Rename files', 4),
    10=>array('Delete files', 4),
    13=>array('Edit boxes', 5),
    14=>array('Link boxes', 5),
    15=>array('File management', 4),
    16=>array('Define aliases', 2),
    21=>array('Move', 2),
    22=>array('Copy', 2),
    23=>array('Delete pages', 3),
    24=>array('Version management', 2),
    25=>array('System configuration', 3),
    26=>array('Publish', 2),
    27=>array('Change folders', 4),
    28=>array('Image browser', 4),
    29=>array('Change edit permissions', 2),
    30=>array('Module management', 3),
    31=>array('Event protocol', 3),
    32=>array('Cronjobs', 3),
    33=>array('Check and correct path index', 3),
    34=>array('Messages', 1),
    35=>array('Edit pages', 2),
    36=>array('Translate', 6),
    37=>array('Clear ouput cache', 2),
    38=>array('View trash', 2)
  );

  var $userOptions = array(
    'PAPAYA_UI_LANGUAGE', 'PAPAYA_UI_THEME', 'PAPAYA_USE_FLASHCONTROLS', 'PAPAYA_USE_RICHTEXT',
    'PAPAYA_OVERVIEW_ITEMS_UNPUBLISHED', 'PAPAYA_OVERVIEW_ITEMS_PUBLISHED',
    'PAPAYA_OVERVIEW_ITEMS_MESSAGES', 'PAPAYA_OVERVIEW_ITEMS_TASKS'
  );
  var $options = array();

  var $defaultUsername = 'new';

  /**
  * Initialisation of parameters
  *
  * @param string $paramName optional, default value 'usr'
  * @access public
  */
  function initialize($paramName = 'usr') {
    $this->paramName = $paramName;
    $this->sessionParamName = 'PAPAYA_SESS_auth_'.$this->paramName;
    $this->initializeParams();

    $this->sessionParams = $this->getSessionValue($this->sessionParamName);
  }

  /**
  * Execute Login
  *
  * @access public
  * @return boolean Login successful
  */
  function execLogin() {
    $globalVariableName = $this->paramName.'_chgpass';
    if (isset($_GET[$globalVariableName]) && preg_match('/^[a-fA-F\d]{32}$/', $_GET[$globalVariableName])) {
      $this->params['chgpass'] = $_GET[$globalVariableName];
      if ($userId = $this->checkChangePasswordId($this->params['chgpass'])) {
        return $this->changeForgottenPassword($userId);
      } else {
        return $this->login();
      }
    } elseif (isset($this->params['chgpass']) && preg_match('/^[a-fA-F\d]{32}$/', $this->params['chgpass'])) {
      if ($userId = $this->checkChangePasswordId($this->params['chgpass'])) {
        return $this->changeForgottenPassword($userId);
      } else {
        return $this->login();
      }
    } else {
      switch (@$this->params['cmd']) {
        case 'logout' :
          if ($this->restoreLogin()) {
            $this->logout();
          }
          return $this->login();
        case 'forgot' :
          $this->forgotPassword();
          return FALSE;
        default :
          return $this->login();
      }
    }
  }

  /**
  * Login function
  *
  * @access public
  * @return boolean
  */
  function login() {
    if (isset($this->params['login'])) {
      if ($this->checkLoginTry($this->params['username'])) {
        if ($this->loadLogin($this->params['username'], $this->getPasswordHash($this->params['password']))) {;
          $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_USER,
            sprintf($this->_gt('User "%s" logged in.'), papaya_strings::escapeHTMLChars($this->user['username'])));
          $GLOBALS['PAPAYA_SESSION']->resetData();
          $this->setSessionToken($this->user['user_id']);
          $GLOBALS['PAPAYA_SESSION']->regenerateSID();
          return TRUE;
        } else {
          $this->logMsg(MSG_WARNING, PAPAYA_LOGTYPE_USER,
            sprintf($this->_gt('Login of "%s" failed.'),
              papaya_strings::escapeHTMLChars($this->params['username'])));
        }
      } else {
        $this->logMsg(MSG_WARNING, PAPAYA_LOGTYPE_USER,
          sprintf($this->_gt('Login of "%s" temporary disabled.'),
          papaya_strings::escapeHTMLChars($this->params['username'])));
      }
    } else {
      if ($this->restoreLogin()) {
        return TRUE;
      }
    }
    $this->getLoginXML();
    return FALSE;
  }

  function setSessionToken($userGuid) {
    $this->sessionParams['user_guid'] = $userGuid;
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  /**
  * Logout function
  *
  * @access public
  */
  function logout() {
    $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_USER,
      sprintf($this->_gt('User "%s" logged out.'), papaya_strings::escapeHTMLChars($this->user['fullname'])));
    $this->isValid = FALSE;
    unset($this->user);
    $this->setSessionToken(0);
  }

  /**
  * Load login data
  * @param string $username
  * @param string $password
  * @return boolean user valid?
  */
  function loadLogin($username, $password) {
    $this->user = array();
    $this->isValid = FALSE;
    $sql = "SELECT user_id
            FROM %s
            WHERE username = '%s'
              AND user_password = '%s'
              AND active = '1'";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableAuthUser, $username, $password))) {
      if ($row = $res->fetchRow()) {
        $this->load($row[0], TRUE);
        $this->setSessionToken($row[0]);
      } elseif ($username == 'admin' && ($password == '' || $password == $this->getPasswordHash(''))) {
        if ($this->addDefaultAdminUser()) {
          return $this->loadLogin($username, $this->getPasswordHash(''));
        }
      }
    }
    return $this->isValid;
  }

  /**
  * Login aus Sessiondaten wiederherstellen
  *
  * @access public
  * @return boolean Login restore successful
  */
  function restoreLogin() {
    $this->user = array();
    $this->isValid = FALSE;
    $this->load($this->sessionParams['user_guid'], TRUE);
    return $this->isValid;
  }

  /**
  * Load user data
  * @param integer $uid
  * @param boolean $login
  * @return boolean loading successful
  */
  function load($uid, $login=FALSE) {
    unset($this->user);
    $this->isValid = FALSE;
    $active = ($login) ? ' AND active = 1' : '';
    $sql = "SELECT user_id, group_id, username, surname, givenname,
                    user_password, email, start_node, sub_level, active, userperm
            FROM %s WHERE user_id = '%s' $active";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableAuthUser, $uid))) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $row['perms'] = $this->decodePermStr($row['userperm']);
        $row['fullname'] = $row['givenname'].' '.$row['surname'];
        $this->user = $row;
        $this->userId = $row['user_id'];
        $this->isValid = TRUE;
        $this->startNode = (int)$row['start_node'];
        $this->subLevel = (int)$row['sub_level'];
        $this->userPerms = $this->loadUserPerms($login);
        if ($row['user_password'] == $this->getPasswordHash('')) {
          $this->addMsg(MSG_WARNING, $this->_gt('User has empty password. Please change it.'));
        }
      }
    }
    $this->loadOptions();
    return $this->isValid;
  }

  function loadOptions() {
    $this->options = array();
    if ($this->isValid) {
      $sql = "SELECT opt_name, opt_value FROM %s WHERE user_id = '%s'";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableOptions, $this->userId))) {
        while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          if (!isset($this->options[$row['opt_name']])) {
            $this->options[$row['opt_name']] = $row['opt_value'];
          }
        }
      }
    }
    foreach($this->userOptions as $opt) {
      if (!isset($this->options[$opt])) {
        if (defined($opt)) {
          $this->options[$opt] = constant($opt);
        } else {
          $this->options[$opt] = NULL;
        }
      }
    }
  }

  /**
  * Load user permissions
  *
  * @param boolean $login optional, default value FALSE
  * @access public
  * @return array permissions
  */
  function loadUserPerms($login = FALSE) {
    $perms = NULL;
    $this->isAuthuser = $login;
    if ($this->isValid) {
      //Zuordnungen User und Gruppen laden
      $sql = "SELECT user_id, group_id
              FROM %s
              WHERE user_id = '%s'";
      $params = array($this->tableAuthUserLinks,$this->userId);
      if ($res = $this->databaseQueryFmt($sql,$params)) {
        while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->user['groups'][] = $row['group_id'];
        }
        if (isset($this->user['groups']) && is_array($this->user['groups'])) {
          if (!in_array($this->user['group_id'], $this->user['groups'])) {
            $this->user['groups'][] = $this->user['group_id'];
          }
        } else {
          $this->user['groups'] = array($this->user['group_id']);
        }
      }
      //Gruppen laden (wenn noch nicht explizit geladen)
      if (!(isset($this->groups) && is_array($this->groups))) {
        $this->loadGroups($this->user['groups']);
      }

      //Admin?
      if (isset($this->user['groups']) && is_array($this->user['groups'])) {
        $activePerms = array();
        if ($this->isAdmin() && $login) {
          $sql = "SELECT perm_id FROM %s WHERE perm_active = '1' OR permgroup_id = 0";
          $params = array($this->tableAuthPermissions);
          if ($res = $this->databaseQueryFmt($sql,$params)) {
            while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
              $activePerms[] = (int)$row['perm_id'];
            }
            $perms = $activePerms;
          }
          if (!(isset($perms) && is_array($perms) && count($perms) > 0) &&
              isset($this->permDefs) && is_array($this->permDefs)) {
            $perms = array_keys($this->permDefs);
          }
        } else {
          //Rechte des Users
          $perms = $this->user['perms'];
          settype($perms, 'array');

          //Rechte der Gruppen dazu
          if (isset($this->user['groups']) && is_array($this->user['groups'])) {
            foreach($this->user['groups'] as $groupId) {
              if (isset($this->groups[(int)$groupId]) && is_array($this->groups[(int)$groupId]) &&
                  isset($this->groups[(int)$groupId]['perms']) && is_array($this->groups[(int)$groupId]['perms'])) {
                  $perms = array_merge($perms, $this->groups[(int)$groupId]['perms']);
              }
            }
          }
          $perms = array_unique($perms);
          //Bei Login deaktivierte Rechte rauswerfen
          if ($login && is_array($perms)) {
            $sql = "SELECT perm_id FROM %s WHERE perm_active = '1'";
            $params = array($this->tableAuthPermissions);
            if ($res = $this->databaseQueryFmt($sql,$params)) {
              while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
                $activePerms[] = (int)$row['perm_id'];
              }
              $perms = array_intersect($perms, $activePerms);
            }
          }
        }
      }
    }
    return $perms;
  }

  /**
  * Load groups
  *
  * @param array $groupIds optional, default value NULL
  * @access public
  */
  function loadGroups($groupIds = NULL) {
    unset($this->groups);
    if (isset($groupIds) && is_array($groupIds)) {
      if (count($groupIds) == 1) {
        $filter = "WHERE group_id = '".$groupIds[0]."'";
      } else {
        $filter = "WHERE group_id IN ('".implode("','",$groupIds)."')";
      }
    } else {
      $filter = '';
    }
    //special group for admins - not in database
    if (isset($groupIds[-1]) || !isset($groupIds)) {
      $this->groups[-1] = array(
        'group_id' => -1,
        'grouptitle' => $this->_gt('Adminstrator'),
        'groupperm' => ''
      );
    }
    //load user defined groups
    $sql = "SELECT group_id, grouptitle, groupperm
            FROM %s $filter
            ORDER BY grouptitle, group_id";
    $params = array($this->tableAuthGroups);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $row['perms'] = $this->decodePermStr($row['groupperm']);
        $this->groups[(int)$row["group_id"]] = $row;
      }
    }
  }

  /**
  * Load Module permissions
  *
  * @param integer $moduleId  optional, default value  NULL
  * @return boolean
  */
  function loadModulePerms($moduleId = NULL) {
    $sql = "SELECT module_id, modperm_id, modperm_active FROM %s ";
    if (isset($moduleId)) {
      $sql .= " WHERE module_id = '%s' ";
      if (isset($this->modulePerms[$moduleId])) {
        unset($this->modulePerms[$moduleId]);
      }
    } else {
      unset($this->modulePerms);
    }
    $sql .= ' ORDER BY module_id, modperm_id';
    if ($res = $this->databaseQueryFmt($sql, array($this->tableModulePermissions, $moduleId))) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->modulePerms[$row['module_id']][$row['modperm_id']] = $row['modperm_active'];
      }
      $res->free();
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load Module permission Links
  *
  * @param integer $groupId optional, default value undefined
  * @param integer $userId optional, default value undefined
  * @param integer $moduleId  optional, default value  NULL
  *
  * @return boolean
  */
  function loadModulePermLinks($groupId, $userId, $moduleId = NULL) {
    $this->loadModulePerms($moduleId);
    $sql = "SELECT group_id, user_id, module_id, module_perms
            FROM %s
            WHERE ((group_id = %d AND user_id = '0') OR (group_id = 0 AND user_id = '%s') ";
    if (isset($this->user['groups']) && is_array($this->user['groups'])) {
      $sql .= (count($this->user['groups']) <= 1)
            ? " OR (group_id = ".(int)$this->user['groups'][0]." AND (user_id = '0'))"
            : " OR (group_id IN ('".implode("','", $this->user['groups'])."') AND (user_id = '0'))";
    }
    $sql .= ') ';
    if (isset($moduleId)) {
      $sql .= " AND module_id = '%s' ";
      $this->modulePermLinks[$moduleId] = array();
    } else {
      unset($this->modulePermLinks);
    }
    $sql .= ' ORDER BY module_id';
    if ($res = $this->databaseQueryFmt($sql, array($this->tableModulePermissionsLinks, $groupId, $userId, $moduleId))) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $perms =  $this->decodePermStr($row['module_perms']);
        if ($row['user_id'] == $userId) {
          $this->modulePermLinks[$row['module_id']]['USER'] =  $perms;
        } elseif ($row['group_id'] == $groupId) {
          $this->modulePermLinks[$row['module_id']]['GROUP'] = $perms;
        }
        if (isset($this->modulePermLinks[$row['module_id']]['ALL']) && is_array($this->modulePermLinks[$row['module_id']]['ALL'])) {
          $this->modulePermLinks[$row['module_id']]['ALL'] =
            array_merge($this->modulePermLinks[$row['module_id']]['ALL'], $perms);
        } else {
          $this->modulePermLinks[$row['module_id']]['ALL'] = $perms;
        }
      }
      $res->free();
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Check permissions
  *
  * @access public
  * @return boolean
  */
  function checkPerms() {
    unset($perms);
    $sql = "SELECT perm_id FROM %s";
    $params = array($this->tableAuthPermissions);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $perms[] = (int)$row["perm_id"];
      }
    }
    if (isset($perms) && is_array($perms)) {
      $rPerms = array_diff(array_keys($this->permDefs), $perms);
    } else {
      $rPerms = array_keys($this->permDefs);
    }
    if (isset($rPerms) && is_array($rPerms) && (count($rPerms) > 0)) {
      $data = array();
      foreach($rPerms as $permId) {
        $perm = $this->permDefs[$permId];
        $dataRow = array(
          'perm_id' => (int)$permId,
          'perm_title' => $perm[0],
          'permgroup_id' => (int)$perm[1]
        );
        $data[] = $dataRow;
      }
      return (FALSE !== $this->databaseInsertRecords($this->tableAuthPermissions, $data));
    }
  }

  /**
  * syncronize Surfer
  *
  * @param integer $userId
  * @access public
  */
  function syncronizeSurfer($userId) {
    if (isset($this->user)) {
      $sql = "SELECT auth_user_id, surfer_id FROM %s WHERE surfer_handle = '%s'";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableCommunitySurfers, $this->user['username']))) {
        if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          if ($row['auth_user_id'] != $userId) {
            $this->databaseDeleteRecord($this->tableCommunitySurfers, 'surfer_id', $row['surfer_id']);
          }
        }
      }
      $sql = "SELECT COUNT(*) FROM %s WHERE auth_user_id ='%s'";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableCommunitySurfers, $userId))) {
        if ($row = $res->fetchRow()) {
          $data = array(
            "surfer_handle" => $this->user['username'],
            "surfer_givenname" => $this->user['givenname'],
            "surfer_surname" => $this->user['surname'],
            "surfer_email" => $this->user['email'],
            "surfer_password" => $this->user['user_password'],
          );
          if ($row[0] > 0) {
            $this->databaseUpdateRecord($this->tableCommunitySurfers, $data, 'auth_user_id', $userId);
          } else {
            $data['auth_user_id'] = $userId;
            $this->databaseInsertRecord($this->tableCommunitySurfers, 'surfer_id', $data);
          }
        }
      }
    }
  }

  /**
  * Check user is in database
  *
  * @param string $username
  * @access public
  * @return mixed valid user id or FALSE
  */
  function userExists($username) {
    if (trim($username) != '') {
      $sql = "SELECT user_id
              FROM %s
              WHERE username = '%s'";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableAuthUser, $username))) {
        if ($row = $res->fetchRow()) {
          return $row[0];
        }
      }
    }
    return FALSE;
  }

  /**
  * Check Surfer is in database
  *
  * @param integer $surferName
  * @access public
  * @return mixed surfer id or FALSE
  */
  function surferExists($surferName) {
    if (trim($surferName) != '') {
      $sql = "SELECT surfer_id
              FROM %s
              WHERE surfer_handle = '%s'";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableCommunitySurfers, $surferName))) {
        if ($row = $res->fetchRow()) {
          return $row[0];
        }
      }
    }
    return FALSE;
  }

  /**
  * Save login data
  *
  * @param integer $userId
  * @access public
  * @return boolean
  */
  function saveLoginData($userId) {
    $result = FALSE;
    if ($chkUserId = $this->userExists($this->params['username'])) {
      if ($chkUserId == $userId) {
        if (trim($this->params['password']) != '') {
            $data = array(
            'user_password' => $this->getPasswordHash($this->params['password']),
            'chg_id' => '',
            'chg_time' => '0'
          );
          if (FALSE !== $this->databaseUpdateRecord($this->tableAuthUser, $data, "user_id", $userId) ) {
            $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Password')));
            $result = TRUE;
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
          }
        }
      } else {
        $this->addMsg(MSG_ERROR, sprintf('Login "%s" already in use.',
          papaya_strings::escapeHTMLChars($this->params['username'])));
      }
    } elseif ($this->surferExists($this->params['username'])) {
      $this->addMsg(MSG_ERROR, sprintf('Login "%s" already in use for community user.',
        papaya_strings::escapeHTMLChars($this->params['username'])));
    } else {
      if (trim($this->params['password']) != '') {
          $data = array(
          'username' => $this->params['username'],
          'user_password' => $this->getPasswordHash($this->params['password']),
          'chg_id' => '',
          'chg_time' => 0
        );
        if (FALSE !== $this->databaseUpdateRecord($this->tableAuthUser, $data, "user_id", $userId)) {
          $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Login and Password')));
          $result = TRUE;
        } else {
          $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
        }
      } else {
          $data = array(
          'username' => $this->params['username'],
          'chg_id' => '',
          'chg_time' => '0'
        );
        if (FALSE !==$this->databaseUpdateRecord($this->tableAuthUser, $data, "user_id", $userId) ) {
          $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Login')));
          $result = TRUE;
        } else {
          $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
        }
      }
    }
    if ($result == TRUE) {
      $this->load($userId);
      $this->syncronizeSurfer($userId);
      return TRUE;
    } else {
      return FALSE;
    }
  }

  /**
  * Save change password id
  *
  * @param integer $userId
  * @param integer $chgId
  * @access public
  * @return mixed user id or FALSE
  */
  function saveChangePasswordId($userId, $chgId) {
    if (preg_match('/^[a-fA-F\d]{32}$/',$userId)) {
      $validUntil = time()+86400;
      $data = array(
        'chg_id' => $chgId,
        'chg_time' => $validUntil
      );
      return (FALSE !== $this->databaseUpdateRecord($this->tableAuthUser, $data, "user_id", $userId));
    }
    return FALSE;
  }

  /**
  * Check change password id
  *
  * @param string $chgId
  * @access public
  * @return mixed user id or FALSE
  */
  function checkChangePasswordId($chgId) {
    if (preg_match('/^[a-fA-F\d]{32}$/',$chgId)) {
      $sql = "SELECT user_id
              FROM %s
              WHERE chg_id = %d
                AND chg_time > %d";
      $params = array($this->tableAuthUser,$chgId,time());
      if ($res = $this->databaseQueryFmt($sql,$params)) {
        if($row = $res->fetchRow()) {
          return $row[0];
        }
      }
    }
    return FALSE;
  }

  /**
  * Decode permission string
  *
  * @param string $str
  * @access public
  * @return mixed
  */
  function decodePermStr($str) {
    if (preg_match_all("/\d+/", $str, $matches)) {
      return $matches[0];
    } else {
      return NULL;
    }
  }

  function addDefaultAdminUser() {
    $sql = "SELECT COUNT(*)
            FROM %s";
    $params = array($this->tableAuthUser);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      list($userCount) = $res->fetchRow();
      if ($userCount == 0) {
        srand((double)microtime()*1000000);
        $userId = md5(uniqid(rand()));
        $data = array(
          'user_id'   => $userId,
          'username'  => 'admin',
          'givenname' => 'Default',
          'surname'   => $this->_gt('Administrator'),
          'group_id'  => -1,
          'active'    => TRUE,
          'start_node'=> 0,
          'sub_level' => 0,
          'user_password' => $this->getPasswordHash('')
        );
        if (FALSE !== $this->databaseInsertRecord($this->tableAuthUser, NULL, $data)) {
          $this->addMsg(MSG_INFO, $this->_gt('New user created.'));
          $this->params['uid'] = $userId;
          //ok now we change all resourcen to this user - because it is the only one.
          $data = array(
            'author_id' => $userId,
            'author_group' => -1
          );
          $this->databaseUpdateRecord(PAPAYA_DB_TBL_TOPICS, $data);
          $this->databaseUpdateRecord(PAPAYA_DB_TBL_TOPICS_PUBLIC, $data);
          $data = array(
            'version_author_id' => $userId
          );
          $this->databaseUpdateRecord(PAPAYA_DB_TBL_TOPICS_VERSIONS, $data);
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  /**
  * Check login input
  *
  * @param boolean $setPassword optional, default value FALSE
  * @access public
  * @return boolean
  */
  function checkLoginInput($setPassword = FALSE) {
    $result = FALSE;
    if (isset($this->params['uid']) && (!$this->isValid)) {
      $this->load($this->params['uid']);
    }
    if (isset($this->user) && is_array($this->user)) {
      if (($this->authUser->userId == $this->userId) && ($this->params['username'] != $this->user['username'])) {
        $this->addMsg(MSG_WARNING, $this->_gt('You can not change your own username.'));
        $this->params['username'] = $this->user['username'];
      }
      if ((trim($this->params['password']) == '') && $setPassword) {
        $this->addMsg(MSG_ERROR, $this->_gt('Please enter a password.'));
        return FALSE;
      } elseif ($this->params['password'] != $this->params['password2']) {
        $this->addMsg(MSG_ERROR, $this->_gt('The passwords didnt match!'));
        return FALSE;
      }
      return TRUE;
    }
    return $result;
  }

  /**
  * Check dialog input
  *
  * @param array $fields
  * @access public
  * @return boolean
  */
  function checkDialogInput($fields) {
    $checkFunctions = get_class_methods('checkit');
    foreach($checkFunctions as $idx=>$functionName) {
      $checkFunctions[$idx] = strtolower($functionName);
    }
    $checkObj = &new checkit;
    unset($this->fieldErrors);
    $result = TRUE;
    if (isset($checkFunctions) && is_array($checkFunctions) && isset($fields) && is_array($fields)) {
      foreach($fields as $key=>$val) {
        $checkFunction = strtolower($val[1]);
        if (in_array($checkFunction, $checkFunctions)) {
            if ($checkObj->$checkFunction(@$this->params[$key], $val[2])) {
              $this->fieldErrors[$key] = 0;
              $this->data[$key] = @$this->params[$key];
            } else  {
              $this->addMsg(MSG_ERROR,
                sprintf($this->_gt('The input in field "%s" is not correct.'), papaya_strings::escapeHTMLChars($this->_gt($val[0]))));
              $this->fieldErrors[$key] = 1;
              $result = FALSE;
            }
        }
      }
    }
    return $result;
  }

  /**
  * Loginformular anzeigen
  */
  /**
  * Get login XML
  *
  * @access public
  */
  function getLoginXML() {
    $result = "";
    $result .= sprintf('<form action="%s" method="post" name="login" id="loginDialog">'.LF, $this->baseLink);
    $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />'.LF, $this->paramName, 'login', 1);
    if (isset($this->params["login"])) {
      $result .= sprintf('<div class="message">%s.</div>'.LF, $this->_gt('Login failed'));
      if ($uid = $this->userExists($this->params['username'])) {
        $href = $this->baseLink.'?'.$this->paramName.'[cmd]=forgot&amp;'.$this->paramName.'[uid]='.$uid;
        $result .= sprintf('<div class="text"><a href="%s">'.$this->_gt('Forgot password?').'</a></div>'.LF,
          $href);
      }
    }
    $result .= sprintf("<label>%s:</label>", $this->_gt('Username'));
    $result .= sprintf('<input type="text" maxlength="20" size="20" name="%s[username]" value="%s" class="dialogInput" />'.LF, $this->paramName, papaya_strings::escapeHTMLChars(@$this->params['username']));
    $result .= sprintf("<label>%s:</label>", $this->_gt('Password'));
    $result .= sprintf('<input type="password" maxlength="20" size="20" name="%s[password]" class="dialogInput" />'.LF, $this->paramName);
    $result .= sprintf('<div class="topbevel" style="text-align: right;"><input type="submit" value="%s" class="dialogButton" /></div>'.LF, $this->_gt('OK'));
    $result .= '</form>'.LF;
    if (@is_object($this->layout)) {
      $this->layout->setParam('PAGE_TITLE', $this->_gt('Login'));
      $this->layout->setParam('PAGE_USER', $this->_gt('Unknown'));
      $this->layout->add(sprintf('<login title="%s" width="300">%s</login>',
          $this->_gt('Login'), $result));
    }
  }

  /**
  * forgot password
  *
  * @access public
  */
  function forgotPassword() {
    $result = '';
    $this->load($this->params['uid']);
    if ($this->isValid){
      if (isset($this->params['send_password'])) {
        srand((double)microtime()*1000000);
        $chgId = md5(uniqid(rand()));

        if ($this->saveChangePasswordId($this->params['uid'], $chgId)) {
          $mailTo = $this->user['givenname'].' '.$this->user['surname'].' <'.$this->user['email'].'>';
          $subject = papaya_strings::escapeHTMLChars(PAPAYA_PROJECT_TITLE.' - '.$this->_gt('Password forgotten'));
          /**
          * @todo Use language file
          */

          $msg = sprintf($this->_gt('You get this mail because here was a password change request for %s'), PAPAYA_PROJECT_TITLE);
          $msg .= $this->_gt('Please ignore this mail if you do not like to change you password.')."\n\n";
          $msg .= $this->_gt('To change your password please click the link:').LF;
          $msg = wordwrap($msg);

          $href = $this->getAbsoluteURL($this->getBasePath().$this->baseLink, '', FALSE);
          $href .= '?'.$this->paramName.'_chgpass='.$chgId;

          if ((checkit::isEMail($this->user['email'], TRUE)) && @mail($mailTo, $subject, $msg.$href)) {
            $result .= '<div align="center">';
            $result .= '<table border="0" cellpadding="4" cellspacing="1">'.LF;
          $result .= '<tr><td colspan="2"><p>'.$this->_gt('The system sent you an email with a link to change your password. Please remember that this link is only valid for 24 hours.').'</p><br/></td></tr>'.LF;
            $result .= '</table>'.LF;
            $result .= '</div>';
          } else {
            $result .= '<div align="center">';
            $result .= '<table border="0" cellpadding="4" cellspacing="1">'.LF;
          $result .= '<tr><td colspan="2"><p>'.$this->_gt('Could not sent email. Please contact the website administrator.').'</p><br/></td></tr>'.LF;
            $result .= '</table>'.LF;
            $result .= '</div>';
          }
        } else {
          $result .= '<div align="center">';
          $result .= '<table border="0" cellpadding="4" cellspacing="1">'.LF;
        $result .= '<tr><td colspan="2"><p>'.$this->_gt('Could not create link. Please contact the website administrator.').'</p><br/></td></tr>'.LF;
          $result .= '</table>'.LF;
          $result .= '</div>';
        }
        $this->layout->add('<login title="'.$this->_gt('Password forgotten').'" width="300">'.$result.'</login>');
      } else {
        $result .= '<div align="center">';
        $result .= sprintf('<form action="%s" method="post" name="login">'.LF, $this->baseLink);
        $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />'.LF,
          $this->paramName, 'cmd', 'forgot');
        $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />'.LF,
          $this->paramName, 'send_password', 1);
        $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />'.LF,
          $this->paramName, 'uid', papaya_strings::escapeHTMLChars($this->params['uid']));
        $result .= '<table border="0" cellpadding="4" cellspacing="1">'.LF;
        $result .= '<tr><td colspan="2"><p>'.$this->_gt('The system will send you an email with a link to change your password. This link is valid for 24 hours.').'</p><br/></td></tr>'.LF;
        $result .= sprintf('<tr><td colspan="2" align="right"><input type="submit" value="E-Mail senden" class="dialogButton" /></td></tr>'.LF, $this->paramName);
        $result .= '</table>'.LF.'</form>'.LF;
        $result .= '</div>';
        $this->layout->setParam('PAGE_TITLE', $this->_gt('Password forgotten'));
        $this->layout->add('<login title="'.$this->_gt('Password forgotten').'" width="300">'.$result.'</login>');
      }
    }
  }

  /**
  * Change forgotten password
  *
  * @param integer $userId
  * @access public
  * @return boolean
  */
  function changeForgottenPassword($userId) {
    $this->load($userId);
    if (isset($this->user)) {
      $changed = FALSE;
      $this->params['username'] = $this->user['username'];
      if (isset($this->params['change_password']) &&
          $this->checkDialogInput(@$this->fieldsPassword) && $this->checkLoginInput(TRUE)) {
          $changed = $this->saveLoginData($userId);
      }
      if ($changed) {
        return $this->login();
      } else {
        $result = '<div align="center">';
        $result .= sprintf('<form action="%s" method="post" name="login">'.LF, $this->baseLink);
        $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />'.LF,
          $this->paramName, 'chgpass', $this->params['chgpass']);
        $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />'.LF,
          $this->paramName, 'change_password', 1);
        $result .= '<table border="0" cellpadding="4" cellspacing="1">'.LF;
        $result .= '<tr><td colspan="2"><p>'.$this->_gt('Please input the new password in both input fields').'</p><br/></td></tr>'.LF;
    $result .= sprintf("<tr><td>%s:</td>", $this->_gt('Password'));
        $result .= sprintf('<td align="right"><input type="password" maxlength="20" size="20" name="%s[password]" class="dialogPassword" /></td></tr>'.LF, $this->paramName);
    $result .= sprintf("<tr><td>%s:</td>", $this->_gt('Repetition'));
        $result .= sprintf('<td align="right"><input type="password" maxlength="20" size="20" name="%s[password2]" class="dialogPassword" /></td></tr>'.LF, $this->paramName);
        $result .= '</table><div class="dialogButtonArea">';
        $result .= sprintf('<input type="submit" value="%s" class="dialogButton" />'.LF, $this->_gt('Change'));
        $result .= '</div>'.LF.'</form>'.LF;
        $result .= '</div>';
        $this->layout->setParam('PAGE_TITLE', $this->_gt('Change password'));
        $this->layout->add(sprintf('<login title="%s" width="300">%s</login>',
          $this->_gt('Password forgotten'), $result));
      }
    }
    return FALSE;
  }

  /**
  * Has module permission
  *
  * @param integer $permId
  * @param integer $moduleId
  * @access public
  * @return mixed
  */
  function hasModulePerm($permId, $moduleId) {
    if (!isset($this->modulePermLinks[$moduleId])) {
      $this->loadModulePermLinks($this->user['group_id'], $this->user['user_id'], $moduleId);
    }
    return (@in_array($permId,$this->modulePermLinks[$moduleId]['ALL']) || $this->isAdmin());
  }

  /**
  * Is module permission active
  *
  * @param integer $permId
  * @param integer $modId
  * @access public
  * @return boolean
  */
  function isModulePermActive($permId, $modId) {
    return (!(isset($this->modulePerms[$modId][$permId]) && ($this->modulePerms[$modId][$permId] == FALSE)));
  }

  /**
  * Has permission
  *
  * @param integer $permId
  * @param mixed $moduleId optional, default value NULL
  * @access public
  * @return boolean
  */
  function hasPerm($permId, $moduleId = NULL) {
    if (isset($moduleId)) {
      return $this->hasModulePerm($permId, $moduleId);
    } elseif (@is_array($this->userPerms) && in_array($permId, $this->userPerms)) {
      return TRUE;
    } elseif ($this->isAdmin() && isset($this->permDefs[$permId]) && $this->permDefs[$permId][1] == 3) {
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Is in group
  *
  * @param integer $groupId
  * @access public
  * @return boolean
  */
  function inGroup($groupId) {
    if ($this->isValid && isset($this->user) && is_array($this->user)) {
      if ((isset($this->user['groups']) && is_array($this->user['groups']) &&
            in_array($groupId, $this->user['groups'])) || ((int)$this->user['group_id'] == $groupId)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * is administrator
  *
  * @access public
  * @return boolean
  */
  function isAdmin() {
    return $this->inGroup(-1);
  }
  
  function getPasswordHash($password) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_auth_secure.php');
    return base_auth_secure::getPasswordHash($password);
  }
  
  function checkLoginTry($username) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_auth_secure.php');
    $secureLoginObj = &new base_auth_secure();
    $result = $secureLoginObj->checkLoginTry($username, 'admin');
    unset($secureLoginObj);
    return $result;
  }
}

?>
