<?php
/**
* Manage action boxes
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_boxes.php,v 1.35 2006/04/26 10:29:00 weinert Exp $
*/

/**
* base class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* Include input check class
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');

/**
* Manage action boxes
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class base_boxes extends base_db {
  var $tableBox = PAPAYA_DB_TBL_BOX;
  var $tableBoxTrans = PAPAYA_DB_TBL_BOX_TRANS;
  var $tableBoxVersions = PAPAYA_DB_TBL_BOX_VERSIONS;
  var $tableBoxVersionsTrans = PAPAYA_DB_TBL_BOX_VERSIONS_TRANS;
  var $tableBoxPublic = PAPAYA_DB_TBL_BOX_PUBLIC;
  var $tableBoxPublicTrans = PAPAYA_DB_TBL_BOX_PUBLIC_TRANS;
  var $tableBoxgroup = PAPAYA_DB_TBL_BOXGROUP;
  var $tableModules = PAPAYA_DB_TBL_MODULES;
  var $tableViews = PAPAYA_DB_TBL_VIEWS;
  var $tableLink = PAPAYA_DB_TBL_BOXLINKS;
  var $tableAuthUser = PAPAYA_DB_TBL_AUTHUSER;
  var $maxVersions = PAPAYA_VERSIONS_MAXCOUNT;

  var $boxesList;
  var $box;
  var $boxModules;
  var $boxNew;
  var $msgs;

  var $paramName;
  var $params;

  /**
  * Constructor
  *
  * @param string $paramName optional, default value 'bb'
  * @access public
  */
  function __construct($paramName='bb') {
    $this->paramName = $paramName;
    $this->initializeParams();
  }

  /**
  * Initialisation of class variable
  *
  * @access public
  */
  function initialize() {
    $this->sessionParamName = 'PAPAYA_SESS_boxes'.$this->paramName;
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);
    //$this->initializeSessionParam('cmd');
    $this->initializeSessionParam('mode');
    $this->initializeSessionParam('gid', array('bid'));
    $this->initializeSessionParam('bid');
    $this->initializeNodes();
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  /**
  * Initialisation of nodes
  *
  * @access public
  */
  function initializeNodes() {
    $this->opened = $this->sessionParams['opened'];
    switch (@$this->params['cmd']) {
      case 'open' :
        if ($this->params['gid'] > 0) {
          $this->opened[$this->params['gid']] = TRUE;
        }
        break;
      case 'close' :
        if (isset($this->opened[$this->params['gid']])) {
          unset($this->opened[$this->params['gid']]);
        }
        break;
    }
    $this->sessionParams['opened'] = $this->opened;
  }

  /**
  * Access function
  *
  * @access public
  */
  function execute() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_language_select.php');
    $this->lngSelect = &base_language_select::getInstance();
    $this->layout->addLeft($this->lngSelect->getComboFormXML());

    $this->loadList();

    //nderungen gefllig
    switch (@$this->params['cmd']) {
      case 'add_translation' :
        if (isset($this->params['bid']) && $this->load($this->params['bid'], $this->lngSelect->currentLanguageId)) {
          if ((!isset($this->box['TRANSLATION'])) && $this->createTranslation($this->lngSelect->currentLanguageId)) {
            $this->loadTranslation($this->box['box_id'], $this->lngSelect->currentLanguageId);
          }
        }
        break;
      case "box_add" :
        if ($newId = $this->create((int)$this->params['gid'], $this->lngSelect->currentLanguageId)) {
          $this->addMsg(MSG_INFO, $this->_gt('New Box created.'));
          $this->params['bid'] = $newId;
          $this->sessionParams['opened'][$this->params['gid']] = TRUE;
          $this->initializeSessionParam('bid');
          $this->loadList();
        }
        break;
      case "box_edit" :
        if ($this->load($this->params['bid'], $this->lngSelect->currentLanguageId)) {
          $this->initializePropertiesDialog();
          if ($this->dialogProperties->checkDialogInput()) {
            if ($this->save()) {
              unset($this->box);
              unset($this->dialogProperties);
              $this->loadList();
            }
          }
        }
        break;
      case "box_delete" :
        if ($str = $this->deleteBox($this->params['bid'])) {
          $this->layout->add($str);
        }
        break;
      case "chg_view" :
        if (isset($this->params['view_id']) && $this->params['view_id'] > 0 &&
            $this->load($this->params['bid'], $this->lngSelect->currentLanguageId)) {
          if ($this->saveView()) {
            $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('View')));
            unset($this->box);
          }
        }
        break;
      case "group_add" :
        if ($newId = $this->createGroup()) {
          $this->params['gid'] = $newId;
          $this->initializeSessionParam('gid', array('bid'));
          $this->loadGroupList();
          $this->addMsg(MSG_INFO, $this->_gt('New group created.'));
        } else {
          $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
        }
        break;
      case "group_edit" :
        if ($this->loadGroup($this->params['gid'])) {
          if ($this->saveGroup()) {
            $this->loadGroupList();
          }
        }
        break;
      case "group_delete" :
        if ($str = $this->deleteGroup($this->params['gid'])) {
          $this->layout->add($str);
        }
        break;
    }
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  /**
  * Build site
  *
  * @access public
  */
  function getXML() {
    if (isset($this->params['bid']) && $this->load($this->params['bid'], $this->lngSelect->currentLanguageId)) {
      if (isset($this->box) && isset($this->box['TRANSLATION']) && is_array($this->box['TRANSLATION'])) {
        if (@$this->params['cmd'] != "box_delete") {
          switch (@$this->params['mode']) {
            case 1 :
              $this->layout->add($this->editContent());
              break;
            case 2 :
              $this->loadVersionsList($this->params['bid']);
              if ($this->loadOldVersion()) {
                $this->layout->add($this->versionExecute());
              }
              $this->loadList();
              $this->loadVersionsList($this->params['bid']);
              if ($this->loadOldVersion()) {
                $this->getVersionInfos();
              }
              $this->layout->add($this->getVersionsList());
              break;
            case 3 :
              $this->layout->add($this->getPreviewFrame('Preview',''));
              break;
            case 4 :
              if ($this->authUser->hasPerm(26)) {
                $this->loadTranslationsInfo();
                $this->layout->add($this->publishExecute());
                $this->load($this->params['bid'], $this->lngSelect->currentLanguageId);
                $this->loadList();
                $this->loadTranslationsInfo();
                $this->layout->add($this->getPublicData());
              }
              break;
            case 5 :
              $this->layout->addRight($this->getBoxInfos());
              $this->layout->add($this->getEditView());
              break;
            default :
              $this->layout->addRight($this->getBoxInfos());
              $this->layout->add($this->getPropertiesDialog());
          }
        }
      } else {
        $this->layout->addRight($this->getBoxInfos());
        $this->layout->add($this->addTranslationDialog());
        $this->layout->add($this->getPropertiesDialog());
      }
    } elseif ($this->loadGroup($this->params['gid'])) {
      if (@$this->params['cmd'] != "group_delete") {
        $this->layout->add($this->editGroup());
      }
    }
    $this->layout->addLeft($this->getXMLList(
      isset($this->params['bid']) ? $this->params['bid'] : 0,
      isset($this->params['gid']) ? $this->params['gid'] : 0));
    $this->layout->add($this->getButtons());
  }

  /**
  * buttons in menu bar
  *
  * @access public
  */
  function getButtons() {
    $editMenu[] = array('-');
    $editMenu[] = array(
      'New group', '', $this->images[117], 13,
        $this->getLink(array('cmd'=>'group_add')), '_self'
    );
    if (isset($this->boxGroups) && is_array($this->boxGroups)) {
      $editMenu[] = array(
        'Delete group', '', $this->images[10], 13,
        $this->getLink(array('cmd'=>'group_delete', 'gid'=>(int)$this->params['gid'])), '_self'
      );
    }
    $editMenu[] = array('-');
    if ((isset($this->boxGroups) && is_array($this->boxGroups)) || (isset($this->box) && is_array($this->box))) {
      $editMenu[] = array(
        'New box', '', $this->images[117], 13,
          $this->getLink(array('cmd'=>'box_add')), '_self'
      );
    }
    if (isset($this->box) && is_array($this->box)) {
      $editMenu[] = array(
        'Delete box', '', $this->images[10], 13,
        $this->getLink(array('cmd'=>'box_delete', 'bid'=>$this->box['box_id'])),
        '_self'
      );
      $editMenu[] = array('-');
      $editMenu[] = array(
        'Publish box', '', $this->images[16], 26,
        $this->getLink(array('cmd'=>'publish', 'mode'=>4,  'bid'=>$this->box['box_id'])),
        '_self'
      );
      if (isset($this->box) && isset($this->box['TRANSLATION']) && is_array($this->box['TRANSLATION'])) {
        $toolbar[] = array(
          'Properties', '', $this->images[44], 13,
          $this->getLink(array('cmd'=>'chg_mode', 'mode'=>0, 'bid'=>$this->box['box_id'])),
          '_self', ($this->params['mode']==0)
        );
        $toolbar[] = array(
          'Module', 'Select module and view', $this->images[3], 13,
          $this->getLink(array('cmd'=>'chg_mode', 'mode'=>5, 'bid'=>$this->box['box_id'])),
          '_self', ($this->params['mode']==5)
        );
        $toolbar[] = array(
          'Content', 'Edit content', $this->images[45], 13,
          $this->getLink(array('cmd'=>'chg_mode', 'mode'=>1, 'bid'=>$this->box['box_id'])),
          '_self', ($this->params['mode']==1)
        );
        $toolbar[] = array(
          'XML Preview', '', $this->images[92], 13,
          $this->getLink(array('cmd'=>'chg_mode', 'mode'=>3, 'bid'=>$this->box['box_id'])),
          '_self', ($this->params['mode']==3)
        );
        $toolbar[] = array(
          'Published', '', $this->images[16], 26,
          $this->getLink(array('cmd'=>'chg_mode', 'mode'=>4, 'bid'=>$this->box['box_id'])),
          '_self', ($this->params['mode']==4)
        );
        $toolbar[] = array('-');
        $toolbar[] = array(
          'Versions', 'Version management', $this->images[74], 13,
          $this->getLink(array('cmd'=>'chg_mode', 'mode'=>2, 'bid'=>$this->box['box_id'])),
          '_self', ($this->params['mode']==2)
        );
        $this->layout->add('<toolbar>'.$this->navObj->getButtons($toolbar).'</toolbar>', 'toolbars');
      }
    }
    $this->navObj->setMenu('edit',$editMenu);
  }

  /**
  * Load list
  *
  * @access public
  * @return mixed boolean
  */
  function loadList() {
    unset($this->boxesList);
    unset($this->boxGroupLinks);
    $sql = "SELECT b.box_id, b.box_name, b.boxgroup_id,
                    b.box_created, b.box_modified,
                    bp.box_modified AS box_published
              FROM %s AS b
              LEFT OUTER JOIN %s AS bp ON bp.box_id = b.box_id
              ORDER BY b.box_name, b.box_id";
    $params = array($this->tableBox, $this->tableBoxPublic);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->boxesList[(int)$row['box_id']] = $row;
        $this->boxGroupLinks[$row['boxgroup_id']][] = (int)$row['box_id'];
      }
      return $this->loadGroupList();
    }
    return FALSE;
  }

  /**
  * Load group list
  *
  * @access public
  * @return boolean
  */
  function loadGroupList() {
    unset($this->boxGroupsList);
    $sql = "SELECT boxgroup_id, boxgroup_title, boxgroup_name
            FROM %s
            ORDER BY boxgroup_title ASC";
    $params = array($this->tableBoxgroup);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->boxGroupsList[$row['boxgroup_id']] = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load Box by id
  *
  * @param integer $boxId
  * @param integer $lngId
  * @access public
  * @return boolean
  */
  function load($boxId, $lngId = 0) {
    unset($this->box);
    if ($boxId > 0) {
      $sql = "SELECT b.box_id, b.box_name, b.boxgroup_id, b.box_created, b.box_modified,
                      bp.box_modified AS box_published
                FROM %s AS b
                LEFT OUTER JOIN %s AS bp ON bp.box_id = b.box_id
                WHERE b.box_id = '%d'";
      $params = array($this->tableBox,$this->tableBoxPublic,$boxId);
      if ($res = $this->databaseQueryFmt($sql,$params)) {
        if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->box = $row;
          if (isset($lngId) && $lngId > 0) {
            $this->loadTranslation($this->box['box_id'], $lngId);
          }
          return TRUE;
        }
      }
    }
    return NULL;
  }

  /**
  * Load translated data for a box
  *
  * @param $boxId
  * @param $lngId
  * @access public
  * @return boolean
  */
  function loadTranslation($boxId, $lngId) {
    unset($this->box['TRANSLATION']);
    $sql = "SELECT b.box_id, b.lng_id, b.view_id, b.box_title, b.box_data, b.box_trans_created, b.box_trans_modified,
                   m.module_guid, m.module_path, m.module_file, m.module_class
              FROM %s AS b
              LEFT OUTER JOIN %s AS v ON (v.view_id = b.view_id)
              LEFT OUTER JOIN %s AS m ON (m.module_guid = v.module_guid AND m.module_active = 1 AND m.module_type = 'box')
              WHERE b.box_id = '%d' AND lng_id = '%d'";
    $params = array($this->tableBoxTrans, $this->tableViews, $this->tableModules, $boxId, $lngId);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->box['TRANSLATION'] =$row;
        return TRUE;
      }
    }
    return FALSE;
  }

  function loadTranslationsInfo() {
    unset($this->box['TRANSLATIONINFOS']);
    $sql = "SELECT bt.box_id, bt.lng_id, bt.box_trans_modified,
                    bt.box_title, bpt.box_trans_modified as box_trans_published
              FROM %s AS bt
              LEFT OUTER JOIN %s AS bpt ON (bpt.box_id = bt.box_id AND bpt.lng_id = bt.lng_id)
              WHERE bt.box_id = %d";
    $params = array($this->tableBoxTrans, $this->tableBoxPublicTrans, (int)$this->box['box_id']);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->box['TRANSLATIONINFOS'][$row['lng_id']] = $row;
      }
    }
  }

  /**
  * Load group by id
  *
  * @param integer $id
  * @access public
  * @return mixed
  */
  function loadGroup($id) {
    unset($this->boxGroups);
    if (isset($this->boxGroupsList) && is_array($this->boxGroupsList) && isset($this->boxGroupsList[$id])) {
      $this->boxGroups = $this->boxGroupsList[$id];
    } elseif ($id > 0) {
      $sql = "SELECT boxgroup_id, boxgroup_title, boxgroup_name
                      FROM %s
                      WHERE boxgroup_id = '%s'";
      $params = array($this->tableBoxgroup,$id);
      if ($res = $this->databaseQueryFmt($sql,$params)) {
        if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->boxGroups = $row;
        }
      }
    } else {
      $this->boxGroups = NULL;
    }
    return $this->boxGroups;
  }

  /**
  * Load versions list
  *
  * @param integer $id
  * @access public
  */
  function loadVersionsList($id) {
    unset($this->versions);
    $sql = "SELECT b.version_id, b.version_time, b.version_author_id,
                b.box_name, u.user_id, u.givenname, u.surname
              FROM %s AS b
                LEFT OUTER JOIN %s AS u ON b.version_author_id = u.user_id
              WHERE b.box_id = '%s'
              ORDER BY b.version_time DESC, b.version_id DESC";
    $params = array($this->tableBoxVersions, $this->tableAuthUser, $id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $row['fullname'] = $row['givenname'].' '.$row['surname'];
        $this->versions[$row["version_id"]] = $row;
      }
    }
  }

  /**
  * Add a box
  *
  * @param integer $groupId
  * @param integer $lngId
  * @access public
  * @return mixed
  */
  function create($groupId, $lngId) {
    if ($this->loadGroup($groupId)) {
      $currentTime = time();
      $data = array(
        'box_name' => $this->_gt('New box'),
        'boxgroup_id' => $groupId,
        'box_modified' => $currentTime,
        'box_created' => $currentTime,
      );
      if ($newId = $this->databaseInsertRecord($this->tableBox, 'box_id', $data)) {
        $data = array(
          'box_id' => (int)$newId,
          'lng_id' => (int)$lngId,
          'box_trans_created' => time(),
          'box_trans_modified' => time()
        );
        $this->databaseInsertRecord($this->tableBoxTrans, NULL, $data);
        return $newId;
      }
    } else {
      $this->addMsg(MSG_WARNING,'Please select a group!');
    }
    return FALSE;
  }

  function createTranslation($lngId) {
    $currentTime = time();
    $data = array(
      'box_id' => (int)$this->box['box_id'],
      'lng_id' => (int)$lngId,
      'box_trans_created' => time(),
      'box_trans_modified' => time()
    );
    return (FALSE !== $this->databaseInsertRecord($this->tableBoxTrans, NULL, $data));
  }

  /**
  * Add a group
  *
  * @access public
  * @return mixed
  */
  function createGroup() {
    $data = array('boxgroup_title' => $this->_gt('New group'));
    return $this->databaseInsertRecord($this->tableBoxgroup, 'boxgroup_id', $data);
  }

  /**
  * Save changes of a box
  *
  * @access public
  * @return mixed
  */
  function save() {
    if (isset($this->box)) {
      $translationModified = FALSE;
      $result = TRUE;
      if (isset($this->box['TRANSLATION'])) {
        $dataTrans = array(
          'box_title' => $this->params['box_title']
        );
        if ($this->checkDataModified($dataTrans, $this->box['TRANSLATION'])) {
          $translationModified = TRUE;
          $dataTrans['box_trans_modified'] = time();
          $filter = array('box_id'=>(int)$this->box['box_id'], 'lng_id'=>$this->box['TRANSLATION']['lng_id']);
          $result = (FALSE !== $this->databaseUpdateRecord($this->tableBoxTrans, $dataTrans, $filter));
        }
      }

      if ($result) {
        $data = array(
          'box_name' => $this->params['box_name'],
          'boxgroup_id' => (int)$this->params['boxgroup_id'],
        );

        if ($translationModified || $this->checkDataModified($data, $this->box)) {
          $data['box_modified'] = time();
          $result = (FALSE !== $this->databaseUpdateRecord($this->tableBox, $data, 'box_id', (int)$this->box['box_id']));
          if ($result) {
            $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
              sprintf($this->_gt('Properties of the box "%s #%d" changed.'),
                papaya_strings::escapeHTMLChars($this->params['box_name']), (int)$this->box['box_id']));
          }
        }
        return $result;
      }
    }
    return FALSE;
  }

  /**
  * Box versionieren
  *
  */


  /**
  * Create verion of Box
  *
  * @access public
  * @return boolean
  */
  function createVersion() {
    $currentTime = time();
    $data = array(
      'version_time' => $currentTime,
      'version_author_id' => $this->authUser->userId,
      'version_message' => $this->params['commit_message'],
      'box_id' => $this->box['box_id'],
      'box_name' => $this->box['box_name'],
      'boxgroup_id' => $this->box['boxgroup_id'],
      'box_modified' => $this->box['box_modified'],
    );
    if ($newVersionId = $this->databaseInsertRecord($this->tableBoxVersions, 'version_id', $data)) {
      $sql = "INSERT INTO %s (version_id, lng_id, box_id, box_title, box_data, view_id)
              SELECT '%d', tt.lng_id, tt.box_id, tt.box_title, tt.box_data, tt.view_id
                FROM %s AS tt
                WHERE tt.box_id = %d";
      $params= array($this->tableBoxVersionsTrans, $newVersionId, $this->tableBoxTrans, $this->box['box_id']);
      if (FALSE !== $this->databaseQueryFmt($sql, $params)) {
        $this->removeOldVersions();
        return TRUE;
      }
    } else {
      return FALSE;
    }
  }

  /**
  * Load old version
  *
  * @access public
  * @return boolean
  */
  function loadOldVersion($lngId = NULL) {
    unset($this->oldVersion);
    $lngParams = ($lngId !== NULL) ? "AND t.lng_id = $lngId" : $lngParams = '';
    $sql = "SELECT v.version_id, v.version_time, v.version_message, v.version_author_id,
                v.box_id, v.box_name, v.boxgroup_id,
                u.user_id, u.givenname, u.surname,
                t.box_title, t.box_data, t.view_id
              FROM %s AS v
                LEFT OUTER JOIN %s AS t ON t.version_id = v.version_id
                LEFT OUTER JOIN %s AS u ON v.version_author_id = u.user_id
              WHERE v.version_id = '%d' $lngParams";
    $params = array($this->tableBoxVersions, $this->tableBoxVersionsTrans, $this->tableAuthUser,
      @(int)$this->params['version_id']);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $row['fullname'] = $row['givenname'].' '.$row['surname'];
        $this->oldVersion = $row;
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Save group
  *
  * @access public
  * @return mixed
  */
  function saveGroup() {
    if ($result = $this->checkGroup()) {
      $values = array(  'boxgroup_title' => $this->params['boxgroup_title'],
                        'boxgroup_name' => $this->params['boxgroup_name']
                );
      if (FALSE !== $this->databaseUpdateRecord($this->tableBoxgroup, $values, 'boxgroup_id', $this->boxGroups['boxgroup_id'])) {
        return TRUE;
      }  else {
        $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
      }
    } else {
      return $result;
    }
  }

  /**
  * Save data
  *
  * @access public
  * @return boolean
  */
  function saveData($dataString) {
    if (isset($this->box['TRANSLATION']) && is_array($this->box['TRANSLATION'])) {
      $dataTrans = array ('box_data' => $dataString);
      if ($this->checkDataModified($dataTrans, $this->topic['TRANSLATION'])) {
        $dataTrans['box_trans_modified'] = time();
        $filter = array('box_id'=>(int)$this->box['box_id'], 'lng_id'=>$this->box['TRANSLATION']['lng_id']);
        if (FALSE !== $this->databaseUpdateRecord($this->tableBoxTrans, $dataTrans, $filter)) {
          $data = array('box_modified' => $dataTrans['box_trans_modified']);
          if (FALSE !== $this->databaseUpdateRecord($this->tableBox, $data, 'box_id', (int)$this->box['box_id'])) {
            $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
              sprintf('Content of box "%s (%s)" changed.',
              papaya_strings::escapeHTMLChars($this->box['box_name']), $this->box['box_id']));
            return TRUE;
          }
        }
      }
      return FALSE;
    } else {
      return FALSE;
    }
  }

  /**
  * Boxes publish function
  *
  * @access public
  * @return boolean
  */
  function publishBox() {
    if (isset($this->box) && is_array($this->box)) {
      if ($this->createVersion()) {
        $data = array(
          'box_name' => $this->box['box_name'],
          'boxgroup_id' => $this->box['boxgroup_id'],
          'box_modified' => time()
        );
        $sql = "SELECT COUNT(*) FROM %s WHERE box_id = %d";
        if ($res = $this->databaseQueryFmt($sql, array($this->tableBoxPublic, $this->box['box_id']))) {
          $row = $res->fetchRow();
          if ($row[0] > 0) {
            $result = (FALSE !== $this->databaseUpdateRecord($this->tableBoxPublic, $data, 'box_id', $this->box['box_id']));
          } else {
            $data['box_id'] = $this->box['box_id'];
            $result = (FALSE !== $this->databaseInsertRecord($this->tableBoxPublic, NULL, $data));
          }
          if ($result) {
            $languages = array();
            if (isset($this->params['public_languages']) && is_array($this->params['public_languages']) &&
                count($this->params['public_languages']) > 0) {
              foreach($this->params['public_languages'] as $lng) {
                $languages[] = (int)$lng;
              }
              $params = array('box_id'=>$this->box['box_id'], 'lng_id'=>$languages);
              if (FALSE !== $this->databaseDeleteRecord($this->tableBoxPublicTrans, $params)) {
                if (count($languages) > 1) {
                  $filter = ' AND bt.lng_id IN ('.implode(',', $languages).')';
                } elseif (count($languages) == 1) {
                  $filter = ' AND bt.lng_id = '.$languages[0];
                } else {
                  $filter = '';
                }
                $sql = "INSERT INTO %s (box_id, lng_id, box_title, box_data, view_id,
                                box_trans_created, box_trans_modified)
                        SELECT bt.box_id, bt.lng_id, bt.box_title, bt.box_data, bt.view_id,
                                bt.box_trans_created, '%d'
                          FROM %s AS bt
                          WHERE bt.box_id = %d $filter";
                $params= array($this->tableBoxPublicTrans, time(), $this->tableBoxTrans, $this->box['box_id']);
                if (FALSE !== $this->databaseQueryFmt($sql, $params)) {
                  $this->deleteCache($this->box['box_id']);
                  return TRUE;
                }
              }
              return TRUE;
            }
          }
        }
      }
    }
    return FALSE;
  }

  /**
  * Delete box output cache
  *
  * @param $boxId
  * @access public
  */
  function deleteCache($boxId) {
    if ($dh = @opendir(PAPAYA_PATH_CACHE)) {
      while($file = readdir($dh)) {
        if (preg_match('#^\.box_'.(int)$boxId.'_\d+_[a-f\d]{32}(\.html)?$#', $file)) {
            unlink(PAPAYA_PATH_CACHE.$file);
        }
      }
      closedir($dh);
    }
  }

  /**
  * Get publish date
  *
  * @access public
  * @return mixed boolean od modified
  */
  function getPublicDate() {
    $sql = "SELECT box_modified FROM %s WHERE box_id = %d";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableBoxPublic, $this->box['box_id']))) {
      if ($row = $res->fetchRow()) {
        return $row[0];
      }
    }
    return FALSE;
  }

  /**
  * Delete public box
  *
  * @access public
  * @return boolean
  */
  function deletePublicBox() {
    return (FALSE !== $this->databaseDeleteRecord($this->tableBoxPublicTrans, 'box_id', $this->box['box_id']) &&
            FALSE !== $this->databaseDeleteRecord($this->tableBoxPublic, 'box_id', $this->box['box_id']));
  }

  /**
  * Delete public trans box
  *
  * @access public
  * @return boolean
  */
  function deletePublicBoxTrans() {
    $condition = array('box_id' => $this->box['box_id'], 'lng_id' => $this->params['lng_id']);
    return ($this->databaseDeleteRecord($this->tableBoxPublicTrans, $condition) !== FALSE);
  }

  /**
  * Group input check function
  *
  * @access public
  * @return mixed boolean or integer
  */
  function checkGroup() {
    $result = TRUE;
    $fieldCount = 0;
    if (!checkit::isNoHTML($this->params['boxgroup_title'], TRUE)) {
      $this->addMsg(MSG_ERROR,
        sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Title')));
      $result = FALSE;
    } else {
      if ($this->params['boxgroup_title'] != $this->boxGroups['boxgroup_title']) {
          $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Title')));
          ++$fieldCount;
      }
    }
    if (!checkit::isAlphaNum($this->params['boxgroup_name'], FALSE)) {
      $this->addMsg(MSG_ERROR,
        sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Name')));
      $result = FALSE;
    } else {
      if ($this->params['boxgroup_name'] != $this->boxGroups['boxgroup_name']) {
          $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Name')));
          ++$fieldCount;
      }
    }
    if ($result) {
      return $fieldCount;
    } else {
      return $result;
    }
  }

  /**
  * Delete box via given id
  *
  * @param integer $id
  * @access public
  * @return mixed formular or ''
  */
  function deleteBox($id) {
    $result = '';
    if ($id > 0) {
      if (isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
        $break = FALSE;
        if (FALSE !== $this->databaseDeleteRecord($this->tableBoxVersionsTrans, 'box_id', $id) &&
            FALSE !== $this->databaseDeleteRecord($this->tableBoxVersions, 'box_id', $id)) {
          $this->addMsg(MSG_INFO,$this->_gt('Versions deleted.'));
        } else {
          $break = TRUE;
        }
        if (!$break) {
          if (FALSE !== $this->databaseDeleteRecord($this->tableBoxPublicTrans, 'box_id', $id) &&
              FALSE !== $this->databaseDeleteRecord($this->tableBoxPublic, 'box_id', $id)) {
            $this->addMsg(MSG_INFO,$this->_gt('Published box deleted.'));
          } else {
            $break = TRUE;
          }
        }
        if (!$break) {
          if (FALSE !== $this->databaseDeleteRecord($this->tableLink, 'box_id', $id)) {
            if (FALSE !== $this->databaseDeleteRecord($this->tableBoxTrans, 'box_id', $id) &&
                FALSE !== $this->databaseDeleteRecord($this->tableBox, 'box_id', $id)) {
              $this->addMsg(MSG_INFO, $this->_gt('Box and links deleted.'));
            } else {
              $this->addMsg(MSG_WARNING, $this->_gt('Database error!').' '.
              $this->_gt('Couldnt delete box. - Links deleted only.'));
            }
          } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error!'));
          }
        }
      } else {
        if ($this->load($id, $this->lngSelect->currentLanguageId)) {
          $result = $this->deleteBoxForm();
        }
      }
      $this->loadList('');
    }
    return $result;
  }

  /**
  * Delete group via given id
  *
  * @param integer $id
  * @param string $cmd optional, default value 'group_delete'
  * @access public
  * @return string formular or ''
  */
  function deleteGroup($id) {
    if ($id > 0) {
      $sql = "SELECT COUNT(box_id) FROM %s WHERE boxgroup_id = '%d'";
      $params = array( $this->tableBox, $id);
      if ($res = $this->databaseQueryFmt($sql,$params)) {
        if ($row = $res->fetchRow()) {
          if ($row[0] == 0) {
            if (isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
              if (FALSE !== $this->databaseDeleteRecord($this->tableBoxgroup, 'boxgroup_id', $id)) {
                $this->addMsg(MSG_INFO, $this->_gt('Group deleted.'));
                unset($this->boxGroups);
                $this->params['gid'] = 0;
                $this->initializeSessionParam('gid', array('bid'));
                $this->loadGroupList('');
              }
            } else {
              $this->loadGroup($this->params['gid']);
              return $this->deleteGroupForm();
            }
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Group is not empty. Please delete all boxes in this group first.'));
          }
        }
      }
    }
    return '';
  }

  function initializePropertiesDialog() {
    if (!(isset($this->dialogProperties) && is_object($this->dialogProperties))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');

      if (isset($this->box) && is_array($this->box)) {
        $data = array(
          'box_name' => $this->box['box_name'],
          'boxgroup_id' => $this->box['boxgroup_id']
        );
        if (isset($this->box['TRANSLATION']) && is_array($this->box['TRANSLATION'])) {
          $data['box_title'] = $this->box['TRANSLATION']['box_title'];
          $data['view_id'] = $this->box['TRANSLATION']['view_id'];
        }
      } else {
        $data = array();
      }
      $hidden = array(
        'cmd' => 'box_edit',
        'bid' => (int)$this->box['box_id']
      );

      $boxGroups = array();
      foreach(@$this->boxGroupsList as $key=>$val) {
        $boxGroups[$key] = $val['boxgroup_title'];
      }

      $fields = array();
      if (isset($this->box['TRANSLATION']) && is_array($this->box['TRANSLATION'])) {
        $fields['box_title'] = array('Title', 'isNoHTML', FALSE, 'input', 100);
      }
      $fields[] = 'Language independent';
      $fields['box_name'] = array('Name', 'isNoHTML', TRUE, 'input', 50);
      $fields['boxgroup_id'] = array('Group', 'isNum', TRUE, 'combo', $boxGroups);

      $this->dialogProperties = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->dialogProperties->msgs = &$this->msgs;
      $this->dialogProperties->loadParams();
      $this->dialogProperties->dialogTitle = $this->_gt('Properties');
      $this->dialogProperties->dialogDoubleButtons = FALSE;
      $this->dialogProperties->textYes = 'Yes';
      $this->dialogProperties->textNo = 'No';
    }
  }

  /**
  * Edit - copy box parameters in $this->params
  *
  * @access public
  * @return string edit formular
  */
  function getPropertiesDialog() {
    $this->initializePropertiesDialog();
    return $this->dialogProperties->getDialogXML();
  }

  function addTranslationDialog() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'add_translation',
      'lng_id' => $this->lngSelect->currentLanguageId,
      'bid' => $this->box['box_id']
    );
    $msg = sprintf($this->_gt('Add content for language "%s" (%s) to box "%s"?'),
        papaya_strings::escapeHTMLChars($this->lngSelect->currentLanguage['lng_title']),
        papaya_strings::escapeHTMLChars($this->lngSelect->currentLanguage['lng_short']),
        (int)$this->box['box_id']);
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Add';
    return $dialog->getMsgDialog();
  }

  /**
  * Edit Group
  *
  * @access public
  * @return string edit formular
  */
  function editGroup() {
    if (isset($this->boxGroups) && is_array($this->boxGroups)) {
      foreach($this->boxGroups as $key=>$val) {
        if (!isset($this->params[$key])) {
          $this->params[$key] = $val;
        }
      }
    }
    $result = sprintf('<dialog action="%s" title="%s" width="100%%">', $this->baseLink, $this->_gt('Properties'));
    $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />',
      $this->paramName, 'cmd', 'group_edit');
    $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />',
      $this->paramName, 'gid', $this->boxGroups['boxgroup_id']);
    $result .= '<lines class="dialogXLarge">';
    $result .= sprintf('<line caption="%s"><input type="text" class="dialogInput dialogScale" size="40" name="%s" value="%s" /></line>',
      $this->_gt('Title'), $this->paramName.'[boxgroup_title]', papaya_strings::escapeHTMLChars($this->params['boxgroup_title']));
    $result .= sprintf('<line caption="%s"><input type="text" class="dialogInput dialogScale" size="40" name="%s" value="%s" /></line>',
      $this->_gt('Name'), $this->paramName.'[boxgroup_name]', papaya_strings::escapeHTMLChars($this->params['boxgroup_name']));
    $result .= '</lines>';
    $result .= sprintf('<dlgbutton class="dialogButton" value="%s" />', $this->_gt('Save'));
    $result .= '</dialog>';
    return $result;
  }

  /**
  * box formular  for delete confirmation
  *
  * @access public
  * @return mixed dialog
  */
  function deleteBoxForm() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'box_delete',
      'bid' => $this->box['box_id'],
      'confirm_delete'=>1,
    );
    $msg = sprintf($this->_gt('Delete box "%s" (%s)?'), $this->box['box_name'], $this->box['box_id']);
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Delete';
    return $dialog->getMsgDialog();
  }

  /**
  * group formular for delete confirmation
  *
  * @access public
  * @return mixed dialog
  */
  function deleteGroupForm() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'group_delete',
      'gid' => $this->boxGroups['boxgroup_id'],
      'confirm_delete'=>1,
    );
    $msg = sprintf($this->_gt('Delete group "%s" (%s)?'),
      $this->boxGroups['boxgroup_title'], $this->boxGroups['boxgroup_id']);
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Delete';
    return $dialog->getMsgDialog();
  }

  /**
  * Edit Content
  *
  * @param integer $userId
  * @access public
  * @return mixed formular
  */
  function editContent() {
    $result = '';
    if (isset($this->box['TRANSLATION']) && $this->box['TRANSLATION']['module_class'] != '') {
      $parent = NULL;
      $obj = &$this->createPluginObject($this->box['TRANSLATION']['module_path'].$this->box['TRANSLATION']['module_file'],
          $this->box['TRANSLATION']['module_class'], $this->box['TRANSLATION']['module_guid'], $parent);
      if (isset($obj) && is_object($obj)) {
        $obj = &new $this->box['TRANSLATION']['module_class']($parent,'box');
        $obj->msgs = &$this->msgs;
        $obj->images = &$this->images;
        $obj->setData($this->box['TRANSLATION']['box_data']);
        $obj->initializeDialog();
        if ($obj->modified()) {
          if ($obj->checkData()) {
            if ($this->saveData($obj->getData())) {
              $this->addMsg(MSG_INFO,$this->_gt('Changes saved.'));
              $this->loadList();
            }
          }
        }
        $result = $obj->getForm();
      }
    }
    return $result;
  }

  /**
  * Get list XML-source
  *
  * @param $aBoxId
  * @param $aGroupId
  * @access public
  * @return string XML-source
  */
  function getXMLList($aBoxId, $aGroupId) {
    $result = '';
    if (isset($this->boxGroupsList) && is_array($this->boxGroupsList)) {
      $result = sprintf('<listview title="%s" width="250"><items>'.LF, $this->_gt('Boxes'));
      foreach($this->boxGroupsList as $groupId=>$group) {
        if (isset($this->boxGroupLinks[$groupId]) && is_array($this->boxGroupLinks[$groupId])) {
          $boxIds = $this->boxGroupLinks[$groupId];
          $indent = 0;
          if (isset($this->opened[$groupId]) && $this->opened[$groupId]) {
            $nodeHref = sprintf('%s?%s[cmd]=%s&amp;%s[gid]=%s',
              $this->baseLink, $this->paramName, 'close', $this->paramName, $groupId);
            $node = sprintf(' node="open" nhref="%s"', $nodeHref);
          } else {
            $nodeHref = sprintf('%s?%s[cmd]=%s&amp;%s[gid]=%s',
              $this->baseLink, $this->paramName, 'open', $this->paramName, $groupId);
            $node = sprintf(' node="close" nhref="%s"', $nodeHref);
          }
        } else {
          $boxIds = NULL;
          $indent = 1;
          $node = '';
        }
        if ($groupId == $aGroupId) {
          $selected = ' selected="selected"';
        } else {
          $selected = '';
        }
        $result .= sprintf('<listitem href="%s?%s[gid]=%s&%s[cmd]=chg_show&%s[bid]=0&p_mode=0" title="%s" image="%s" indent="%s"  %s></listitem>',
          $this->baseLink, $this->paramName, $groupId, $this->paramName, $this->paramName,
          $group["boxgroup_title"], $this->images[51], $indent, $selected.$node);
        if (isset($boxIds) && is_array($boxIds) &&
            isset($this->opened[$groupId]) && ($this->opened[$groupId])) {
          foreach($boxIds as $boxId) {
            $box = $this->boxesList[$boxId];
            if (isset($box) && is_array($box)) {
              if ($boxId == $aBoxId) {
                $selected = ' selected="selected"';
              } else {
                $selected = '';
              }
              if ($pubDate = $box['box_published']) {
                $pubDateStr = date('Y-m-d H:i:s', $pubDate);
                if ($pubDate >= $box['box_modified']) {
                  $imageIndex = 5;
                } else {
                  $imageIndex = 53;
                }
              } else {
                $pubDateStr = '';
                $imageIndex = 52;
              }
              $result .= sprintf('<listitem href="%s?%s[bid]=%s&%s[cmd]=chg_show&p_mode=1" title="%s" image="%s" indent="2" %s></listitem>',
                  $this->baseLink, $this->paramName,$boxId, $this->paramName,
                  $box["box_name"], $this->images[$imageIndex], $selected);
            }
          }
        }
      }
      $result .= "</items></listview>\n";
    }
    return $result;
  }

  /**
  * Get versions list XML-source
  *
  * @access public
  * @return string XML-source
  */
  function getVersionsList() {
    $result = '';
    if (isset($this->versions) && is_array($this->versions)) {
      $result .= sprintf('<listview title="%s" width="100%%">', $this->_gt('Versions'));
      $result .= '<cols>';
      $result .= sprintf('<col>%s</col>', $this->_gt('Version time'));
      $result .= sprintf('<col>%s</col>', $this->_gt('User'));
      /* $result .= '<col/>'; */
      $result .= '<col/>';
      $result .= '</cols>';
      $result .= '<items>';
      foreach($this->versions as $id=>$version) {
        if (isset($this->params['version_id']) && $id == $this->params['version_id']) {
          $selected = ' selected="selected"';
        } else {
          $selected = '';
        }
        $result .= sprintf('<listitem title="%s" href="%s?%s[version_id]=%s"%s>',
          date('Y-m-d H:i:s' ,$version['version_time']),
          $this->baseLink, $this->paramName, $id, $selected);
        $result .= '<subitem>'.$version['fullname'].'</subitem>';
        $href = sprintf('%s?%s[version_id]=%s&amp;%s[cmd]=',
          $this->baseLink, $this->paramName, $id, $this->paramName);
        $result .= sprintf('<subitem align="center"><a href="%s%s"><glyph src="%s" hint="%s" /></a></subitem>',
          $href, 'restore_version', $this->images[46], $this->_gt('Restore'));
        $result .= '</listitem>';
      }
      $result .= '</items>';
      $result .= '</listview>';
    }
    return $result;
  }

  /**
  * Get version Information XML-source
  *
  * @access public
  * @return string XML-source
  */
  function getVersionInfos() {
    if (isset($this->oldVersion) && is_array($this->oldVersion)) {
      $result = sprintf('<listview title="Infos [%s]" width="300">',
        date('Y-m-d H:i:s' ,$this->oldVersion['version_time']));
      $result .= '<items>';
      $result .= sprintf('<listitem title="%s" indent="%s"><subitem>%s</subitem></listitem>',
        $this->_gt('Name'), 0, $this->oldVersion['box_name']);
      $result .= sprintf('<listitem title="%s" indent="%s"><subitem>%s</subitem></listitem>',
        $this->_gt('Group'), 0, $this->boxGroupsList[$this->oldVersion['boxgroup_id']]['boxgroup_title']);
      $result .= sprintf('<listitem title="%s" indent="%s"><subitem>%s</subitem></listitem>',
        $this->_gt('User'), 0, $this->oldVersion['fullname']);
      $result .= '</items>';
      $result .= '</listview>';
      $this->layout->addRight($result);
      $result = '<panel title="'.papaya_strings::escapeHTMLChars($this->_gt('Message')).'"><sheet width="100%" align="center"><text><div style="padding: 0px 5px 0px 5px; ">'.papaya_strings::escapeHTMLChars($this->oldVersion['version_message']).'</div></text></sheet></panel>';
      $this->layout->add($result);
    }
  }

  /**
  * Execute version
  *
  * @access public
  * @return mixed
  */
  function versionExecute() {
    $result = '';
    if (isset($this->oldVersion) && is_array($this->oldVersion)) {
      switch (@$this->params['cmd']) {
        case 'version_delete' :
          //Version lschen
          if (isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
            if ($this->deleteVersion()) {
              $this->addMsg(MSG_INFO, $this->_gt('Version deleted.'));
              unset($this->oldVersion);
            }
          } else {
            $result = $this->getVersionDeleteForm();
          }
          break;
        case 'restore_version' :
          //Version wiederherstellen
          if ($this->loadOldVersion()) {
            if ($this->params['confirm_restore']) {
              if (is_array($this->params['restore_languages']) && count($this->params['restore_languages'] > 0)) {
                foreach (@$this->params['restore_languages'] as $lngId) {
                  $this->restoreVersionTrans($lngId);
                }
              }
              if (isset($this->params['meta_info']) && $this->params['meta_info'] == 'Yes') {
                $this->restoreVersion();
              }
              $this->addMsg(MSG_INFO, $this->_gt('Version restored'));
            } else {
              $result = $this->getVersionRestoreForm();
            }
          }
        break;
      }
    }
    return $result;
  }

  /**
  * Get version delete fromular
  *
  * @access public
  * @return string fromular
  */
  function getVersionDeleteForm() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'version_delete',
      'bid' => $this->box['box_id'],
      'version_id' => (int)$this->params['version_id'],
      'confirm_delete'=>1,
    );
    $msg = sprintf($this->_gt('Delete version from "%s"?'), date('Y-m-d H:i:s' ,$this->oldVersion['version_time']));
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Delete';
    return $dialog->getMsgDialog();
  }

  /**
  * Get version restore formular
  *
  * @access public
  * @return mixed dialog
  */
  function getVersionRestoreForm() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
    $hidden = array(
      'cmd' => 'restore_version',
      'bid' => $this->box['box_id'],
      'version_id' => (int)$this->params['version_id'],
      'confirm_restore'=>1,
    );

    $data = array();
    $msg = sprintf($this->_gt('Restore version for "%s" from %s?'),
      papaya_strings::escapeHTMLChars($this->box['TRANSLATION']['box_title']), date('Y-m-d H:i:s' ,$this->oldVersion['version_time']));
    $fields = array(
      $msg,
      'meta_info' => array('Language independent information', 'isSomeText', FALSE, 'checkbox'),
      'restore_languages' => array('Languages', 'isSomeText', FALSE, 'function', 'callbackRestoreLanguages', '', '', 'left')
    );
    $dialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
    $dialog->dialogTitle = $this->_gt('Restore Version');
    $dialog->buttonTitle = $this->_gt('Restore');
    $dialog->inputFieldSize = 'x-large';
    return $dialog->getDialogXML();
  }

  function callbackRestoreLanguages($name, $element, $data) {
    $result = '';
    if (isset($this->lngSelect->languages) && is_array($this->lngSelect->languages)) {
      $sql = 'SELECT lng_id FROM %s WHERE box_id=%d AND version_id=%d';
      $params = array($this->tableBoxVersionsTrans, (int)$this->box['box_id'], $this->params['version_id']);
      if ($res = $this->databaseQueryFmt($sql, $params)) {
        while($row = $res->fetchRow()) {
          $lngId = $row[0];
          $languages[$lngId] = $this->lngSelect->languages[$lngId];
        }
      }
      foreach($languages as $lngId=>$lng) {
        $result .= sprintf('<input type="checkbox" name="%s[%s][]" value="%d" %s%s/>'.LF,
          $this->paramName, $name, $lngId, @$selected, @$disabled);
        $result .= papaya_strings::escapeHTMLChars($languages[$lngId]['lng_title']).' ('.
          papaya_strings::escapeHTMLChars($this->lngSelect->languages[$lngId]['lng_short']).')';
      }
    }
    return $result;
  }

  /**
  * Delete version
  *
  * @access public
  * @return boolean
  */
  function deleteVersion() {
    return ($this->databaseDeleteRecord($this->tableBoxVersions, 'version_id', $this->params['version_id']) !== FALSE);
  }

  function restoreVersion() {
    $data = array(
      'box_name' => $this->oldVersion['box_name'],
      'boxgroup_id' => $this->oldVersion['boxgroup_id'],
      'box_modified' => time()
    );
    if ($this->databaseUpdateRecord($this->tableBox, $data, 'box_id', $this->box['box_id']) !== FALSE) {
      return TRUE;
    }  else {
      $this->addMsg(MSG_ERROR,$this->_gt('Database error! Changes not saved.'));
    }
    return FALSE;
  }

  function restoreVersionTrans($lngId) {
    $this->loadOldVersion($lngId);
    $data = array(
      'view_id' => $this->oldVersion['view_id'],
      'box_title' => $this->oldVersion['box_title'],
      'box_data' => $this->oldVersion['box_data'],
      'box_trans_modified' => time()
    );
    if ($this->databaseUpdateRecord($this->tableBoxTrans, $data, 'box_id', $this->box['box_id']) !== FALSE) {
      return TRUE;
    }  else {
      $this->addMsg(MSG_ERROR,$this->_gt('Database error! Changes not saved.'));
    }
    return FALSE;
  }

  /**
  * Remove old versions
  *
  * @access public
  * @return mixed
  */
  function removeOldVersions() {
    $sql = "SELECT version_time
            FROM %s
            WHERE box_id = %d
            ORDER BY version_time DESC";
    $counter = 0;
    if ($res = $this->databaseQueryFmt($sql, array($this->tableBoxVersions, $this->box['box_id']))) {
      while (($row = $res->fetchRow()) && ($counter < $this->maxVersions)) {
        $border = $row[0];
        ++$counter;
      }
      if ($counter >= $this->maxVersions) {
        $sql = "DELETE
                  FROM %s
                  WHERE box_id = %d
                    AND version_time < %d";
        return $this->databaseQueryFmt($sql, array($this->tableBoxVersion, $this->box['box_id'], $border));
      }
    }
    return FALSE;
  }

  /**
  * publication execute
  *
  * @access public
  * @return string formular or ''
  */
  function publishExecute() {
    $result = '';
    switch (@$this->params['cmd']) {
      case 'publish' :
        if (isset($this->params['confirm_publish']) && $this->params['confirm_publish'] &&
            isset($this->params['commit_message']) && checkit::isSomeText($this->params['commit_message'], TRUE)) {
          if ($this->publishBox()) {
            $this->sessionParams['last_publish_message'] = @$this->params['commit_message'];
            $this->addMsg(MSG_INFO, $this->_gt('Box published.'));
            $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
              sprintf($this->_gt('Box "%s (%s)" published.'),
                papaya_strings::escapeHTMLChars($this->box['box_name']), $this->box['box_id']));
          }
        } else {
          $result = $this->getPublishForm();
        }
        break;
      case 'deletepublic' :
        if (isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
          if ($this->deletePublicBox()) {
            $this->addMsg(MSG_INFO, $this->_gt('Published box deleted.'));
          }
        } else {
          $result = $this->getDeletePublicForm();
        }
        break;
      case 'deletepublic_trans' :
        if (isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
          if ($this->deletePublicBoxTrans()) {
            $this->addMsg(MSG_INFO, $this->_gt('Published box translation deleted.'));
          }
        } else {
          $result = $this->getDeletePublicTransForm();
        }
        break;
    }
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
    return $result;
  }

  /**
  * Get public data
  *
  * @param boolean $sel
  * @access public
  * @return string XML
  */
  function getPublicData() {
    $result = sprintf('<listview title="%s" width="100%%">', $this->_gt('Published'));
    $result .= '<cols>';
    $result .= '<col/>';
    $result .= sprintf('<col>%s</col>', $this->_gt('Modified'));
    $result .= '<col/>';
    $result .= sprintf('<col>%s</col>', $this->_gt('Published'));
    $result .= '<col/>';
    $result .= '</cols>';
    $result .= '<items>';
    if ($pubDate = $this->getPublicDate()) {
      $pubDateStr = date('Y-m-d H:i:s', $pubDate);
    } else {
      $pubDateStr = '';
    }
    if (isset($this->box['box_published']) && $this->box['box_published'] < $this->box['box_modified']) {
      $imageIndex = 48; //published and modified
      $imageHint = 'Modified';
    } elseif (isset($this->box['box_published']) && $this->box['box_published'] >= $this->box['box_modified']) {
      $imageIndex = 16; //published and up to date
      $imageHint = 'Published';
    } else {
      $imageIndex = 15; //created
      $imageHint = 'Created';
    }
    $result .= sprintf('<listitem title="%s">', $this->_gt('General'));
    $result .= sprintf('<subitem align="center">%s</subitem>', date('Y-m-d H:i:s' ,$this->box['box_modified']));
    $result .= sprintf('<subitem align="center"><glyph src="%s" hint="%s"/></subitem>', $this->images[$imageIndex],
      papaya_strings::escapeHTMLChars($this->_gt($imageHint)));
    $result .= sprintf('<subitem align="center">%s</subitem>', $pubDateStr);
    if ($pubDate) {
      $result .= sprintf('<subitem align="center"><a href="%s"><glyph src="%s"/></a></subitem>',
        $this->getLink(array('cmd' => 'deletepublic')), $this->images[10]);
    } else {
      $result .= '<subitem/>';
    }
    $result .= '</listitem>';
    if (isset($this->lngSelect->languages) && is_array($this->lngSelect->languages)) {
      foreach($this->lngSelect->languages as $lngId=>$lng) {
        if ($lng['lng_glyph'] != '' && file_exists($this->getBasePath(TRUE).'pics/language/'.$lng['lng_glyph'])) {
          $image = ' image="./pics/language/'.papaya_strings::escapeHTMLChars($lng['lng_glyph']).'"';
        } else{
          $image = '';
        }
        $result .= sprintf('<listitem title="%s" indent="1"%s>'.LF,  papaya_strings::escapeHTMLChars($lng['lng_title']), $image);
        if (isset($this->box['TRANSLATIONINFOS']) && isset($this->box['TRANSLATIONINFOS'][$lngId])) {
          $result .= sprintf('<subitem align="center">%s</subitem>'.LF,
            date('Y-m-d H:i:s', $this->box['TRANSLATIONINFOS'][$lngId]['box_trans_modified']));
          if (isset($this->box['TRANSLATIONINFOS'][$lngId]['box_trans_published'])) {
            if ($this->box['TRANSLATIONINFOS'][$lngId]['box_trans_published'] >=
                $this->box['TRANSLATIONINFOS'][$lngId]['box_trans_modified']) {
              $result .= sprintf('<subitem align="center"><glyph src="%s" hint="%s"/></subitem>', $this->images[16],
                  papaya_strings::escapeHTMLChars($this->_gt('Published')));
            } else {
              $result .= sprintf('<subitem align="center"><glyph src="%s" hint="%s"/></subitem>', $this->images[48],
                  papaya_strings::escapeHTMLChars($this->_gt('Modified')));
            }
            $result .= sprintf('<subitem align="center">%s</subitem>'.LF,
              date('Y-m-d H:i:s', $this->box['TRANSLATIONINFOS'][$lngId]['box_trans_published']));
            $result .= sprintf('<subitem align="center"><a href="%s"><glyph src="%s"/></a></subitem>',
              $this->getLink(array('cmd' => 'deletepublic_trans', 'lng_id' => $lngId)), $this->images[10]);
          } else {
            $result .= sprintf('<subitem align="center"><glyph src="%s" hint="%s"/></subitem>', $this->images[15],
                papaya_strings::escapeHTMLChars($this->_gt('Created')));
            $result .= '<subitem/>';
            $result .= '<subitem/>';
          }
        } else {
          $result .= '<subitem/><subitem/><subitem/><subitem/>'.LF;
        }
        $result .= '</listitem>'.LF;
      }
    }
    $result .= '</items>';
    $result .= '</listview>';
    return $result;
  }

  /**
  * Get information about box
  *
  * @access public
  * @return string XML
  */
  function getBoxInfos() {
    $result = '';
    if (isset($this->box) && is_array($this->box)) {
      $this->loadTranslationsInfo();
      $result .= '<listview title="Infos" width="300">';
      $result .= '<items>';
      $result .= sprintf('<listitem title="%s" indent="%s"><subitem>%s</subitem></listitem>',
        $this->_gt('Name'), 0, papaya_strings::escapeHTMLChars($this->box['box_name']));
      $result .= sprintf('<listitem title="%s" indent="%s"><subitem>%s</subitem></listitem>',
        $this->_gt('Group'), 0, papaya_strings::escapeHTMLChars($this->boxGroupsList[$this->box['boxgroup_id']]['boxgroup_title']));
      $result .= sprintf('<listitem title="%s" indent="1"><subitem>%s</subitem></listitem>',
        $this->_gt('Created'),  date('Y-m-d H:i:s', $this->box['box_created']));
      $result .= sprintf('<listitem title="%s" indent="1"><subitem>%s</subitem></listitem>'.LF,
        $this->_gt('Modified'), date('Y-m-d H:i:s', $this->box['box_modified']));
      if (isset($this->lngSelect->languages) && is_array($this->lngSelect->languages)) {
        foreach($this->lngSelect->languages as $lngId=>$lng) {
          if ($lng['lng_glyph'] != '' && file_exists($this->getBasePath(TRUE).'pics/language/'.$lng['lng_glyph'])) {
            $image = ' image="./pics/language/'.papaya_strings::escapeHTMLChars($lng['lng_glyph']).'"';
          } else{
            $image = '';
          }
          $result .= sprintf('<listitem title="%s"%s><subitem>%s</subitem></listitem>'.LF,
            papaya_strings::escapeHTMLChars($lng['lng_title']), $image, papaya_strings::escapeHTMLChars($lng['lng_short']));
          if (isset($this->box['TRANSLATIONINFOS']) && isset($this->box['TRANSLATIONINFOS'][$lngId])) {
            $result .= sprintf('<listitem title="%s" indent="2"><subitem>%s</subitem></listitem>'.LF,
              papaya_strings::escapeHTMLChars($this->_gt('Title')), papaya_strings::escapeHTMLChars($this->box['TRANSLATIONINFOS'][$lngId]['box_title']));
            if (isset($this->box['TRANSLATIONINFOS'][$lngId]['box_trans_published'])) {
              if ($this->box['TRANSLATIONINFOS'][$lngId]['box_trans_published'] <
                  $this->box['TRANSLATIONINFOS'][$lngId]['box_trans_modified']) {
                $result .= sprintf('<listitem title="%s" indent="2"><subitem>%s</subitem></listitem>'.LF,
                  papaya_strings::escapeHTMLChars($this->_gt('Status')), papaya_strings::escapeHTMLChars($this->_gt('Modified')));
              } else {
                $result .= sprintf('<listitem title="%s" indent="2"><subitem>%s</subitem></listitem>'.LF,
                  papaya_strings::escapeHTMLChars($this->_gt('Status')), papaya_strings::escapeHTMLChars($this->_gt('Public')));
              }
              $result .= sprintf('<listitem title="%s" indent="2"><subitem>%s</subitem></listitem>'.LF,
                papaya_strings::escapeHTMLChars($this->_gt('Published')),
                date('Y-m-d H:i:s', $this->box['TRANSLATIONINFOS'][$lngId]['box_trans_published']));
            } else {
              $result .= sprintf('<listitem title="%s" indent="2"><subitem>%s</subitem></listitem>'.LF,
                papaya_strings::escapeHTMLChars($this->_gt('Status')), papaya_strings::escapeHTMLChars($this->_gt('Created')));
            }
            $result .= sprintf('<listitem title="%s" indent="2"><subitem>%s</subitem></listitem>'.LF,
              papaya_strings::escapeHTMLChars($this->_gt('Modified')),
              date('Y-m-d H:i:s', $this->box['TRANSLATIONINFOS'][$lngId]['box_trans_modified']));
          } else {
            $result .= sprintf('<listitem title="%s" indent="2"><subitem>%s</subitem></listitem>'.LF,
              papaya_strings::escapeHTMLChars($this->_gt('Status')), papaya_strings::escapeHTMLChars($this->_gt('No Content')));
          }
        }
      }
      $result .= '</items>';
      $result .= '</listview>';
    }
    return $result;
  }


  /**
  * Get publish formular
  *
  * @access public
  * @return mixed dialog
  */
  function getPublishForm() {
    if (isset($this->box['TRANSLATION'])) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array(
        'cmd' => 'publish',
        'bid' => $this->box['box_id'],
        'confirm_publish'=>1,
      );
      $data = array();
      if (isset($this->sessionParams['last_publish_message']) && trim($this->sessionParams['last_publish_message']) != '') {
        $publishVersionMessage = $this->sessionParams['last_publish_message'];
      } else {
        $publishVersionMessage = '';
      }
      $fields = array(
        'commit_message' => array('Message', 'isSomeText', TRUE, 'input', 200, '', $publishVersionMessage),
        'public_languages' => array('Languages', 'isSomeText', TRUE, 'function', 'callbackPublishLanguages', '', '', 'left')
      );
      $msg = sprintf($this->_gt('Publish box "%s" (%s)?'), @papaya_strings::escapeHTMLChars($this->box['box_name']), (int)$this->box['box_id']);
      $dialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $dialog->msgs = &$this->msgs;
      $dialog->dialogTitle = $this->_gt('Publish');
      $dialog->buttonTitle = 'Publish';
      $dialog->inputFieldSize = 'x-large';

      return $dialog->getDialogXML();
    }
    return '';
  }

  function callbackPublishLanguages($name, $element, $data) {
    $result = '';
    if (isset($this->lngSelect->languages) && is_array($this->lngSelect->languages)) {
      foreach($this->lngSelect->languages as $lngId=>$lng) {
        if (isset($this->box['TRANSLATIONINFOS']) && isset($this->box['TRANSLATIONINFOS'][$lngId])) {
          $selected = @in_array($lngId, $data) ? ' checked="checked"' : '';
          if ((!is_array($data)) && $this->lngSelect->currentLanguageId == $lngId) {
            $selected = ' checked="checked"';
          }
          if (isset($this->box['TRANSLATIONINFOS'][$lngId]['box_trans_published']) &&
                $this->box['TRANSLATIONINFOS'][$lngId]['box_trans_published'] >=
                $this->box['TRANSLATIONINFOS'][$lngId]['box_trans_modified']) {
            $disabled = ' disabled="disabled"';
          } else {
            $disabled = '';
          }
          $result .= sprintf('<input type="checkbox" name="%s[%s][]" value="%d" %s%s/>'.LF,
            $this->paramName, $name, $lngId, $selected, $disabled);
          $result .= papaya_strings::escapeHTMLChars($this->lngSelect->languages[$lngId]['lng_title']).' ('.
            papaya_strings::escapeHTMLChars($this->lngSelect->languages[$lngId]['lng_short']).')';
        }
      }
    }
    return $result;
  }

  /**
  * Get public delete formular
  *
  * @access public
  * @return mixed dialog
  */
  function getDeletePublicForm() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'deletepublic',
      'bid' => $this->box['box_id'],
      'confirm_delete'=>1,
    );
    $msg = $this->_gt('Delete published box?');
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Delete';
    return $dialog->getMsgDialog();
  }

  /**
  * Get public delete translation formular
  *
  * @access public
  * @return mixed dialog
  */
  function getDeletePublicTransForm() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'deletepublic_trans',
      'bid' => $this->box['box_id'],
      'lng_id' => $this->params['lng_id'],
      'confirm_delete'=>1,
    );
    $msg = sprintf($this->_gt('Delete published box for language %s (%s)?'),
      $this->lngSelect->languages[$this->params['lng_id']]['lng_title'], $this->lngSelect->languages[$this->params['lng_id']]['lng_short']);
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Delete';
    return $dialog->getMsgDialog();
  }

  /**
  * Get preview frame
  *
  * @param string $caption
  * @param integer $sid
  * @access public
  * @return string
  */
  function getPreviewFrame($caption, $sid) {
    $result = sprintf('<panel width="100%%" title="%s [%s]"><iframe width="100%%" noresize="noresize" hspace="0" vspace="0" align="center" scrolling="auto" height="550" src="../%s" class="inset" id="preview" /></panel>',
      $this->_gt($caption), papaya_strings::escapeHTMLChars($this->box['box_name']),
      $this->getWebLink('', '', 'preview', array('box' => $this->box['box_id'], 'lng_id' => @(int)$this->box['TRANSLATION']['lng_id']), NULL, '')
    );
    return $result;
  }

  /**
  * Parsed Box
  *
  * @access public
  * @return mixed preview
  */
  function parsedBox() {
    $outputString = '';
    if (isset($this->box) && is_array($this->box) && isset($this->box['TRANSLATION'])) {
      $parent = NULL;
      $obj = &$this->createPluginObject($this->box['TRANSLATION']['module_path'].$this->box['TRANSLATION']['module_file'],
        $this->box['TRANSLATION']['module_class'], $this->box['TRANSLATION']['module_guid'], $parent);
      if (isset($obj) && is_object($obj)) {
        if ($obj->preview) {
          $obj->setData($this->box['TRANSLATION']['box_data']);
          $str = @$obj->getParsedData();
          header('Content-type: text/xml; charset=utf-8');
          $xmlTree = simple_xmltree::create();
          $xmlTree->loadFromString($str);
          if (isset($xmlTree->root)) {
            $outputString = '<box>'.$str.'</box>';
          } else {
            $outputString = '<box><![CDATA['.$str.']]></box>';
          }
          unset($xmlTree);
        } else {
          $outputString = $this->_gt('No preview for this box possible.');
        }
      }
    }
    return $outputString;
  }

  /**
  * List of modules and views
  *
  * @return string
  */
  function getEditView() {
    if (isset($this->box['TRANSLATION'])) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_selectview.php');
      $selectView = &new base_selectview();
      $selectView->authUser = &$this->authUser;
      $selectView->msgs = &$this->msgs;
      $selectView->images = &$this->images;
      $selectView->load((int)$this->box['TRANSLATION']['view_id'], 'box');
      $selectView->actionLink = $this->getLink(array('cmd'=>'chg_view', 'bid'=>$this->box['box_id']), $this->paramName).'&'.$this->paramName.'[view_id]=';
      return $selectView->getXMLViewList();
    } else {
      return '';
    }
  }

  /**
  * save language specific view
  *
  * @access public
  * @return boolean
  */
  function saveView() {
    if (isset($this->box['TRANSLATION']) && is_array($this->box['TRANSLATION'])) {
      $dataTrans = array(
        'view_id' => (int)$this->params['view_id']
      );
      if ($this->checkDataModified($dataTrans, $this->box['TRANSLATION'])) {
        $dataTrans['box_trans_modified'] = time();
        $filter = array('box_id'=>(int)$this->box['box_id'], 'lng_id'=>$this->box['TRANSLATION']['lng_id']);
        if (FALSE !== $this->databaseUpdateRecord($this->tableBoxTrans, $dataTrans, $filter)) {
          $data = array('box_modified' => $dataTrans['box_trans_modified']);
          if (FALSE !== $this->databaseUpdateRecord($this->tableBox, $data, 'box_id', (int)$this->box['box_id'])) {
            $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
              sprintf('View of box "%s (%s)" changed.',
              papaya_strings::escapeHTMLChars($this->box['TRANSLATION']['box_title']), $this->box['box_id']));
            return TRUE;
          }
        }
      }
    }
    return FALSE;
  }
}
?>