<?php
/**
* Link Box with page
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_boxeslinks.php,v 1.22 2006/06/02 10:29:52 weinert Exp $
*/

/**
* Database basic access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* Input check function
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');

/**
* Link Box with page
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class base_boxeslinks extends base_db {
  var $tableBox = PAPAYA_DB_TBL_BOX;
  var $tableBoxTrans = PAPAYA_DB_TBL_BOX_TRANS;
  var $tableBoxgroup = PAPAYA_DB_TBL_BOXGROUP;
  var $tableModules = PAPAYA_DB_TBL_MODULES;
  var $tableViews = PAPAYA_DB_TBL_VIEWS;
  var $tableViewLinks = PAPAYA_DB_TBL_VIEWLINKS;
  var $tableLink = PAPAYA_DB_TBL_BOXLINKS;
  var $tableTopics = PAPAYA_DB_TBL_TOPICS;
  var $tableTopicsTrans = PAPAYA_DB_TBL_TOPICS_TRANS;

  var $data;
  var $boxGroupsList;

  /**
  * PHP 5 constructor
  *
  * @param array &$topic
  * @param string $paramName optional, default value 'bl'
  * @access public
  */
  function __construct(&$topic, $paramName='bl') {
    $this->topicObj = &$topic;
    $this->paramName = $paramName;
  }

  /**
  * PHP 5 constructor redirect
  *
  * @param array &$topic
  * @param string $paramName optional, default value 'bl'
  * @access public
  */
  function base_boxeslinks(&$topic, $paramName='bl') {
    $this->__construct($topic, $paramName);
  }

  /**
  * Load box group list
  *
  * @access public
  * @return boolean
  */
  function loadBoxGroupList() {
    unset($this->boxGroupsList);
    $sql = "SELECT boxgroup_id, boxgroup_title, boxgroup_name
            FROM %s
            ORDER BY boxgroup_title";
    $params = array($this->tableBoxgroup);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->boxGroupsList[$row['boxgroup_id']] = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load data list
  *
  * @access public
  * @return boolean
  */
  function loadDataList($lngId, $viewModeId) {
    unset($this->data);
    if ($viewModeId > 0) {
      $sql = "SELECT bl.topic_id, bl.box_id, bl.box_sort, b.box_name, b.boxgroup_id,
                      bt.box_title, bt.box_data, bt.view_id,
                      m.module_guid, m.module_useoutputfilter, m.module_path, m.module_file, m.module_class
              FROM %s AS bl,
                    %s AS b,
                    %s AS bt,
                    %s AS v,
                    %s AS vl,
                    %s AS m
              WHERE bl.topic_id = '%d'
                AND b.box_id = bl.box_id
                AND bt.box_id = bl.box_id
                AND bt.lng_id = '%d'
                AND v.view_id = bt.view_id
                AND vl.view_id = bt.view_id
                AND vl.viewmode_id = %d
                AND m.module_guid = v.module_guid
                AND m.module_active = 1
                AND m.module_type = 'box'
              ORDER BY bl.topic_id, bl.box_sort, bl.box_id";
      $params = array($this->tableLink, $this->tableBox, $this->tableBoxTrans,
        $this->tableViews, $this->tableViewLinks, $this->tableModules, $this->topicId, $lngId, $viewModeId);
    } else {
      $sql = "SELECT bl.topic_id, bl.box_id, bl.box_sort, b.box_name, b.boxgroup_id,
                      bt.box_title, bt.box_data, bt.view_id,
                      m.module_guid, m.module_useoutputfilter, m.module_path, m.module_file, m.module_class
              FROM %s AS bl,
                    %s AS b,
                    %s AS bt,
                    %s AS v,
                    %s AS m
              WHERE bl.topic_id = '%d'
                AND b.box_id = bl.box_id
                AND bt.box_id = bl.box_id
                AND bt.lng_id = '%d'
                AND v.view_id = bt.view_id
                AND m.module_guid = v.module_guid
                AND m.module_active = 1
                AND m.module_type = 'box'
              ORDER BY bl.topic_id, bl.box_sort, bl.box_id";
      $params = array($this->tableLink, $this->tableBox, $this->tableBoxTrans,
        $this->tableViews, $this->tableModules, $this->topicId, $lngId);
    }
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $row['module_file'] = $row['module_path'].$row['module_file'];
        $this->data[$row['box_id']] = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Parse box ( for output )
  *
  * @param array $data
  * @access public
  * @return string '' or XML
  */
  function parsedBox($data, $lngId, $boxId, $viewModeId, $cache = TRUE) {
    $result = '';
    $obj = &$this->createPluginObject($data['module_file'],
      $data['module_class'], $data['module_guid'], $this->topicObj);
    if (isset($obj) && is_object($obj)) {
      $obj->boxId = $boxId;
      if ($cache && ($str = $this->loadBoxCache($obj, $lngId, $viewModeId))) {
        $result .= sprintf('<box title="%s" group="%s" guid="%s" module="%s">',
          $data['box_title'], @$this->boxGroupsList[$data['boxgroup_id']]['boxgroup_name'],
          $data['module_guid'], $data['module_class']);
        $result .= '<![CDATA['.$str.']]>';
        $result .= '</box>'.LF;
      } else {
        $obj->setData($data['box_data']);
        if ($str = $obj->getParsedData()) {
          $outputString = NULL;
          if ($viewModeId > 0) {
            if ($data['module_useoutputfilter']) {
              $outputObj = &new papaya_output;
              if ($outputObj->loadViewModeData('', $viewModeId)) {
                if ($filter = $outputObj->getFilter($data['view_id'])) {
                  if ($filter->checkConfiguration('box')) {
                    if ($output = $filter->parseBox($this->topicObj, $data, $str)) {
                      $this->writeBoxCache($obj, $lngId, $viewModeId, $output);
                      $outputString = '<![CDATA['.$output.']]>';
                    } else {
                      $outputString = '<![CDATA[<!--'.$filter->errorMessage.'-->]]>';
                    }
                  } else {
                    $outputString = '<![CDATA[<!--'.$filter->errorMessage.'-->]]>';
                  }
                }
              }
              unset($outputObj);
            } else {
              $outputString = '<![CDATA['.$str.']]>';
              $this->writeBoxCache($obj, $lngId, $viewModeId, $str);
            }
          } else {
            $xmlTree = simple_xmltree::create();
            $xmlTree->loadFromString($str);
            if (isset($xmlTree->root)) {
              $outputString = $str;
            } else {
              $outputString = '<![CDATA['.$str.']]>';
            }
            unset($xmlTree);
          }
          if ($outputString) {
            $result .= sprintf('<box title="%s" group="%s" guid="%s" module="%s">',
              $data['box_title'], @$this->boxGroupsList[$data['boxgroup_id']]['boxgroup_name'],
              $data['module_guid'], $data['module_class']);
            $result .= $outputString;
            $result .= '</box>'.LF;
          }
          unset($outputString);
        }
      }
      unset($obj);
    }
    return $result;
  }

  function getBoxCacheId(&$box, $lngId, $viewModeId) {
    if (isset($box) && is_object($box) && isset($GLOBALS['PAPAYA_PAGE']) && $GLOBALS['PAPAYA_PAGE']->public) {
      if ($boxCacheId = $box->getCacheId()) {
        $boxId = @(int)$box->boxId;
        $surferId = @(int)$GLOBALS['PAPAYA_SURFER']->surferId;
        return 'box_'.$lngId.'_'.$boxId.'_'.$surferId.'_'.$boxCacheId.'_'.md5($_SERVER['HTTP_HOST']).'.'.$viewModeId;
      }
    }
    return FALSE;
  }

  function loadBoxCache(&$box, $lngId, $viewModeId) {
    if (defined('PAPAYA_CACHE_TIME') && PAPAYA_CACHE_TIME > 0) {
      if ($cacheId = $this->getBoxCacheId($box, $lngId, $viewModeId)) {
        $cacheFileName = PAPAYA_PATH_CACHE.'.'.$cacheId;
        if (file_exists($cacheFileName)) {
          $now = time();
          $cacheTime = filectime($cacheFileName);
          if ($now < ($cacheTime + (int)PAPAYA_CACHE_TIME)) {
            $str = file_get_contents($cacheFileName);
            return $str;
          } else {
            @unlink($cacheFileName);
          }
        }
      }
    }
    return FALSE;
  }

  function writeBoxCache(&$box, $lngId, $viewModeId, $str) {
    if (defined('PAPAYA_CACHE_TIME') && PAPAYA_CACHE_TIME > 0) {
      if ($str != '' && $cacheId = $this->getBoxCacheId($box, $lngId, $viewModeId)) {
        $cacheFileName = PAPAYA_PATH_CACHE.'.'.$cacheId;
        if (file_exists(PAPAYA_PATH_CACHE) && (int)PAPAYA_CACHE_TIME > 0) {
          if ($fh = fopen($cacheFileName, 'w')) {
            fwrite($fh, $str);
            fclose($fh);
          }
        }
      }
    }
    return FALSE;
  }

  /**
  * Parse function
  *
  * @param integer $id
  * @access public
  * @return string '' or XML
  */
  function parsed($id, $lngId, $viewModeId) {
    $result = '';
    include_once(PAPAYA_INCLUDE_PATH.'system/papaya_parser.php');
    $parser = &new papaya_parser;
    $parser->tableTopics = $this->tableTopics;
    $parser->tableTopicsTrans = $this->tableTopicsTrans;
    if ($id > 0) {
      $this->topicId = $id;
      $this->loadBoxGroupList();
      if ($this->loadDataList($lngId, $viewModeId)) {
        if (isset($this->data) && is_array($this->data)) {
          foreach($this->data as $boxId=>$box) {
            $str = $this->parsedBox($box, $lngId, $boxId, $viewModeId);
            $result .= $parser->parse($str, $lngId);
          }
        }
      }
    }
    unset($parser);
    return $result;
  }
}
?>
