<?php
/**
* Base class of dynamic image plugins
*
* image plugins must be inherited from this superclass
*
* @package papaya_modules
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_dynamicimage.php,v 1.5 2006/04/20 18:24:00 weinert Exp $
*/

/**
* Base class for plugins
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_plugin.php');

/**
* Base class of dynamic image plugins
*
* image  plugins must be inherited from this superclass
*
* @package papaya_modules
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_dynamicimage extends base_plugin {

  /**
  * can be cached
  * @var boolean
  */
  var $cacheable = TRUE;

  /**
  * Css class
  * @var string
  */
  var $inputFieldSize = 'x-large';
  
  /**
  * Allowed options / attributes in the papaya:image tags
  * @var array
  */
  var $attributeFields = array();
  
  /**
  * Array with the current attributes
  * @var array
  */
  var $attributes = array();

  /**
  * generate the image
  *
  * @access public
  * @return mixed Filename or FALSE
  */
  function generateImage() {
    $this->lastError = 'No image generator implemented.';
    return FALSE;
  }  
  
  /**
  * get attribute dialog (for preview)
  *
  * @access public
  * @return string
  */
  function getAttributeDialog() {
    if (is_array($this->attributeFields) && count($this->attributeFields) > 0) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array('set_attr' => 1);
      $dialog = &new base_dialog($this, $this->paramName, $this->attributeFields, $this->attributes, $hidden);
      $dialog->msgs = &$this->msgs;
      $dialog->loadParams();
      $dialog->inputFieldSize = $this->inputFieldSize;
      $dialog->baseLink = $this->baseLink;
      $dialog->dialogTitle = $this->_gt('Attributes');
      $dialog->buttonTitle = $this->_gt('Show');
      $dialog->dialogDoubleButtons = TRUE;
      if ($dialog->modified('set_attr')) {
        if ($dialog->checkDialogInput()) {
          $this->attributes = $dialog->data;
        }
      }
      return $dialog->getDialogXML();      
    }
  }
  
  /**
  * check an set the attributes
  *
  * @param $attributes
  * @access public
  * @return boolean
  */
  function setAttributes($attributes) {
    $checkFunctions = get_class_methods('checkit');
    foreach($checkFunctions as $idx=>$functionName) {
      $checkFunctions[$idx] = strtolower($functionName);
    }
    $checkObj = &new checkit;
    unset($this->inputErrors);
    $result = TRUE;
    if (isset($checkFunctions) && is_array($checkFunctions)) {
      foreach($this->attributeFields as $key=>$val) {
        if (isset($val) && is_array($val)) {
          if (strpos($val[3], 'disabled_') === 0) {
            $elementType = substr($val[3], strlen('disabled_'));
            //field does not allow user inputs - no data
            $this->attributes[$key] = NULL;
          } else {
            $elementType = $val[3];
            //active field
            $checkFunction = strtolower($val[1]);
            if (in_array($checkFunction, $checkFunctions)) {
              if ($checkObj->$checkFunction(isset($attributes[$key]) ? $attributes[$key] : '', $val[2])) {
                $this->attributes[$key] = isset($attributes[$key]) ? $attributes[$key] : '';
              } else  {
                $result = FALSE;
              }
            } else {
              if (@preg_match($checkFunction, isset($attributes[$key]) ? $attributes[$key] : '')) {
                $this->attributes[$key] = isset($attributes[$key]) ? $attributes[$key] : '';
              } elseif ($val[2] == FALSE && ((!isset($attributes[$key])) || $attributes[$key] == ''))  {
                $this->attributes[$key] = isset($attributes[$key]) ? $attributes[$key] : '';
              } else {
                $result = FALSE;
              }
            }
          }
        }
      }
    }
    return $result;  
  }

  /**
  * Get output cache identifier
  *
  * @access public
  * @return mixed Cache Id or FALSE
  */
  function getCacheId() {
    if ($this->cacheable) {
      return md5(serialize($this->attributes));
    } else {
      return FALSE;
    }
  }
}
?>