<?php
/**
* Dynamic formular contains of XML-data
*
* @package papaya
* @author Thomas Weinert <thomas@papaya-cms.com>
* @version $Id: base_dynform.php,v 1.6 2006/04/26 10:29:00 weinert Exp $
*/


/**
* basic class object
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_object.php');

/**
* Dynamic formular contains of XML-data
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_dynamic_form extends base_object {

  var $groups = NULL;
  var $fields = NULL;

  var $dialogTitle = '';
  var $buttonTitle = '';

  var $paramName = NULL;

  var $data;

  /**
  * Constructor
  *
  * @param string $xml
  * @param mixed $data optional, default value NULL
  * @param mixed $hidden optional, default value NULL
  * @access public
  */
  function __construct($xml, $data = NULL, $hidden = NULL) {
    $this->xmlToFields($xml);
    $this->data = $data;
    $this->hidden = $hidden;
  }

  /**
  * Get dialog XML
  *
  * @access public
  * @return string XML
  */
  function getDialogXML() {
    $result = sprintf('<form action="%s" method="%s" enctype="multipart/form-data">', $this->baseLink, 'POST');
    $result .= $this->getHiddenFieldsXML();
    if (isset($this->groups) && is_array($this->groups)) {
      foreach($this->groups as $group) {
        $result .= '<fieldset>';
        $result .= '<legend>'.papaya_strings::escapeHTMLChars($group['title']).'</legend>';
        if (isset($group['fields']) && is_array($group['fields'])) {
          foreach($group['fields'] as $fieldId) {
            $field = $this->fields[$fieldId];
            if (isset($this->inputErrors[$field['name']]) && $this->inputErrors[$field['name']]) {
              $error = ' class="error"';
            } else {
              $error = '';
            }
            $result .= sprintf('<label for="%s"%s>%s%s</label>',
              papaya_strings::escapeHTMLChars($this->getFieldName($field['name'])), $error, papaya_strings::escapeHTMLChars($field['title']),
              ($field['needed'] ? '<needed/>' : ''));
            $result .= $this->getFieldXML($field);
          }
        }
        $result .= '<br clear="all" class="clear"/></fieldset>';
      }
      $result .= '<input type="submit" value="'.papaya_strings::escapeHTMLChars($this->buttonTitle).'" class="form_button"/>';
    }
    $result .= '</form>';
    return $result;
  }

  /**
  * Get field XML
  *
  * @param array &$field
  * @access public
  * @return string XML
  */
  function getFieldXML(&$field) {
    $result = '';
    $cssClassName = $this->getFieldCSS($field);
    if (isset($this->params[$field['name']])) {
      $value = $this->params[$field['name']];
    } elseif (isset($this->data[$field['name']])) {
      $value = $this->data[$field['name']];
    } elseif (isset($field['default'])) {
      $value = $field['default'];
    } else {
      $value = '';
    }
    switch ($field['type']) {
    case 'input' :
      $result .= $this->getFieldXMLInput($field, $value, $cssClassName);
      break;
    case 'combobox' :
      $result .= $this->getFieldXMLCombobox($field, $value, $cssClassName);
      break;
    case 'checkbox' :
      $result .= $this->getFieldXMLCheckbox($field, $value, $cssClassName);
      break;
    case 'checkboxes' :
      $result .= $this->getFieldXMLCheckboxes($field, $value, $cssClassName);
      break;
    case 'file' :
      $result .= $this->getFieldXMLFile($field, $value, $cssClassName);
      break;
    case 'image' :
      $result .= $this->getFieldXMLImage($field, $value, $cssClassName);
      break;
    case 'radio' :
      $result .= $this->getFieldXMLRadio($field, $value, $cssClassName);
      break;
    case 'textarea' :
      $result .= $this->getFieldXMLTextarea($field, $value, $cssClassName);
      break;
    case 'url' :
      $result .= $this->getFieldXMLInput($field, $value, $cssClassName);
      break;
    }
    return $result;
  }

  /**
  *
  *
  * @param string $fieldName
  * @access public
  * @return string
  */
  function getFieldName($fieldName) {
    return (isset($this->paramName) && ($this->paramName != ''))
      ? $this->paramName.'['.$fieldName.']'
      : $fieldName;
  }

  function getFieldCSS(&$field) {
    $result = (isset($field['css']) && ($field['css'] != '')) ? $field['css'] : 'form_'.$field['type'];
    if (isset($this->inputErrors[$field['name']]) && $this->inputErrors[$field['name']]) {
      $result .= ' error';
    }
    return $result;
  }

  /**
  * Load parameters
  *
  * @access public
  */
  function loadParams() {
    if (isset($this->paramName) && $this->paramName != '') {
      if (isset($_REQUEST[$this->paramName]) && is_array($_REQUEST[$this->paramName])) {
        $this->params = $_REQUEST[$this->paramName];
      }
    }
  }

  /**
  * Get hidden fields XML
  *
  * @access public
  * @return string
  */
  function getHiddenFieldsXML() {
    $result = '';
    if (isset($this->hidden) && is_array($this->hidden)) {
      foreach($this->hidden as $key=>$val) {
        if (isset($val) && is_array($val)) {
          foreach($val as $subKey=>$subValue)  {
            $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s"/>',
              papaya_strings::escapeHTMLChars($key), papaya_strings::escapeHTMLChars($subKey), papaya_strings::escapeHTMLChars((string)$subValue));
          }
        } elseif (isset($val)) {
          $result .= sprintf('<input type="hidden" name="%s" value="%s"/>',
            papaya_strings::escapeHTMLChars($key), papaya_strings::escapeHTMLChars($val));
        }
      }
    }
    return $result;
  }

  /**
  * Get query String
  *
  * @param array $params
  * @access public
  * @return mixed
  */
  function getQueryString($params) {
    $result = '';
    if (isset($params) && is_array($params)) {
      foreach($params as $key=>$val) {
        if (isset($val) && is_array($val)) {
          foreach($val as $subKey=>$subValue)  {
            $result .= '&amp;'.urlencode($key).'['.urlencode($subKey).']='.urlencode((string)$subValue);
          }
        } elseif (isset($val)) {
          $result .= '&amp;'.urlencode($key).'='.urlencode((string)$val);
        }
      }
    }
    if (isset($this->hidden) && is_array($this->hidden)) {
      foreach($this->hidden as $key=>$val) {
        if (isset($val) && is_array($val)) {
          foreach($val as $subKey=>$subValue)  {
            $result .= '&amp;'.urlencode($key).'['.urlencode($subKey).']='.urlencode((string)$subValue);
          }
        } elseif (isset($val)) {
          $result .= '&amp;'.urlencode($key).'='.urlencode((string)$val);
        }
      }
    }
    if ($result != '') {
      return substr($result, 5);
    } else {
      return '';
    }
  }

  /**
  * Check dialog inputs
  *
  * @param mixed $oldData optional, default value NULL
  * @access public
  * @return boolean
  */
  function checkDialogInputs($oldData = NULL) {
    unset($this->validatedData);
    unset($this->inputErrors);
    unset($this->uploadFiles);
    $checkFunctions = get_class_methods('checkit');
    foreach($checkFunctions as $idx=>$functionName) {
      $checkFunctions[$idx] = strtolower($functionName);
    }
    $checkObj = &new checkit;
    if (isset($this->fields) && is_array($this->fields) &&
        isset($checkFunctions) && is_array($checkFunctions)) {
      $checkFunctions = array_flip($checkFunctions);
      foreach($this->fields as $field) {
        if ($field['type'] == 'file' || $field['type'] == 'image') {
          if (isset($oldData[$field['name']])) {
            $this->validatedData[$field['name']] = $oldData[$field['name']];
          }
          if (($uploadData = $this->getFileUploadData($field['name'])) &&
              is_uploaded_file($uploadData['tmp_name'])) {
            //the upload is executed later - only if the data is requested
            if ($field['type'] == 'image') {
              list(,,$imageType) = getimagesize($uploadData['tmp_name']);
              if ($imageType >= 1 && $imageType <= 3) {
                $uploadData['folder'] = (int)@$field['typeparams']['folder'];
                $this->uploadFiles[$field['name']] = $uploadData;
              } else {
                $this->addMsg(MSG_ERROR, $this->_gtf('The input in field "%s" is not correct.', $field['title']));
                $this->inputErrors[$field['name']] = TRUE;
              }
            } else {
              $uploadData['folder'] = (int)@$field['typeparams']['folder'];
              $this->uploadFiles[$field['name']] = $uploadData;
            }
          }
        } elseif ($field['type'] == 'checkbox') {
          $valueOn = (isset($field['typeparams']['value_on']) && trim($field['typeparams']['value_on']) != '')
            ? $field['typeparams']['value_on']
            : 'X';
          $valueOff = (isset($field['typeparams']['value_off']) && trim($field['typeparams']['value_off']) != '')
            ? $field['typeparams']['value_off']
            : 'O';
          if (isset($this->params[$field['name']]) && $this->params[$field['name']] == $valueOn) {
            $this->validatedData[$field['name']] = $valueOn;
          } else {
            $this->validatedData[$field['name']] = $valueOff;
          }
        } elseif ($field['type'] == 'checkboxes') {
          $items = preg_split("/(\r\n)|(\n\r)|[\r\n]/", $field['typeparams']['items']);
          $inputs = @array_flip($this->params[$field['name']]);
          $data = FALSE;
          foreach($items as $item) {
            if (strpos($item, '=') > 0) {
              list($itemKey) = explode('=', $item);
            } else {
              $itemKey = $item;
            }
            if (isset($inputs[$itemKey])) {
              $data[] = $itemKey;
            }
          }
          if (is_array($data)) {
            $this->validatedData[$field['name']] = implode("\n", $data);
          } elseif (!$field['needed']) {
            $this->validatedData[$field['name']] = '';
          } else {
            $this->addMsg(MSG_ERROR, $this->_gtf('The input in field "%s" is not correct.', $field['title']));
            $this->inputErrors[$field['name']] = TRUE;
          }
        } elseif (isset($this->params[$field['name']]) && (!is_array($this->params[$field['name']])) &&
            strlen($this->params[$field['name']]) > 0) {
          if ($field['validatefunc'] == -1 && isset($field['validateregex'])) {
            if (@preg_match($field['validateregex'], $this->params[$field['name']])) {
              $this->validatedData[$field['name']] = $this->params[$field['name']];
            } else {
              $this->addMsg(MSG_ERROR, $this->_gtf('The input in field "%s" is not correct.', $field['title']));
              $this->inputErrors[$field['name']] = TRUE;
            }
          } elseif (isset($checkFunctions[$field['validatefunc']])) {
            $checkFunction = $field['validatefunc'];
            if ($checkObj->$checkFunction($this->params[$field['name']], $field['needed'])) {
              $this->validatedData[$field['name']] = $this->params[$field['name']];
            } else {
              $this->addMsg(MSG_ERROR, $this->_gtf('The input in field "%s" is not correct.', $field['title']));
              $this->inputErrors[$field['name']] = TRUE;
            }
          } else {
            $this->addMsg(MSG_ERROR, $this->_gtf('Invalid check function for field "%s"', $field['name']));
            $this->inputErrors[$field['name']] = TRUE;
          }
        } elseif ($field['needed'] == FALSE)  {
          $this->validatedData[$field['name']] = '';
        } else {
          $this->addMsg(MSG_ERROR, $this->_gtf('The input in field "%s" is not correct.', $field['title']));
          $this->inputErrors[$field['name']] = TRUE;
        }
      }
    }
    if (isset($this->inputErrors) && count($this->inputErrors) > 0) {
      return FALSE;
    } else {
      return TRUE;
    }
  }

  /**
  * Get file upload data
  *
  * @param string $fieldName
  * @access public
  * @return mixed
  */
    function getFileUploadData($fieldName) {
      if (isset($_FILES) && is_array($_FILES) && isset($_FILES[$this->paramName]) &&
          isset($_FILES[$this->paramName]['name'][$fieldName]) &&
          isset($_FILES[$this->paramName]['tmp_name'][$fieldName]) &&
          is_uploaded_file($_FILES[$this->paramName]['tmp_name'][$fieldName])) {
        return array(
          'name' => $_FILES[$this->paramName]['name'][$fieldName],
          'tmp_name' => $_FILES[$this->paramName]['tmp_name'][$fieldName],
          'error' => $_FILES[$this->paramName]['error'][$fieldName]
        );
      } else {
        return FALSE;
      }
  }

  /**
  * Get dialog inputs
  *
  * @access public
  * @return mixed
  */
  function getDialogInputs() {
    $this->uploadMediaFiles();
    if (isset($this->validatedData)) {
      return $this->validatedData;
    } else {
      return NULL;
    }
  }

  /**
  * Upload media files
  *
  * @access public
  */
  function uploadMediaFiles() {
    if (isset($this->uploadFiles) && is_array($this->uploadFiles)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/papaya_fileman.php');
      $fileManager = &new papaya_filemanager;
      $fileManager->dataDirectory = PAPAYA_PATH_MEDIAFILES;
      $fileManager->thumbnailDirectory = PAPAYA_PATH_THUMBFILES;
      $fileManager->dataLoadFolderList($this->lngId);
      foreach($this->uploadFiles as $fieldName=>$uploadData) {
        if (isset($uploadData['folder']) && isset($fileManager->folders[$uploadData['folder']])) {
          if ($fileId = $fileManager->addFile($uploadData, $uploadData['name'], $uploadData['folder'])) {
            if (isset($this->validatedData[$fieldName]) && $this->validatedData[$fieldName] != '') {
              $fileManager->deleteFile($this->validatedData[$fieldName]);
            }
            $this->validatedData[$fieldName] = $fileId;
          }
        }
      }
      unset($fileManager);
    }
  }

  /**
  * Tansform XML to fields
  *
  * @param string $xml
  * @access public
  */
  function xmlToFields($xml) {
    unset($this->groups);
    unset($this->fields);
    $xmlTree = $this->createXMLTree($xml);
    if (is_object($xmlTree) && is_array($xmlTree->children)) {
      foreach($xmlTree->children as $node) {
        if (is_object($node)) {
          switch ($node->name) {
            case 'group' :
              unset($group);
              if (isset($node->children) &&  is_array($node->children) &&
                  count($node->children) > 0) {
                $group = array(
                  'title' => isset($node->attributes['title']) ? $node->attributes['title'] : $this->_gt('Group')
                );
                foreach($node->children as $fieldNode) {
                  unset($field);
                  if (is_object($fieldNode) && ($fieldNode->name == 'field')) {
                    if (isset($fieldNode->attributes['name'])) {
                      $field = array(
                        'group' => $group['title'],
                        'name' => @$fieldNode->attributes['name'],
                        'title' => @$fieldNode->attributes['title'],
                        'css' => @$fieldNode->attributes['css'],
                        'validatefunc' => @$fieldNode->attributes['validatefunc'],
                        'validateregex'=> @$fieldNode->attributes['validateregex'],
                        'needed' => @$fieldNode->attributes['needed'],
                        'type' => 'input',
                        'typeparams' => array()
                      );
                      if (isset($fieldNode->children) &&  is_array($fieldNode->children)) {
                        foreach($fieldNode->children as $typeNode) {
                          if ($typeNode->name != '') {
                            $field['type'] = $typeNode->name;
                            if (isset($typeNode->children) && is_array($typeNode->children)) {
                              foreach($typeNode->children as $paramNode) {
                                $s = '';
                                $paramNode->printText($s);
                                $field['typeparams'][$paramNode->name] = simple_xmltree::XMLToStr($s);
                              }
                            }
                            break;
                          }
                        }
                      }
                      $this->fields[$field['name']] = $field;
                      $group['fields'][] = $field['name'];
                    }
                  }
                }
                $this->groups[$group['title']] = $group;
              }
              break;
          }
        }
      }
    }
  }

  /**
  * Create XML tree
  * @param string $xmlData XML-string
  * @return object simple_xmlnode $xmlTree XML-tree
  */
  function createXMLTree($xmlData) {
    if (trim($xmlData) != '') {
      $xmlTree = simple_xmltree::create();
      if (is_object($xmlTree)) {
        if (0 === $xmlTree->loadFromString($xmlData)) {
          if (is_object($xmlTree->root)) {
            return $xmlTree->root;
          }
        } elseif (defined('PAPAYA_DBG_XML_USERINPUT') && PAPAYA_DBG_XML_USERINPUT === '1') {
          $this->lastXMLError = @$xmlTree->lastError;
          unset($xmlTree);
        }
      }
    }
    return FALSE;
  }

  /**
  * Get field XML for an input field
  *
  * @param array &$field
  * @param string $value
  * @param string $cssClassName
  * @access public
  * @return string HTML
  */
  function getFieldXMLInput(&$field, $value, $cssClassName) {
    if (isset($field['typeparams']['maxlength']) && $field['typeparams']['maxlength'] > 0) {
      $maxLength = ' maxlength="'.(int)$field['typeparams']['maxlength'].'"';
    } else {
      $maxLength = '';
    }
    return sprintf('<input type="text" name="%s" class="%s" value="%s" %s/>',
      papaya_strings::escapeHTMLChars($this->getFieldName($field['name'])), $cssClassName,
      papaya_strings::escapeHTMLChars($value), $maxLength
    );
  }

  /**
  * Get field XML for a checkbox
  *
  * @param array &$field
  * @param string $value
  * @param string $cssClassName
  * @access public
  * @return string HTML
  */
  function getFieldXMLCheckbox(&$field, $value, $cssClassName) {
    $valueOn = (isset($field['typeparams']['value_on']) && trim($field['typeparams']['value_on']) != '')
      ? $field['typeparams']['value_on']
      : 'X';
    $valueOff = (isset($field['typeparams']['value_off']) && trim($field['typeparams']['value_off'])  != '')
      ? $field['typeparams']['value_off']
      : 'O';
    $caption = (isset($field['typeparams']['caption']) && trim($field['typeparams']['caption']) != '')
      ? $field['typeparams']['caption']
      : $valueOn;
    if ((!isset($value)) || (trim($value) == '')) {
      $checked = (@$field['typeparams']['default'] > 0) ? ' checked="checked"' : '';
    } elseif ($valueOn == $value) {
      $checked = ' checked="checked"';
    } else {
      $checked = '';
    }
    return sprintf('<input type="checkbox" name="%s" class="%s" value="%s"%s>%s</input>',
      papaya_strings::escapeHTMLChars($this->getFieldName($field['name'])), $cssClassName,
      papaya_strings::escapeHTMLChars($valueOn), $checked, papaya_strings::escapeHTMLChars($caption)
    );
  }

  /**
  * Get field XML for a group of checkboxes
  *
  * @param array &$field
  * @param string $value
  * @param string $cssClassName
  * @access public
  * @return string HTML
  */
  function getFieldXMLCheckboxes(&$field, $value, $cssClassName) {
    $result = '';
    if (isset($value) && is_array($value)) {
      $values = $value;
    } else {
      $values = preg_split("/(\r\n)|(\n\r)|[\r\n]/", $value);
    }
    if (isset($values) && is_array($values)) {
      $values = array_flip(array_values($values));
    } else {
      $values = array();
    }
    if (isset($field['typeparams']['items'])) {
      $items = preg_split("/(\r\n)|(\n\r)|[\r\n]/", $field['typeparams']['items']);
    } else {
      $items = NULL;
    }
    if (isset($items) && count($items) > 0) {
      foreach($items as $item) {
        if (strpos($item, '=') > 0) {
          list($itemKey,$itemVal) = explode('=', $item, 2);
        } else {
          $itemKey = $item;
          $itemVal = $item;
        }
        $selected = (isset($values[$itemKey])) ? ' checked="checked"' : '';
        $result .= sprintf('<input type="checkbox" name="%s[]" value="%s" class="%s" %s>%s</input>',
            papaya_strings::escapeHTMLChars($this->getFieldName($field['name'])),
            papaya_strings::escapeHTMLChars($itemKey), $cssClassName, $selected,  papaya_strings::escapeHTMLChars($itemVal));
      }
    }
    return $result;
  }

  /**
  * Get field XML for a combobox
  *
  * @param array &$field
  * @param string $value value that willbe signed as selected
  * @param string $cssClassName
  * @access public
  * @return string
  */
  function getFieldXMLCombobox(&$field, $value, $cssClassName) {
    $result = sprintf('<select name="%s" class="%s">',
      papaya_strings::escapeHTMLChars($this->getFieldName($field['name'])), $cssClassName);
    if (isset($field['typeparams']['items'])) {
      $items = preg_split("/(\r\n)|(\n\r)|[\r\n]/", $field['typeparams']['items']);
    } else {
      $items = NULL;
    }
    if (isset($items) && count($items) > 0) {
      foreach($items as $item) {
        if (strpos($item, '=') > 0) {
          list($itemKey,$itemVal) = explode('=', $item, 2);
        } else {
          $itemKey = $item;
          $itemVal = $item;
        }
        $selected = ($value == $itemKey) ? ' selected="selected"' : '';
        $result .= sprintf('<option value="%s" %s>%s</option>',
            papaya_strings::escapeHTMLChars($itemKey), $selected,  papaya_strings::escapeHTMLChars($itemVal));
      }
    }
    $result .= '</select>';
    return $result;
  }

  /**
  * Get XML for a file upload field
  *
  * @param array &$field
  * @param string $value
  * @param string $cssClassName
  * @access public
  * @return string
  */
  function getFieldXMLFile(&$field, $value, $cssClassName) {
    $result = '';
    if (isset($this->data[$field['name']]) && $this->data[$field['name']] != '') {
      include_once(PAPAYA_INCLUDE_PATH.'system/papaya_parser.php');
      $parser = &new papaya_parser;
      $parser->files[$this->data[$field['name']]] = FALSE;
      $parser->getMediaInfos();
      if ($parser->mediaFileExists($this->data[$field['name']])) {
        $result .= '<div style="text-align: center; padding: 4px 0px 10px 0px">';
        $params = array(
          'src'=>$this->data[$field['name']],
          'download'=>'yes'
        );
        $result .= $parser->createMediaTag($params);
        $queryParams = array(
          $this->paramName => array(
            'dynform_cmd' => 'delfile',
            'dynform_field' => $field['name']
          )
        );
        $result .= sprintf('<br clear="all"/><a href="%s"><button name="del">'.papaya_strings::escapeHTMLChars(@$field['typeparams']['delcaption']).'</button></a>',
          $this->baseLink.'?'.$this->getQueryString($queryParams));
        $result .= '</div>';
      }
    }
    $result .= sprintf('<input type="file" name="%s" class="%s"/>',
      papaya_strings::escapeHTMLChars($this->getFieldName($field['name'])), $cssClassName);
    return $result;
  }

  /**
  * Get field XML for an image upload field
  *
  * @param array &$field
  * @param string $value
  * @param string $cssClassName
  * @access public
  * @return string
  */
  function getFieldXMLImage(&$field, $value, $cssClassName) {
    $result = '';
    if (isset($this->data[$field['name']]) && $this->data[$field['name']] != '') {
      include_once(PAPAYA_INCLUDE_PATH.'system/papaya_parser.php');
      $parser = &new papaya_parser;
      $parser->files[$this->data[$field['name']]] = FALSE;
      $parser->getMediaInfos();
      if ($parser->mediaFileExists($this->data[$field['name']])) {
        $result .= '<div style="text-align: center; padding: 4px 0px 10px 0px">';
        $params = array(
          'width'=>@(int)$field['typeparams']['width'],
          'height'=>@(int)$field['typeparams']['height'],
          'src'=>$this->data[$field['name']],
          'align'=>'middle'
        );
        $result .= $parser->createMediaTag($params);
        $queryParams = array(
          $this->paramName => array(
            'dynform_cmd' => 'delfile',
            'dynform_field' => $field['name']
          )
        );
        $result .= sprintf('<br clear="all"/><a href="%s"><button name="del">'.papaya_strings::escapeHTMLChars($field['typeparams']['delcaption']).'</button></a>',
          $this->baseLink.'?'.$this->getQueryString($queryParams));
        $result .= '</div>';
      }
    }
    $result .= sprintf('<input type="file" name="%s" class="%s"/>',
      papaya_strings::escapeHTMLChars($this->getFieldName($field['name'])), $cssClassName);
    return $result;
  }

  /**
  * Get field XML for a group of radio fields
  *
  * @param array &$field
  * @param string $value value that willbe signed as selected
  * @param string $cssClassName
  * @access public
  * @return string
  */
  function getFieldXMLRadio(&$field, $value, $cssClassName) {
    $result = '';
    if (isset($field['typeparams']['items'])) {
      $items = preg_split("/(\r\n)|(\n\r)|[\r\n]/", $field['typeparams']['items']);
    } else {
      $items = NULL;
    }
    if (isset($items) && count($items) > 0) {
      foreach($items as $item) {
        if (strpos($item, '=') > 0) {
          list($itemKey,$itemVal) = explode('=', $item, 2);
        } else {
          $itemKey = $item;
          $itemVal = $item;
        }
        $selected = ($value == $itemKey) ? ' checked="checked"' : '';
        $result .= sprintf('<input type="radio" name="%s" value="%s" class="%s" %s>%s</input>',
            papaya_strings::escapeHTMLChars($this->getFieldName($field['name'])),
            papaya_strings::escapeHTMLChars($itemKey), $cssClassName, $selected,  papaya_strings::escapeHTMLChars($itemVal));
      }
    }
    return $result;
  }
  /**
  * Get field XML for a textarea
  *
  * @param array &$field
  * @param string $value
  * @param string $cssClassName
  * @access public
  * @return string
  */
  function getFieldXMLTextarea(&$field, $value, $cssClassName) {
    if (isset($field['typeparams']['maxlength']) && $field['typeparams']['maxlength'] > 0) {
      $rows = (int)$field['typeparams']['maxlength'];
    } else {
      $rows = 5;
    }
    return sprintf('<textarea name="%s" class="%s" rows="%d">%s</textarea>',
      papaya_strings::escapeHTMLChars($this->getFieldName($field['name'])), $cssClassName,
      $rows, papaya_strings::escapeHTMLChars($value)
    );
  }
}

?>