<?php
/**
* Language managment UI and Content
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_languages.php,v 1.2 2005/10/21 10:02:27 weinert Exp $
*/

/**
* Base class for database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* Language managment UI and Content
*
* @package papaya_modules
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_languages extends base_db {

  var $paramName = 'phr';

  var $tableLanguage = PAPAYA_DB_TBL_LNG;
  var $tableTranslation = PAPAYA_DB_TBL_PHRASE_TRANS;

  var $languages = array();

  function initialize() {
    $this->sessionParamName = 'PAPAYA_SESS_languages_'.$this->paramName;
    $this->initializeParams();
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);

    $this->initializeSessionParam('pagemode');
  }

  function execute() {
    switch (@$this->params['pagemode']) {
      case 1 :
        $this->loadLanguages();
        switch(@$this->params['cmd']) {
          case 'lng_add' :
            $this->initializeDialog();
            if ($this->dialog->checkDialogInput()) {
              if ($newId = $this->addLanguage()) {
                $this->params['lng_id'] = $newId;
                $this->addMsg(MSG_INFO, sprintf($this->_gt('%s added.'), $this->_gt('Language')));
                unset($this->dialog);
                $this->loadLanguages();
              }
            }
            break;
          case 'lng_chg' :
            if (isset($this->params['lng_id']) && $this->params['lng_id'] > 0 && isset($this->languages[$this->params['lng_id']])) {
              $this->initializeDialog();
              if ($this->dialog->checkDialogInput()) {
                if ($this->saveLanguage()) {
                  $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Language')));
                  unset($this->dialog);
                  $this->loadLanguages();
                }
              }
            }
            break;
          case 'lng_del' :
            if (isset($this->params['lng_id']) && $this->params['lng_id'] > 0 && isset($this->languages[$this->params['lng_id']])) {
              if (!$this->checkLanguageHasPhrases($this->params['lng_id'])) {
                if (isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
                  if ($this->deleteLanguage()) {
                    unset($this->languages[$this->params['lng_id']]);
                    unset($this->params['lng_id']);
                    unset($this->params['cmd']);
                    $this->addMsg(MSG_INFO, sprintf($this->_gt('%s deleted.'), $this->_gt('Language')));
                  }
                }
              } else {
                unset($this->params['cmd']);
                $this->addMsg(MSG_ERROR, $this->_gt('Language has phrase translations for admininterface.'));
              }
            }
            break;
        }
        break;
      default :
        include_once(PAPAYA_INCLUDE_PATH.'system/base_phraseeditor.php');
        $this->phraseEditor = &new base_phraseeditor();
        $this->phraseEditor->layout = &$this->layout;
        $this->phraseEditor->images = &$this->images;
        $this->phraseEditor->msgs = &$this->msgs;
        $this->phraseEditor->authUser = &$this->authUser;

        $this->phraseEditor->initialize();
        $this->phraseEditor->execute();
        break;
    }
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  function getXML() {
    switch (@$this->params['pagemode']) {
      case 1 :
        $this->getLanguagesListXML();
        $this->getXMLButtons();
        $this->getDelForm();
        $this->getDialogXML();
        break;
      default :
        return $this->phraseEditor->getXML();
        break;
    }
  }


  function getXMLButtons() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $toolbar = &new base_btnbuilder;
    $toolbar->images = &$this->images;
    $toolbar->addButton('Phrases', $this->baseLink.'?'.$this->paramName.'[pagemode]=0', 16, '', FALSE);
    $toolbar->addButton('Languages', $this->baseLink.'?'.$this->paramName.'[pagemode]=1', 16, '', TRUE);
    $toolbar->addSeperator();
    $toolbar->addButton('New Language', $this->baseLink.'?'.$this->paramName.'[lng_id]=0', 41, '', FALSE);
    if (isset($this->params['lng_id']) && $this->params['lng_id'] > 0 && isset($this->languages[$this->params['lng_id']])) {
      $toolbar->addButton('Delete',
          $this->baseLink.'?'.$this->paramName.'[cmd]=lng_del&'.$this->paramName.'[lng_id]='.(int)$this->params['lng_id'],
          10, '', FALSE);
    }
    $toolbar->addSeperator();
    if ($result = $toolbar->getXML()) {
      $this->layout->add('<menu>'.$result.'</menu>', 'menus');
    }
  }

  function loadLanguages() {
    unset($this->languages);
    $sql = "SELECT lng_id, lng_ident, lng_short, lng_title, lng_glyph, is_interface_lng, is_content_lng
              FROM %s
              ORDER BY lng_title, lng_short, lng_ident, lng_id";
    if ($res = $this->databaseQueryFmt($sql, $this->tableLanguage)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->languages[$row['lng_id']] = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  function saveLanguage() {
    $data = array(
      'lng_short' => @$this->params['lng_short'],
      'lng_ident' => @$this->params['lng_ident'],
      'lng_title' => @$this->params['lng_title'],
      'lng_glyph' => @$this->params['lng_glyph'],
      'is_interface_lng' => @(int)$this->params['is_interface_lng'],
      'is_content_lng' => @(int)$this->params['is_content_lng']
    );
    if ((!$data['is_interface_lng']) && $this->checkLanguageHasPhrases($this->params['lng_id'])) {
      $this->addMsg(MSG_WARNING, $this->_gt('Language has phrase translations for admininterface.'));
      $data['is_interface_lng'] = 1;
    }
    return FALSE !== $this->databaseUpdateRecord($this->tableLanguage, $data, 'lng_id', (int)$this->params['lng_id']);
  }

  function addLanguage() {
    $data = array(
      'lng_short' => @$this->params['lng_short'],
      'lng_ident' => @$this->params['lng_ident'],
      'lng_title' => @$this->params['lng_title'],
      'lng_glyph' => @$this->params['lng_glyph'],
      'is_interface_lng' => @(int)$this->params['is_interface_lng'],
      'is_content_lng' => @(int)$this->params['is_content_lng']
    );
    return $this->databaseInsertRecord($this->tableLanguage, 'lng_id', $data);
  }

  function deleteLanguage() {
    return $this->databaseDeleteRecord($this->tableLanguage, 'lng_id', (int)$this->params['lng_id']);
  }

  function checkLanguageHasPhrases($lngId) {
    $sql = "SELECT COUNT(*) FROM %s WHERE lng_id = %d";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableTranslation, (int)$lngId))) {
      list($count) = $res->fetchRow();
      return ($count > 0);
    }
    return FALSE;
  }

  function getLanguagesListXML() {
    if (isset($this->languages) && is_array($this->languages) && count($this->languages) > 0) {
      $result = sprintf('<listview title="%s">', papaya_strings::escapeHTMLChars($this->_gt('Languages')));
      $result .= '<items>';
      foreach($this->languages as $lng) {
        $selected = (@$this->params['lng_id'] == $lng['lng_id']) ? ' selected="selected"' : '';
        $href = $this->baseLink.'?'.$this->paramName.'[lng_id]='.(int)$lng['lng_id'];
        $result .= sprintf('<listitem href="%s" title="%s"%s>', $href, papaya_strings::escapeHTMLChars($lng['lng_title']), $selected);
        if ($lng['lng_glyph'] != '' && file_exists($this->getBasePath(TRUE).'pics/language/'.$lng['lng_glyph'])) {
          $result .= '<subitem>';
          $result .= '<glyph src="'.papaya_strings::escapeHTMLChars('./pics/language/'.$lng['lng_glyph']).'"/>';
          $result .= '</subitem>';
        } else {
          $result .= '<subitem/>';
        }
        $result .= '</listitem>';
      }
      $result .= '</items>';
      $result .= '</listview>';
      $this->layout->addLeft($result);
    }
  }

  function initializeDialog() {
    if (!(isset($this->dialog) && is_object($this->dialog))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');

      if (isset($this->params['lng_id']) && $this->params['lng_id'] > 0 && isset($this->languages[$this->params['lng_id']])) {
        $hidden = array(
          'save'=> 1,
          'cmd'=> 'lng_chg',
          'lng_id' => @$this->params['lng_id']
        );
        $data = $this->languages[$this->params['lng_id']];
        $caption = 'Edit';
      } else {
        $hidden = array(
          'save'=> 1,
          'cmd'=> 'lng_add'
        );
        $data = array();
        $caption = 'Add';
      }

      $path = $this->getBasePath(TRUE).'pics/language/';
      $fields = array(
        'lng_short' => array('Language', '/^[a-z]{2,3}_[a-z]{2,3}$/i', TRUE, 'input', 7, '', 'en_EN'),
        'lng_ident' => array('Ident', '/^[a-z]{2,3}$/', TRUE, 'input', 3, '', 'en'),
        'lng_title' => array('Title', 'isNoHTML', TRUE, 'input', 30, '', 'New Language'),
        'lng_glyph' => array ('Image file', 'isFile', FALSE, 'filecombo', array($path, '/^\w+\.gif$/i', TRUE), ''),
        'is_interface_lng' => array('Interface', 'isNum', TRUE, 'yesno', '', '', 1),
        'is_content_lng' => array('Content', 'isNum', TRUE, 'yesno', '', '', 1),
      );

      $this->dialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->dialog->dialogTitle = $this->_gt($caption);
      $this->dialog->baseLink = $this->baseLink;
      $this->dialog->msgs = &$this->msgs;
      $this->dialog->loadParams();
    }
  }

  function getDialogXML() {
    $this->initializeDialog();
    $this->layout->add($this->dialog->getDialogXML());
  }


  /**
  * Get delete language form
  *
  * @access public
  */
  function getDelForm() {
      if (isset($this->params['lng_id']) && $this->params['lng_id'] > 0 && isset($this->languages[$this->params['lng_id']]) &&
          isset($this->params['cmd']) && $this->params['cmd'] == 'lng_del') {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'lng_del',
        'lng_id' => (int)$this->params['lng_id'],
        'confirm_delete'=>1,
      );
      $msg = sprintf($this->_gt('Delete language "%s"?'), $this->languages[$this->params['lng_id']]['lng_title']);
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Delete';
      $this->layout->add($dialog->getMsgDialog());
    }
  }
}
?>