<?php
/**
* sitemap basic class
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_sitemap.php,v 1.12 2006/04/20 18:24:00 weinert Exp $
*/


/**
* basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');
/**
* basic class check input
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');

/**
* sitemap basic class
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class base_sitemap extends base_db {
  var $paramName = 'tt';
  var $sessionParamname;

  var $params;
  var $sessionParams;

  var $baseLink;

  var $useSurfer = TRUE;
  var $useCookie = FALSE;

  var $tableTopics = PAPAYA_DB_TBL_TOPICS;
  var $tableTopicsTrans = PAPAYA_DB_TBL_TOPICS_TRANS;
  var $tableLanguages = PAPAYA_DB_TBL_LNG;
  
  var $tableViewLinks = PAPAYA_DB_TBL_VIEWLINKS;
  var $tableViewModes = PAPAYA_DB_TBL_VIEWMODES;

  /**
  * configurations
  *@var array $data
  */
  var $data;

  var $surfer;

  var $topicMin;
  var $topicMax;
  var $topicFocused;

  var $topicObj;
  var $topicObjPrevs;

  var $topicRoot;

  var $topics;

  var $topicTree;
  var $rootIds;

  var $openNodes;

  /**
  * base sitemap
  *
  * @param integer &$topic
  * @param boolean $fields optional, default value FALSE
  * @access public
  */
  function __construct(&$topic, $fields=FALSE) {
    $this->topicObj = &$topic;
    $this->tableTopics = $this->topicObj->tableTopics;
    $this->tableTopicsTrans = $this->topicObj->tableTopicsTrans;
    $this->topicObjPrevs = $this->idStringToArray($this->topicObj->topic['prev_path'], $this->topicObj->topic['prev']);
    $this->data = Array('xslfile' =>  (isset($fields['xslfile'])) ? $fields['xslfile'] : '',
                        'title' =>    (isset($fields['title'])) ? $fields['title'] : '',
                        'root'=>      @(int)$fields['root'],
                        'focus' =>    @$fields['focus'],
                        'forstart' => @(int)$fields['forstart'],
                        'forend' =>   @(int)$fields['forend'],
                        'foclevels' =>@(int)$fields['foclevels'],
                        'format' =>   @$fields['format'],
                        'sort' =>     (isset($fields['sort'])) ? $fields['sort'] : FALSE);
    $this->paramName = $this->paramName.'_'.$this->data['root'];
    $this->sessionParamName = 'PAPAYA_SESS_sitemap_'.$this->paramName;
    $this->initializeParams();
    $this->baseLink = 'index'.PAPAYA_URL_EXTENSION;
  }

  /**
  * PHP 4 constuctor redirect
  *
  * @access public
  */
  function base_sitemap(&$topic, $fields=FALSE) {
    $this->__construct($topic, $fields);
  }

  /**
  * initialize seassion data
  *
  * @access public
  */
  function initializeSessionData() {
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);
    $this->openNodes = $this->sessionParams['opened'];
    if (isset($this->params['open'])) {
      $this->openNodes[(int)$this->params['open']] = TRUE;
    } elseif (isset($this->params['close'])) {
      unset($this->openNodes[(int)$this->params['close']]);
    }
    $this->sessionParams['opened'] = $this->openNodes;
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  /**
  * Initialize cookie data
  *
  * @access public
  */
  function initializeCookieData() {
    $cookieName = 'c_sitemap_'.$this->paramName.'_opened';
    $cookieData = $_COOKIE['cookiename'];
    if (preg_match_all('#\d+#',$cookieData, $matches, PREG_PATTERN_ORDER)) {
      $this->openNodes = array_flip($matches[0]);
    }
    if ($this->topicObj->topicId > 0) {
      switch (@$this->params['cmd']) {
        case 'open' :
          $this->openNodes[$id] = TRUE;
          $cookieData = implode(';',array_keys($this->openNodes));
          setcookie($cookieName, $cookieData);
          break;
        case 'close' :
          if (isset($this->openNodes) && is_array($this->openNodes)) {
            unset($this->openNodes[$id]);
            $cookieData = implode(';',array_keys($this->openNodes));
            setcookie($cookieName, $cookieData);
          }
          break;
      }
    }
  }

  /**
  * Get XML
  *
  * @access public
  * @return ''
  */
  function getXML() {
    if ($this->initializeNavTree()) {
      $result = $this->getSitemap();
      return $result;
    }
    return '';
  }

  /**
  * Get urls
  *
  * @access public
  * @return string
  */
  function getUrls() {
    $result = '';
    $sql = "SELECT tt.topic_id, tt.topic_title, 
                   l.lng_id, l.lng_ident, vm.viewmode_ext
              FROM %s AS tt, %s AS l, %s AS vl, %s AS vm
             WHERE l.lng_id = tt.lng_id
               AND vl.view_id = tt.view_id
               AND vm.viewmode_id = vl.viewmode_id
               AND vm.viewmode_ext = '%s'";
    $params = array($this->tableTopicsTrans, $this->tableLanguages, 
      $this->tableViewLinks, $this->tableViewModes, PAPAYA_URL_EXTENSION);
    
    echo '<html><head><title>URLs</title><meta name="robots" content="noindex, follow"></head><body>'.LF;
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $link = $this->getWebLink($row['topic_id'], $row['lng_ident'], $row['viewmode_ext'], NULL, NULL, $row['topic_title']);
        $href = $this->getAbsoluteURL($link, NULL, FALSE);
        printf('<a href="%s">%s</a>'.LF, $href, $href);
      }
    }
    echo '</body></html>';
    return '';
  }

  /**
  * Initialize nav tree
  *
  * @param boolean $navigationOnly optional, default value TRUE
  * @access public
  * @return boolean
  */
  function initializeNavTree($navigationOnly=TRUE) {
    //get content language
    $lngId = $this->topicObj->getContentLanguageId();
    //verschiedene Einstellungen nach Typ prfen und abarbeiten
    switch ($this->data['format']) {
      case 'breadcrumb' :
      case 'path' :
        $this->checkDepth(0);
        $this->topicPathRoot = $this->topicObj->getParentIDFromMaster($this->data['root'],$this->data['forstart']-1);
        break;
      case 'active' :
        //Beim dynamischen Baum mssen die Knoten initialisiert werden
          if ($this->useCookie) {
          //entweder aus einem Cookie
          $this->initializeCookieData();
        } else {
          //oderaus einer Session
          $this->initializeSessionData();
        }
        $this->checkDepth(1);
        break;
      case 'static' :
        $this->checkDepth(1);
        break;
    }
    //Basis laden
    if ($this->loadBase()) {
      //eventuell Surfer laden
      $this->initializeSurfer();
      //minimale Generationstiefe (Anzahl der Vorgnger)
      $this->topicMin = $this->data['forstart'] + count($this->topicRoot['prevs']);
      //maximale Generationstiefe
      $this->topicMax = $this->topicMin + $this->data['depth'];
      //Focus-Id ermittlen
      $this->topicFocused = $this->getFocusId();

      if ($this->data['format'] == 'breadcrumb') {
        return $this->loadPrevTree($lngId, $navigationOnly);
      } elseif ($this->loadTree($lngId, $navigationOnly)) {
        return TRUE;
      }
    } elseif ($this->data['root'] == 0) {
      //eventuell Surfer laden
      $this->initializeSurfer();
      //minimale Generationstiefe (Anzahl der Vorgnger)
      $this->topicMin = 0;
      //maximale Generationstiefe
      $this->topicMax = 999;
      //Focus-Id ermittlen
      $this->topicFocused = 0;

      $this->data['subpath'] = ';0;';

      if ($this->data['format'] == 'breadcrumb') {
        return $this->loadPrevTree($lngId, $navigationOnly);
      } elseif ($this->loadTree($lngId, $navigationOnly)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Check depth
  *
  * @param integer $min
  * @access public
  */
  function checkDepth($min) {
    $this->data['depth'] = $this->data['forend'] - $this->data['forstart'];
    if($this->data['depth'] < $min) {
      $this->data['forend'] = $this->data['forend']+$min;
      $this->data['depth'] = $min;
    }
    if ($this->data['forstart'] <= 0) {
      $this->data['forstart'] = 1;
    }
  }

  /**
  * Check path
  *
  * @param integer $id
  * @param array &$row
  * @access public
  * @return array
  */
  function checkPath($id, &$row) {
    if (isset($row) && is_array($row)) {
      if ($this->topicObj->topicId == $id) {
        //aktuelle Seite
        $row['focuspath'] = TRUE;
        return TRUE;
      } elseif (isset($row['prevs']) && is_array($row['prevs']) && in_array($this->topicPathRoot, $row['prevs'])) {
        //im aktuellen Pfad
        if (isset($row['prevs']) && is_array($row['prevs']) && in_array($this->topicObj->topicId, $row['prevs'])) {
          //Nachfolger von aktueller Seite
          if ($row['prev'] == $this->topicObj->topicId) {
            //direkter Nachfolger
            $row['focuspath'] = FALSE;
            return TRUE;
          }
        } elseif (isset($this->topicObjPrevs) && is_array($this->topicObjPrevs) && in_array($row['prev'], $this->topicObjPrevs)) {
          //Kind von Vorgnger
          if (in_array($id, $this->topicObjPrevs)) {
            $row['focuspath'] = TRUE;
          } else {
            $row['focuspath'] = FALSE;
          }
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  /**
  * Check open node
  *
  * @param integer $id
  * @param array &$row
  * @access public
  * @return boolean
  */
  function checkOpenNode($id, &$row) {
    if (isset($row) && is_array($row)) {
      if (isset($this->openNodes) && is_array($this->openNodes) && isset($this->openNodes[$row['prev']])) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Check filter
  *
  * @param integer $id
  * @param array &$row
  * @access public
  * @return boolean
  */
  function checkFilter($id, &$row) {
    if (isset($row) && is_array($row)) {
      if ($this->useSurfer && is_object($this->surfer)) {
        if (!$this->surfer->canView($id)) {
          return FALSE;
        }
      }
      $depth = count($row['prevs']);
      if (isset($this->topicTree[$row['prev']]['childcount'])) {
        $this->topicTree[$row['prev']]['childcount']++;
      } else {
        $this->topicTree[$row['prev']]['childcount'] = 1;
      }
      if ($this->topicMin > $depth) {
        return FALSE;
      } elseif ($this->topicMax <= $depth) {
        switch ($this->data['format']) {
          case 'breadcrumb' :
          case 'path' :
            if (!$this->checkPath($id, $row)) {
              return FALSE;
            }
            break;
          case 'active' :
            if (!$this->checkOpenNode($id, $row)) {
              return FALSE;
            }
            break;
          default:
            return FALSE;
        }
      } else {
        switch ($this->data['format']) {
          case 'breadcrumb' :
          case 'path' :
            $this->checkPath($id, $row);
            break;
          case 'active' :
            $this->checkOpenNode($id, $row);
            break;
        }
      }
      if ($this->topicMin == $depth) {
        $this->rootIds[] = $id;
      } else {
        $this->topicTree[$row['prev']]['childs'][] = $id;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * ids to array
  *
  * @param string $idString
  * @param integer $id
  * @access public
  * @return array
  */
  function idStringToArray($idString, $id) {
    if (preg_match_all('#\d+#', $idString, $matches, PREG_PATTERN_ORDER)) {
      $result = $matches[0];
      $result[] = $id;
    } else {
      $result = array($id);
    }
    return $result;
  }

  /**
  * Load base
  *
  * @access public
  * @return boolean
  */
  function loadBase() {
    unset($this->topicRoot);
    $sql = "SELECT topic_id, prev, prev_path
            FROM %s
            WHERE topic_id = %d";
    $params = array($this->tableTopics,$this->data['root']);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $row['prevs'] = $this->idStringToArray($row['prev_path'], $row['prev']);
        $row['subpath'] = ';'.implode(';',$row['prevs']).';';
        $this->topicRoot = $row;
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Load tree internal
  *
  * @param string $filter
  * @param string $navigationFilter
  * @param string $sort
  * @access public
  * @return boolean
  */
  function loadTreeInternal($lngId, $filter, $navigationFilter, $sort) {
    $sql = "SELECT t.topic_id, tt.topic_title, t.prev, t.prev_path, t.topic_weight, t.topic_created
            FROM %s AS t, %s AS tt
            WHERE tt.topic_id = t.topic_id
              AND tt.lng_id = %d
              AND ".str_replace('%', '%%', $filter)."
              AND ".str_replace('%', '%%', $navigationFilter)."
            ORDER BY t.topic_weight %s, t.topic_created %s";
    $sortString = (($sort == 'DESC') ? 'DESC' : 'ASC');
    $params = array($this->tableTopics, $this->tableTopicsTrans, $lngId, $sortString, $sortString);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $row['prevs'] = $this->idStringToArray($row['prev_path'], $row['prev']);
        if ($this->checkFilter($row['topic_id'], $row)) {
          $this->topics[$row['topic_id']] = $row;
        }
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load tree
  *
  * @param boolean $navigationOnly optional, default value TRUE
  * @access public
  * @return boolean
  */
  function loadTree($lngId, $navigationOnly = TRUE) {
    unset($this->topics);
    $sort = ($this->data['sort']) ? 'DESC' : 'ASC';
    $navigationFilter = ($navigationOnly) ? 't.is_navigation = 1' : '1=1';

    if (@($this->data['subpath'] != ';0;')) {
      $filter = " t.prev = ".(int)$this->data['root']." ";
      if ($this->loadTreeInternal($lngId, $filter, $navigationFilter, $sort)) {
        $filter = " t.prev_path LIKE '{$this->topicRoot['subpath']}{$this->data['root']};%'";
        return $this->loadTreeInternal($lngId, $filter, $navigationFilter, $sort);
      }
    } else {
      $filter = '1=1';
      return $this->loadTreeInternal($lngId, $filter, $navigationFilter, $sort);
    }
    return FALSE;
  }

  function loadPrevTree($lngId, $navigationOnly = TRUE) {
    unset($this->topics);
    $sort = 'ASC';
    $navigationFilter = ($navigationOnly) ? 't.is_navigation = 1' : '1';

    $filter = ' t.topic_id IN (';
    if (preg_match_all('/\d+/', $this->topicObj->topic['prev_path'], $regs, PREG_PATTERN_ORDER)) {
      foreach($regs[0] as $id) {
        $filter .= (int)$id . ',';
      }
    }

    $filter .= (int)$this->topicObj->topic['prev'].','.(int)$this->topicObj->topicId.') ';
    return $this->loadTreeInternal($lngId, $filter, $navigationFilter, $sort);
  }

  /**
  * Initialize surfer
  *
  * @access public
  */
  function initializeSurfer() {
    if ($this->useSurfer) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_surfer.php');
      if (isset($GLOBALS['PAPAYA_SURFER'])) {
        $this->surfer = &$GLOBALS['PAPAYA_SURFER'];
      } else {
        $this->surfer = &new base_surfer(TRUE);
        $this->surfer->loadTopicIdList(TRUE);
        $GLOBALS['PAPAYA_SURFER'] = &$this->surfer;
      }
    }
  }

  /**
  * Get focus id
  *
  * @access public
  * @return integer
  */
  function getFocusId() {
    $result = 0;
    switch ($this->data['focus']) {
      case 'root' :
        $result = $this->topicObj->getParentIDFromMaster($this->data['root'], $this->data['foclevels']);
        break;
      case 'dyna' :
        if ($this->data['foclevels'] == 0) {
          $result = $this->topicObj->topicId;
        } else {
          $f = count($this->topicObjPrevs) - $this->data['foclevels'];
          $result = $this->topicObjPrevs[$f];
        }
        break;
    }
    return $result;
  }

  /**
  * Get sitemap
  *
  * @access public
  * @return string
  */
  function getSitemap() {
    $result = sprintf('<sitemap format="%s" date="%s">'.LF,
      papaya_strings::escapeHTMLChars($this->data['format']),
      papaya_strings::escapeHTMLChars(date('Y-m-d H:i:s'))
      );
    $result .= $this->getItems($this->rootIds);
    $result .= '</sitemap>'.LF;
    return $result;
  }

  /**
  * Get items
  *
  * @param $nodes
  * @access public
  * @return string
  */
  function getItems($nodes) {
    $result = '';
    if (isset($this->rootIds) && is_array($this->rootIds)) {
      foreach($this->rootIds as $id) {
        $counter = 0;
        $gen = 0;
        $result .= $this->getItem($id, $counter, $gen);
      }
    }
    return $result;
  }

  /**
  * Get tiem
  *
  * @param integer $id
  * @param integer &$counter
  * @param integer &$gen
  * @access public
  * @return string
  */
  function getItem($id, &$counter, &$gen) {
    $result = '';
    $row = $this->topics[$id];
    if (isset($row) && is_array($row)) {
      if (isset($row['focuspath']) && $row['focuspath']) {
        $focusPath = ' focuspath="focuspath"';
      } else {
        $focusPath = '';
      }
      if ($id == $this->topicFocused) {
        $focus = ' focus="focus"';
      } else {
        $focus = '';
      }
      $subResult = '';
      $subCounter = 0;
      if (isset($this->topicTree[$id]['childs']) && is_array($this->topicTree[$id]['childs'])) {
        foreach($this->topicTree[$id]['childs'] as $subId) {
          $subResult .= $this->getItem($subId, $subCounter, $gen);
          $gen++;
        }
      }
      $result .= sprintf('<mapitem id="%d" href="%s" title="%s" enctitle="%s" children="%d" allchildren="%d" visible="1" gens="%d" %s>'.LF,
        $id, $this->getWebLink($id, '', 'page', NULL, NULL, $row['topic_title'], 0),
        papaya_strings::escapeHTMLChars($row['topic_title']), rawurlencode($row['topic_title']),
        (isset($this->topicTree[$id]['childcount'])) ? (int)$this->topicTree[$id]['childcount'] : 0,
        $subCounter,
        $gen, $focus, $focusPath);
      $result .= $subResult;
      $result .= '</mapitem>'.LF;
      $counter += $subCounter;
      $counter++;
    }
    return $result;

  }
}

?>