<?php
/**
* Basic statistic object with logging for actions
*
* @package papaya
* @author David Rekowski <rekowski@papaya-cms.com>
* @version $Id: base_statistic_actions.php,v 1.9 2006/02/14 12:31:18 rekowski Exp $
*/


/**
* Basic class object
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* Basic statistic object for actions
* @package papaya
* @author David Rekowski <rekowski@papaya-cms.com>
*/
class base_statistic_actions extends base_db {

  function __construct() {
    parent::__construct();
    $this->tableStatisticActions = PAPAYA_DB_TABLEPREFIX.'_statistic_actions';
    $this->tableTopicPublicTrans = PAPAYA_DB_TABLEPREFIX.'_topic_public_trans';
    $this->statisticRequestId    = @$GLOBALS['PAPAYA_PAGE']->statisticRequestId;
  }

  function initializeStatisticObj() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_statistic.php');
    $this->statisticObj = &new base_statistic();
  }

  function setStatisticObj(&$statisticObj) {
    if (isset($statisticObj) && is_object($statisticObj)) {
      $this->statisticObj = &$statisticObj;
    } else {
      $this->initializeStaticObject();
    }
  }

  /**
  * logs page request
  *
  * @param integer $actionId action id of module object
  * @param string $moduleGUID module GUID
  * @param string $message message added to action
  */
  function logAction($guid, $actionId, $message, $refererPage = '', $refererParams = '') {
    if (isset($this->statisticRequestId) && $this->statisticRequestId != '') {
      $data['statistic_action_id']       = NULL;
      $data['module_guid']               = $guid;
      $data['statistic_moduleaction_id'] = $actionId;
      $data['statistic_referer_page']    = $refererPage;
      $data['statistic_referer_params']  = $refererParams;
      $data['statistic_request_id']      = $this->statisticRequestId;
      $data['statistic_message']         = $message;
      if ($this->databaseInsertRecord($this->tableStatisticActions, 'statistic_action_id', $data)) {
        return TRUE;
      }
    }
    return FALSE;
  }


  /**
  * fetches n most executed actions for module (guid) and action (actionId)
  *
  * @param integer $n top n results
  * @param array $viewIds views to evaluate
  * @param string $guid module guid
  * @param integer $actionId id of action for selected module
  * @param string $mode default = html or other output mode
  * @return array $result database results: count, topic_id, categ_id, statistic_moduleaction_params
  */
  function getModuleAction($n, $guid, $actionId) {
    $result = array();
    $sql = "SELECT COUNT(*) AS count, statistic_moduleaction_id, statistic_message, statistic_referer_page, statistic_referer_params
            FROM %s
            WHERE module_guid = '%s' AND statistic_moduleaction_id = '%d'
            GROUP BY statistic_referer_page, statistic_referer_params
            LIMIT 0, %d
            ";
    $params = array($this->tableStatisticActions, $guid, $actionId, $n);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $result[] = $row;
      }
    }
    return $result;
  }

}
?>