<?php
/**
* Base surfer
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_surfer.php,v 1.5 2006/05/08 12:01:29 weinert Exp $
*/


/**
* basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

define('SURFER_ERROR_UNKNOWN', 700);
define('SURFER_ERROR_PASSWORD', 701);
define('SURFER_ERROR_USERNAME', 702);
define('SURFER_ERROR_BLOCKED', 703);
define('SURFER_ERROR_PERMISSIONS', 704);

/**
* Surfer object for load and check users
* @package papaya_base
*/
class base_surfer extends base_db {
  /**
  * Table name surfer
  * @var string $tableSurfers
  */
  var $tableSurfers = PAPAYA_DB_TBL_SURFER;

  /**
  * Table name group
  * @var string $tableSurferGroups
  */
  var $tableSurferGroups = PAPAYA_DB_TBL_SURFERGROUPS;

  /**
  * Table name rights
  * @var string $tableSurferPermissions
  */
  var $tableSurferPermissions = PAPAYA_DB_TBL_SURFERPERM;

  /**
  * Link table name surfer<->right
  * @var string $tableSurferLinks
  */
  var $tableSurferLinks = PAPAYA_DB_TBL_SURFERPERMLINK;

  /**
  * Table name topic
  * @var string $tableTopics
  */
  var $tableTopics = PAPAYA_DB_TBL_TOPICS;

  /**
  * Session variable name for surfer handle
  * @var string $surfernameVar
  */
  var $surfernameVar = 'papaya_surfer_handle';

  /**
  * Name of the global surfer object
  * @var string surferobj_var
  */
  var $surferObjectVariableName  = 'surfer';

  /**
  * Session variable name for topics that the surfer can see
  * @var string
  */
  var $topiclistVar = 'surfer_topics';

  /**
  * Param name for login form fields
  * @var string
  */
  var $logformVar = 'surf';

  /**
  * Visitor name
  * @access public
  * @var string $handle
  */
  var $surferHandle = "";

  /**
  * Valid surfer
  * @access public
  * @var boolean $isValid
  */
  var $isValid = FALSE;

  /**
  * Active
  * @access public
  * @var boolean $active
  */
  var $surferValid = FALSE;

  /**
  * Surfer id
  * @access public
  * @var int $id
  */
  var $surferId = "";

  /**
  * Surfer email
  * @access public
  * @var string $email
  */
  var $surferEmail = "";

  /**
  * Surfer acessible topics
  * @access public
  * @var array $topicList
  */
  var $topicList = FALSE;
  /**
  * Topic rights
  * @var boolean
  */
  var $topicPermissions = FALSE;
  /**
  * Temporary
  * @var boolean
  */
  var $tempTopicPermissions = FALSE;

  /**
  * Errors
  * @var array
  */
  var $errs;

  /**
  * Constructor base surfer
  *
  * @param boolean $login optional, default value TRUE
  * @param string $surferHandle optional, default value ""
  * @access public
  */
  function __construct($login=TRUE,$surferHandle="") {
    $this->baseLink = $this->getBaseLink();
    if ($login) {
      if (isset($_REQUEST[$this->logformVar.'_logout']) && $_REQUEST[$this->logformVar.'_logout'] == 1){
        $this->logout();
      } elseif (!@$GLOBALS[$this->surferObjectVariableName]->isValid) {
        $userHandle = $this->getSessionValue($this->surfernameVar);
        if (isset($userHandle) && trim($userHandle) != '') {
          $this->load($userHandle);
        }
        if ((!$this->isValid) && isset($_POST[$this->logformVar."_surfer"]) &&
            $_POST[$this->logformVar."_surfer"] != $this->getSessionValue($this->surfernameVar)) {
          if ($this->login(@$_POST[$this->logformVar."_surfer"], @$_POST[$this->logformVar."_password"])) {
            $this->logLoginTime();
          }
        }
      }
    } else {
      $this->load($surferHandle);
    }
  }

  /**
  * Load function
  *
  * @param string $surferName
  * @param boolean $login optional, default value FALSE
  * @param string $password optional, default value ""
  * @access public
  * @return boolean
  */
  function load($surferName,$login=FALSE,$password="") {
    $this->isValid = FALSE;
    unset($this->errs);
    if (trim($surferName) != '') {
      $sql = "SELECT  s.surfer_id, s.surfergroup_id, s.surfer_handle, s.surfer_password,
                      s.surfer_givenname, s.surfer_surname,
                      s.surfer_email, s.surfer_email_alt,
                      s.surfer_phone, s.surfer_phone_alt,
                      s.surfer_valid, sg.surfergroup_title
              FROM %s AS s
              LEFT OUTER JOIN %s AS sg ON sg.surfergroup_id = s.surfergroup_id
              WHERE s.surfer_handle = '%s'";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableSurfers, $this->tableSurferGroups, strtolower($surferName)))) {
        if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          if ($row['surfer_password'] || (!$login)) {
            if ((!$login) ||
                ($this->getPasswordHash($password) == $row["surfer_password"] && 
                (bool)$row['surfer_valid'])) {
              $this->surfer = $row;
              $this->surferId = $row["surfer_id"];
              $this->surfergroupId = $row['surfergroup_id'];
              $this->surferHandle = $row["surfer_handle"];
              $this->surferGivenname = $row["surfer_givenname"];
              $this->surferSurname = $row["surfer_surname"];
              $this->surferEmail = $row["surfer_email"];
              $this->surferValid = $row["surfer_valid"];
              $this->isValid = TRUE;
              $this->loadTopicIdList();
              if ($login) {
                $this->setSessionValue($this->surfernameVar, $this->surferHandle);
                $GLOBALS[$this->topiclistVar] = $this->topicList;
              }
              $GLOBALS[$this->surferObjectVariableName] = $this;
            } elseif ($this->getPasswordHash($password) != $row["surfer_password"]) {
              $this->errs[SURFER_ERROR_PASSWORD] = $this->_gt("Invalid password");
            } elseif (!(bool)$row['surfer_valid']) {
              $this->errs[SURFER_ERROR_BLOCKED] = $this->_gt('Account blocked');
            } else {
              $this->errs[SURFER_ERROR_UNKNOWN] = $this->_gt('Unknown Error');
            }
          }
        } else {
          $this->errs[SURFER_ERROR_USERNAME] = $this->_gt("Invalid username");
        }
      } else {
        $this->errs[SURFER_ERROR_USERNAME] = $this->_gt("Invalid username");
      }
    }
    return $this->isValid;
  }

  /**
  * Login attempt
  *
  * @param string $username
  * @param string $password
  * @return boolean login valid?
  */
  function login($surferName ,$password) {
    if ($this->checkLoginTry($surferName)) {
      $this->load($surferName, TRUE, $password);
      if ($this->isValid) {
        $GLOBALS['PAPAYA_SESSION']->regenerateSID();
      }
    }
  }

  /**
  * Load list of all accessible sites
  */
  function loadTopicIdList($default=FALSE) {
    unset($this->topicList);
    unset($this->livePerms);
    unset($this->topicPermissions);
    unset($this->tempTopicPermissions);

    $this->livePerms = array();
    if ($default){
    } else if (isset($this->surferId) && ($this->surferId > 0)) {
      $sql = "SELECT l.surfer_permid
                FROM %s AS l, %s AS s, %s AS p
                WHERE l.surfergroup_id = s.surfergroup_id
                  AND s.surfer_id = '%d'
                  AND l.surfer_permid = p.surferperm_id
                  AND p.surferperm_active > 0";
      $params = array($this->tableSurferLinks, $this->tableSurfers, $this->tableSurferPermissions, $this->surferId);
      if ($res = $this->databaseQueryFmt($sql, $params)) {
        while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->livePerms[$row['surfer_permid']] = $row['surfer_permid'];
        }
      }
    }
    $sql = "SELECT topic_id, surfer_useparent, surfer_permids, prev
              FROM %s";
    if ($res = $this->databaseQueryFmt($sql, $this->tableTopics)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        if ($row['surfer_useparent'] == 1) {
          $this->topicPermissions[$row['topic_id']] = $this->permStr2Array($row['surfer_permids']);
        } else {
          $this->tempTopicPermissions[$row['topic_id']] = $row;
        }
      }
    }
    //rechte fr knoten holen
    if (isset($this->tempTopicPermissions) && is_array($this->tempTopicPermissions)) {
      foreach($this->tempTopicPermissions as $key=>$val) {
        if (!isset($this->topicPermissions[$key])) {
          $this->topicPermissions[$key] = $this->getNodeRights($key);
        }
      }
    }
    unset($this->tempTopicPermissions);
    //rechte prfen
    if (isset($this->topicPermissions) && is_array($this->topicPermissions)) {
      foreach($this->topicPermissions as $id=>$rights) {
        if ($rights) {
          if (count(array_intersect($rights, $this->livePerms))) {
            //ja
            $this->topicList[$id] = TRUE;
          }
        } else {
          //ja
          $this->topicList[$id] = TRUE;
        }
      }
    }
    unset($this->topicPermissions);
  }

  /**
  * Load permission list
  *
  * @param boolean $forceReload optional, default value FALSE
  * @access public
  * @return boolean
  */
  function loadPermissionList($forceReload = FALSE) {
    if ($forceReload && isset($this->permissions)) {
      unset($this->permissions);
    }
    if (!isset($this->permissions)) {
      $sql = "SELECT surferperm_id, surferperm_title, surferperm_active
                FROM %s
            ORDER BY surferperm_title, surferperm_id";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableSurferPermissions))) {
        while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->permissions[$row['surferperm_id']] = $row;
        }
        return TRUE;
      }
    } else {
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Check user by data
  *
  * @param string $username
  * @param string $userEmail
  * @access public
  * @return mixed
  */
  function checkUserByData($username, $userEmail) {
    $sql = "SELECT surfer_handle
              FROM %s
              WHERE surfer_handle = '%s' OR surfer_email = '%s'";
    $params = array($this->tableSurfers, $username, $userEmail);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if($row = $res->fetchRow()) {
        return $row[0];
      }
    }
    return FALSE;
  }

  /**
  * Request password change
  *
  * @access public
  * @return boolean
  */
  function requestPasswordChange() {
    srand((double)microtime()*1000000);
    $chgId = md5(uniqid(rand()));
    $username = trim(@$_REQUEST[$this->logformVar]['username']);
    $userEmail = trim(@$_REQUEST[$this->logformVar]['email']);
    if ($username = $this->checkUserByData($username, $userEmail)) {
      if ($this->saveChangePasswordId($username, $chgId)) {
        $aSurfer = &new base_surfer();
        if ($aSurfer->load($username)) {
          $mailTo = $aSurfer->surfer['surfer_givenname'].''.$aSurfer->surfer['surfer_surname'].' <'.$aSurfer->surfer['surfer_email'].'>';
          $subject = papaya_strings::escapeHTMLChars(PAPAYA_PROJECT_TITLE).' - '.$this->_gt('Password forgotten');

          $msg = "Sie erhalten diese E-Mail, da eine Passwortnderung fr ".PAPAYA_PROJECT_TITLE." angefordert wurde. ";
          $msg .= "Falls Sie Ihr Passwort nicht ndern mchten, knnen Sie diese E-Mail ignorieren.\n\n";
          $msg .= "Um Ihr Passwort zu ndern, klicken Sie auf den folgenden Link: \n";
          $msg = wordwrap($msg);

          $href = $this->getAbsoluteURL($this->baseLink).'?'.$this->logformVar.'_chg='.$chgId;
          unset($GLOBALS['PAPAYA_SURFER']);
          $GLOBALS['PAPAYA_SURFER'] = &new base_surfer;
          $GLOBALS['PAPAYA_SURFER']->loadTopicIdList(!$GLOBALS['PAPAYA_SURFER']->isValid);
          if (@mail($mailTo, $subject, $msg.$href)) {
            return TRUE;
          }
        }
      }
    }
    return FALSE;
  }

  /**
  * Get password request formular
  *
  * @access public
  * @return string
  */
  function getPasswordRequestForm() {
    $result = sprintf('<passwordrequest action="%s" name="%s">', $this->getBaselink(), $this->logformVar);
    $result .= '<element type="hidden" name="newpwd" value="2"/>';
    $result .= sprintf('<element type="input" name="username" value="%s"/>',
      papaya_strings::escapeHTMLChars(@$_POST[$this->logformVar]['username']));
    $result .= sprintf('<element type="input" name="email" value="%s"/>',
      papaya_strings::escapeHTMLChars(@$_POST[$this->logformVar]['email']));
    $result .= '</passwordrequest>';
    return $result;
  }

  /**
  * Save change password id
  *
  * @param string $username
  * @param integer $chgId
  * @access public
  * @return boolean
  */
  function saveChangePasswordId($username, $chgId) {
    $validUntil = time()+86400;
    $data = array(
      'chg_id'   => $chgId,
      'chg_time' => $validUntil,
    );
    return ($this->databaseUpdateRecord($this->tableSurfers, $data, 'surfer_handle', $username) !== FALSE);
  }

  function logLoginTime() {
    if ($this->isValid) {
      $data = array('surfer_lastlogin'=>time());
      return ($this->databaseUpdateRecord($this->tableSurfers, $data, 'surfer_id', $this->surferId) !== FALSE);
    }
    return FALSE;
  }

  /**
  * Check change password id
  *
  * @param integer $chgId
  * @access public
  * @return mixed
  */
  function checkChangePasswordId($chgId) {
    if (preg_match('/^[a-fA-F\d]{32}$/',$chgId)) {
      $sql = "SELECT surfer_id, surfer_handle
              FROM %s
              WHERE chg_id = '%s' AND (NOT(chg_id = ''))
                AND chg_time > '%d'";
      $params = array($this->tableSurfers, $chgId, time());
      if ($res = $this->databaseQueryFmt($sql, $params)) {
        if($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          return $row;
        }
      }
    }
    return FALSE;
  }

  /**
  * Get node rights
  *
  * @param integer $id
  * @access public
  * @return mixed array or boolean
  */
  function getNodeRights($id) {
    if (!isset($this->topicPermissions[$id])) {
      if (isset($this->tempTopicPermissions[$id])) {
        $row = $this->tempTopicPermissions[$id];
      } else {
        $row = NULL;
      }
      if (isset($row) && is_array($row)) {
        $rights = $this->getNodeRights($row['prev']);
        if ($row['surfer_useparent'] == 3) {
          $addRights = $this->permStr2Array($row['surfer_permids']);
        } else{
          $addRights = FALSE;
        }
        if (isset($rights) && is_array($rights) && is_array($addRights)) {
          return array_merge($rights, $addRights);
        } elseif (isset($rights) && is_array($rights)) {
          return $rights;
        } elseif (isset($addRights) && is_array($addRights)) {
          return $addRights;
        } else {
          return FALSE;
        }
      } else {
        return FALSE;
      }
    } else {
      return $this->topicPermissions[$id];
    }
  }

  /**
  * Permission string to array
  *
  * @param string $permString
  * @access public
  * @return mixed
  */
  function permStr2Array($permString) {
    if (preg_match_all('#\d+#', $permString, $matches, PREG_PATTERN_ORDER)) {
      return $matches[0];
    } else {
      return FALSE;
    }
  }

  /**
  * Test if surfer can enter a given topic
  *
  * @access public
  * @param integer $topicId
  */
  function canView($topicId) {
    return (isset($this->topicList[$topicId]) && $this->topicList[$topicId]);
  }


  /**
  * Get viewable
  *
  * @access public
  * @return array
  */
  function getViewable(){
    return array_keys($this->topicList);
  }
  /**
  * Surfer logout, clean up variable
  *
  * @access public
  */
  function logout(){
    $this->setSessionValue($this->surfernameVar, NULL);
    $this->setSessionValue($this->topiclistVar, NULL);
    unset($this->surferId);
    unset($this->surferHandle);
    unset($this->surferGivenname);
    unset($this->surferSurname);
    unset($this->surferEmail);
    unset($this->surferPassword);
    unset($this->surferValid);
    unset($this->isValid);
    $this->isValid = FALSE;
    $this->loadTopicIdList();
    $GLOBALS[$this->surferObjectVariableName] = $this;
  }

  /**
  * Get formular XML
  *
  * @param boolean $login optional, default value TRUE
  * @access public
  * @return string
  */
  function getFormXML($login=TRUE){
    if($login){
      if ($this->isValid) {
        $this->errs[SURFER_ERROR_PERMISSIONS] = $this->_gt("Invalid Permissions");
      }
      $username = @$_POST[$this->logformVar.'_surfer'];
      $changeHref = $this->baseLink.'?'.$this->logformVar.'[newpwd]=1';
      $return  = sprintf('<login name="%s" action="%s" title="%s" chglink="%s">',
        $this->logformVar, $this->baseLink, 'Please fill in your data.', $changeHref);
      $return .= sprintf('<element type="username" title="%s" name="surfer" value="%s"/>', $this->_gt('Username'),
        papaya_strings::escapeHTMLChars($username));
      $return .= sprintf('<element type="password" title="%s" name="password" value=""/>', $this->_gt('Password'));
      if(isset($this->errs) && is_array($this->errs)){
        foreach($this->errs as $errorNumber=>$err){
          $return .= "<error no=\"$errorNumber\">$err</error>";
        }
      }
      $return .= '</login>';
    } else {
      $return  = sprintf('<logout name="%s" action="%s" username="%s" fullname="%s">',
        $this->logformVar, $this->baseLink,
        papaya_strings::escapeHTMLChars($this->getSessionValue($this->surfernameVar)),
        papaya_strings::escapeHTMLChars($this->surferGivenname.' '.$this->surferSurname));
      $return .= '</logout>';
    }
    return $return;
  }

  /**
  * Get password formular XML
  *
  * @param string $token
  * @access public
  * @return string
  */
  function getPassFormXML($token) {
    $result  = sprintf('<passwordchange name="%s" action="%s" title="%s">',
      $this->logformVar, $this->baseLink, 'Please input your new password.');
    $result .= '<element type="hidden" name="chg" value="'.papaya_strings::escapeHTMLChars($token).'"/>';
    $result .= '<element type="hidden" name="save" value="1"/>';
    $result .= '<element type="password" name="password_1" value=""/>';
    $result .= '<element type="password" name="password_2" value=""/>';
    $result .= '</passwordchange>';
    return $result;
  }

  /**
  * Has permission ?
  *
  * @param integer $permId
  * @access public
  * @return boolean
  */
  function hasPerm($permId) {
    return ($this->isValid && is_array($this->livePerms) && isset($this->livePerms[$permId]));
  }

  /**
  * Has on permissions of
  *
  * @param integer $permIds
  * @access public
  * @return mixed
  */
  function hasOnPermOf($permIds) {
    if (isset($permIds) && is_array($permIds)) {
      if (count($permIds) > 1) {
        $filter = "IN ('".implode("','", $permIds)."')";
      } elseif (count($permIds) == 1) {
        $filter = ' = '.(int)$permIds[0];
      } else {
        return TRUE;
      }
    } else {
      return TRUE;
    }
    $sql = "SELECT COUNT(*)
              FROM %s sl, %s p
              WHERE surfergroup_id = '%d'
                AND sl.surfer_permid = p.surferperm_id
                AND p.surferperm_active = 1
                AND p.surferperm_id $filter";
    $params = array(
      $this->tableSurferLinks, $this->tableSurferPermissions, (int)@$this->surfergroupId
    );
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        return ($row[0] > 0);
      }
    }
    return FALSE;
  }
  
  function getPasswordHash($password) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_auth_secure.php');
    return base_auth_secure::getPasswordHash($password);
  }
  
  function checkLoginTry($username) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_auth_secure.php');
    $secureLoginObj = &new base_auth_secure();
    $result = $secureLoginObj->checkLoginTry($username, 'community');
    unset($secureLoginObj);
    return $result;
  }
}

?>