<?php
/**
* Connect sufer permissions and page
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
* @version $Id: base_surferlinks.php,v 1.5 2006/01/23 16:35:06 weinert Exp $
*/


/**
* basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');
/**
* basic class for check conditions
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');

/**
* Connect sufer permissions and page
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class base_surferlinks extends base_db {
  var $tableSurferPerm = PAPAYA_DB_TBL_SURFERPERM;
  var $tableTopics = PAPAYA_DB_TBL_TOPICS;
  var $tableTopicsTrans = PAPAYA_DB_TBL_TOPICS_TRANS;

  var $paramName;
  var $params;
  var $baseLink;

  var $msgs;

  var $surferPermissionList;
  var $surferPermissions;
  var $livePerm;
  var $topicList;
  var $used;
  var $data;

  /**
  * base surfer links
  *
  * @param integer $id
  * @param string $paramName optional, default value 'sl'
  * @access public
  */
  function __construct($id, $paramName='sl') {
    $this->paramName = $paramName;
    $this->initializeParams();
    $this->topicId = (int)$id;
  }

  /**
  * Load mode list
  *
  * @access public
  * @return boolean
  */
  function loadModeList() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_statictables.php');
    foreach(base_statictables::getTableAccessStates() as $key=>$val) {
      $this->modeList[$key] = $this->_gt($val);
    }
    return TRUE;
  }

  /**
  * Load list
  *
  * @access public
  */
  function loadList(){
    unset($this->surferPermissionList);
    $sql = "SELECT surferperm_id, surferperm_title, surferperm_active
              FROM %s";
    if ($res = $this->databaseQueryFmt($sql,array($this->tableSurferPerm))) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->surferPermissionList[$row['surferperm_id']] = $row;
      }
    }
  }

  /**
  * Load function
  *
  * @access public
  */
  function load(){
    unset($this->surferPermissions);
    $sql = "SELECT  surfer_useparent, surfer_permids
              FROM  %s
            WHERE topic_id = %d";
    $params = array($this->tableTopics, (int)$this->topicId);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->surferPermissions = $row;
      }
    }
  }

  /**
  * Load topics
  *
  * @param integer $lngId Sprach Id
  * @access public
  */
  function loadTopics($lngId){
    unset($this->topicList);
    unset($this->livePerm);
    $sqlFunction = $this->databaseGetSQLSource('LOCATE',
      $this->databaseGetSQLSource('CONCAT', ';', TRUE, 'a.topic_id', FALSE, ';', TRUE), FALSE,
      $this->databaseGetSQLSource('CONCAT', 'b.prev_path', FALSE, 'b.prev', FALSE, ';', TRUE), FALSE);
    $sql = "SELECT a.topic_id, a.surfer_useparent, a.surfer_permids, a.prev,
                   b.prev_path, tt.topic_title
              FROM %s as a,
                   %s as b
              LEFT OUTER JOIN %s AS tt ON (tt.topic_id = a.topic_id AND tt.lng_id = '%d')
             WHERE (b.topic_id = %d AND ($sqlFunction != 0) OR  a.topic_id = %d)";
    $params = array($this->tableTopics, $this->tableTopics, $this->tableTopicsTrans, $lngId, $this->topicId, $this->topicId);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->topicList[$row['topic_id']] = $row;
      }
      $current = $this->topicId;
      while ($current != 0) {
        $rec = $this->topicList[$current]['surfer_useparent'];
        $permString = $this->topicList[$current]['surfer_permids'];
        if ($rec != 2 && preg_match_all('/\d+/',$permString,$matches, PREG_PATTERN_ORDER)){
          foreach($matches[0] as $val){
            if (!(isset($this->livePerm) && is_array($this->livePerm) && in_array($val, array_keys($this->livePerm)))) {
              $this->livePerm[$val] = $this->topicList[$current]['topic_id'];
            }
          }
        }
        if ($rec == 1) break;
        $current = $this->topicList[$current]['prev'];
      }
    } else {
      $this->addMsg(MSG_WARNING,"Database error!");
    }
  }

  /**
  * Execute - basic class for parameter handling
  *
  * @access public
  */
  function execute() {
    $this->load();
    switch (@$this->params['cmd']) {
      case 'chg_mode' :
        $this->updateSurferPermUse((int)$this->params['mode']);
        break;
      case 'add' :
        preg_match_all('/\d+/',$this->surferPermissions['surfer_permids'],$matches, PREG_PATTERN_ORDER);
        $argh = array_flip($matches[0]);
        $argh[(int)$this->params['perm_id']] = TRUE;
        $this->updateSurferPerms(array_keys($argh));
        break;
      case 'del' :
        preg_match_all('/\d+/',$this->surferPermissions['surfer_permids'],$matches, PREG_PATTERN_ORDER);
        $argh = array_flip($matches[0]);
        unset($argh[(int)$this->params['perm_id']]);
        $this->updateSurferPerms(array_keys($argh));
        break;
    }
  }

  /**
  * Update surfer permissions
  *
  * @param array $perms
  * @access public
  */
  function updateSurferPerms($perms){
    $this->loadList();
    if (isset($perms) && is_array($perms)){
      $perms = ";".implode(";",$perms).";";
    } else {
      $perms = '';
    }
    $values = array(
      'surfer_permids' => $perms,
      'topic_modified' => time()
    );
    if($this->databaseUpdateRecord($this->tableTopics, $values, 'topic_id', $this->topicId) !== FALSE) {
      $this->addMsg(MSG_INFO, $this->_gtf('%s modified.', $this->_gt('Permissions')));
    } else {
      $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
    }
  }

  /**
  * update surfer permission use
  *
  * @param integer $mode
  * @access public
  */
  function updateSurferPermUse($mode){
    $this->loadList();
    $values = array(
      'surfer_useparent' => (int)$mode,
      'topic_modified' => time()
    );
    if($this->databaseUpdateRecord($this->tableTopics, $values, 'topic_id', $this->topicId) !== FALSE) {
      $this->addMsg(MSG_INFO, $this->_gtf('%s modified.', $this->_gt('Permissions')));
    } else {
      $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
    }
  }

  /**
  * Get
  *
  * @param int $lngId
  * @access public
  * @return string
  */
  function get($lngId) {
    $this->loadList();
    $this->loadTopics($lngId);
    return $this->getPermList($this->topicList[$this->topicId]['surfer_useparent']);
  }

  /**
  * Get permission list
  *
  * @param integer $mode
  * @access public
  * @return string
  */
  function getPermList($mode) {
    unset($inhPerm);
    unset($ownPerm);
    unset($avPerm);
    $result = '';

    if (isset($this->surferPermissionList) && is_array($this->surferPermissionList)) {
      foreach($this->surferPermissionList as $perm) {
        if (isset($this->livePerm[(int)$perm['surferperm_id']]) && $this->livePerm[(int)$perm['surferperm_id']] != $this->topicId){
          $inhPerm[$perm['surferperm_id']] = $this->livePerm[$perm['surferperm_id']];
        } elseif (isset($this->livePerm[(int)$perm['surferperm_id']])) {
          $ownPerm[$perm['surferperm_id']] = TRUE;
        } else {
          $avPerm[$perm['surferperm_id']] = TRUE;
        }
      }
    }
    if (isset($ownPerm) && is_array($ownPerm) && (($mode == 1) || ($mode == 3))) {
      $result .= sprintf('<listview title="%s" width="100%%">', $this->_gt('Linked permissions'));
      $result .= '<items>';
      foreach($ownPerm as $id=>$dummy) {
        $result .= sprintf('<listitem title="%s" image="%s">',
          $this->surferPermissionList[$id]['surferperm_title'], $this->images[1]);
        $result .= '<subitem align="right">';
        $result .= sprintf('<a href="%s"><glyph src="%s" hint="%s"/></a>',
          $this->getLink(array('page_id'=> $this->topicId, 'cmd' => 'del', 'perm_id' => $this->surferPermissionList[$id]['surferperm_id'])), $this->images[10], $this->_gt('Delete'));
        $result .= '</subitem>';
        $result .= '</listitem>';
      }
      $result .= '</items>';
      $result .= '</listview>';
    }
    if (isset($inhPerm) && is_array($inhPerm) && (($mode == 2) || ($mode == 3))) {
      $result .= sprintf('<listview title="%s" width="100%%">', $this->_gt('Inherited permissions'));
      $result .= '<cols>';
      $result .= '<col>'.$this->_gt('Permission').'</col>';
      $result .= '<col>'.$this->_gt('Page').'</col>';
      $result .= '</cols>';
      $result .= '<items>';
      foreach($inhPerm as $id=>$pid) {
        $result .= sprintf('<listitem title="%s" image="%s">',
          $this->surferPermissionList[$id]['surferperm_title'], $this->images[2]);
        if ($this->topicList[$pid]['topic_title'] != '') {
          $title = papaya_strings::escapeHTMLChars($this->topicList[$pid]['topic_title']);
        } else {
          $title = '<i>'.papaya_strings::escapeHTMLChars($this->_gt('No title')).'</i>';        
        }          
        $result .= sprintf('<subitem><a href="%s">%s</a></subitem>',
          $this->getLink(array('page_id'=> $pid)), $title);
        $result .= '</listitem>';
      }
      $result .= '</items>';
      $result .= '</listview>';
    }
    if (isset($avPerm) && is_array($avPerm) && (($mode == 1) || ($mode == 3))) {
      $result .= sprintf('<listview title="%s" width="100%%">', $this->_gt('Avaliable permissions'));
      $result .= '<items>';
      foreach($avPerm as $id=>$dummy) {
        $result .= sprintf('<listitem title="%s" image="%s">',
          $this->surferPermissionList[$id]['surferperm_title'], $this->images[1]);
        $result .= '<subitem align="right">';
        $result .= sprintf('<a href="%s"><glyph src="%s" hint="%s"/></a>',
          $this->getLink(array('page_id'=> $this->topicId, 'cmd' => 'add', 'perm_id' => $this->surferPermissionList[$id]['surferperm_id'])), $this->images[11], $this->_gt('Link'));
        $result .= '</subitem>';
        $result .= '</listitem>';
      }
      $result .= '</items>';
      $result .= '</listview>';
    }
    return $result;
  }

  /**
  * Get mode dialog
  *
  * @access public
  * @return string
  */
  function getModeDlg() {
    $this->loadModeList();
    $actId = $this->topicList[$this->topicId]['surfer_useparent'];
    if (isset($this->modeList) && is_array($this->modeList)) {
      $result = sprintf('<dialog action="%s" title="%s" width="100%%">', $this->baseLink, $this->_gt('Permission inheritance'));
      $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />',
        $this->paramName, 'cmd', 'chg_mode');
      $result .= sprintf('<input type="hidden" name="%s[%s]" value="%d" />',
        $this->paramName, 'page_id', $this->topicId);
      $result .= '<lines class="dialogSmall">';
      $result .= sprintf('<line caption="%s">', $this->_gt('Mode'));
      $result .= sprintf('<select name="%s[mode]" class="dialogSelect dialogScale">', $this->paramName);
      foreach($this->modeList as $id=>$mode) {
        $selected = ($id == $actId) ? 'selected="selected"' : '';
        $result .= sprintf('<option value="%s" %s>%s</option>', $id, $selected, $mode);
      }
      $result .= '</select>';
      $result .= '</line>';
      $result .= '</lines>';
      $result .= sprintf('<dlgbutton value="%s" />', $this->_gt('Edit'));
      $result .= '</dialog>';
    }
    return $result;
  }
}
?>