<?php
/**
* thumbnail creation
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_thumbnail.php,v 1.7 2006/05/31 14:29:50 weinert Exp $
*/


/**
* thumbnail creation
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_thumbnail {
  /**
  * base path for images
  * @var string $baseDirectory
  */
  var $baseDirectory = '';

  /**
  * base path for thumbnails
  * @var string $thumbnailDirectory
  */
  var $thumbnailDirectory = '';

  /**
  * thumbnail width
  * @var string $width
  */
  var $width = 100;

  /**
  * thumbnail height
  * @var string $height
  */
  var $height = 100;

  /**
  * resize mode (max = maximal, min = minimal, mincrop = minimal cropped, abs = absolute)
  * @var string $resizeFlag
  */
  var $resizeFlag = 'max';

  /**
  * ignore cache and create
  * @var string $createAll
  */
  var $createAll = FALSE;

  /**
  * output filetype
  * @var string $thumbnailType
  */
  var $thumbnailType = PAPAYA_THUMBS_FILETYPE;

  /**
  * jpeg quality for thumbnail output
  * @var string $jpegQuality
  */
  var $jpegQuality = PAPAYA_THUMBS_JPEGQUALITY;

  /**
  * path to a template file for thumbnails
  * @var string $tempFileName
  */
  var $tempFileName = './border.jpg';

  /**
  * supported features in gd
  * @var string $imageTypes
  */
  var $imageTypes;

  /**
  * error messages
  * @var string $error
  */
  var $error;
  
  var $backgroundColor = array('r' => 255, 'g' => 255, 'b' => 255);

  /**
  * Constructor
  */
  function __construct() {
    if (function_exists('gd_info')) {
      $gdInfo = gd_info();
      $this->imageTypes = array(
        'gif_read' => $gdInfo['GIF Read Support'],
        'gif_write' => $gdInfo['GIF Create Support'],
        'png' => $gdInfo['PNG Support'],
        'jpg' => $gdInfo['JPG Support']
      );
    } elseif (function_exists('imagetypes')) {
      $imageTypes = imagetypes();
      $this->imageTypes = array(
        'gif_read' => $imageTypes & IMG_GIF,
        'gif_write' => $imageTypes & IMG_GIF,
        'png' => $imageTypes & IMG_PNG,
        'jpg' => $imageTypes & IMG_JPG
      );
    } else {
      $this->imageTypes = array(
        'gif_read' => FALSE,
        'gif_write' => FALSE,
        'png' => FALSE,
        'jpg' => FALSE
      );
    }
  }

  function base_thumbnail() {
    $this->__construct();
  }

  /**
  * check that the gd has support for the image type
  *
  * @param integer $type image type
  * @param boolean $readOnly Only read access needed
  * @return boolean
  */
  function checkTypeInGD($type, $readOnly = FALSE) {
    switch ($type) {
      case 1 :
        if ($readOnly) {
          if (!$this->imageTypes['gif_read']) {
            $this->setError(1, 'GIF (read) not supported.');
            return FALSE;
          }
        } elseif (!$this->imageTypes['gif_write']) {
          $this->setError(1, 'GIF (write) not supported.');
          return FALSE;
        }
        break;
      case 2 :
        if (!$this->imageTypes['jpg']) {
          $this->setError(1, 'JPEG not supported.');
          return FALSE;
        }
        break;
      case 3 :
        if (!$this->imageTypes['png']) {
          $this->setError(1, 'PNG not supported.');
          return FALSE;
        }
        break;
      default :
        $this->setError(1,'Unknown image file format.');
        return FALSE;
    }
    return TRUE;
  }

  /**
  * Calculates thumbnail size
  *
  * @param integer $orgWidth original width
  * @param integer $orgHeight original height
  * @param integer $mode resize mode (abs, max, min, mincrop)
  *
  * @return array array($newWidth, $newHeight)
  */
  function calcSize($orgWidth, $orgHeight, $mode = NULL) {
    if ($this->width < 1) {
      $this->width = $orgWidth;
    }
    if ($this->height < 1) {
      $this->height = $orgHeight;
    }
    $iMode = isset($mode) ? $mode : $this->resizeFlag;
    if ($iMode == 'abs' || $iMode == 'mincrop') {
        $newWidth = $this->width;
        $newHeight = $this->height;
    } else {
      $divWidth = $orgWidth / $this->width;
      $divHeight = $orgHeight / $this->height;
      if ($iMode == 'min') {
        if ($divWidth <= $divHeight) {
          $newWidth = $this->width;
          $newHeight = round($orgHeight / $divWidth);
        } else {
          $newHeight = $this->height;
          $newWidth = round($orgWidth / $divHeight);
        }
      } else {
        if ($divWidth >= $divHeight) {
          $newWidth = $this->width;
          $newHeight = round($orgHeight / $divWidth);
        } else {
          $newHeight = $this->height;
          $newWidth = round($orgWidth / $divHeight);
        }
      }
    }
    return array($newWidth,$newHeight);
  }

  /**
  * Es wird im Speicher eine Kopie der in den Parametern uebergebenen Datei angelegt und als Ergebnis zurueckgeliefert.
  *
  * @param string $fileName Dateiname der Bilddatei
  * @param integer $fileType Bildtyp der Datei
  * @return object Das im Speicher neu erzeugte Bild
  */
  function loadFile($fileName,$fileType) {
    $result = FALSE;
    switch ($fileType) {
      case 1 :
        $result = imagecreatefromGIF($fileName);
        break;
      case 2 :
        $result = imagecreatefromJPEG($fileName);
        break;
      case 3 :
        $result = imagecreatefromPNG($fileName);
        break;
    }
    return $result;
  }

  /**
  * saves an image
  *
  * @param object $im image resource
  * @param string $fileName target file name
  * @param integer $fileType target file type
  * @return mixed FALSE or absolute file name
  */
  function saveFile($im, $fileName, $fileType) {
    switch ($fileType) {
      case 1 :
        $saved = imageGIF($im, $this->thumbnailDirectory.$fileName);
        break;
      case 2 :
        $saved = imageJPEG($im, $this->thumbnailDirectory.$fileName, 
          $this->jpegQuality);
        break;
      case 3 :
        $saved = imagePNG($im, $this->thumbnailDirectory.$fileName);
        break;
      default :
        $this->setError(1,'Thumbnailformat wird nicht untersttzt.');
        return FALSE;
    }
    return $fileName;
  }

  /**
  * creates a filename for the thumbnail cache
  *
  * @param string $string
  * @param integer $width image width
  * @param integer $height image height
  * @param integer $imageType image file type
  * @param string $filter filter (resize mode)
  * @return string
  */
  function getFilename($string, $width, $height, $imageType, $filter = '') {
    switch ($imageType) {
      case 1  : $ext = '.gif'; break;
      case 2  : $ext = '.jpg'; break;
      default : $ext = '.png';
    }
    $fileName = substr($string, 0, strrpos($string, '.'));
    return $fileName.'_'.$filter.'_'.$width.'x'.$height.$ext;
  }

  /**
  * Convert memory amounts to bytes
  *
  * @param string $val
  * @access public
  * @return integer
  */
  function returnBytes($val) {
    $val = trim($val);
    $last = $val{strlen($val)-1};
    switch($last) {
    case 'k':
    case 'K':
      return (int) $val * 1024;
      break;
    case 'm':
    case 'M':
      return (int) $val * 1048576;
      break;
    default:
      return (int) $val;
    }
  }

  /**
  * create a new thubmail
  *
  * @param string $imageFileName source filename
  * @param string $thumbFileName destination filename (thumbnail)
  * @param integer $orgType source image type
  * @param integer $orgWidth orignal width
  * @param integer $orgHeight original height
  * @param integer $thumbWidth thumbnail width
  * @param integer $thumbHeight thumbnail height
  * @return mixed cache filename or FALSE
  */
  function createThumb($imageFileName, $thumbFileName, $orgType, 
      $orgWidth, $orgHeight, $thumbWidth, $thumbHeight) {
      
    $memoryNeeded = (($orgWidth * $orgHeight * 4) + 
      ($thumbWidth * $thumbHeight * 4)) * 1.5;
      
    if (function_exists('memory_get_usage')) {
      $memoryNeeded += memory_get_usage();
    } else {
      $memoryNeeded += 4194304;
    }
    if ($memoryAvail = @ini_get('memory_limit')) {
      $memoryAvail = $this->returnBytes($memoryAvail);
      if ($memoryAvail != -1 && $memoryAvail <  $memoryNeeded) {
        ini_set('memory_limit', $memoryNeeded);
      }
      $memoryAvail = $this->returnBytes(ini_get('memory_limit'));
    } else {
      $memoryAvail = 8388608; //8M
    }

    if ($memoryAvail == -1 || $memoryAvail >=  $memoryNeeded) {
      if ($im = $this->loadFile($this->baseDirectory.$imageFileName,$orgType)) {
        if (function_exists('imagecreatetruecolor')) {
          $thumb = @imagecreatetruecolor($thumbWidth,$thumbHeight);
        }
        if (!$thumb) {
          $thumb = imagecreate($thumbWidth,$thumbHeight);
        }
        if (isset($this->backgroundColor)) {
          $bgColorIdx = imagecolorallocate($thumb, 
            $this->backgroundColor['r'], 
            $this->backgroundColor['g'], 
            $this->backgroundColor['b']);
          imagefilledrectangle($thumb, 0, 0, $thumbWidth, $thumbHeight, $bgColorIdx);
        }
        if ($thumb) {
          if (function_exists('imagecopyresampled')) {
            if (!@imagecopyresampled($thumb, $im, 0, 0, 0, 0, 
                $thumbWidth, $thumbHeight, $orgWidth, $orgHeight)) {
              imagecopyresized($thumb, $im, 0, 0, 0, 0, 
                $thumbWidth, $thumbHeight, $orgWidth, $orgHeight);
            }
          } else {
            imagecopyresized($thumb, $im, 0, 0, 0, 0, 
              $thumbWidth, $thumbHeight, $orgWidth, $orgHeight);
          } 
          if (file_exists($this->tempFileName)) {
            list($tmpWidth, $tmpHeight, $tmpType) = 
              getimagesize($this->tempFileName);
            if ($tmp = $this->loadFile($this->tempFileName, $tmpType)) {
              $left = round(($tmpWidth - $thumbWidth) / 2);
              $top = round(($tmpHeight - $thumbHeight) / 2);
              imagecopy($tmp, $thumb, $left, $top, 0, 0, 
                $thumbWidth, $thumbHeight);
              imagedestroy($thumb);
              $thumb = $tmp;
            }
          }

          $saved = $this->saveFile($thumb, $thumbFileName, 
            $this->thumbnailType);
          imagedestroy($im);
          imagedestroy($thumb);
          return $saved;
        } else {
          $this->setError(1,'Thumbnail kann nicht erzeugt werden.');
          imagedestroy($im);
          return FALSE;
        }
      } else {
        $this->setError(1,'Bild kann nicht gelesen werden.');
        return FALSE;
      }
    } else {
      $this->setError(1,'Not enough memory avaliable.');
      return FALSE;
    }
  }

  /**
  * Delete all thumbnail files for a fileId
  *
  * @param $fileName media file id
  * @return boolean 
  */
  function deleteThumbs($fileId) {    
    $tmp = explode('.',$fileId);
    $thumbFileName = preg_quote($tmp[0]);
    if ($dh = opendir($this->thumbnailDirectory)) {
      while($file = readdir($dh)) {
        if (preg_match('/^'.$thumbFileName.'/', $file)) {
          unlink($this->thumbnailDirectory.'/'.$file);
        }
      }
      closedir($dh);
    }
  }
  
  
  /**
  * ??? helpme
  *
  * @param $fileName Dateiname
  * @return ??? helpme
  */
  function getThumb($fileName, $bgColor = '#FFFFFF') {
    $this->backgroundColor = $this->htmlToColor($bgColor);  
    if (file_exists($this->baseDirectory.$fileName)) {
      //load image infos
      list($orgWidth, $orgHeight, $orgType) = 
        @getimagesize($this->baseDirectory.$fileName);
      //image?
      if (!$orgType) {
        $this->setError(1, 'No image.');
        return FALSE;
      }
      //calculate size
      list($thumbWidth, $thumbHeight) = $this->calcSize($orgWidth, $orgHeight);

      //get thumbnail filename
      $thumbFileName = $this->getFilename($fileName, 
        $thumbWidth, $thumbHeight, $this->thumbnailType, $this->resizeFlag);

      //check cache
      if (file_exists($this->thumbnailDirectory.$thumbFileName)) {
        $this->setError(0, 'Thumbnail already exists.');
        //force create?
        if (!$this->createAll) {
          //use cached thumbnail
          return $thumbFileName;
        }
      }
      //check image file type
      if ($this->checkTypeInGD($orgType, TRUE)) {
        //create thumbnail
        if ($this->resizeFlag == 'mincrop') {
          list($scaledWidth,$scaledHeight) = 
            $this->calcSize($orgWidth, $orgHeight, 'min');
          $thumbFileMin = $this->getFilename($fileName, 
            $scaledWidth, $scaledHeight, $this->thumbnailType, 'min');
          $this->createThumb($fileName, $thumbFileMin, $orgType, 
            $orgWidth, $orgHeight, $scaledWidth, $scaledHeight);
          return $this->imageCrop($thumbFileMin, $thumbFileName, 
            $thumbWidth, $thumbHeight);
        } else {
          return $this->createThumb($fileName, $thumbFileName, $orgType, 
            $orgWidth, $orgHeight, $thumbWidth, $thumbHeight);
        }
      } else {
        return FALSE;
      }
    } else {
      $this->setError(1, 'File not found.');
      return FALSE;
    }
  }

  /**
  * set error message
  *
  * @param $level Level
  * @param $msg Message
  */
  function setError($level, $msg) {
    unset($this->error);
    $this->error = array('level' => $level, 'msg' => $msg);
  }

  /**
  * crop image
  *
  * @param string $srcFilename source
  * @param string $destFilename destination
  * @param integer $width Width
  * @param integer $height Height
  * @return boolean Success
  */
  function imageCrop($srcFilename, $destFilename, $width, $height) {
    if ($im = $this->loadFile($this->thumbnailDirectory.$srcFilename, 
        $this->thumbnailType)) {
      if (function_exists('imagecreateTRUEcolor')) {
        $thumb = @imagecreateTRUEcolor($width, $height);
      }
      if (!$thumb) {
        $thumb = imagecreate( $width, $height);
      }
      if ($thumb) {
        $orgWidth = imagesx($im);
        $orgHeight = imagesy($im);
        if ($orgWidth != $width) {
          $left = Round(($orgWidth - $width) / 2);
        } else {
          $left = 0;
        }
        if ($orgHeight != $height) {
          $top = Round(($orgHeight - $height) / 2);
        } else {
          $top = 0;
        }

        imagecopy($thumb, $im, 0, 0, $left, $top, $width, $height);

        $saved = $this->saveFile($thumb, $destFilename, $this->thumbnailType);
        imagedestroy($im);
        imagedestroy($thumb);
        return $saved;
      } else {
        $this->setError(1, 'Can not create thumbnail.');
        imagedestroy($im);
        return FALSE;
      }
    } else {
      $this->setError(1, 'can not read image.');
      return FALSE;
    }
  }
  
  function htmlToColor($htmlColor) {
		if (strpos($htmlColor, '#') === FALSE) {
			$offset = 0;
		} else {
			$offset = 1;
		}
		$result['r'] = hexdec(substr($htmlColor, $offset, 2));
		$result['g'] = hexdec(substr($htmlColor, $offset+2, 2));
		$result['b'] = hexdec(substr($htmlColor, $offset+4, 2));
		return $result;  
  }
}
?>
