<?php
/**
* Topic basic objekt
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_topic.php,v 1.31 2006/05/18 15:49:59 weinert Exp $
*/


/**
* Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');
/**
* Basic class check conditions
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');

/**
* file owner
*/
define('PERM_OWNER', 1);
/**
* file group
*/
define('PERM_GROUP', 2);
/**
* Alle rights
*/
define('PERM_ALL', 4);

/**
* Read permission
*/
define('PERM_READ', 0);
/**
* Write permission
*/
define('PERM_WRITE', 1);
/**
* Create permission
*/
define('PERM_CREATE', 2);


/**
* Topic basic objekt
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_topic extends base_db {
  /**
  * Paramter name for formulars
  */
  var $paramName = 'tt';
  /**
  * Table with pages (topics)
  * @var string $tableTopics
  */
  var $tableTopics = PAPAYA_DB_TBL_TOPICS;
  /**
  * Table with public pages (Topics)
  * @var string $tableTopics
  */
  var $tableTopicsPublic = PAPAYA_DB_TBL_TOPICS_PUBLIC;
  /**
  * Table with released version of topics
  * @var string $tableTopicsVersions
  */
  var $tableTopicsVersions = PAPAYA_DB_TBL_TOPICS_VERSIONS;
  /**
  * Table with language content for pages
  * @var string $tableTopicsTrans
  */
  var $tableTopicsTrans = PAPAYA_DB_TBL_TOPICS_TRANS;
  /**
  * Table with language content for page version
  * @var string $tableTopicsVersionsTrans
  */
  var $tableTopicsVersionsTrans = PAPAYA_DB_TBL_TOPICS_VERSIONS_TRANS;
  /**
  * Table with language content for public pages
  * @var string $tableTopicsPublicTrans
  */
  var $tableTopicsPublicTrans = PAPAYA_DB_TBL_TOPICS_PUBLIC_TRANS;
  /**
  * Table with Module classes
  * @var string $tableModules
  */
  var $tableModules = PAPAYA_DB_TBL_MODULES;
  /**
  * Table with views
  * @var string $tableViews
  */
  var $tableViews = PAPAYA_DB_TBL_VIEWS;
  /**
  * Links to boxes
  * @var string $tableBoxesLinks
  */
  var $tableBoxesLinks = PAPAYA_DB_TBL_BOXLINKS;
  /**
  * Table with users
  * @var string $tableAuthUser
  */
  var $tableAuthUser = PAPAYA_DB_TBL_AUTHUSER;
  /**
  * Table contains groups
  * @var string $tableAuthGroups
  */
  var $tableAuthGroups = PAPAYA_DB_TBL_AUTHGROUPS;
  /**
  * Table contains languages
  * @var string $tableLanguages
  */
  var $tableLanguages = PAPAYA_DB_TBL_LNG;

  /**
  * Topic-ID - number of entries
  * @var integer $topicId
  */
  var $topicId = 0;

  /**
  * Topic - all data as array
  * @var array $topic
  */
  var $topic;

  /**
  * Processing content-module
  * @var object base_content $moduleObj
  */
  var $moduleObj = FALSE;

  /**
  * Number of topic versons
  * @var integer $maxVersions
  */
  var $maxVersions = 200;

  /**
  * Storable in cache?
  * @var boolean $cachable
  */
  var $cacheable = TRUE;

  /**
  * current content language
  * @var array $currentLanguage
  */
  var $currentLanguage = NULL;

  /**
  * load topic basics
  *
  * @param $topicId
  * @access public
  * @return boolean
  */
  function loadOutput($topicId, $lngId) {
    $sql = "SELECT topic_id, prev, prev_path,
                    topic_mainlanguage,
                    topic_created, topic_modified,
                    box_useparent, meta_useparent, surfer_useparent
              FROM %s
              WHERE topic_id = %d";
    $params = array($this->tableTopics, $topicId);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->topicId = (int)$row['topic_id'];
        $this->topic = $row;
        if (isset($lngId)) {
          $this->loadTranslatedData($topicId, $lngId);
        }
        return true;
      }
    }
    return false;
  }
  
  function getLastValidateTime($topicId) {
    $sql = "SELECT MAX(topic_audited) 
              FROM %s
             WHERE topic_id = %d";
    $params = array($this->tableTopicsVersions, $topicId);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      return $res->fetchField();
    }
    return NULL;
  }

  function languageIdentToId($lngIdent) {
    $sql = "SELECT lng_id FROM %s WHERE lng_ident = '%s'";
    if ($res = $this->databaseQueryFmt($sql , array($this->tableLanguages, trim($lngIdent)))) {
      if ($row = $res->fetchRow()) {
        return (int)$row[0];
      }
    }
    return 0;
  }

  function loadCurrentLanguage($lngId) {
    $sql = "SELECT lng_ident, lng_id, lng_title, lng_short FROM %s WHERE lng_id = '%s'";
    if ($res = $this->databaseQueryFmt($sql , array($this->tableLanguages, trim($lngId)))) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->currentLanguage = $row;
        return TRUE;
      }
    }
    return FALSE;
  }

  function getViewId() {
    if (isset($this->topic) && $this->topic['TRANSLATION']) {
      return (int)$this->topic['TRANSLATION']['view_id'];
    } else {
      return 0;
    }
  }

  /**
  * Check path
  *
  * @access public
  */
  function checkPath() {
    if ($this->topic['prev'] == 0) {
      $result = ';';
    } elseif (preg_match_all('/\d+/',$this->topic['prev_path'], $regs, PREG_PATTERN_ORDER)) {
      $result = ';'.implode(';',$regs[0]).';';
    } else {
      $result = ';0;';
    }
    return $result;
  }

  /**
  * Get module output
  *
  * @param boolean $pageContent page output or short text
  * @param array $parseParams parase parameter from output filter
  *
  * @return string
  */
  function parseContent($pageContent = TRUE, $parseParams = NULL) {
    $result = FALSE;
    $this->moduleObj = &$this->createPluginObject(
      $this->topic['TRANSLATION']['module_path'].$this->topic['TRANSLATION']['module_file'],
      $this->topic['TRANSLATION']['module_class'],
      $this->topic['TRANSLATION']['module_guid'], $this,
      $this->topic['TRANSLATION']['topic_content']);
    if (isset($this->moduleObj) && is_object($this->moduleObj)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/papaya_parser.php');
      $parser = &new papaya_parser;
      $parser->tableTopics = $this->tableTopics;
      $parser->tableTopicsTrans = $this->tableTopicsTrans;
      if (!$pageContent) {
        if (method_exists($this->moduleObj,'getParsedTeaser')) {
          $result = sprintf('<subtopic no="%s" href="%s" author="%s %s" created="%s" createdRFC822="%s" published="%s" module="%s" guid="%s">'."\r\n",
             $this->topicId,
             $this->getWebLink((int)$this->topicId, NULL, NULL, NULL, NULL, $this->topic['TRANSLATION']['topic_title']),
             papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['givenname']),
             papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['surname']),
             date('Y-m-d H:i:s', $this->topic['topic_created']),
             date('D, d M Y H:i:s O', $this->topic['topic_created']),
             date('Y-m-d H:i:s', $this->topic['topic_modified']),
             get_class($this->moduleObj), $this->topic['TRANSLATION']['module_guid']);
          $result .= $this->moduleObj->getParsedTeaser($parseParams);
          $result .= '</subtopic>';
        }
      } else {
        $str = $this->moduleObj->getParsedData($parseParams);
        if (FALSE !== $str) {
          $result = sprintf('<topic no="%s" href="%s" author="%s %s" created="%s" published="%s" audited="%s" module="%s" guid="%s">'.LF,
            $this->topicId,
            $this->getWebLink((int)$this->topicId, NULL, NULL, NULL, NULL, $this->topic['TRANSLATION']['topic_title']),
            papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['givenname']), 
            papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['surname']),
            date('Y-m-d H:i:s', @(int)$this->topic['topic_created']),
            date('Y-m-d H:i:s', @(int)$this->topic['topic_modified']),
            (($audited = $this->getLastValidateTime((int)$this->topicId)) > 0) ? date('Y-m-d H:i:s', $audited) : '',
            get_class($this->moduleObj), $this->topic['TRANSLATION']['module_guid']);
          $result .= $parser->parse($str, $this->getContentLanguageId());
          $result .= '</topic>'.LF;
          if ($str = $parser->getParsedData()) {
            $result .= '<parser>';
            $result .= $str;
            $result .= '</parser>';
          }
          if (!$this->moduleObj->cacheable) {
            $this->cacheable = FALSE;
          }
        }
      }
      unset($parser);
      unset($this->moduleObj);
    }
    return $result;
  }
  
  function loadTranslationsData($currentLngId = 0) {
    $this->topicTranslations = NULL;
    $sql = "SELECT tt.topic_id, tt.lng_id, tt.topic_title, 
                   l.lng_short, l.lng_title, l.lng_ident
              FROM %s AS tt, %s AS l
             WHERE tt.topic_id = %d AND tt.lng_id = l.lng_id
             ORDER BY l.lng_title";
    $params = array($this->tableTopicsTrans, $this->tableLanguages, (int)$this->topicId);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->topicTranslations[$row['lng_id']] = $row;
      }
    }
    return $this->topicTranslations;
  }
  
  function getTranslationsData($currentLngId = 0) {
    $result = '';
    if ($this->topicId && ($translations = $this->loadTranslationsData($currentLngId))) {
      foreach($translations as $translation) {
        $selected = ($translation['lng_id'] == $currentLngId) ? ' selected="selected"' : '';
        $href = $this->getWebLink($this->topicId, $translation['lng_ident'], NULL, NULL, NULL, $translation['topic_title']);
        $result .= sprintf('<translation lng_short="%s" lng_title="%s" href="%s"%s>%s</translation>',
          papaya_strings::escapeHTMLChars($translation['lng_short']),
          papaya_strings::escapeHTMLChars($translation['lng_title']),
          $href, $selected, 
          papaya_strings::escapeHTMLChars($translation['topic_title'])
        );
      }
    }
    return $result;
  }

  function loadMetaData() {
    if ($this->topicId) {
      $metaDate = date('Y-m-d\TH:i:s+0200', $this->topic['topic_modified']);
      if (!$this->topic['meta_useparent']) {
        $metaTitle = @$this->topic['TRANSLATION']['meta_title'];
        $metaKeywords = @$this->topic['TRANSLATION']['meta_keywords'];
        $metaDescr = @$this->topic['TRANSLATION']['meta_descr'];
      } else {
        $previousIds = False;
        $metaTitle = False;
        $metaKeywords = False;
        $metaDescr = False;
        if (preg_match_all('/\d+/',$this->topic['prev_path'], $matches, PREG_PATTERN_ORDER)) {
          $previousIds = $matches[0];
        }
        if ($this->topic['prev'] > 0) {
          $previousIds[] = $this->topic['prev'];
        }
        if (isset($previousIds) && is_array($previousIds)) {
          $previousIds = array_unique($previousIds);
          if (count($previousIds) > 1) {
            $filter = ' IN ('.implode(',', $previousIds).') ';
          } else {
            $filter = ' = '.(int)$previousIds[0].' ';
          }
          $sql = "SELECT t.topic_id, tt.lng_id, tt.meta_keywords, tt.meta_descr, tt.meta_title
                    FROM %s t
                    LEFT OUTER JOIN %s AS tt ON (tt.topic_id = t.topic_id AND tt.lng_id = %d)
                    WHERE t.topic_id $filter
                      AND t.meta_useparent = 0";
          unset($rows);
          $params = array($this->tableTopics, $this->tableTopicsTrans, $this->currentLanguage['lng_id']);
          if ($res = $this->databaseQueryFmt($sql,$params)) {
            while (0 === $res->fetchInto($row,DB_FETCHMODE_ASSOC)) {
              $rows[$row['topic_id']] = $row;
            }
          }
          if (isset($rows) && is_array($rows)) {
            for($i = count($previousIds); $i > 0; $i--) {
              if (isset($previousIds[$i]) && isset($rows[$previousIds[$i]])) {
                $no = $previousIds[$i];
                $metaKeywords = $rows[$no]['meta_keywords'];
                $metaDescr = $rows[$no]['meta_descr'];
                $metaTitle = $rows[$no]['meta_title'];
                break;
              }
            }
          }
        }
      }
      $result = array(
        'meta_date' => $metaDate,
        'meta_keywords' => $metaKeywords,
        'meta_descr' => $metaDescr,
        'meta_title' => $metaTitle,
        'meta_language' => $this->currentLanguage['lng_short']
      );
      return $result;
    }
    return FALSE;
  }

  /**
  * Metainformationen fr den aktuellen Topic
  *
  * @return string XML
  */
  function getMetaInfos() {
    $result = '';
    if ($this->topicId) {
      $data = $this->loadMetaData();
      $result .= "<metatags>\n";
      if (isset($data['meta_title']) && $data['meta_title'] != '') {
        $result .= sprintf('<pagetitle>%s</pagetitle>'.LF, papaya_strings::escapeHTMLChars($data['meta_title']));
      }
      if (isset($data['meta_language'])) {
        $result .= sprintf('<language>%s</language>'.LF, papaya_strings::escapeHTMLChars($data['meta_language']));
      }
      $result .= sprintf('<metatag type="%s">%s</metatag>'.LF, 'date', $data['meta_date']);
      if (isset($data['meta_keywords']) && $data['meta_keywords'] != '') {
        $result .= sprintf('<metatag type="%s">%s</metatag>'.LF, 'keywords', papaya_strings::escapeHTMLChars($data['meta_keywords']));
      }
      if (isset($data['meta_descr']) && $data['meta_descr'] != '') {
        $result .= sprintf('<metatag type="%s">%s</metatag>'.LF, 'description', papaya_strings::escapeHTMLChars($data['meta_descr']));
      }
      $result .= "</metatags>\n";
    }
    return $result;
  }

  /**
  * Check if entry is $id child
  *
  * @param integer $id
  * @return boolean
  */
  function hasParent($id) {
    if ($id == 0) return TRUE;
    if (preg_match_all('/\d+/',$this->topic['prev_path'], $regs, PREG_PATTERN_ORDER)) {
      $ids = array_flip($regs[0]);
    } else {
      $ids = array();
    }
    if (isset($ids[$id]) || ((int)$this->topic['prev'] == $id) || ($this->topicId === $id) || ($id == 0)) {
      $result = TRUE;
    } else {
      $result = FALSE;
    }
    return $result;
  }

  /**
  * Abstand zum Startknoten ermitteln
  *
  * @param integer $startId
  * @return integer
  */
  function getLevel($startId) {
    if ($startId == $this->topicId) {
      return 0;
    }
    if ($this->topic['prev']) {
      $path = ";0".$this->topic['prev_path'].";".$this->topic['prev'].";";
    } else {
      $path = ";0;";
    }
    $path = str_replace(";;",";",$path);
    $path = substr($path,strpos($path,";$startId;"));
    $elements = explode(";",$path);
    $counted = count($elements)-2;
    return $counted;
  }

  /**
  * Detect parent ID
  * @param integer $level level higher
  */
  function getParentID($level=0) {
    if ($level<=0) {
      return $this->topicId;
    } elseif ($level==1) {
      return $this->topic['prev'];
    } else {
      if (preg_match_all('#\d+#', $this->topic['prev_path'], $regs, PREG_PATTERN_ORDER)) {
        $ups = $regs[0];
      } else {
        return 0;
      }
      $idx = count($ups)-$level+1;
      return $ups[$idx];
    }
  }

  /**
  * Get parent id from master
  *
  * @param integer $masterNode id of given node
  * @param integer $level optional, default value 0
  * @access public
  * @return string
  */
  function getParentIDFromMaster($masterNode,$level=0) {
    $result = $masterNode;
    if ($level>0) {
      $pathList = explode(" ",trim(strtr($this->topic['prev_path'],";"," ")));
      $pathList[] = $this->topic['prev'];
      $pathList[] = $this->topicId;
      if (isset($pathList) && is_array($pathList)) {
        foreach($pathList as $key=>$val) {
          if ($val == $masterNode) {
            $idx = $key+$level;
            $result = $pathList[$idx];
            break;
          }
        }
      }
    }
    return $result;
  }

  /**
  * Get box parent
  *
  * @access public
  * @return mixed
  */
  function getBoxParent() {
    if (preg_match_all('/\d+/',@(string)$this->topic['prev_path'], $matches, PREG_PATTERN_ORDER)) {
      $previousIds = $matches[0];
    }
    if (isset($this->topic['prev']) && $this->topic['prev'] > 0) {
      $previousIds[] = $this->topic['prev'];
    }
    if (isset($previousIds) && is_array($previousIds) && count($previousIds)>1) {
      $filter = "topic_id in ('".implode("','",$previousIds)."')";
    } elseif (isset($previousIds) && is_array($previousIds)) {
      $filter = "topic_id = '{$previousIds[0]}'";
    } else {
      $filter = "topic_id = '0'";
    }
    $sql = "SELECT topic_id FROM %s
            WHERE box_useparent = 0
              AND $filter
            ORDER BY topic_id DESC";
    unset($rows);
    $params = array($this->tableTopics);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $rows[$row['topic_id']] = $row;
      }
    }
    if (isset($rows) && is_array($rows)) {
      for($i = count($previousIds); $i > 0; $i--) {
        if (isset($previousIds[$i])) {
          $no = $previousIds[$i];
        } else {
          $no = 0;
        }
        if (isset($rows[$no])) {
          return $no;
        }
      }
    }
    return FALSE;
  }

  /**
  * Get boxes topic id
  *
  * @access public
  * @return integer
  */
  function getBoxesTopicID() {
    if ($this->topic['box_useparent']) {
      return $this->getBoxParent();
    }  else {
      return $this->topicId;
    }
  }

  /**
  * Get surfer permission ids
  *
  * @access public
  * @return mixed
  */
  function getSurferPermIDs() {
    if (preg_match_all('/\d+/',$this->topic['prev_path'], $matches, PREG_PATTERN_ORDER)) {
      $previousIds = $matches[0];
    }
    if ($this->topic['prev'] > 0) {
      $previousIds[] = $this->topic['prev'];
    }
    if ($this->topic['surfer_useparent'] != 1) {
      if ($this->topic['surfer_useparent'] > 2) {
        $perms = $this->topic['surfer_permids'];
      } else {
        $perms = '';
      }

      if (isset($previousIds) && is_array($previousIds)) $previousIds = array_unique($previousIds);
      if (isset($previousIds) && is_array($previousIds) && count($previousIds)>1) {
        $filter = "topic_id in ('".implode("','",$previousIds)."')";
      } elseif (isset($previousIds[0])) {
        $filter = "topic_id = '".(int)$previousIds[0]."'";
      } else {
        $filter = "topic_id = '0'";
      }
      $sql = "SELECT topic_id, surfer_useparent, surfer_permids FROM %s
              WHERE (NOT(surfer_useparent = 2))
                AND $filter
              ORDER BY topic_id DESC";
      unset($rows);
      if ($res = $this->databaseQueryFmt($sql,array($this->tableTopics))) {
        while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $rows[$row['topic_id']] = $row;
        }
      }
      if (isset($rows) && is_array($rows)) {
        for($i = count($previousIds)-1; $i >= 0; $i--) {
          $no = $previousIds[$i];
          if (isset($rows[$no]) && is_array($rows[$no])) {
            $perms .= ';'.$rows[$no]['surfer_permids'];
            if ($rows[$no]['surfer_useparent'] == 1) {
              break;
            }
          }
        }
      }
    } else {
      $perms = $this->topic['surfer_permids'];
    }
    if (preg_match_all('/\d+/',$perms, $matches, PREG_PATTERN_ORDER)) {
      $permIds = array_unique($matches[0]);
      return $permIds;
    }
    return FALSE;
  }


  /**
  * Topic exists
  *
  * @param $id
  * @access public
  * @return mixed
  */
  function topicExists($id) {
    $sql = "SELECT COUNT(topic_id) FROM %s WHERE topic_id = %d";
    if ($res = $this->databaseQueryFmt($sql,array($this->tableTopics,(int)$id))) {
      if ($row = $res->fetchRow()) {
        return ($row[0] > 0);
      }
    }
    return FALSE;
  }

  /**
  * Delete cache
  *
  * @access public
  */
  function deleteCache() {
    $page = $this->topicId;
    if (file_exists(PAPAYA_PATH_CACHE) && (trim(PAPAYA_PATH_CACHE) != '')) {
      if ($dh = @opendir(PAPAYA_PATH_CACHE)) {
        while($file = @readdir($dh)) {
          if (preg_match('#^\.'.(int)$page.'_\d+_[a-z]{0,4}_[a-f\d]{32}\.([a-z]+)(\.gz)?$#i', $file)) {
            @unlink(PAPAYA_PATH_CACHE.$file);
          }
        }
        @closedir($dh);
      }
    }
  }

  function getContentLanguageId() {
    if (isset($this->topic)) {
      if (isset($this->topic['TRANSLATION'])) {
        return (int)$this->topic['TRANSLATION']['lng_id'];
      } elseif (isset($this->topic['topic_mainlanguage']) && $this->topic['topic_mainlanguage'] > 0) {
        return (int)$this->topic['topic_mainlanguage'];
      }
    }
    return (int)PAPAYA_CONTENT_LANGUAGE;
  }
}

?>