<?php
/**
* Topic basic objekt
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_topic_edit.php,v 1.80 2006/05/26 12:21:45 weinert Exp $
*/


/**
* Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_topic.php');



/**
* Topic basic objekt
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_topic_edit extends base_topic {
  /**
  * use richtext editor
  * @var boolean
  */
  var $useRichtext = TRUE;

  /**
  * language selector
  * @var base_language_select object $lngSelect
  */
  var $lngSelect = NULL;

  /**
  * @var string $infoSheetViews informational message about views, set in initialize()
  */
  var $infoSheetViews = '<sheet><![CDATA[&nbsp;]]</sheet>';

  /**
  * Surfer permissions table name
  * @var string $tableSurferPerm
  */
  var $tableSurferPerm = PAPAYA_DB_TBL_SURFERPERM;

  /**
  * Initialize for parameters
  *
  * @param mixed $id optional, default value NULL
  * @access public
  */
  function initialize($id = NULL) {
    $this->sessionParamName = 'PAPAYA_SESS_'.$this->paramName;
    $this->initializeParams();

    $this->sessionParams = $this->getSessionValue($this->sessionParamName);

    if (isset($this->params['page_id']) && ($this->params['page_id'] > 0)) {
      $this->sessionParams['page_id'] = (int)$this->params['page_id'];
    } elseif (isset($id) && ($id > 0)) {
      $this->params['page_id'] = (int)$id;
      $this->sessionParams['page_id'] = (int)$this->params['page_id'];
    } elseif (isset($this->sessionParams['page_id'])) {
      $this->params['page_id'] = (int)$this->sessionParams['page_id'];
    } else {
      $this->params['page_id'] = 0;
    }

    $this->initializeSessionParam('mode');

    if (isset($this->params['open_mod']) && preg_match('/[a-z\d]{32}/',$this->params['open_mod'])) {
      $this->sessionParams['openmods'][$this->params['open_mod']] = TRUE;
    } elseif (isset($this->params['close_mod']) && preg_match('/[a-z\d]{32}/',$this->params['close_mod'])) {
      unset($this->sessionParams['openmods'][$this->params['close_mod']]);
    }

    $this->initializeSessionParam('version_datetime');
    if (isset($this->params['version_date']) && isset($this->params['version_time']) &&
        $this->params['version_date'] != '' && $this->params['version_time'] != '') {
      $versionDate = explode('-', $this->params['version_date']);
      $versionTime = explode(':', $this->params['version_time']);
      $versionDateTime = @mktime($versionTime[0],$versionTime[1],$versionTime[2],$versionDate[1],$versionDate[2],$versionDate[0]);
      $this->params['version_datetime'] = $versionDateTime;
      $this->sessionParams['version_datetime'] = $versionDateTime;
    }

    $useRichtext = $this->getSessionValue('PAPAYA_SESS_USE_RICHTEXT');
    $this->useRichtext = (isset($useRichtext)) ? (bool)$useRichtext : TRUE;

    $this->setSessionValue($this->sessionParamName, $this->sessionParams);

    $this->infoSheetViews = '<sheet width="100%" align="center"><text><div style="padding: 10px;"><noescape><![CDATA[';
    $this->infoSheetViews .= $this->_gtfile('viewselect.txt');
    $this->infoSheetViews .= ']]></noescape></div></text></sheet>';

  }

  function execute() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_language_select.php');
    $this->lngSelect = &base_language_select::getInstance();
    $this->layout->addLeft($this->lngSelect->getComboFormXML());

    $this->load($this->params['page_id'], $this->lngSelect->currentLanguageId);
    $this->currentLanguage = $this->lngSelect->currentLanguage;

    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $this->menubar = &new base_btnbuilder;
    $this->menubar->images = &$this->images;

    switch (@$this->params['cmd']) {
      case 'add_translation' :
        if ($this->hasPermUser(PERM_WRITE, $this->authUser) && $this->editable($this->authUser)) {
          if (isset($this->params['lng_id']) && $this->params['lng_id'] > 0 &&
              $this->lngSelect->currentLanguageId == $this->params['lng_id'] &&
              isset($this->lngSelect->languages[(int)$this->params['lng_id']])) {
            if ($this->createTranslation($this->lngSelect->currentLanguageId)) {
              $this->loadTranslatedData($this->topicId, $this->lngSelect->currentLanguageId);
              $this->currentLanguage = $this->lngSelect->currentLanguage;
            }
          }
        }
        break;
      case 'add_topic' :
        if (($this->authUser->hasPerm(1) && ($this->hasPermUser(PERM_CREATE, $this->authUser))) &&
            //User darf in diesem Zweig anlegen
            (($this->authUser->startNode == 0) || ($this->hasParent($this->authUser->startNode))) &&
            //User darf bis in diese Tiefe arbeiten
            (($this->getLevel($this->authUser->startNode) < $this->authUser->subLevel) ||
              ($this->authUser->subLevel == 0))) {
            if ($newId = $this->create()) {
              $this->params['page_id'] = $newId;
              $this->initializeSessionParam('page_id', array('mode', 'cmd'));
              $this->load($this->params['page_id'], $this->lngSelect->currentLanguageId);
            }
          }
          break;
        case 'switch_richtext' :
          $this->useRichtext = @(bool)$this->params['switch_richtext'];
          $this->setSessionValue('PAPAYA_SESS_USE_RICHTEXT', $this->useRichtext);
          $protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ? 'https' : 'http';
          //Zurck auf Los
          $toUrl = $protocol."://".$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'];
          header("Location: $toUrl");
          exit;
      case 'del_trans' :
        if (isset($this->params['del_trans_confirm']) && $this->params['del_trans_confirm']) {
          foreach($this->params['del_trans_language'] as $lng) {
            if ($this->deleteTrans($lng)) {
              $this->addMsg(MSG_INFO, 'Language version of page deleted.');
              $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
                sprintf('Language version of page "%s (%s)" deleted.',
                  papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), $this->topicId));
            } else {
              $this->addMsg(MSG_ERROR, 'Couldn\'t delete language version %s of page.');
            }
          }
        } else {
          $this->layout->add($this->getDelTransForm());
        }
      break;
    }

    if (!(($this->topicId) && ($this->hasParent($this->authUser->user['start_node'])))) {
      if ($this->params['page_id'] != $this->authUser->user['start_node'] && (!@$this->params['redirected'])) {
        $protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ? 'https' : 'http';
        //Zurck auf Los
        $toUrl = $protocol."://".$_SERVER['HTTP_HOST'].$this->getBasePath().
          $this->getLink(array('redirected'=>1, 'page_id'=>$this->authUser->user['start_node']));
        header("Location: $toUrl");
        exit;
      }
      //jo spielen schon nur is hier nix
      $this->addMsg(MSG_INFO,$this->_gt('Please select a page.'));
      $this->menubar->addButton('New page',
        $this->getLink(array('cmd'=>'add_topic', 'mode'=>0, 'page_id' => $this->topicId)),
        70, 'Create a new page');
    } elseif (!($this->hasPermUser(PERM_WRITE, $this->authUser) && $this->editable($this->authUser))) {
      $this->addMsg(MSG_ERROR,$this->_gt('You have no edit permissions for this page.'));
      switch ($this->params['mode']) {
        case 5 :
          if (@$this->params['version_datetime'] > 0) {
            $linkParams = array('p_date'=>@(int)$this->params['version_datetime']);
          } else {
            $linkParams = NULL;
          }
          $this->layout->add(
            $this->getContentFrame('../'.$this->getWebLink($this->topicId, '', 'preview', $linkParams), $this->_gt('Preview'))
          );
          break;
        default :
          $this->layout->add($this->getPropertiesReadOnly());
          $this->layout->addRight($this->getInformation());
      }
      $this->getToolbar();
    } elseif (isset($this->topic['TRANSLATION']) && is_array($this->topic['TRANSLATION'])) {
      $this->setMenuBar(TRUE);
      switch (@$this->params['mode']) {
        case 1: //edit content
          if ($str = $this->getEditContent()) {
            $this->layout->add($str);
          } else {
            $this->addMsg(MSG_WARNING, 'No edit dialog for this module.');
          }
          break;
        case 2 :  //boxes
          if ($this->authUser->hasPerm(14)) {
            include_once(PAPAYA_INCLUDE_PATH.'system/papaya_boxeslinks.php');
            $boxLinks = &new papaya_boxeslinks($this);
            $boxLinks->msgs = &$this->msgs;
            $boxLinks->authUser = &$this->authUser;
            $boxLinks->images = &$this->images;
            $boxLinks->initialize();

            if (@(bool)$this->params['chgusep']) {
              if ($this->saveBoxUseParent()) {
                $this->load($this->topicId, $this->lngSelect->currentLanguageId);
              }
            }
            $this->layout->addRight($this->getEditUseParentForm($this->getBoxParent()));
            if (!$this->topic['box_useparent']) {
              $boxLinks->topicId = $this->topicId;
              $boxLinks->loadList();
              $boxLinks->loadBoxList();
              $boxLinks->loadBoxGroupList();
              $boxLinks->execute();
              $this->layout->add($boxLinks->getList($this->images, TRUE, $this->_gt('Linked boxes')));
              $this->layout->add($boxLinks->getBoxList($this->images));
            } elseif ($boxLinks->topicId = $this->getBoxParent()) {
              $boxLinks->loadList();
              $boxLinks->loadBoxList();
              $boxLinks->loadBoxGroupList();
              $this->layout->add($boxLinks->getList($this->images, FALSE, $this->_gt('Inherited boxes')));
            }
          } else {
            $this->addMsg(MSG_WARNING,"You have no permissions to edit box links.");
          }
          break;
        case 5:  // preview
          if (@$this->params['version_datetime'] > 0) {
            $linkParams = array('p_date'=>@(int)$this->params['version_datetime']);
          } else {
            $linkParams = NULL;
          }
          $lngIdent = $this->lngSelect->currentLanguage['lng_ident'];
          $this->layout->add(
            $this->getContentFrame('../'.$this->getWebLink($this->topicId, $lngIdent, 'preview', $linkParams), $this->_gt('Preview'))
          );
          $this->menubar->addSeperator();
          $this->menubar->addButton('Full window', '../'.$this->getWebLink($this->topicId,
              $lngIdent, 'preview', $linkParams), 92, 'Full window preview');
          $this->menubar->addButton('XML preview', '../'.$this->getWebLink($this->topicId,
              $lngIdent, 'xmlpreview', $linkParams), 103, 'XML output preview');
          break;
        case 4: // surfer permissions
          if ($this->authUser->hasPerm(2,'88236ef1454768e23787103f46d711c2')) {
            include_once(PAPAYA_INCLUDE_PATH.'system/base_surferlinks.php');
            $sfl = &new base_surferlinks($this->topicId);
            $sfl->msgs = &$this->msgs;
            $sfl->images = &$this->images;
            $sfl->execute();
            $this->layout->add($sfl->get($this->lngSelect->currentLanguageId));
            $this->layout->addRight($sfl->getModeDlg());
          } else {
            $this->addMsg(MSG_WARNING,"You have no permissions to edit access permissions");
          }
          break;
        case 6 : //view and module
          if (isset($this->params['cmd']) && $this->params['cmd'] == 'chg_view') {
            if ($this->checkEditView()) {
              if ($this->saveView()) {
                $this->loadTranslatedData($this->topicId, $this->lngSelect->currentLanguageId);
                $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('View')));
              }
            }
          }
          $this->layout->add($this->getEditView());
          $this->layout->addRight($this->getInformation());
          $this->layout->addRight($this->infoSheetViews);
          break;
        case 7: //versions
          if ($this->authUser->hasPerm(24)) {
            $this->loadVersionsList();

            if (isset($this->params['version_id']) && $this->loadVersion($this->params['version_id'])) {
              $this->layout->add($this->versionExecute($this->msgs, $this->authUser));
              $this->loadVersionTranslatedData($this->params['version_id'], $this->lngSelect->currentLanguageId);
              $this->loadVersionTranslationsInfo();
            }

            $this->loadVersionsList();

            $this->menubar->addSeperator();
            $this->getPagesNav($this->menubar, 10, $this->versionsCount, 'version_offset',
                array('page_id'=>$this->topicId), 'Versions');


            if (isset($this->savedVersion) && is_array($this->savedVersion)) {
              $this->getVersionInfos();
            }
            $this->layout->add($this->getVersionsList());
          }
          break;
        case 8: //editor permissions
          if ($this->authUser->hasPerm(29)) {
            include_once(PAPAYA_INCLUDE_PATH.'system/papaya_user.php');
              $editUser = &new papaya_user();
            $editUser->initialize('edit_usr');

            $editUser->loadGroups();
            $editUser->loadUsers();
            $editUser->loadGroupTree();
            $editUser->load($this->topic['author_id']);

            $this->changePermExecute($editUser);
            $this->layout->add($this->getEditPerm($editUser));
            $this->layout->addRight($this->getEditUser($editUser));
          }
          break;
        case 9:
          $this->loadTranslationsInfo();
          $this->layout->add($this->publishExecute($this->msgs, $this->authUser));
          $this->load($this->topicId, $this->lngSelect->currentLanguageId);
          $this->loadTranslationsInfo();
          $this->layout->add($this->getPublicData());
          break;
        default :
          $this->execPageActions();
          $this->setMenuBar(TRUE);
          $this->layout->add($this->getPropertiesDialog());
          $this->layout->addRight($this->getInformation());
      }
      $this->getToolbar();
    } else {
      //Show a dialog to create a translation
      $this->setMenuBar(TRUE);
      $this->execPageActions();
      $this->layout->add($this->addTranslationDialog());
      $this->layout->add($this->getPropertiesDialog());
      $this->layout->addRight($this->getInformation());
    }

    include_once(PAPAYA_INCLUDE_PATH.'system/papaya_topic_tree.php');
    $topicTree = &new papaya_topic_tree('ptt');
    $topicTree->images = &$this->images;
    $topicTree->msgs = &$this->msgs;
    $topicTree->layout = &$this->layout;
    $topicTree->authUser = &$this->authUser;
    $topicTree->paramName = 'tt';
    if (@$this->params['mode'] == 5) {
      $this->addVersionPreviewDialog();
    }
    $this->layout->addLeft($topicTree->getPartTree($this));
    $this->addTopicIdDialog();
    $this->addSwitchRichtextDialog();

    if ($str = $this->menubar->getXML()) {
      $this->layout->addMenu(sprintf('<menu ident="%s">%s</menu>'.LF, 'edit2', $str));
    }

    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  function execPageActions() {
    switch (@$this->params['cmd']) {
      case 'save_properties' :
        $this->initializePropertiesDialog();
        if ($this->dialogProperties->checkDialogInput()) {
          if ($this->saveProperties()) {
            unset($this->dialogProperties);
            $this->load($this->topicId, $this->lngSelect->currentLanguageId);
            $this->addMsg(MSG_INFO,$this->_gt('Changes saved.'));
          } else {
            $this->addMsg(MSG_WARNING,$this->_gt('Database error! Changes not saved.'));
          }
        }
        break;
      case 'del_topic' :
        if (isset($this->params['del_topic_confirm']) && isset($this->topic)) {
          $prev = @(int)$this->topic['prev'];
          if ($this->topic['is_deleted']) {
            if ($this->destroy()) {
              $this->addMsg(MSG_INFO,$this->_gtf('Page #%d deleted.', $this->topicId),
                TRUE, PAPAYA_LOGTYPE_PAGES);
              unset($topic);
              $this->params['page_id'] = $prev;
              $this->initializeSessionParam('page_id', array('cmd','mode'));
              $this->load($this->params['page_id'], $this->lngSelect->currentLanguageId);
            } else {
              $this->addMsg(MSG_ERROR,$this->_gt('Can not delete this page.'));
            }
          } elseif ($this->delete()) {
            $this->addMsg(MSG_INFO,$this->_gtf('Page #%d moved to trash.', $this->topicId), TRUE, PAPAYA_LOGTYPE_PAGES);
            unset($topic);
            $this->params['page_id'] = $prev;
            $this->initializeSessionParam('page_id', array('cmd','mode'));
            $this->load($this->params['page_id'], $this->lngSelect->currentLanguageId);
          } else {
            $this->addMsg(MSG_ERROR,$this->_gt('Can not delete this page.'));
          }
        } else {
          $this->layout->add($this->getDelForm());
        }
      break;
    }
  }

  function loadTranslationsInfo() {
    unset($this->topic['TRANSLATIONINFOS']);
    $sql = "SELECT tt.topic_id, tt.lng_id, tt.topic_trans_modified,
                    tt.topic_title, ttp.topic_trans_modified as topic_trans_published
              FROM %s AS tt
              LEFT OUTER JOIN %s AS ttp ON (ttp.topic_id = tt.topic_id AND ttp.lng_id = tt.lng_id)
              WHERE tt.topic_id = %d";
    $params = array($this->tableTopicsTrans, $this->tableTopicsPublicTrans, (int)$this->topicId);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->topic['TRANSLATIONINFOS'][$row['lng_id']] = $row;
      }
    }
  }

  /**
  * Get toolbar
  *
  * @access public
  */
  function getToolbar() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $toolbar = &new base_btnbuilder;
    $toolbar->images = &$this->images;

    if ($this->hasPermUser(PERM_WRITE, $this->authUser) && $this->editable($this->authUser)) {
      $toolbar->addButton('Properties',
        $this->getLink(array('cmd'=>'chg_mode', 'mode'=>0, 'page_id' => $this->topicId)),
        44, '', @($this->params['mode'] == 0));
      $toolbar->addButton('Module',
        $this->getLink(array('cmd'=>'chg_mode', 'mode'=>6, 'page_id' => $this->topicId)),
        3, 'Select module and view', @($this->params['mode'] == 6));
      $toolbar->addButton('Content',
        $this->getLink(array('cmd'=>'chg_mode', 'mode'=>1, 'page_id' => $this->topicId)),
        45, 'Edit content', @($this->params['mode'] == 1));
      if ($this->authUser->hasPerm(14)) {
        $toolbar->addButton('Boxes',
          $this->getLink(array('cmd'=>'chg_mode', 'mode'=>2, 'page_id' => $this->topicId)),
          5, 'Connect boxes', @($this->params['mode'] == 2));
      }
      $toolbar->addButton('Preview',
        $this->getLink(array('cmd'=>'chg_mode', 'mode'=>5, 'page_id' => $this->topicId)),
        92, 'Page preview', @($this->params['mode'] == 5));
      $toolbar->addButton('Publish',
        $this->getLink(array('cmd'=>'chg_mode', 'mode'=>9, 'page_id' => $this->topicId)),
        16, 'Publish Page', @($this->params['mode'] == 9));

      $toolbar->addSeperator();
      if ($this->authUser->hasPerm(2,'88236ef1454768e23787103f46d711c2')) {
        $toolbar->addButton('Access',
          $this->getLink(array('cmd'=>'chg_mode', 'mode'=>4, 'page_id' => $this->topicId)),
          1, 'Restrict access', @($this->params['mode'] == 4));
      }
      $toolbar->addSeperator();
      if ($this->authUser->hasPerm(24)) {
        $toolbar->addButton('Versions',
          $this->getLink(array('cmd'=>'chg_mode', 'mode'=>7, 'page_id' => $this->topicId)),
          74, 'Version management', @($this->params['mode'] == 7));
      }
      if ($this->authUser->hasPerm(29)) {
        $toolbar->addButton('Permissions',
          $this->getLink(array('cmd'=>'chg_mode', 'mode'=>8, 'page_id' => $this->topicId)),
          25, 'Edit permissions', @($this->params['mode'] == 8));
      }
    } elseif ($this->hasPermUser(PERM_READ, $this->authUser)) {
      $toolbar->addButton('Properties',
        $this->getLink(array('cmd'=>'chg_mode', 'mode'=>0, 'page_id' => $this->topicId)),
        44, '', @($this->params['mode'] != 5));
      $toolbar->addButton('Preview',
        $this->getLink(array('cmd'=>'chg_mode', 'mode'=>5)),
        92, 'Page preview', @($this->params['mode'] == 5));
    }
    if ($str = $toolbar->getXML()) {
      $this->layout->add('<toolbar>'.$str.'</toolbar>', 'toolbars');
    }
  }

  /**
  * Set menu bar
  *
  * @param boolean $editable optional, default value TRUE
  * @access public
  */
  function setMenuBar($editable = TRUE) {
    $this->menubar->clear();
    if ($this->hasPermUser(PERM_CREATE, $this->authUser, $this->topic['prev'])) {
      $this->menubar->addButton('New page',
        $this->getLink(array('cmd'=>'add_topic', 'page_id' => $this->topic['prev'], 'mode'=>0)),
        70, 'Create a new page');
    }
    if ($this->hasPermUser(PERM_CREATE, $this->authUser)) {
      $this->menubar->addButton('New subpage',
        $this->getLink(array('cmd'=>'add_topic', 'page_id' => $this->topicId, 'mode'=>0, 'page_id' => $this->topicId)),
        93, 'Create a new subpage');
    }
    if ($editable && $this->authUser->hasPerm(23)) {
      $this->menubar->addSeperator();
      $this->menubar->addButton('Delete page',
        $this->getLink(array('cmd'=>'del_topic', 'page_id' => $this->topicId, 'mode'=>0, 'page_id' => $this->topicId)),
        10, 'Delete this page');
    }
    if ($editable && $this->authUser->hasPerm(2) && isset($this->topic['TRANSLATIONINFOS'])) {
      $this->menubar->addButton('Delete Content',
        $this->getLink(array('cmd'=>'del_trans', 'page_id' => $this->topicId, 'mode'=>9, 'page_id' => $this->topicId)),
        10, 'Delete a language version of this page');
    }
    $this->menubar->addSeperator();
    if ($editable && $this->authUser->hasPerm(26)) {
      $this->menubar->addButton('Publish page',
        $this->getLink(array('cmd'=>'publish', 'page_id' => $this->topicId, 'mode'=>9, 'page_id' => $this->topicId)),
        16, 'Publish this page');
      if (@($this->params['mode'] == 9)) {
        $this->menubar->addButton('Handoff page',
          $this->getLink(array('cmd'=>'handoff', 'page_id' => $this->topicId, 'mode'=>9, 'page_id' => $this->topicId)),
          151, 'Handoff this page');
      }
    } else {
      $this->menubar->addButton('Handoff page',
        $this->getLink(array('cmd'=>'handoff', 'page_id' => $this->topicId, 'mode'=>9, 'page_id' => $this->topicId)),
        151, 'Handoff this page');
    }
  }

  function load($id, $lngId) {
    $result = FALSE;
    $sql = "SELECT t.topic_id,
                t.prev, t.prev_path,
                t.topic_mainlanguage, t.topic_weight,
                t.topic_created, t.topic_modified,
                t.author_id, t.author_perm, t.author_group,
                t.is_navigation, t.is_deleted,
                t.meta_useparent, t.box_useparent, t.surfer_useparent, t.surfer_permids,
                tp.topic_modified AS topic_published,
                tp.topic_created AS topic_published_created,
                tp.published_from, tp.published_to,
                u.user_id, u.givenname AS author_givenname, u.surname AS author_surname,
                l.lng_short, l.lng_title
              FROM %s AS t
                LEFT OUTER JOIN %s AS tp ON tp.topic_id = t.topic_id
                LEFT OUTER JOIN %s AS u ON t.author_id = u.user_id
                LEFT OUTER JOIN %s AS l ON l.lng_id = t.topic_mainlanguage
              WHERE t.topic_id = %d
              ORDER BY topic_id";
    $params = array($this->tableTopics, $this->tableTopicsPublic, $this->tableAuthUser, $this->tableLanguages, $id);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->topicId = (int)$row["topic_id"];
        $this->topic = $row;
        $row['TRANSLATION'] = NULL;
        $this->loadTranslatedData($this->topicId, $lngId);
        $result = TRUE;
      }
      $res->free();
    }
    return $result;
  }

  function loadTranslatedData($id, $lng) {
    $result = FALSE;
    $sql = "SELECT t.topic_id, t.topic_title, t.topic_content, t.lng_id,
                    t.topic_trans_created, t.topic_trans_modified,
                    t.meta_title, t.meta_keywords, t.meta_descr,
                    t.view_id, v.view_title, v.view_file, v.view_printfile,
                    m.module_guid , m.module_title, m.module_path, m.module_file, m.module_class,
                    u.user_id, u.username, u.givenname, u.surname, u.group_id
              FROM %s AS t
              LEFT OUTER JOIN %s AS u ON t.author_id = u.user_id
              LEFT OUTER JOIN %s AS v ON v.view_id = t.view_id
              LEFT OUTER JOIN %s AS m ON m.module_guid = v.module_guid
              WHERE t.topic_id = %d AND t.lng_id = %d";
    $params = array($this->tableTopicsTrans, $this->tableAuthUser, $this->tableViews, $this->tableModules, $id, $lng);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->topic['TRANSLATION'] = $row;
        $this->loadCurrentLanguage($row['lng_id']);
        $result = TRUE;
      }
      $res->free();
    }
    return $result;
  }

  /**
  * Store topic in database without content(ohne Inhalt)
  *
  * @return mixed
  */
  function saveProperties() {
    if ($this->topicId == $this->dialogProperties->params['page_id']) {

      $translationModified = FALSE;
      $result = TRUE;

      if (isset($this->topic['TRANSLATION'])) {
        $dataTrans = array(
          'topic_title' => $this->dialogProperties->data['topic_title']
        );
        if ($this->authUser->hasPerm(7) && $this->dialogProperties->data['meta_useparent']) {
          if (isset($this->dialogProperties->params['meta_title'])) {
            $dataTrans['meta_title'] = $this->dialogProperties->data['meta_title'];
          }
          if (isset($this->dialogProperties->params['meta_keywords'])) {
            $dataTrans['meta_keywords'] = $this->dialogProperties->data['meta_keywords'];
          }
          if (isset($this->dialogProperties->params['meta_descr'])) {
            $dataTrans['meta_descr'] = $this->dialogProperties->data['meta_descr'];
          }
        }

        if ($this->checkDataModified($dataTrans, $this->topic['TRANSLATION'])) {
          $translationModified = TRUE;
          $dataTrans['topic_trans_modified'] = time();
          $filter = array('topic_id'=>(int)$this->topicId, 'lng_id'=>$this->topic['TRANSLATION']['lng_id']);
          $result = (FALSE !== $this->databaseUpdateRecord($this->tableTopicsTrans, $dataTrans, $filter));
        }
      } else {
        $dataTrans = array('topic_title'=>$this->_gt('No Title'));
      }

      if ($result) {
        $data = array(
          'is_navigation' => $this->dialogProperties->data['is_navigation'],
          'prev_path' => $this->checkPath(),
          'is_deleted' => '0',
          'topic_weight' => $this->dialogProperties->data['topic_weight'],
          'topic_mainlanguage' => $this->dialogProperties->data['topic_mainlanguage']
        );

        if ($this->authUser->hasPerm(7)) {
          $data['meta_useparent'] = (int)(!$this->dialogProperties->data['meta_useparent']);
        }

        if ($translationModified || $this->checkDataModified($data, $this->topic)) {
          $data['topic_modified'] = time();
          $result = (FALSE !== $this->databaseUpdateRecord($this->tableTopics, $data, 'topic_id', (int)$this->topicId));

          if ($result) {
            $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
              sprintf($this->_gt('Properties of the page "%s #%d" changed.'),
                papaya_strings::escapeHTMLChars($dataTrans['topic_title']), (int)$this->topicId));
          }
        }
      }

      return $result;
    }
    return FALSE;
  }

  /**
  * save language specific content
  *
  * @return boolean
  */
  function saveContent() {
    if (isset($this->moduleObj) && is_object($this->moduleObj) &&
        isset($this->topic['TRANSLATION']) && is_array($this->topic['TRANSLATION'])) {
      $dataTrans = array(
        'topic_content' => $this->moduleObj->getData()
      );
      if ($this->checkDataModified($dataTrans, $this->topic['TRANSLATION'])) {
        $dataTrans['topic_trans_modified'] = time();
        $filter = array('topic_id'=>(int)$this->topicId, 'lng_id'=>$this->topic['TRANSLATION']['lng_id']);
        if (FALSE !== $this->databaseUpdateRecord($this->tableTopicsTrans, $dataTrans, $filter)) {
          $data = array('topic_modified' => $dataTrans['topic_trans_modified'], 'is_deleted' => '0');
          if (FALSE !== $this->databaseUpdateRecord($this->tableTopics, $data, 'topic_id', (int)$this->topicId)) {
            $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
              sprintf('Content of page "%s (%s)" changed.',
              papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), $this->topicId));
            return TRUE;
          }
        }
      }
      return FALSE;
    } else {
      return FALSE;
    }
  }

  /**
  * save language specific view
  *
  * @access public
  * @return boolean
  */
  function saveView() {
    if (isset($this->topic['TRANSLATION']) && is_array($this->topic['TRANSLATION'])) {
      $dataTrans = array(
        'view_id' => (int)$this->params['view_id']
      );
      if ($this->checkDataModified($dataTrans, $this->topic['TRANSLATION'])) {
        $dataTrans['topic_trans_modified'] = time();
        $filter = array('topic_id'=>(int)$this->topicId, 'lng_id'=>$this->topic['TRANSLATION']['lng_id']);
        if (FALSE !== $this->databaseUpdateRecord($this->tableTopicsTrans, $dataTrans, $filter)) {
          $data = array('topic_modified' => $dataTrans['topic_trans_modified'], 'is_deleted' => '0');
          if (FALSE !== $this->databaseUpdateRecord($this->tableTopics, $data, 'topic_id', (int)$this->topicId)) {
            $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
              sprintf('View of page "%s (%s)" changed.',
              papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), $this->topicId));
            return TRUE;
          }
        }
      }
    }
    return FALSE;
  }

  /**
  * Create topic
  *
  * @return boolean
  */
  function create() {
    if ($this->topicId > 0 && isset($this->topic) && is_array($this->topic)) {
      $data = array(
        'topic_modified' => time(),
        'topic_created' => time(),
        'topic_mainlanguage' => (int)$this->topic['topic_mainlanguage'],
        'author_id' => $this->authUser->userId,
        'author_group' => (int)$this->topic['author_group'],
        'author_perm' => '7'.substr($this->topic['author_perm'],1),
        'is_navigation' => (int)$this->topic['is_navigation'],
        'prev' => (int)$this->topicId,
        'prev_path' => str_replace(';;', ';', $this->topic['prev_path'].$this->topic['prev'].';'),
        'meta_useparent' => 1,
        'box_useparent' => 1,
        'surfer_useparent' => 2
      );
    } else {
      $data = array(
        'topic_modified' => time(),
        'topic_created' => time(),
        'topic_mainlanguage' => $this->lngSelect->currentLanguageId,
        'author_id' => $this->authUser->userId,
        'author_group' => $this->authUser->user['group_id'],
        'author_perm' => '777',
        'is_navigation' => 1,
        'prev' => 0,
        'prev' => ';',
        'meta_useparent' => 0,
        'box_useparent' => 0,
        'surfer_useparent' => 1
      );
    }
    if ($newId = $this->databaseInsertRecord($this->tableTopics, 'topic_id', $data)) {
      $data = array(
        'topic_id' => (int)$newId,
        'lng_id' => (int)$this->lngSelect->currentLanguageId,
        'author_id' => $this->authUser->userId,
        'topic_trans_created' => time(),
        'topic_trans_modified' => time()
      );
      $this->databaseInsertRecord($this->tableTopicsTrans, NULL, $data);
      return $newId;
    }
    return FALSE;
  }

  function createTranslation($lngId) {
    $data = array(
      'topic_id' => (int)$this->topicId,
      'lng_id' => (int)$lngId,
      'author_id' => $this->authUser->userId,
      'topic_trans_created' => time(),
      'topic_trans_modified' => time()
    );
    return (FALSE !== $this->databaseInsertRecord($this->tableTopicsTrans, NULL, $data));
  }

  function delete() {
    //move current topic to trash (mark as deleted)
    if ($this->deletePublicTopic()) {
      $data = array(
        'is_deleted' => '1'
      );
      return (FALSE !== $this->databaseUpdateRecord($this->tableTopics, $data, 'topic_id', $this->topicId));
    }
  }

  /**
  * Delete this entry and all subentries, all versions and all additional data
  *
  * @return boolean
  */
  function destroy() {
    //load Ids
    $ids = FALSE;
    $oldPath = $this->topic["prev_path"].$this->topic["prev"].';'.$this->topic["topic_id"].';';
    $sql = "SELECT topic_id
              FROM %s
              WHERE topic_id = %d
              OR prev = '%d'
              OR prev_path like '%s%%'";
    $params = array($this->tableTopics, $this->topicId, $this->topicId, $oldPath);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while($row = $res->fetchRow()) {
        $ids[] = $row[0];
      }
    }
    if (isset($ids) && is_array($ids)) {
      $filter = array("topic_id" =>$ids);
      //delete boxes
      if (FALSE !== $this->databaseDeleteRecord($this->tableBoxesLinks, $filter)) {
        //delete public pages
        if (FALSE !== $this->databaseDeleteRecord($this->tableTopicsPublicTrans, $filter)) {
          if (FALSE !== $this->databaseDeleteRecord($this->tableTopicsPublic, $filter)) {
          //delete backups
            if (FALSE !== $this->databaseDeleteRecord($this->tableTopicsVersionsTrans, $filter)) {
              if (FALSE !== $this->databaseDeleteRecord($this->tableTopicsVersions, $filter)) {
                //delete page
                if (FALSE !== $this->databaseDeleteRecord($this->tableTopicsTrans, $filter)) {
                  return (FALSE !== $this->databaseDeleteRecord($this->tableTopics, $filter));
                }
              }
            }
          }
        }
      }
    }
    return FALSE;
  }

  /**
  * Create version
  *
  * @access public
  * @return boolean
  */
  function createVersion() {
    $currentTime = time();

    if(isset($this->params['change_level'])) {
      $changeLevel = (int)$this->params['change_level'];
    } else {
      $changeLevel = -1;
    }

    $versionData = array(
      'topic_id'           => $this->topicId,
      'topic_weight'       => $this->topic['topic_weight'],
      'topic_mainlanguage' => (int)$this->topic['topic_mainlanguage'],
      'topic_modified'     => $this->topic['topic_modified'],
      'topic_change_level' => $changeLevel,
      'is_navigation'      => (int)$this->topic['is_navigation'],
      'meta_useparent'     => (int)$this->topic['meta_useparent'],
      'box_useparent'      => (int)$this->topic['box_useparent'],
      'version_time'       => $currentTime,
      'version_author_id'  => $this->authUser->userId,
      'version_message'    => $this->params['commit_message'],
    );

    if ($newVersionId = $this->databaseInsertRecord($this->tableTopicsVersions, 'version_id', $versionData)) {
      $sql = "INSERT INTO %s (version_id, lng_id, topic_id, topic_title, topic_content, author_id,
                              view_id, meta_title, meta_keywords, meta_descr)
                      SELECT '%d', tt.lng_id, tt.topic_id, tt.topic_title, tt.topic_content, tt.author_id,
                              tt.view_id, tt.meta_title, tt.meta_keywords, tt.meta_descr
                        FROM %s AS tt
                        WHERE tt.topic_id = %d";
      $params= array($this->tableTopicsVersionsTrans, $newVersionId, $this->tableTopicsTrans, $this->topicId);
      if (FALSE !== $this->databaseQueryFmt($sql, $params)) {
        $this->removeOldVersions();
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Get public date
  *
  * @access public
  * @return mixed
  */
  function getPublicDate() {
    $sql = "SELECT topic_modified FROM %s WHERE topic_id = %d";
    $params = array($this->tableTopicsPublic,$this->topicId);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      if ($row = $res->fetchRow()) {
        return $row[0];
      }
    }
    return FALSE;
  }

  /**
  * Publish topic
  *
  * @access public
  * @return boolean
  */
  function publishTopic() {
    if (isset($this->topic) && is_array($this->topic)) {
      if ($this->createVersion()) {

        $sql = "SELECT COUNT(*) FROM %s WHERE topic_id = %d";
        
        $publishedFrom = checkit::convertISODateTimeToUnix(@$this->params['published_from']);
        $publishedTo = checkit::convertISODateTimeToUnix(@$this->params['published_to']);
        if (!(isset($this->params['published_from']) && $publishedFrom > 0)) {
          $publishedFrom = $this->topic['published_from'];
        }
        if (!(isset($this->params['published_to']) && $publishedTo > 0)) {
          $publishedTo = $this->topic['published_to'];
        }
        if ($publishedTo < $publishedFrom) {
          $publishedTo = $publishedFrom;
        }
        
        $data = array(
          'topic_weight'   => $this->topic['topic_weight'],
          'topic_mainlanguage' => $this->topic['topic_mainlanguage'],
          'topic_modified' => $this->topic['topic_modified'],
          'prev'           => $this->topic['prev'],
          'prev_path'      => $this->topic['prev_path'],
          'is_navigation'  => (int)$this->topic['is_navigation'],
          'author_id'      => $this->topic['author_id'],
          'meta_useparent' => (int)$this->topic['meta_useparent'],
          'box_useparent'  => (int)$this->topic['box_useparent'],
          'surfer_useparent' => (int)$this->topic['surfer_useparent'],
          'surfer_permids'   => $this->topic['surfer_permids'],
          'published_from'   => $publishedFrom,
          'published_to'     => $publishedTo
        );

        if ($res = $this->databaseQueryFmt($sql, array($this->tableTopicsPublic, $this->topicId))) {
          $row = $res->fetchRow();
          if ($row[0] > 0) {
            $topicCreated = checkit::convertISODateTimeToUnix(@$this->params['topic_created']);
            if (!(isset($this->params['topic_created']) && $topicCreated > 0)) {
              $data['topic_created'] = $topicCreated;
            }
            $result =  ($this->databaseUpdateRecord($this->tableTopicsPublic, $data, 'topic_id', $this->topicId) !== FALSE);
          } else {
            $data['topic_id'] = $this->topicId;
            $topicCreated = checkit::convertISODateTimeToUnix(@$this->params['topic_created']);
            if (!(isset($this->params['topic_created']) && $topicCreated > 0)) {
              $topicCreated = $this->topic['topic_created'];
            }
            $data['topic_created'] = $topicCreated;
            $result = ($this->databaseInsertRecord($this->tableTopicsPublic, NULL, $data) !== FALSE);
          }
          if ($result) {
            $languages = array();
            if (isset($this->params['public_languages']) && is_array($this->params['public_languages']) &&
                count($this->params['public_languages']) > 0) {
              foreach($this->params['public_languages'] as $lng) {
                $languages[] = (int)$lng;
              }
              $params = array('topic_id'=>$this->topicId, 'lng_id'=>$languages);
              if (FALSE !== $this->databaseDeleteRecord($this->tableTopicsPublicTrans, $params)) {
                if (count($languages) > 1) {
                  $filter = ' AND t.lng_id IN ('.implode(',', $languages).')';
                } elseif (count($languages) == 1) {
                  $filter = ' AND t.lng_id = '.$languages[0];
                } else {
                  $filter = '';
                }
                $sql = "INSERT INTO %s (topic_id, lng_id, topic_title, topic_content, author_id, view_id,
                                topic_trans_created, topic_trans_modified, topic_trans_checked, meta_title, meta_keywords, meta_descr)
                        SELECT t.topic_id, t.lng_id, t.topic_title, t.topic_content, t.author_id, t.view_id,
                                t.topic_trans_created, '%d', '%d', t.meta_title, t.meta_keywords, t.meta_descr
                          FROM %s AS t
                          WHERE t.topic_id = %d $filter";
                $params= array($this->tableTopicsPublicTrans, time(), time(), $this->tableTopicsTrans, $this->topicId);
                if (FALSE !== $this->databaseQueryFmt($sql, $params)) {
                  $this->deleteCache();
                  return TRUE;
                }
              }
            }
            return TRUE;
          }
        }
      }
    }
    return FALSE;
  }

  /**
  * del public topic
  *
  * @access public
  * @return boolean
  */
  function deletePublicTopic() {
    if ($this->databaseDeleteRecord($this->tableTopicsPublicTrans, 'topic_id', $this->topicId) !== FALSE) {
      if ($this->databaseDeleteRecord($this->tableTopicsPublic, 'topic_id', $this->topicId) !== FALSE) {
        $this->deleteCache();
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * del public topic translation
  */
  function deletePublicTopicTrans() {
    $condition = array('topic_id' => $this->topicId, 'lng_id' => $this->params['lng']);
    $sql = "SELECT COUNT(*) FROM %s WHERE topic_id = %d";
    if ($this->databaseDeleteRecord($this->tableTopicsPublicTrans, $condition) !== FALSE) {
      if ($res = $this->databaseQueryFmt($sql, array($this->tableTopicsPublicTrans, $this->topicId))) {
        $row = $res->fetchRow();
        if ($row[0] == 0) {
          $this->deletePublicTopic();
        }
      }
      $this->deleteCache();
      return TRUE;
    }
    return FALSE;
  }

  /**
  * delete language editing version of topic
  */
  function deleteTrans($lngId) {
    $condition = array('topic_id' => $this->topicId, 'lng_id' => $lngId);
    if ($this->databaseDeleteRecord($this->tableTopicsTrans, $condition) !== FALSE) {
      $this->deleteCache();
      return TRUE;
    }
    return FALSE;
  }


  /**
  * Content Objekte laden
  *
  * @return mixed
  */
  function loadContentList() {
    $result = FALSE;
    $sql = "SELECT module_guid, module_title, module_path, module_file, module_class
              FROM %s
              WHERE module_type='page' AND module_active = 1
              ORDER BY module_title, module_guid";
    $params = array($this->tableModules);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $row['module_file'] = $row['module_path'].$row['module_file'];
        $result[$row["module_guid"]] = $row;
      }
    }
    return $result;
  }

  /**
  * Load views/stylsheets
  *
  * @return mixed
  */
  function loadViewList() {
    $result = FALSE;
    $sql = "SELECT view_title, view_id, view_file, view_printfile, view_editfile, module_guid
              FROM %s
              ORDER BY view_title, view_id";
    if ($res = $this->databaseQueryFmt($sql,array($this->tableViews))) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $result[$row["view_id"]] = $row;
      }
    }
    return $result;
  }

  /**
  * Load states of a page
  *
  * @return array
  */
  function loadStateList() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_statictables.php');
    foreach(base_statictables::getTableStates() as $key=>$val) {
      $result[$key] = $this->_gt($val);
    }
    return $result;
  }

  function loadLanguageList() {
    unset($this->languages);
    $sql = "SELECT lng_id, lng_title FROM %s WHERE is_content_lng = 1 ORDER BY lng_title";
    if ($res = $this->databaseQueryFmt($sql, $this->tableLanguages)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->languages[$row['lng_id']] = $row['lng_title'];
      }
      $res->free();
    }
  }

  /**
  * Load versions list
  *
  * @access public
  */
  function loadVersionsList() {
    unset($this->versions);
    $this->versionsCount = 0;
    $sql = "SELECT t.version_id, t.version_time, t.version_author_id,
                u.user_id, u.givenname, u.surname
              FROM %s AS t
                LEFT OUTER JOIN %s AS u ON t.version_author_id = u.user_id
              WHERE t.topic_id = %d
              ORDER BY t.version_time DESC, t.version_id DESC";
    $params = array($this->tableTopicsVersions,$this->tableAuthUser,(int)$this->topicId);
    if ($res = $this->databaseQueryFmt($sql, $params, 10, @(int)$this->params['version_offset'])) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $row['fullname'] = $row['givenname'].' '.$row['surname'];
        $this->versions[$row["version_id"]] = $row;
      }
      $this->versionsCount = $res->absCount();
    }
  }

  /**
  * Load version
  *
  * @access public
  * @return boolean
  */
  function loadVersion($versionId, $topicId = NULL, $versionTime = 0) {
    unset($this->savedVersion);
    if (isset($topicId) && $versionTime > 0) {
      $sql = "SELECT t.version_id, t.version_time, t.version_message, t.version_author_id,
                  t.topic_id, t.topic_mainlanguage, t.topic_weight,
                  t.topic_modified,
                  t.is_navigation,
                  t.meta_useparent, t.box_useparent,
                  u.user_id, u.givenname, u.surname
                FROM %s AS t
                  LEFT OUTER JOIN %s AS u ON t.version_author_id = u.user_id
                WHERE t.topic_id = '%d' AND t.version_time <= %d
                ORDER BY t.version_time DESC, t.version_id DESC";
      $params = array($this->tableTopicsVersions, $this->tableAuthUser, $topicId, $versionTime);
    } else {
      $sql = "SELECT t.version_id, t.version_time, t.version_message, t.version_author_id,
                  t.topic_id, t.topic_mainlanguage, t.topic_weight,
                  t.topic_modified,
                  t.is_navigation,
                  t.meta_useparent, t.box_useparent,
                  u.user_id, u.givenname, u.surname
                FROM %s AS t
                  LEFT OUTER JOIN %s AS u ON t.version_author_id = u.user_id
                WHERE t.version_id = '%d'";
      $params = array($this->tableTopicsVersions, $this->tableAuthUser, $versionId);
    }
    if ($res = $this->databaseQueryFmt($sql,$params, 1)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->savedVersion = $row;
        return TRUE;
      }
    }
    return FALSE;
  }

  function loadVersionTranslatedData($versionId, $lngId) {
    $result = FALSE;
    $sql = "SELECT t.topic_id, t.topic_title, t.topic_content, t.lng_id,
                    t.meta_title, t.meta_keywords, t.meta_descr,
                    t.view_id, v.view_title, v.view_file, v.view_printfile,
                    m.module_guid , m.module_title, m.module_path, m.module_file, m.module_class,
                    u.user_id, u.username, u.givenname, u.surname, u.group_id
              FROM %s AS t
              LEFT OUTER JOIN %s AS u ON t.author_id = u.user_id
              LEFT OUTER JOIN %s AS v ON v.view_id = t.view_id
              LEFT OUTER JOIN %s AS m ON m.module_guid = v.module_guid
              WHERE t.version_id = %d AND t.lng_id = %d";
    $params = array($this->tableTopicsVersionsTrans, $this->tableAuthUser, $this->tableViews, $this->tableModules, $versionId, $lngId);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->savedVersion['TRANSLATION'] = $row;
        $result = TRUE;
      }
      $res->free();
    }
    return $result;
  }

  function loadVersionTranslationsInfo() {
    unset($this->savedVersion['TRANSLATIONINFOS']);
    $sql = "SELECT ttv.topic_id, ttv.lng_id, ttv.topic_title
              FROM %s AS ttv
              WHERE ttv.version_id = %d";
    $params = array($this->tableTopicsVersionsTrans, (int)$this->savedVersion['version_id']);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->savedVersion['TRANSLATIONINFOS'][$row['lng_id']] = $row;
      }
    }
  }

  /**
  * Remove old versions
  *
  * @access public
  * @return mixed
  */
  function removeOldVersions() {
    $sql = "SELECT version_time
            FROM %s
            WHERE topic_id = %d
            ORDER BY version_time DESC";
    $counter = 0;
    $params = array($this->tableTopicsVersions,$this->topicId);
    if ($res = $this->databaseQueryFmt($sql, $params, 1, $this->maxVersions)) {
      if ($row = $res->fetchRow()) {
        $border = $row[0];
        $sql = "SELECT version_id
                  FROM %s
                  WHERE topic_id = %d
                    AND version_time < '%d'";
        $params = array($this->tableTopicsVersions,$this->topicId,$border);
        if ($res = $this->databaseQueryFmt($sql,$params)) {
          $ids = array();
          while ($row = $res->fetchRow()) {
            $ids[] = (int)$row[0];
          }
          if (count($ids) > 0) {
            if (FALSE === $this->databaseDeleteRecord($this->tableTopicsVersionsTrans, array('version_id'=>$ids))) {
              return FALSE;
            }
          }
          return (FALSE !== $this->databaseDeleteRecord($this->tableTopicsVersions, array('version_id'=>$ids)));
        }
      }
    }
    return FALSE;
  }

  /**
  * Deliver i-frame deklaration for output of content edit/preview
  *@param string $fileName
  */
  function getContentFrame($fileName, $caption) {
    $result = sprintf('<panel width="100%%" title="%s [%s]"><iframe width="100%%" noresize="noresize" hspace="0" vspace="0" align="center" scrolling="auto" height="1400" src="%s" class="inset" id="preview" /></panel>',
      papaya_strings::escapeHTMLChars($caption),
      papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), 
      $fileName);
    return $result;
  }

  /**
  * Edit section as XML
  */
  function getEditContent() {
    $result = FALSE;
    $moduleData = $this->topic['TRANSLATION'];
    $this->moduleObj = &$this->createPluginObject($moduleData['module_path'].$moduleData['module_file'],
      $moduleData['module_class'], $moduleData['module_guid'], $this, $moduleData['topic_content']);
    if (isset($this->moduleObj) && is_object($this->moduleObj)) {
      $this->moduleObj->images = &$this->images;
      $this->moduleObj->msgs = &$this->msgs;
      $this->moduleObj->initialize();
      $this->moduleObj->execute();
      $this->moduleObj->initializeDialog(array('page_id'=>$this->topicId));
      if ($this->moduleObj->modified()) {
        if ($this->moduleObj->checkData()) {
          if ($this->saveContent()) {
            $this->addMsg(MSG_INFO, $this->_gt('Changes saved.'));
          }
        }
      }
      $result = $this->moduleObj->getForm();
      unset($this->moduleObj);
    }
    return $result;
  }

  function initializePropertiesDialog() {
    if (!(isset($this->dialogProperties) && is_object($this->dialogProperties))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array(
        'page_id' => $this->topicId,
        'cmd' => 'save_properties'
      );

      $data = array();
      $fields = array();
      if (isset($this->topic['TRANSLATION'])) {
        $data = $this->topic['TRANSLATION'];
        $fields['topic_title'] = array('Title', 'isNoHTML', TRUE, 'input', 400, '', 1);
        if ($this->authUser->hasPerm(7) && (!($this->topic['meta_useparent']))) {
          $fields[] = 'Metatags';
          $fields['meta_title'] = array('Page Title', 'isNoHTML', FALSE, 'input', 400, '', 1);
          $fields['meta_keywords'] = array('Keywords', 'isNoHTML', FALSE, 'input', 400, '', 1);
          $fields['meta_descr'] = array('Description', 'isNoHTML', FALSE, 'textarea', 6, '', 1);
        }
      }
      $fields[] = 'Language independent';
      $data['topic_weight'] = $this->topic['topic_weight'];
      $fields['topic_weight'] = array ('Sort', 'isnoHTML', FALSE, 'input', '30', '', 'zzz',);
      $data['topic_mainlanguage'] = $this->topic['topic_mainlanguage'];
      $fields['topic_mainlanguage'] = array ('Default language', 'isNum', TRUE, 'function', 'getContentLanguageCombo', '', 1);
      $data['is_navigation'] = $this->topic['is_navigation'];
      $fields['is_navigation'] = array('Navigation', 'isNum', TRUE, 'combo', array(1=>$this->_gt('visible'), 0=>$this->_gt('hidden')), '', 1);
      if ($this->authUser->hasPerm(7)) {
        $data['meta_useparent'] = !($this->topic['meta_useparent']);
        $fields['meta_useparent'] = array('Define Metatags', 'isNum', TRUE, 'yesno', '', '', 1);
      }

      $this->dialogProperties = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->dialogProperties->msgs = &$this->msgs;
      $this->dialogProperties->loadParams();
      $this->dialogProperties->dialogTitle = $this->_gt('Properties');
      $this->dialogProperties->dialogDoubleButtons = FALSE;
      $this->dialogProperties->textYes = 'Yes';
      $this->dialogProperties->textNo = 'No';
    }
  }

  /**
  * Change general data
  *
  * @return string
  */
  function getPropertiesDialog() {
    $this->initializePropertiesDialog();
    return $this->dialogProperties->getDialogXML();
  }

  /**
  * Get Page properties info dialog
  *
  * @todo Implement this
  *
  * @access public
  * @return string
  */
  function getPropertiesReadOnly() {
    return '';
  }

  /**
  * Zugriffsrechte ndern
  *
  * @return string
  */
  function getEditPerm() {
    $result = sprintf('<listview title="%s" width="100%%">', $this->_gt('Edit permissions'));
    $result .= '<cols>';
    $result .= '<col/>';
    $result .= sprintf('<col align="center">%s</col>', $this->_gt('Author'));
    $result .= sprintf('<col align="center">%s</col>', $this->_gt('Group'));
    $result .= sprintf('<col align="center">%s</col>', $this->_gt('All'));
    $result .= '</cols>';
    $result .= '<items>';
    $result .= $this->getEditPermElement($this->_gt('Read'), PERM_READ);
    $result .= $this->getEditPermElement($this->_gt('Write'), PERM_WRITE);
    $result .= $this->getEditPermElement($this->_gt('Create'), PERM_CREATE);
    $result .= '</items>';
    $result .= '</listview>';
    return $result;
  }

  /**
  * Get edit permission element
  *
  * @param string $caption
  * @param integer $perm
  * @access public
  * @return string
  */
  function getEditPermElement($caption, $perm) {
    $result  = sprintf('<listitem title="%s">', $caption);
    $href = $this->baseLink.'?'.$this->paramName.'[cmd]=chmod&amp;'.$this->paramName.'[perm]='.(int)$perm;
    $result .= '<subitem align="center">';
    if ($this->authUser->userId == $this->topic['author_id']) {
      $result .= sprintf('<glyph src="%s"/>', ($this->hasPerm($perm, PERM_OWNER)) ? $this->images[29] : $this->images[30]);
    } else {
      $result .= sprintf('<a href="%s&amp;%s[val]=%d"><glyph src="%s"/></a>',
        $href, $this->paramName, $this->getPermValue($perm, PERM_OWNER),
        ($this->hasPerm($perm, PERM_OWNER)) ? $this->images[27] : $this->images[28]);
    }
    $result .= '</subitem>';
    $result .= '<subitem align="center">';
    if ($this->authUser->inGroup($this->topic['author_group']) && ($this->authUser->userId != $this->topic['author_id']) && !$this->authUser->isAdmin()) {
      $result .= sprintf('<glyph src="%s"/>', ($this->hasPerm($perm, PERM_GROUP)) ? $this->images[29] : $this->images[30]);
    } else {
      $result .= sprintf('<a href="%s&amp;%s[val]=%d"><glyph src="%s"/></a>',
        $href, $this->paramName, $this->getPermValue($perm, PERM_GROUP),
        ($this->hasPerm($perm, PERM_GROUP)) ? $this->images[27] : $this->images[28]);
    }
    $result .= '</subitem>';
    $result .= '<subitem align="center">';
    $result .= sprintf('<a href="%s&amp;%s[val]=%d"><glyph src="%s"/></a>',
      $href, $this->paramName, $this->getPermValue($perm, PERM_ALL),
      ($this->hasPerm($perm, PERM_ALL)) ? $this->images[27] : $this->images[28]);
    $result .= '</subitem>';
    $result .= '</listitem>';
    return $result;
  }

  /**
  * Get edit user
  *
  * @access public
  * @return string
  */
  function getEditUser($editUser) {
    $result = '';

    if (isset($editUser->users) && is_array($editUser->users) && is_array($editUser->groups)) {
      $result = sprintf('<dialog title="%s" action="%s" width="300">'.LF, $this->_gt('Author'), $this->baseLink);
      $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s"/>'.LF,
        $this->paramName, 'cmd', 'chown');
      $result .= '<lines class="dialogSmall">'.LF;
      $result .= sprintf('<line caption="%s" align="left">'.LF, $this->_gt('Group'));
      $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">'.LF, $this->paramName, 'gid');
      foreach($editUser->groups as $groupId=>$group) {
        if ($groupId == $this->topic['author_group']) {
          $result .= sprintf('<option value="%s" selected="selected">&raquo;%s&laquo;</option>'.LF,
            $groupId, papaya_strings::escapeHTMLChars($group['grouptitle']));
        } else {
          $result .= sprintf('<option value="%s">%s</option>'.LF,
            $groupId, papaya_strings::escapeHTMLChars($group['grouptitle']));
        }
      }
      $result .= '</select>'.LF;
      $result .= '</line>'.LF;
      $result .= '<line caption="Autor">'.LF;
      $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">'.LF, $this->paramName, 'uid');
      foreach ($editUser->users as $username=>$aUser) {
        if ($aUser['user_id'] == $this->topic['author_id']) {
          $result .= sprintf('<option value="%s" selected="selected">&raquo;%s (%s)&laquo;</option>'.LF,
            $aUser['user_id'], papaya_strings::escapeHTMLChars($aUser['fullname']),
            papaya_strings::escapeHTMLChars($editUser->groups[(int)$aUser['group_id']]['grouptitle']));
        } else {
          $result .= sprintf('<option value="%s">%s (%s)</option>'.LF,
            $aUser['user_id'], papaya_strings::escapeHTMLChars($aUser['fullname']),
            papaya_strings::escapeHTMLChars($editUser->groups[(int)$aUser['group_id']]['grouptitle']));
        }
      }
      $result .= '</select>'.LF;
      $result .= '</line>'.LF;
      $result .= '</lines>'.LF;
      $result .= sprintf('<dlgbutton value="%s"/>', $this->_gt('Change'));
      $result .= '</dialog>'.LF;
    }
    return $result;
  }


  /**
  * List of modules and views
  *
  * @return string
  */
  function getEditView() {
    if (isset($this->topic['TRANSLATION'])) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_selectview.php');
      $selectView = &new base_selectview();
      $selectView->authUser = &$this->authUser;
      $selectView->msgs = &$this->msgs;
      $selectView->images = &$this->images;
      $selectView->load((int)$this->topic['TRANSLATION']['view_id'], 'page');
      $selectView->actionLink = $this->getLink(array('cmd'=>'chg_view'), $this->paramName).'&'.$this->paramName.'[view_id]=';
      return $selectView->getXMLViewList();
    } else {
      return '';
    }
  }

  /**
  * Check input and change if needed
  *
  * @return boolean
  */
  function checkEdit() {
    $result = FALSE;
    //Title changed
    if ($this->params['topic_title'] != $this->topic['TRANSLATION']['topic_title']) {
      //new title - nice title
      if (checkit::isNoHTML($this->params['topic_title'],TRUE)) {
        $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Title')));
        $result = TRUE;
        $this->topic['TRANSLATION']['topic_title'] = $this->params['topic_title'];
      } else {
        $this->addMsg(MSG_ERROR, sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Title')));
      }
    }

    //Visibility changed
    if ($this->params['is_navigation'] != $this->topic['is_navigation']) {
      $this->topic['is_navigation'] = $this->params['is_navigation'];
      $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Navigation visibility')));
      $result = TRUE;
    }
    //printable changed
    if ($this->params['is_printable'] != $this->topic['is_printable']) {
      $this->topic['is_printable'] = $this->params['is_printable'];
      if ($this->topic['is_printable']) {
        $this->addMsg(MSG_INFO, $this->_gt('Print view activated.'));
      } else {
        $this->addMsg(MSG_INFO, $this->_gt('Print view disabled.'));
      }
      $result = TRUE;
    }

    //sort field changed
    if ($this->params['topic_weight'] != $this->topic['topic_weight']) {
      if (checkit::isNoHTML($this->params['topic_weight'],TRUE)) {
        $this->topic['topic_weight'] = $this->params['topic_weight'];
        $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Sort')));
        $result = TRUE;
      } else {
        $this->addMsg(MSG_ERROR, sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Sort')));
      }
    }
    return $result;
  }

  /**
  * Check edit content module
  *
  * @access public
  * @return boolean
  */
  function checkEditView() {
    if (isset($this->params['view_id']) && $this->params['view_id'] > 0){
      include_once(PAPAYA_INCLUDE_PATH.'system/base_selectview.php');
      $selectView = &new base_selectview();
      if ($selectView->loadView($this->params['view_id'])) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Get permission value
  *
  * @param integer $perm
  * @param integer $for optional, default value 0
  * @access public
  * @return integer
  */
  function getPermValue($perm, $for = 0) {
    $a = (int)$this->topic['author_perm'][$perm];
    if ($for > 0) {
      if ($this->hasPerm($perm, $for)) {
        $a = ($a & (~$for));
      } else {
        $a = ($a | $for);
      }
    }
    return $a;
  }

  /**
  * Set user
  *
  * @param integer $uid
  * @param integer $gid
  * @access public
  * @return boolean
  */
  function setUser($uid, $gid) {
    $sql = "SELECT user_id FROM %s WHERE user_id = '%s'";
    if ($res = $this->databaseQueryFmt($sql,array($this->tableAuthUser,$uid))) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $cUid = $row['user_id'];
        $sql = "SELECT group_id FROM %s WHERE group_id = '%d'";
        if ($gid == -1) {
          $cGid = -1;
        } elseif ($res = $this->databaseQueryFmt($sql,array($this->tableAuthGroups,$gid))) {
          if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
            $cGid = $row['group_id'];
          }
        }
        if ($cUid == $uid && $cGid == $gid && ($cGid > 0 || $cGid == -1)) {
          $data = array(
            'author_id'=>$cUid,
            'author_group'=>$cGid
          );
          if (FALSE !== $this->databaseUpdateRecord($this->tableTopics, $data, 'topic_id', (int)$this->topicId)) {
            $this->topic['author_id'] = $cUid;
            $this->topic['author_group'] = $cGid;
            return TRUE;
          }
        }
      }
    }
    return FALSE;
  }

  /**
  * Set permission value
  *
  * @param integer $perm
  * @param integer $value
  * @access public
  * @return mixed
  */
  function setPermValue($perm, $value) {
    $this->topic['author_perm'][$perm] = $value;
    $this->topic['author_perm'] = str_pad($this->topic['author_perm'], 3, 0);
    $data = array('author_perm'=>$this->topic['author_perm']);
    return $this->databaseUpdateRecord($this->tableTopics, $data, 'topic_id', (int)$this->topicId);
  }

  /**
  * Has permission
  *
  * @param integer $perm
  * @param integer $for
  * @access public
  * @return integer
  */
  function hasPerm($perm, $for) {
    return (((int)$this->topic['author_perm'][$perm]) & $for);
  }

  /**
  * Has permission user
  *
  * @param integer $perm
  * @param object auth_user &$user
  * @param mixed $topicId optional, default value NULL
  * @param mixed $permData optional, default value NULL
  * @access public
  * @return mixed
  */
  function hasPermUser($perm, &$user, $topicId = NULL, $permData = NULL) {
    if (is_object($user)) {
      if ($user->isAdmin()) return TRUE;
      //aktueller Topic oder anderen laden?
      $s = 0;
      $authorId = '';
      $authorGroup = 0;
      if ((!isset($topicId)) || ($topicId == $this->topicId)) {
        if (isset($permData)) {
          $s = (int)$permData;
        } else {
          $s = (int)@$this->topic['author_perm'][$perm];
        }
        $authorId = $this->topic['author_id'];
        $authorGroup = $this->topic['author_group'];
      } elseif ($topicId > 0) {
        $sql = "SELECT t.topic_id, t.author_id, t.author_group, t.author_perm, u.group_id
                FROM %s AS t, %s AS u
                WHERE t.topic_id = %d
                  AND u.user_id = t.author_id";
        $params = array($this->tableTopics,$this->tableAuthUser,(int)$topicId);
        if ($res = $this->databaseQueryFmt($sql,$params)) {
          if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
            $s = (int)$row['author_perm'][$perm];
            $authorId = $row['author_id'];
            $authorGroup = ($row['author_group'] != 0) ? (int)$row['author_group'] : (int)$row['group_id'];
          }
        }
      }
      if ($user->userId == $authorId) {
        return ($s & PERM_OWNER);
      }
      if ($user->inGroup($authorGroup)) {
        return ($s & PERM_GROUP);
      }
      return ($s & PERM_ALL);
    }
    return FALSE;
  }

  /**
  * Change permission execute
  *
  * @access public
  */
  function changePermExecute(&$user) {
    //print_r($this->params);
    switch (@$this->params['cmd']) {
      case 'chmod' :
        if ($this->params['val'] > 0 && isset($this->params['perm'])) {
          $perm = (int)$this->params['perm'];
          $permVal = (int)$this->params['val'];
          if ($this->hasPermUser($perm, $user) == $this->hasPermUser($perm, $this->authUser, NULL, $permVal)) {
            if ($this->setPermValue($perm, $permVal)) {
              $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Permissions')));
              $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
                sprintf('Edit permissions of page "%s (%s)" changed.',
                  papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), $this->topicId));
            } else {
              $this->addMsg(MSG_WARNING,$this->_gt('Database error! Changes not saved.'));
            }
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('You can not change your own permissions.'));
          }
        } else {
          $this->addMsg(MSG_WARNING,$this->_gt('Input error! Changes not saved.'));
        }
        break;
      case 'chown' :
        if ($this->setUser($this->params['uid'],$this->params['gid'])) {
          $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Owner')));
          $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
                sprintf('Owner of page "%s (%s)" changed.',
                  papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), $this->topicId));
        } else {
          $this->addMsg(MSG_WARNING,$this->_gt('Database error! Changes not saved.'));
        }
        break;
    }
  }

  /**
  * Change Box use parent - on deactivate fetch the current inherited box ids and insert them
  *
  * @access public
  * @return boolean
  */
  function saveBoxUseParent() {
    if (FALSE !== $this->databaseDeleteRecord($this->tableBoxesLinks, 'topic_id', $this->topicId)) {
      if ($this->topic['box_useparent']) {
        $boxTopicId = $this->getBoxParent();
        $sql = "SELECT box_id, box_sort
                  FROM %s
                 WHERE topic_id = %d";
        $linkData = array();
        if ($res = $this->databaseQueryFmt($sql, array($this->tableBoxesLinks, $boxTopicId))) {
          while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
            $row['topic_id'] = $this->topicId;
            $linkData[] = $row;
          }
        }
        if (is_array($linkData) && count($linkData) > 0) {
          $this->databaseInsertRecords($this->tableBoxesLinks, $linkData);
        }
      }
      $data = array(
        'box_useparent' => (int)!$this->topic['box_useparent'],
        'topic_modified' => time()
      );
      if (FALSE !== $this->databaseUpdateRecord($this->tableTopics, $data, 'topic_id', $this->topicId)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Change box use parent form
  *
  * @param integer $gotoId
  * @access public
  * @return string
  */
  function getEditUseParentForm($gotoId) {
    $result = '';
    if ($this->topic['box_useparent']) {
      $str = 'Define own boxes for this page?';
      $linkString = 'Yes, activate!';
    } else {
      $str = 'This page has own boxes.';
      $linkString = 'Disable';
    }

    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'chg_boxes',
      'page_id' => $this->topicId,
      'chgusep'=>1,
    );
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $this->_gt($str), 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = $linkString;
    $result .= $dialog->getMsgDialog();

    if ($gotoId > 0 && $this->topic['box_useparent']) {
      $hidden = array(
        'cmd' => 'show',
        'page_id' => (int)$gotoId,
      );
      $msg = $this->_gt('Inherit boxes from a parent page.');
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'information');
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Show';
      $result .= $dialog->getMsgDialog();
    }
    return $result;
  }

  /**
  * Get surfer permission list items
  *
  * @access public
  * @return string
  */
  function getSurferPermListItems() {
    $permsIds = $this->getSurferPermIDs();
    $result = FALSE;
    if (isset($permsIds) && is_array($permsIds)) {
      unset($ownPermIds);
      if ($this->topic['surfer_useparent'] == 3) {
        if (preg_match_all('/\d+/',$this->topic['surfer_permids'], $matches, PREG_PATTERN_ORDER)) {
          $ownPermIds = array_unique($matches[0]);
        }
      }
      if (count($permsIds) == 1) {
        $filter = "surferperm_id = '{$permsIds[0]}'";
      } else {
        $filter = "surferperm_id IN ('".implode("','", $permsIds)."')";
      }
      $sql = "SELECT surferperm_id, surferperm_title, surferperm_active
              FROM %s
              WHERE $filter";
      if ($res = $this->databaseQueryFmt($sql,array($this->tableSurferPerm))) {
        while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          if ($row['surferperm_active']) {
            $active = $this->_gt('Yes');
          } else {
            $active = $this->_gt('No');
          }
          if (@in_array($row['surferperm_id'], $ownPermIds)) {
            $image = $this->images[1];
          } else {
            $image = $this->images[2];
          }
          $result .= sprintf('<listitem title="%s" image="%s"><subitem>%s</subitem></listitem>',
            $row['surferperm_title'], $image, $active);
        }
      }
    }
    if ($result) {
      $result = sprintf('<listview title="%s" hint="%s" width="100%%"><cols><col>%s</col><col>%s</col></cols><items>%s</items></listview>',
        $this->_gt('Access permissions'),
        $this->_gt('Access to the page is restricted to registered visitors with the defined permissions. A page without any permission is available to all visitors.'),
        $this->_gt('Permission'),
        $this->_gt('Active'), $result);
      return $result;
    }
    return '';
  }

  /**
  * Get topic information
  *
  * @access public
  * @return string
  */
  function getTopicInformation() {
    $metaData = $this->loadMetaData();
    $this->loadTranslationsInfo();
    $dateCreated = date('Y-m-d H:i:s', $this->topic['topic_created']);
    $dateModified = date('Y-m-d H:i:s', $this->topic['topic_modified']);
    $datePublishCreated = date('Y-m-d H:i:s', $this->topic['topic_published_created']);
    $publishFields = '';

    if($this->topic['published_from'] > 0 && $this->topic['published_from'] < $this->topic['published_to']) {
      $publishFields .= sprintf('<listitem title="%s" indent="1"><subitem>%s</subitem></listitem>'.LF,
      $this->_gt('Published from'), date('Y-m-d H:i:s', $this->topic['published_from']));
    }
    if($this->topic['published_to'] > 0 && $this->topic['published_from'] < $this->topic['published_to']) {
      $publishFields .= sprintf('<listitem title="%s" indent="1"><subitem>%s</subitem></listitem>'.LF,
      $this->_gt('Published to'), date('Y-m-d H:i:s', $this->topic['published_to']));
    }
    if($this->topic['published_from'] > 0 && $this->topic['published_from'] == $this->topic['published_to']) {
      $publishFields .= sprintf('<listitem title="%s" indent="1"><subitem>%s</subitem></listitem>'.LF,
      $this->_gt('Published'), 'unlimited');
    }
    $result = sprintf('<listview title="%s" width="100%%">', $this->_gt('Information'));
    $result .= '<items>';
    $result .= sprintf('<listitem title="%s"><subitem/></listitem>'.LF,
        papaya_strings::escapeHTMLChars($this->_gt('General')));
    $result .= sprintf('<listitem title="%s" indent="1"><subitem>%s</subitem></listitem>'.LF,
        papaya_strings::escapeHTMLChars($this->_gt('Page title')), papaya_strings::escapeHTMLChars($metaData['meta_title']));
    $result .= sprintf('<listitem title="%s" indent="1"><subitem>%s</subitem></listitem>',
      $this->_gt('Author'), papaya_strings::escapeHTMLChars($this->topic['author_givenname'].' '.$this->topic['author_surname']));
    $result .= sprintf('<listitem title="%s" indent="1"><subitem>%s</subitem></listitem>',
      $this->_gt('Created'), $dateCreated);
    $result .= sprintf('<listitem title="%s" indent="1"><subitem>%s</subitem></listitem>'.LF,
      $this->_gt('Modified'), $dateModified);
    if($this->topic['topic_published_created'] > 0) {
      $result .= sprintf('<listitem title="%s" indent="1"><subitem>%s</subitem></listitem>'.LF,
        $this->_gt('Prepared'), $datePublishCreated);
    }
    $result .= $publishFields;
    if (isset($this->lngSelect->languages) && is_array($this->lngSelect->languages)) {
      foreach($this->lngSelect->languages as $lngId=>$lng) {
        if ($lng['lng_glyph'] != '' && file_exists($this->getBasePath(TRUE).'pics/language/'.$lng['lng_glyph'])) {
          $image = ' image="./pics/language/'.papaya_strings::escapeHTMLChars($lng['lng_glyph']).'"';
        } else{
          $image = '';
        }
        $result .= sprintf('<listitem title="%s"%s><subitem>%s</subitem></listitem>'.LF,
          papaya_strings::escapeHTMLChars($lng['lng_title']), $image, papaya_strings::escapeHTMLChars($lng['lng_short']));
        if (isset($this->topic['TRANSLATIONINFOS']) && isset($this->topic['TRANSLATIONINFOS'][$lngId])) {
          $result .= sprintf('<listitem title="%s" indent="2"><subitem>%s</subitem></listitem>'.LF,
            papaya_strings::escapeHTMLChars($this->_gt('Title')), papaya_strings::escapeHTMLChars($this->topic['TRANSLATIONINFOS'][$lngId]['topic_title']));
          if (isset($this->topic['TRANSLATIONINFOS'][$lngId]['topic_trans_published'])) {
            if ($this->topic['TRANSLATIONINFOS'][$lngId]['topic_trans_published'] <
                $this->topic['TRANSLATIONINFOS'][$lngId]['topic_trans_modified']) {
              $result .= sprintf('<listitem title="%s" indent="2"><subitem>%s</subitem></listitem>'.LF,
                papaya_strings::escapeHTMLChars($this->_gt('Status')), papaya_strings::escapeHTMLChars($this->_gt('Modified')));
            } else {
              $result .= sprintf('<listitem title="%s" indent="2"><subitem>%s</subitem></listitem>'.LF,
                papaya_strings::escapeHTMLChars($this->_gt('Status')), papaya_strings::escapeHTMLChars($this->_gt('Public')));
            }
            $result .= sprintf('<listitem title="%s" indent="2"><subitem>%s</subitem></listitem>'.LF,
              papaya_strings::escapeHTMLChars($this->_gt('Published')),
              date('Y-m-d H:i:s', $this->topic['TRANSLATIONINFOS'][$lngId]['topic_trans_published']));
          } else {
            $result .= sprintf('<listitem title="%s" indent="2"><subitem>%s</subitem></listitem>'.LF,
              papaya_strings::escapeHTMLChars($this->_gt('Status')), papaya_strings::escapeHTMLChars($this->_gt('Created')));
          }
          $result .= sprintf('<listitem title="%s" indent="2"><subitem>%s</subitem></listitem>'.LF,
            papaya_strings::escapeHTMLChars($this->_gt('Modified')),
            date('Y-m-d H:i:s', $this->topic['TRANSLATIONINFOS'][$lngId]['topic_trans_modified']));
        } else {
          $result .= sprintf('<listitem title="%s" indent="2"><subitem>%s</subitem></listitem>'.LF,
            papaya_strings::escapeHTMLChars($this->_gt('Status')), papaya_strings::escapeHTMLChars($this->_gt('No Content')));
        }
      }
    }
    $result .= '</items>';
    $result .= '</listview>';
    return $result;
  }

  /**
  * Get information
  *
  * @access public
  * @return string
  */
  function getInformation() {
    $result = $this->getTopicInformation();
    $result .= $this->getSurferPermListItems();
    return $result;
  }

  /**
  * Get delete formular
  *
  * @access public
  * @return string
  */
  function getDelForm() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'del_topic',
      'page_id' => $this->topicId,
      'del_topic_confirm'=>1,
    );
    $title = (isset($this->topic['TRANSLATION']['topic_title'])) ? $this->topic['TRANSLATION']['topic_title'] : $this->_gt('No Title');
    if ($this->topic['is_deleted']) {
      $msg = sprintf($this->_gt('Delete page "%s" (%s)?'), @papaya_strings::escapeHTMLChars($title), (int)$this->topicId);
    } else {
      $msg = sprintf($this->_gt('Move page "%s" (%s) to trash folder?'), @papaya_strings::escapeHTMLChars($title), (int)$this->topicId);
    }
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->dialogTitle = $this->_gt('Delete page');
    $dialog->buttonTitle = 'Delete';
    return $dialog->getMsgDialog();
  }

  /**
  * Get Versions list
  *
  * @access public
  * @return string
  */
  function getVersionsList() {
    $result = '';
    if (isset($this->versions) && is_array($this->versions)) {
      $result .= sprintf('<listview title="%s" width="100%%">', $this->_gt('Versions'));
      $result .= '<cols>';
      $result .= sprintf('<col>%s</col>', $this->_gt('Commit time'));
      $result .= sprintf('<col>%s</col>', $this->_gt('User'));
      $result .= '<col/>';
      $result .= '<col/>';
      $result .= '</cols>';
      $result .= '<items>';
      foreach($this->versions as $id=>$version) {
        if (isset($this->params['version_id']) && $id == $this->params['version_id']) {
          $selected = ' selected="selected"';
        } else {
          $selected = '';
        }
        $result .= sprintf('<listitem title="%s" href="%s?%s[page_id]=%d&amp;%s[version_id]=%s"%s>',
          date('Y-m-d H:i:s' ,$version['version_time']),
          $this->baseLink, $this->paramName, $this->topicId, $this->paramName, $id, $selected);
        $result .= '<subitem>'.$version['fullname'].'</subitem>';
        $href = sprintf('%s?%s[page_id]=%d&amp;%s[version_id]=%s&amp;%s[cmd]=',
          $this->baseLink, $this->paramName, $this->topicId, $this->paramName, $id, $this->paramName);
        $result .= sprintf('<subitem align="center"><a href="%s%s"><glyph src="%s" hint="%s" /></a></subitem>',
          $href, 'restore_version', $this->images[46], $this->_gt('Restore'));
        $href = sprintf('%s?%s[page_id]=%d&amp;%s[version_datetime]=%d&amp;%s[cmd]=chg_mode&amp;%s[mode]=5',
          $this->baseLink, $this->paramName, $this->topicId,
          $this->paramName, (int)$version['version_time'],
          $this->paramName, $this->paramName);
        $result .= sprintf('<subitem align="center"><a href="%s"><glyph src="%s" hint="%s" /></a></subitem>',
          $href, $this->images[4], $this->_gt('Preview'));
        $result .= '</listitem>';
      }
      $result .= '</items>';
      $result .= '</listview>';
    }
    return $result;
  }

  /**
  * Get version infos
  *
  * @access public
  * @return string
  */
  function getVersionInfos() {
    if (isset($this->savedVersion) && is_array($this->savedVersion)) {
      $yesno = array($this->_gt('No'), $this->_gt('Yes'));
      $result = sprintf('<listview title="Infos [%s]" width="300">',
        date('Y-m-d H:i:s' ,$this->savedVersion['version_time']));
      $result .= '<items>';
      $result .= sprintf('<listitem title="%s" indent="%s"><subitem>%s</subitem></listitem>',
        $this->_gt('Properties'), 0, '');
      $result .= sprintf('<listitem title="%s" indent="%s"><subitem>%s %s</subitem></listitem>',
        $this->_gt('User'), 1, papaya_strings::escapeHTMLChars($this->savedVersion['givenname']), papaya_strings::escapeHTMLChars($this->savedVersion['surname']));
      $result .= sprintf('<listitem title="%s" indent="%s"><subitem>%s</subitem></listitem>',
        $this->_gt('Navigation'), 1, $yesno[$this->savedVersion['is_navigation']]);
      $result .= sprintf('<listitem title="%s" indent="%s"><subitem>%s</subitem></listitem>',
        $this->_gt('Metatags'), 1, $yesno[!$this->savedVersion['meta_useparent']]);
      $result .= sprintf('<listitem title="%s" indent="%s"><subitem>%s</subitem></listitem>',
        $this->_gt('Sort'), 1, $this->savedVersion['topic_weight']);

      if (isset($this->lngSelect->languages) && is_array($this->lngSelect->languages)) {
        foreach($this->lngSelect->languages as $lngId=>$lng) {
          if ($lng['lng_glyph'] != '' && file_exists($this->getBasePath(TRUE).'pics/language/'.$lng['lng_glyph'])) {
            $image = ' image="./pics/language/'.papaya_strings::escapeHTMLChars($lng['lng_glyph']).'"';
          } else{
            $image = '';
          }
          $result .= sprintf('<listitem title="%s"%s><subitem>%s</subitem></listitem>'.LF,
            papaya_strings::escapeHTMLChars($lng['lng_title']), $image, papaya_strings::escapeHTMLChars($lng['lng_short']));
          if (isset($this->savedVersion['TRANSLATIONINFOS']) && isset($this->savedVersion['TRANSLATIONINFOS'][$lngId])) {
            $result .= sprintf('<listitem title="%s" indent="2"><subitem>%s</subitem></listitem>'.LF,
              papaya_strings::escapeHTMLChars($this->_gt('Title')), papaya_strings::escapeHTMLChars($this->savedVersion['TRANSLATIONINFOS'][$lngId]['topic_title']));
          } else {
            $result .= sprintf('<listitem title="%s" indent="2"><subitem>%s</subitem></listitem>'.LF,
              papaya_strings::escapeHTMLChars($this->_gt('Status')), papaya_strings::escapeHTMLChars($this->_gt('No Content')));
          }
        }
      }


      $result .= '</items>';
      $result .= '</listview>';



      $this->layout->addRight($result);

      $result = '<panel title="'.papaya_strings::escapeHTMLChars($this->_gt('Message')).'"><sheet width="100%" align="center"><text><div style="padding: 0px 5px 0px 5px; ">'.papaya_strings::escapeHTMLChars($this->savedVersion['version_message']).'</div></text></sheet></panel>';
      $this->layout->add($result);
    }
  }

  /**
  * Version execute
  *
  * @access public
  * @return string
  */
  function versionExecute() {
    $result = '';
    if (isset($this->savedVersion) && is_array($this->savedVersion)) {
      switch (@$this->params['cmd']) {
        case 'del_version' :
          //Version lschen
          if (isset($this->params['delconfirm']) && $this->params['delconfirm']) {
            if ($this->delVersion()) {
              $this->addMsg(MSG_INFO, $this->_gt('Version deleted.'));
              $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
                sprintf('Version of "%s (%s)" deleted.',
                  papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), $this->topicId));
              unset($this->savedVersion);
            }
          } else {
            $result = $this->getVersionDelForm();
          }
          break;
        case 'restore_version' :
          //Version wiederherstellen
          if (isset($this->params['restoreconfirm']) && $this->params['restoreconfirm']) {
            if (isset($this->params['meta_info']) && $this->params['meta_info'] == 'Yes') {
              if ($this->restoreVersion($this->params['version_id'])) {
                $this->addMsg(MSG_INFO, $this->_gt('Metainfo for version restored.'));
              }
            }
            if (isset($this->params['restore_languages']) &&
                is_array($this->params['restore_languages']) &&
                count($this->params['restore_languages']) != 0)
            {
              if (!isset($this->params['meta_info']) || $this->params['meta_info'] != 'Yes') {
                if ($this->setTopicModified()) {
                  $this->addMsg(MSG_INFO, $this->_gt('Modification date set.'));
                } else {
                  $this->addMsg(MSG_ERROR, $this->_gt('Modification date couldn\'t be set.'));
                }
              }
              foreach($this->params['restore_languages'] as $i => $lngId) {
                if (!$this->restoreVersionTrans($this->params['version_id'], $lngId)) {
                  $this->addMsg(MSG_INFO, sprintf($this->_gt('Version for language %s restored.'), $lngId));
                }
              }
            }
          } else {
            $result = $this->getVersionRestoreForm();
          }
          break;
      }
    }
    return $result;
  }

  /**
  * Get version delete formular
  *
  * @access public
  * @return string
  */
  function getVersionDelForm() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'del_version',
      'page_id' => $this->topicId,
      'version_id' => (int)$this->params['version_id'],
      'delconfirm'=>1,
    );
    $msg = sprintf($this->_gt('Delete version for "%s" from %s?'),
      papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), date('Y-m-d H:i:s' ,$this->savedVersion['version_time']));
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->dialogTitle = $this->_gt('Delete version');
    $dialog->buttonTitle = 'Delete';
    return $dialog->getMsgDialog();
  }

  /**
  * Get version restore formular
  *
  * @access public
  * @return string
  */
  function getVersionRestoreForm() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
    $hidden = array(
      'cmd' => 'restore_version',
      'page_id' => $this->topicId,
      'version_id' => (int)$this->params['version_id'],
      'restoreconfirm'=>1,
    );

    $data = array();
    $msg = sprintf($this->_gt('Restore version for "%s" from %s?'),
      papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), date('Y-m-d H:i:s' ,$this->savedVersion['version_time']));
    $fields = array(
      $msg,
      'meta_info' => array('Language independent information', 'isSomeText', FALSE, 'checkbox'),
      'restore_languages' => array('Languages', 'isSomeText', FALSE, 'function', 'callbackRestoreLanguages', '', '', 'left')
    );
    $dialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
    $dialog->dialogTitle = $this->_gt('Restore Version');
    $dialog->buttonTitle = $this->_gt('Restore');
    $dialog->inputFieldSize = 'x-large';
    return $dialog->getDialogXML();
  }

  function callbackRestoreLanguages($name, $element, $data) {
    $result = '';
    if (isset($this->lngSelect->languages) && is_array($this->lngSelect->languages)) {
      $sql = 'SELECT lng_id FROM %s WHERE topic_id=%d AND version_id=%d';
      $params = array($this->tableTopicsVersionsTrans, (int)$this->topicId, $this->params['version_id']);
      if ($res = $this->databaseQueryFmt($sql, $params)) {
        while($row = $res->fetchRow()) {
          $lngId = $row[0];
          $languages[$lngId] = $this->lngSelect->languages[$lngId];
        }
      }
      foreach($languages as $lngId=>$lng) {
        $result .= sprintf('<input type="checkbox" name="%s[%s][]" value="%d" %s%s/>'.LF,
          $this->paramName, $name, $lngId, @$selected, @$disabled);
        $result .= papaya_strings::escapeHTMLChars($languages[$lngId]['lng_title']).' ('.
          papaya_strings::escapeHTMLChars($this->lngSelect->languages[$lngId]['lng_short']).')';
      }
    }
    return $result;
  }


  /**
  * Delete version
  *
  * @access public
  * @return mixed
  */
  function delVersion() {
    return $this->databaseDeleteRecord($this->tableTopicsVersions, 'version_id', (int)$this->params['version_id']);
  }

  /**
  * Restore version
  *
  * @access public
  * @return boolean
  */
  function restoreVersion($versionId) {
    $res = FALSE;
    $sql = "SELECT is_navigation,
                   meta_useparent,
                   box_useparent,
                   topic_mainlanguage,
                   topic_weight
              FROM %s
              WHERE topic_id = %d AND version_id = %d";
    $params = array($this->tableTopicsVersions, $this->topicId, $versionId);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        $data['is_navigation']  = $row[0];
        $data['meta_useparent'] = $row[1];
        $data['box_useparent']  = $row[2];
        $data['topic_mainlanguage']  = $row[3];
        $data['topic_weight']   = $row[4];
        $data['topic_id']       = $this->topicId;

        $condition = array('topic_id' => $this->topicId);
        $res = ($this->databaseUpdateRecord($this->tableTopics, $data, $condition) !== FALSE);
      }
    }
    return $res;
  }

  /**
  * Restore version translation
  *
  * @access public
  * @return boolean
  */
  function restoreVersionTrans($versionId, $lngId) {
    $res = FALSE;
    $sql = "SELECT topic_title, topic_content,
                   view_id, topic_trans_weight,
                   meta_title, meta_keywords,
                   meta_descr
              FROM %s
              WHERE topic_id = %d AND lng_id = %d AND version_id = %d";
    $params = array($this->tableTopicsVersionsTrans, $this->topicId, $lngId, $versionId);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        $data['topic_title']   = $row[0];
        $data['topic_content'] = $row[1];
        $data['view_id']       = $row[2];
        $data['topic_trans_weight'] = $row[3];
        $data['meta_title']    = $row[4];
        $data['meta_keywords'] = $row[5];
        $data['meta_descr']    = $row[6];
        $data['topic_id']      = $this->topicId;
        $data['lng_id']        = $lngId;

        $condition = array('topic_id' => $this->topicId, 'lng_id' => $lngId);
        $res = ($this->databaseUpdateRecord($this->tableTopicsTrans, $data, $condition) !== FALSE);
      }
    }
    return $res;
  }

  function setTopicModified() {
    $data = array('topic_modified' => time());
    return ($this->databaseUpdateRecord($this->tableTopics, $data, 'topic_id', (int)$this->topicId));
  }

  /**
  * Get public data
  *
  * @param boolean $sel
  * @access public
  * @return string
  */
  function getPublicData() {
    $result = sprintf('<listview title="%s" width="100%%">', $this->_gt('Published'));
    $result .= '<cols>';
    $result .= '<col/>';
    $result .= sprintf('<col>%s</col>', $this->_gt('Modified'));
    $result .= '<col/>';
    $result .= sprintf('<col>%s</col>', $this->_gt('Published'));
    $result .= '<col/>';
    $result .= '</cols>';
    $result .= '<items>';
    if ($pubDate = $this->getPublicDate()) {
      $pubDateStr = date('Y-m-d H:i:s', $pubDate);
    } else {
      $pubDateStr = '';
    }
    if ($this->topic['is_deleted']) {
      $imageIndex = 10; //deleted
      $imageHint = 'Deleted';
    } elseif (isset($this->topic['topic_published']) && $this->topic['topic_published'] < $this->topic['topic_modified']) {
      $imageIndex = 48; //published and modified
      $imageHint = 'Modified';
    } elseif (isset($this->topic['topic_published']) && $this->topic['topic_published'] >= $this->topic['topic_modified']) {
      $imageIndex = 16; //published and up to date
      $imageHint = 'Published';
    } else {
      $imageIndex = 15; //created
      $imageHint = 'Created';
    }

    $result .= sprintf('<listitem title="%s">', $this->_gt('General'));
    $result .= sprintf('<subitem align="center">%s</subitem>', date('Y-m-d H:i:s' ,$this->topic['topic_modified']));
    $result .= sprintf('<subitem align="center"><glyph src="%s" hint="%s"/></subitem>', $this->images[$imageIndex],
      papaya_strings::escapeHTMLChars($this->_gt($imageHint)));
    $result .= sprintf('<subitem align="center">%s</subitem>', $pubDateStr);
    if ($pubDate) {
      $result .= sprintf('<subitem align="center"><a href="%s"><glyph src="%s"/></a></subitem>',
        $this->baseLink.'?tt[cmd]=del_public', $this->images[10]);
    } else {
      $result .= '<subitem/>';
    }
    $result .= '</listitem>';

    if (isset($this->lngSelect->languages) && is_array($this->lngSelect->languages)) {
      foreach($this->lngSelect->languages as $lngId=>$lng) {
        if ($lng['lng_glyph'] != '' && file_exists($this->getBasePath(TRUE).'pics/language/'.$lng['lng_glyph'])) {
          $image = ' image="./pics/language/'.papaya_strings::escapeHTMLChars($lng['lng_glyph']).'"';
        } else{
          $image = '';
        }
        $href = $this->baseLink.'?tt[cmd]=';
        $result .= sprintf('<listitem title="%s" indent="1"%s>'.LF,  papaya_strings::escapeHTMLChars($lng['lng_title']), $image);
        if (isset($this->topic['TRANSLATIONINFOS']) && isset($this->topic['TRANSLATIONINFOS'][$lngId])) {
          $result .= sprintf('<subitem align="center">%s</subitem>'.LF,
            date('Y-m-d H:i:s', $this->topic['TRANSLATIONINFOS'][$lngId]['topic_trans_modified']));
          if (isset($this->topic['TRANSLATIONINFOS'][$lngId]['topic_trans_published'])) {
            if ($this->topic['TRANSLATIONINFOS'][$lngId]['topic_trans_published'] >=
                $this->topic['TRANSLATIONINFOS'][$lngId]['topic_trans_modified']) {
              $result .= sprintf('<subitem align="center"><glyph src="%s" hint="%s"/></subitem>', $this->images[16],
                  papaya_strings::escapeHTMLChars($this->_gt('Published')));
            } else {
              $result .= sprintf('<subitem align="center"><glyph src="%s" hint="%s"/></subitem>', $this->images[48],
                  papaya_strings::escapeHTMLChars($this->_gt('Modified')));
            }
            $result .= sprintf('<subitem align="center">%s</subitem>'.LF,
              date('Y-m-d H:i:s', $this->topic['TRANSLATIONINFOS'][$lngId]['topic_trans_published']));
            $result .= sprintf('<subitem align="center"><a href="%s%s&tt[lng]=%s"><glyph src="%s"/></a></subitem>',
              $href, 'del_public_trans', $lngId, $this->images[10]);
          } else {
            $result .= sprintf('<subitem align="center"><glyph src="%s" hint="%s"/></subitem>', $this->images[15],
                papaya_strings::escapeHTMLChars($this->_gt('Created')));
            $result .= '<subitem/>';
            $result .= '<subitem/>';
          }
        } else {
          $result .= '<subitem/><subitem/><subitem/><subitem/>'.LF;
        }
        $result .= '</listitem>'.LF;
      }
    }

    $result .= '</items>';
    $result .= '</listview>';
    return $result;
  }

  /**
  * publish execute
  *
  * @access public
  * @return string
  */
  function publishExecute() {
    $result = '';
    switch (@$this->params['cmd']) {
      case 'handoff' :
        //hand off form
        $this->initializeHandoffDialog();
        $this->dialogHandoff->loadParams();
        if (isset($this->params['handoff_confirm']) && $this->params['handoff_confirm'] &&
          $this->dialogHandoff->checkDialogInput()) {
          $this->todo->params = $this->params;
          if($this->todo->createTodo()) {
            $this->addMsg(MSG_INFO, $this->_gt('Page forwarded.'));
            $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
                  sprintf($this->_gt('Page "%s (%s)" forwarded.'),
                    papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), $this->topicId));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Couldn\'t handoff this page.'));
          }
        } else {
          $result = $this->getHandoffForm();
        }
        break;
      case 'publish' :
        if ($this->authUser->hasPerm(26)) {
          if(!isset($_POST['audit'])) {
            $this->initializePublishDialog();
            $this->dialogPublish->loadParams();
            if (isset($this->params['publish_confirm']) && $this->params['publish_confirm'] &&
              $this->dialogPublish->checkDialogInput()) {
              $this->sessionParams['last_publish_message'] = $this->params['commit_message'];
              if ($this->publishTopic()) {
                $this->addMsg(MSG_INFO, $this->_gt('Page published.'));
                $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
                  sprintf($this->_gt('Page "%s (%s)" published.'),
                    papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), $this->topicId));
              } else {
                $this->addMsg(MSG_ERROR, $this->_gt('Couldn\'t publish this page.'));
              }
            } else {
              $result = $this->getPublishForm();
            }
          } else {
            if($this->addTopicAudit()) {
              $this->addMsg(MSG_INFO, $this->_gt('Page audited.'));
                $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
                  sprintf($this->_gt('Page "%s (%s)" audited.'),
                    papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), $this->topicId));
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Couldn\'t audit this page.'));
            }
          }
        }
        break;
      case 'del_public' :
        if ($this->authUser->hasPerm(26)) {
          if (isset($this->params['del_public_confirm']) && $this->params['del_public_confirm']) {
            if ($this->deletePublicTopic()) {
              $this->addMsg(MSG_INFO, 'Published page deleted.');
              $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
                sprintf('Published page "%s (%s)" deleted.',
                  papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), $this->topicId));
            } else {
              $this->addMsg(MSG_ERROR, 'Couldn\'t delete published page.');
            }
          } else {
            $result = $this->getDelPublicForm();
          }
        }
        break;
      case 'del_public_trans' :
        if ($this->authUser->hasPerm(26)) {
          if (isset($this->params['del_public_trans_confirm']) && $this->params['del_public_trans_confirm']) {
            if ($this->deletePublicTopicTrans()) {
              $this->addMsg(MSG_INFO, 'Translation of published page deleted.');
              $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
                sprintf('Translation of published page "%s (%s)" deleted.',
                  papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), $this->topicId));
            } else {
              $this->addMsg(MSG_ERROR, 'Couldn\'t delete translation of published page.');
            }
          } else {
            $result = $this->getDelPublicTransForm();
          }
        }
        break;
    }
    return $result;
  }

  /**
  * Get Pages Navigation
  *
  * @param &$toolbar
  * @param $step
  * @param $max
  * @param $offsetParam
  * @param $params
  * @access public
  */
  function getPagesNav(&$toolbar, $step, $max, $offsetParam, $params, $caption) {
    $offset = 0;
    if (isset($this->params[$offsetParam]) && $this->params[$offsetParam] > 0 &&
        $this->params[$offsetParam] <= $max) {
      $offset = (int)$this->params[$offsetParam];
    }
    $linkParams = $params;
    if (($max > $step) && is_object($toolbar)) {
      if ($offset > 0) {
        $i = (($offset - $step) > 0) ? ($offset - $step) : 0;
        $linkParams[$offsetParam] = $i;
        $toolbar->addButton('Prior', $this->getLink($linkParams), 8, '');
      }
      for($i=0,$x=1;$i<$max;$i+=$step,$x++) {
        $pages[$i] = $x;
      }
      $toolbar->addCombo($caption, $this->getLink($params),
        $this->paramName.'['.$offsetParam.']', $offset, $pages);
      if ($offset < ($max-$step)) {
        $i = (($offset + $step) < $max) ? ($offset + $step) : ($max-$step);
        $linkParams[$offsetParam] = $i;
        $toolbar->addButton('Next', $this->getLink($linkParams), 9, '');
      }
    }
  }
  /**
  * Initialize publish formular
  *
  * @access public
  */
  function initializePublishDialog() {
    if (!(isset($this->dialogPublish) && is_object($this->dialogPublish)) &&
        isset($this->topic['TRANSLATION'])) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array(
        'cmd' => 'publish',
        'page_id' => $this->topicId,
        'id' => $this->topicId,
        'publish_confirm'=>1,
      );
      $data = array();
      if (isset($this->sessionParams['last_publish_message']) && 
          trim($this->sessionParams['last_publish_message']) != '') {
        $publishVersionMessage = $this->sessionParams['last_publish_message'];
      } else {
        $publishVersionMessage = '';
      }
      $created = (@$this->topic['topic_published_created'] > 0) 
        ? $this->topic['topic_published_created'] : time();
      $publishedFrom = (@$this->topic['published_from'] > 0) 
        ? $this->topic['published_from'] : time();
      $publishedTo = (@$this->topic['published_to'] > 0) 
        ? $this->topic['published_to'] : time();

      $created = date("Y-m-d H:i", $created);
      $publishedFrom = date("Y-m-d H:i", $publishedFrom);
      $publishedTo = date("Y-m-d H:i", $publishedTo);

      require_once(PAPAYA_INCLUDE_PATH.'system/base_statictables.php');
      $fields = array(
        'commit_message' => array('Message', 'isSomeText', TRUE, 'input', 200, 
          '', $publishVersionMessage),
        'change_level' => array('Changelevel', 'isNum', TRUE, 'combo', 
          base_statictables::getChangeLevels(), '', 1),
        'topic_created' => array('Created', 'isISODateTime', TRUE, 'input', 30, 
          'ISO Format YYYY-DD-MM HH:MM', $created),
        'Publish Period',
        'published_from' => array('Published from', 'isISODateTime', FALSE, 
          'input', 30, 'ISO Format YYYY-DD-MM HH:MM', $publishedFrom),
        'published_to' => array('Published to', 'isISODateTime', FALSE, 
          'input', 30, 'ISO Format YYYY-DD-MM HH:MM', $publishedTo),
        'Language',
        'public_languages' => array('Languages', NULL, TRUE, 'function', 
          'callbackPublishLanguages', '', '', 'left')
      );

      $sql = "SELECT version_id, topic_id
                FROM %s
               WHERE topic_id = ".$this->topic['topic_id'];

      if($res = $this->databaseQueryFmt($sql, $this->tableTopicsVersions)) {
        if(!$row = $res->fetchRow()) {
          unset($fields['change_level']);
        }
      }

      $msg = sprintf($this->_gt('Publish page "%s" (%s)?'),
        @papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), 
        (int)$this->topicId);
      $this->dialogPublish = &new base_dialog($this, $this->paramName, $fields, 
        $data, $hidden);
      $this->dialogPublish->msgs = &$this->msgs;
      $this->dialogPublish->fields = $fields;
      $this->dialogPublish->data = $data;
      $this->dialogPublish->hidden = $hidden;

      $this->dialogPublish->dialogTitle = $this->_gt('Publish');
      $this->dialogPublish->buttonTitle = 'Publish';
      $this->dialogPublish->addButton('audit', 'Audited');
      $this->dialogPublish->inputFieldSize = 'x-large';
    }
  }

  /**
  * Get publish formular
  *
  * @access public
  * @return string
  */
  function getPublishForm() {
    $this->initializePublishDialog();
    if (isset($this->dialogPublish) && is_object($this->dialogPublish)) {
      return $this->dialogPublish->getDialogXML();
    }
    return '';
  }

  /**
  * Initialize publish formular
  *
  * @access public
  */
  function initializeHandoffDialog() {
    if (!(isset($this->dialogHandoff) && is_object($this->dialogHandoff)) &&
        isset($this->topic['TRANSLATION'])) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array(
        'cmd' => 'handoff',
        'topic_id' => $this->topicId,
        'title' => $this->_gt('Check and Publish'),
        'handoff_confirm'=>1,
        'priority'=>0,
        'status'=>0,
        'date_to'=>0,
        'user_id_from' => $this->authUser->userId,
      );
      require_once(PAPAYA_INCLUDE_PATH.'system/papaya_todo.php');
      $this->todo = new papaya_todo;

      $fields = array(
        'user_id_to' => array('User', 'isSometext', TRUE, 'combo', 
          $this->todo->getUserList(26), '', ''),
        'comment' => array('Comment', 'isSometext', FALSE, 'input', 30, '', '')
      );

      $msg = sprintf($this->_gt('Publish page "%s" (%s)?'),
        @papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), 
        (int)$this->topicId);
      $this->dialogHandoff = &new base_dialog($this, $this->paramName, 
        $fields, $data, $hidden);
      $this->dialogHandoff->msgs = &$this->msgs;
      $this->dialogHandoff->fields = $fields;
      $this->dialogHandoff->data = $data;
      $this->dialogHandoff->hidden = $hidden;

      $this->dialogHandoff->dialogTitle = $this->_gt('Handoff to');
      $this->dialogHandoff->buttonTitle = 'Handoff page';
      $this->dialogHandoff->inputFieldSize = 'x-large';
    }
  }

  /**
  * Get hendof formular
  *
  * @access public
  * @return string
  */
  function getHandoffForm() {
    $this->initializeHandoffDialog();
    if (isset($this->dialogHandoff) && is_object($this->dialogHandoff)) {
      return $this->dialogHandoff->getDialogXML();
    }
    return '';
  }

  function callbackPublishLanguages($name, $element, $data) {
    $result = '';
    if (isset($this->lngSelect->languages) && is_array($this->lngSelect->languages)) {
      foreach($this->lngSelect->languages as $lngId=>$lng) {
        if (isset($this->topic['TRANSLATIONINFOS']) && isset($this->topic['TRANSLATIONINFOS'][$lngId])) {
          $selected = @in_array($lngId, $data) ? ' checked="checked"' : '';
          if ((!is_array($data)) && $this->lngSelect->currentLanguageId == $lngId) {
            $selected = ' checked="checked"';
          }
          if (isset($this->topic['TRANSLATIONINFOS'][$lngId]['topic_trans_published']) &&
                $this->topic['TRANSLATIONINFOS'][$lngId]['topic_trans_published'] >=
                $this->topic['TRANSLATIONINFOS'][$lngId]['topic_trans_modified']) {
            $disabled = ' disabled="disabled"';
          } else {
            $disabled = '';
          }
          $result .= sprintf('<input type="checkbox" name="%s[%s][]" value="%d" %s%s/>'.LF,
            $this->paramName, $name, $lngId, $selected, $disabled);
          $result .= papaya_strings::escapeHTMLChars($this->lngSelect->languages[$lngId]['lng_title']).' ('.
            papaya_strings::escapeHTMLChars($this->lngSelect->languages[$lngId]['lng_short']).')';
        }
      }
    }
    return $result;
  }



  /**
  * Get delete public formular
  *
  * @access public
  * @return string
  */
  function getDelPublicForm() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'del_public',
      'page_id' => $this->topicId,
      'del_public_confirm'=>1,
    );
    $msg = sprintf($this->_gt('Delete published page "%s" (%s)?'),
      papaya_strings::escapeHTMLChars($this->topic['TRANSLATION']['topic_title']), (int)$this->topicId);
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Delete';
    return $dialog->getMsgDialog();
  }

  function getDelTransForm() {
    if (isset($this->topic['TRANSLATIONINFOS']) && is_array($this->topic['TRANSLATIONINFOS']) &&
        count($this->topic['TRANSLATIONINFOS']) > 0) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array(
        'cmd' => 'del_trans',
        'page_id' => $this->topicId,
        'del_trans_confirm'=>1,
      );
      $data = array();
      if (isset($this->topic['TRANSLATION']) && $this->topic['TRANSLATION']['topic_title'] == '') {
        $topicTitle = $this->topic['TRANSLATION']['topic_title'];
      } else {
        $topicTitle = $this->_gt('No Title');
      }
      $msg = sprintf($this->_gt('Delete language versions of page "%s" (%s)?'),
        papaya_strings::escapeHTMLChars($topicTitle), (int)$this->topicId);
      $fields = array(
        $msg,
        'del_trans_language' => array('Languages', 'isSomeText', FALSE, 'function', 'callbackDeleteLanguages', '', '', 'left')
      );
      $dialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $dialog->dialogTitle = $this->_gt('Delete language content');
      $dialog->buttonTitle = 'Delete';
      $dialog->inputFieldSize = 'x-large';
      return $dialog->getDialogXML();
    }
    return '';
  }

  function callbackDeleteLanguages($name, $element, $data) {
    $result = '';
    if (isset($this->lngSelect->languages) && is_array($this->lngSelect->languages)) {
      $sql = 'SELECT lng_id FROM %s WHERE topic_id=%d';
      $params = array($this->tableTopicsTrans, (int)$this->topicId);
      if ($res = $this->databaseQueryFmt($sql, $params)) {
        while($row = $res->fetchRow()) {
          $lngId = $row[0];
          $languages[$lngId] = $this->lngSelect->languages[$lngId];
        }
      }
      if (isset($languages) && is_array($languages)) {
        foreach($languages as $lngId=>$lng) {
          $result .= sprintf('<input type="checkbox" name="%s[%s][]" value="%d" %s%s/>'.LF,
            $this->paramName, $name, $lngId, @$selected, @$disabled);
          $result .= papaya_strings::escapeHTMLChars($languages[$lngId]['lng_title']).' ('.
            papaya_strings::escapeHTMLChars($this->lngSelect->languages[$lngId]['lng_short']).')';
        }
      }
    }
    return $result;
  }


  /**
  * Get delete public translation formular
  *
  * @access public
  * @return string
  */
  function getDelPublicTransForm() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'del_public_trans',
      'page_id' => $this->topicId,
      'lng' => $this->params['lng'],
      'del_public_trans_confirm'=>1,
    );
    if (isset($this->topic['TRANSLATION']) && $this->topic['TRANSLATION']['topic_title'] == '') {
      $topicTitle = $this->topic['TRANSLATION']['topic_title'];
    } else {
      $topicTitle = $this->_gt('No Title');
    }
    $msg = sprintf($this->_gt('Delete published translation of page "%s" (%s)?'),
      papaya_strings::escapeHTMLChars($topicTitle), (int)$this->topicId);
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Delete';
    return $dialog->getMsgDialog();
  }

  /**
  * Editable
  *
  * @param object auth_user &$user
  * @access public
  * @return boolean
  */
  function editable(&$user) {
    if (//edit topic
      ($user->hasPerm(35) || $user->isAdmin()) &&
      //Indent level OK or all allowed
      (($this->getLevel($user->startNode) <= $user->subLevel) || ($user->subLevel == 0))) {
      //ok - go!
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Add topic ID dialog
  *
  * @access public
  */
  function addTopicIdDialog() {
    $result = sprintf('<dialog action="%s" title="%s" width="250" name="gotoform">', 'topic.php', papaya_strings::escapeHTMLChars($this->_gt('Goto Page')));
    $result .= '<lines class="dialogXSmall">';
    $result .= sprintf('<line caption="%s"><input type="text" name="%s[page_id]" maxlength="5" value="%d" class="dialogInput dialogScale"/></line>',
      papaya_strings::escapeHTMLChars($this->_gt('Page Id')), $this->paramName, $this->topicId);
    $result .= '</lines>';
    $result .= sprintf('<dlgbutton value="%s" />', $this->_gt('GoTo'));
    $result .= '</dialog>';
    $this->layout->addLeft($result);
  }

  function addSwitchRichtextDialog() {
    if (isset($this->authUser->options['PAPAYA_USE_RICHTEXT']) && $this->authUser->options['PAPAYA_USE_RICHTEXT']) {
      $result = sprintf('<dialog action="%s" title="%s" width="250" name="switchrichtextform">', $this->baseLink, papaya_strings::escapeHTMLChars($this->_gt('Switch Richtext Editor')));
      $useRichtext = $this->getSessionValue('PAPAYA_SESSION_USE_RICHTEXT');
      if (isset($this->useRichtext) && $this->useRichtext) {
        $hiddenVal = '0';
        $btnCaption = 'Disable';
        $msg = 'The richtext editor is active. Disable it?';
      } else {
        $hiddenVal = '1';
        $msg = 'Do you like to use the richtext editor?';
        $btnCaption = 'Yes, activate!';
      }
      $result .= sprintf('<input type="hidden" name="%s[cmd]" value="switch_richtext"/>', $this->paramName);
      $result .= sprintf('<input type="hidden" name="%s[switch_richtext]" value="%d"/>', $this->paramName, $hiddenVal);
      $result .= '<lines>';
      $result .= sprintf('<line align="center"><info>%s</info></line>', papaya_strings::escapeHTMLChars($this->_gt($msg)));
      $result .= '</lines>';
      $result .= sprintf('<dlgbutton value="%s" />', $this->_gt($btnCaption));
      $result .= '</dialog>';
      $this->layout->addLeft($result);
    }
  }

  function getVersionsPreviewList() {
    $result = '';
    $this->loadVersionsList();
    $this->menubar->addSeperator();
    $this->getPagesNav($this->menubar, 10, $this->versionsCount, 'version_offset',
        array('page_id'=>$this->topicId), 'Versions');

    $result .= sprintf('<listview title="%s" width="100%%">', $this->_gt('Versions'));
    $result .= '<items>';
    if (@$this->params['version_datetime'] <= 0) {
      $selected = ' selected="selected"';
      $haveSelect = TRUE;
    } else {
      $selected = '';
      $haveSelect = FALSE;
    }
    $href = sprintf('%s?%s[page_id]=%d&amp;%s[version_datetime]=0',
      $this->baseLink, $this->paramName, $this->topicId, $this->paramName);
    $result .= sprintf('<listitem title="%s (%s)" image="%s" href="%s"%s><subitem/></listitem>',
      date('Y-m-d H:i:s' , $this->topic['topic_modified']),
      $this->_gt('Current'), $this->images[4], $href, $selected);
    
    if (isset($this->versions) && is_array($this->versions)) {
      foreach($this->versions as $id=>$version) {
        if ((!$haveSelect) && $version['version_time'] <= @$this->params['version_datetime']) {
          $selected = ' selected="selected"';
          $haveSelect = TRUE;
        } else {
          $selected = '';
        }
        $href = sprintf('%s?%s[page_id]=%d&amp;%s[version_datetime]=%d',
          $this->baseLink, $this->paramName, (int)$this->topicId,
          $this->paramName, (int)$version['version_time']);
        $result .= sprintf('<listitem title="%s" image="%s" href="%s"%s>',
          date('Y-m-d H:i:s' ,$version['version_time']), $this->images[4], $href, $selected);
        $href = sprintf('%s?%s[page_id]=%d&amp;%s[version_id]=%d&amp;%s[cmd]=restore_version&amp;%s[mode]=7',
          $this->baseLink, $this->paramName, (int)$this->topicId,
          $this->paramName, (int)$id, $this->paramName, $this->paramName);
        $result .= sprintf('<subitem align="right"><a href="%s"><glyph src="%s" title="%s"/></a></subitem>',
          $href, $this->images[46], papaya_strings::escapeHTMLChars($this->_gt('Restore')));
        $result .= '</listitem>';
      }
    }
    $result .= '</items>';
    $result .= '</listview>';
    return $result;
  }

  function addVersionPreviewDialog() {
    $result = $this->getVersionsPreviewList();
    $result .= sprintf('<dialog action="%s" title="%s" width="250" name="selectPreviewVersion">', $this->baseLink, papaya_strings::escapeHTMLChars($this->_gt('Select Version')));
    $result .= '<lines>';
    $selectionTime = (@$this->params['version_datetime'] > 0) ? $this->params['version_datetime'] : time();
    $result .= sprintf('<line caption="%s"><input type="text" name="%s[version_date]" maxlength="10" value="%s"/></line>',
      papaya_strings::escapeHTMLChars($this->_gt('Date')), $this->paramName, date('Y-m-d', $selectionTime));
    $result .= sprintf('<line caption="%s"><input type="text" name="%s[version_time]" maxlength="10" value="%s"/></line>',
      papaya_strings::escapeHTMLChars($this->_gt('Time')), $this->paramName, date('H:i:s', $selectionTime));
    $result .= '</lines>';
    $result .= sprintf('<dlgbutton value="%s" />', $this->_gt('Preview'));
    $result .= '</dialog>';
    $this->layout->addLeft($result);
  }

  function addTranslationDialog() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'add_translation',
      'lng_id' => $this->lngSelect->currentLanguageId,
      'page_id' => $this->topicId
    );
    $msg = sprintf($this->_gt('Add content for language "%s" (%s) to page "%s"?'),
        papaya_strings::escapeHTMLChars($this->lngSelect->currentLanguage['lng_title']),
        papaya_strings::escapeHTMLChars($this->lngSelect->currentLanguage['lng_short']),
        (int)$this->topicId);
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Add';
    return $dialog->getMsgDialog();
  }

  /**
  * Get language combo
  *
  * @param string $name
  * @param array $element
  * @param mixed $data
  * @access public
  * @return string XML
  */
  function getContentLanguageCombo($name, $element, $data) {
    return $this->lngSelect->getContentLanguageCombo($this->paramName, $name, $element, $data);
  }

  /**
  * add Audittime to last Version
  *
  * @return boolean
  */
  function addTopicAudit() {
    $sql = "SELECT v.version_id, v.version_time, v.version_author_id, v.version_message,
                   v.topic_id, v.topic_modified, v.topic_change_level, v.topic_audited, v.is_navigation,
                   v.topic_weight, v.meta_useparent, v.box_useparent, topic_mainlanguage
              FROM %s AS v
             WHERE v.topic_id = '%d'
             ORDER BY v.version_id DESC";
    $params = array($this->tableTopicsVersions, $this->topic["topic_id"]);
    if ($res = $this->databaseQueryFmt($sql, $params, 1)) {
      $row = $res->fetchRow(DB_FETCHMODE_ASSOC);
      $row['version_id'] = '';
      $row['topic_audited'] = time();
      $newVersionId = $this->databaseInsertRecord("papaya_topic_versions", "version_id", $row);

      $sql = "INSERT INTO %s (version_id, lng_id, topic_id, topic_title, topic_content, author_id,
                              view_id, meta_title, meta_keywords, meta_descr)
                      SELECT '%d', tt.lng_id, tt.topic_id, tt.topic_title, tt.topic_content, tt.author_id,
                              tt.view_id, tt.meta_title, tt.meta_keywords, tt.meta_descr
                        FROM %s AS tt
                       WHERE tt.topic_id = %d";
      $params= array($this->tableTopicsVersionsTrans, $newVersionId, $this->tableTopicsTrans, $this->topic["topic_id"]);
      if($this->databaseQueryFmt($sql,$params)) {
        return TRUE;
      }
    }
    return FALSE;
  }
}
?>
