<?php
/**
* Edit URL basic class
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
* @version $Id: base_urlmounter.php,v 1.7 2006/02/20 12:14:31 weinert Exp $
*/


/**
* Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* Edit URL basic class
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class base_urlmounter extends base_db {
  var $tableAliases = PAPAYA_DB_TBL_URLS;
  var $tableLanguages = PAPAYA_DB_TBL_LNG;

  var $baseLink;  
  var $aError;

  /**
  * Load
  *
  * @param string $path
  * @access public
  * @return mixed
  */
  function load($path) {
    $sql = "SELECT a.topic_id, l.lng_ident, a.url_params 
              FROM %s AS a
              LEFT OUTER JOIN %s AS l ON (l.lng_id = a.lng_id)
             WHERE a.path='%s'";
    $params = array($this->tableAliases,$this->tableLanguages,$this->checkPathSlashes($path));
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        return $row;
      }
    }
    return FALSE;
  }
  
  /**
  * Check path slashes
  *
  * @param string $path
  * @access public
  * @return string
  */
  function checkPathSlashes($path) {
    $result = trim($path);
    if (substr($path, 0, 1) != '/') {
      $result = '/'.$path;
    }
    if (substr($result, -1) != '/') {
      $result .= '/';
    }
    return preg_replace('#//+#', '/', $result);
  }

  /**
  * Locate
  *
  * @access public
  * @return string
  */
  function locate() {
    $result = FALSE;
    if (preg_match('#^/(sid\w*[\da-fA-F]{32}/)?([^\?]+)(.*)$#',$_SERVER['REQUEST_URI'],$regs)) {
      list(,,$path,$addString) = $regs;
      if ($path != '') {
        $data = $this->load($path);
        if (!$data && PAPAYA_PATH_WEB != '/' && strpos($path, PAPAYA_PATH_WEB) == 0) {
          $data = $this->load(substr($path, strlen(PAPAYA_PATH_WEB)-1));
        }
        if (isset($data) && is_array($data)) {
          $queryParams = array_merge($this->parseQueryString($addString), $this->parseQueryString($data['url_params']));
          $url = PAPAYA_PATH_WEB.$this->getWebLink((int)$data['topic_id'], $data['lng_ident'], NULL, NULL, NULL, $path);
          $url = $this->getAbsoluteURL($url).$this->getQueryString($queryParams);
          return $url;
        }
      }
    }
    return $result;
  }
  
  function parseQueryString($queryString) {
    $result = array();
    if (preg_match_all('~(^|[?&])(([^=&?]+)=([^&]+))|([^&]+)~', $queryString, $matches, PREG_SET_ORDER)) {
      foreach($matches as $match) {
        if (isset($match[3]) && $match[3] != '') {
          $result[$match[3].'='] = $match[4];
        } elseif (isset($match[5])) {
          $result[$match[5]] = TRUE;
        }
      }
    }
    return $result;
  }
  
  function getQueryString($queryParams) {
    if (isset($queryParams) && is_array($queryParams) && count($queryParams) > 0) {
      $result = '';
      foreach($queryParams as $paramName => $paramValue) {
        if(substr($paramName, -1) == '=') {
          $result .= '&'.urlencode(substr($paramName, 0, -1)).'='.urlencode($paramValue);
        } else {
          $result .= '&'.urlencode($paramName);        
        }
      }
      return '?'.substr($result, 1);
    }
  }
}

?>