<?php
/**
* View list basic class
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
* @version $Id: base_viewlist.php,v 1.22 2006/04/25 10:25:00 weinert Exp $
*/


/**
* Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');
/**
* Basic classcheck conditions
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');

/**
* View list basic class
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class base_viewlist extends base_db {
  var $tableViews = PAPAYA_DB_TBL_VIEWS;
  var $tableViewModes = PAPAYA_DB_TBL_VIEWMODES;
  var $tableViewLinks = PAPAYA_DB_TBL_VIEWLINKS;
  var $tableModules = PAPAYA_DB_TBL_MODULES;
  var $tableModuleGroups = PAPAYA_DB_TBL_MODULEGROUPS;


  var $moduleGroups = NULL;
  var $modules = NULL;

  var $views = NULL;
  var $viewModes = NULL;
  var $outputModules = NULL;

  var $view = NULL;
  var $viewMode = NULL;
  var $viewLink = NULL;

  var $brokenViews = NULL;

  var $openModuleGroups = array();

  var $reservedModes = array('php', 'urls', 'xml', 'media', 'thumb', 'download', 'popup', 'image');

  /**
  * Constructor
  *
  * @param string $paramName optional, default value 'vl'
  * @access public
  */
  function __construct($paramName='vl') {
    $this->paramName = $paramName;
    $this->sessionParamName = 'PAPAYA_SESS_papaya_views_'.$this->paramName;
  }

  /**
  * Initialize parameters
  *
  * @access public
  */
  function initialize() {
    $this->initializeParams();
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);
    $this->initializeSessionParam('mode', array('cmd'));
    if (!(isset($this->sessionParams['open_modulegroups']) && is_array($this->sessionParams['open_modulegroups']))) {
      $this->sessionParams['open_modulegroups'] = array();
    }
    $this->openModuleGroups = &$this->sessionParams['open_modulegroups'];
  }

  function execute() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_language_select.php');
    $this->lngSelect = &base_language_select::getInstance();
    $this->loadModuleGroups();
    switch ($this->params['mode']) {
      case 1 :
        $this->loadOutputModulesList();
        switch(@$this->params['cmd']) {
          case 'viewmode_add' :
            //add a new filter
            $this->initializeViewModeDialog();
            if ($this->dialogViewMode->checkDialogInput() && $this->checkViewModeData()) {
              if ($newId = $this->createViewMode()) {
                $this->addMsg(MSG_INFO, $this->_gt('Filter added.'));
                $this->params['viewmode_id'] = $newId;
                unset($this->dialogViewMode);
              }
            }
            break;
          case 'viewmode_edit' :
            //edit current filter
            if (isset($this->params['viewmode_id']) && $this->params['viewmode_id'] > 0 &&
                $this->loadViewMode($this->params['viewmode_id'])) {
              $this->initializeViewModeDialog();
              if ($this->dialogViewMode->checkDialogInput() && $this->checkViewModeData()) {
                if ($this->saveViewMode()) {
                  $this->addMsg(MSG_INFO, $this->_gt('Filter modified.'));
                  unset($this->viewMode);
                  unset($this->dialogViewMode);
                }
              }
            }
            break;
          case 'viewmode_delete' :
            //delete a filter
            if (isset($this->params['viewmode_id']) && $this->params['viewmode_id'] > 0 &&
                isset($this->params['confirm_delete']) && $this->params['confirm_delete'] &&
                $this->loadViewMode($this->params['viewmode_id'])) {
              if ($this->deleteViewMode()) {
                $this->addMsg(MSG_INFO, $this->_gt('Filter deleted.'));
                unset($this->viewMode);
              }
            }
            break;
        }
        $this->loadViewModesList();
        if (isset($this->params['viewmode_id']) && $this->params['viewmode_id'] > 0) {
          $this->loadViewMode($this->params['viewmode_id']);
        }
        break;
      default :
        $this->loadModulesList();
        $this->loadViewModesList();
        switch(@$this->params['cmd']) {
          case 'group_open' :
            if (isset($this->params['group_id']) && $this->params['group_id'] > 0) {
              $this->openModuleGroups[(int)$this->params['group_id']] = TRUE;
            }
            break;
          case 'group_close' :
            if (isset($this->params['group_id']) && $this->params['group_id'] > 0 &&
                isset($this->openModuleGroups[(int)$this->params['group_id']])) {
              unset($this->openModuleGroups[(int)$this->params['group_id']]);
            }
            break;
          case 'view_link' :
            if (isset($this->params['view_id']) && $this->params['view_id'] > 0 &&
                isset($this->params['viewmode_id']) && $this->params['viewmode_id'] > 0) {
              $this->addViewLink($this->params['view_id'], $this->params['viewmode_id']);
            }
          case 'viewlink_edit' :
            if (isset($this->params['view_id']) && $this->params['view_id'] > 0 &&
                isset($this->params['viewmode_id']) && $this->params['viewmode_id'] > 0) {
              $this->executeViewLinkEdit($this->params['view_id'], $this->params['viewmode_id']);
            }
            break;
          case 'view_unlink' :
            if (isset($this->params['view_id']) && $this->params['view_id'] > 0 &&
                isset($this->params['viewmode_id']) && $this->params['viewmode_id'] > 0 &&
                $this->loadViewLink($this->params['view_id'], $this->params['viewmode_id']) &&
                isset($this->params['confirm_unlink']) && $this->params['confirm_unlink']) {
              if ($this->deleteViewLink($this->params['view_id'], $this->params['viewmode_id'])) {
                unset($this->viewLink);
              };
            }
            break;
          case 'view_add' :
            //add a new view
            $this->initializeViewDialog();
            if ($this->dialogView->checkDialogInput()) {
              if ($newId = $this->createView()) {
                $this->addMsg(MSG_INFO, $this->_gt('View added.'));
                $this->params['view_id'] = $newId;
                unset($this->dialogView);
              }
            }
            break;
          case 'view_edit' :
            //edit current view
            if (isset($this->params['view_id']) && $this->params['view_id'] > 0 &&
                $this->loadView($this->params['view_id'])) {
              $this->initializeViewDialog();
              if ($this->dialogView->checkDialogInput()) {
                if ($this->saveView()) {
                  $this->addMsg(MSG_INFO, $this->_gt('View modified.'));
                  unset($this->view);
                  unset($this->dialogView);
                }
              }
            }
            break;
          case 'view_delete' :
            //delete a view
            if (isset($this->params['view_id']) && $this->params['view_id'] > 0 &&
                isset($this->params['confirm_delete']) && $this->params['confirm_delete'] &&
                $this->loadView($this->params['view_id'])) {
              if ($this->deleteView()) {
                $this->addMsg(MSG_INFO, $this->_gt('View deleted.'));
                unset($this->view);
              }
            }
            break;
        }
        $this->loadViewsList();
        if (isset($this->params['view_id']) && $this->params['view_id'] > 0) {
          $this->loadView($this->params['view_id']);
        }
        break;
    }
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  function executeViewLinkEdit($viewId, $viewModeId) {
    $result = FALSE;
    if ($this->loadView($viewId) && $this->loadViewLink($viewId, $viewModeId)) {
      if ($this->view['module_useoutputfilter']) {
        $parent = NULL;
        $moduleObj = &$this->createPluginObject($this->viewLink['module_path'].$this->viewLink['module_file'],
          $this->viewLink['module_class'], $this->viewLink['module_guid'], $parent);
        if (isset($moduleObj) && is_object($moduleObj)) {
          $moduleObj->images = &$this->images;
          $moduleObj->paramName = $this->paramName;
          $moduleObj->setData($this->viewLink['viewlink_data']);
          $hidden = array(
            'view_id' => $this->viewLink['view_id'],
            'viewmode_id' => $this->viewLink['viewmode_id'],
            'cmd' => 'viewlink_edit'
          );
          $moduleObj->initializeDialog($hidden);
          if ($moduleObj->modified()) {
            if ($moduleObj->checkData()) {
              if ($this->saveViewLinkContent($this->viewLink['view_id'], $this->viewLink['viewmode_id'], $moduleObj->getData())) {
                $this->addMsg(MSG_INFO, $this->_gt('Changes saved.'));
              }
            }
          }
          $this->layout->add($moduleObj->getForm($hidden));
          unset($moduleObj);
          return TRUE;
        }
      } else {
        $str = sprintf('<msgdialog type="info" width="100%%"><message>%s</message></msgdialog>', $this->_gt('No Configuration needed.'));
        $this->layout->add($str);
        return TRUE;
      }
    }
  }

  function loadViewLink($viewId, $viewModeId) {
    unset($this->viewLink);
    $sql = "SELECT vl.view_id, vl.viewmode_id, vl.viewlink_data,
                   vm.viewmode_ext,
                   m.module_guid, m.module_path, m.module_file, m.module_class, m.module_title
              FROM %s AS vl
              LEFT OUTER JOIN %s AS vm ON (vm.viewmode_id = vl.viewmode_id)
              LEFT OUTER JOIN %s AS m ON (m.module_guid = vm.module_guid)
             WHERE vl.view_id = %d AND vl.viewmode_id = %d";
    $params = array($this->tableViewLinks, $this->tableViewModes, $this->tableModules, $viewId, $viewModeId);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->viewLink = $row;
        return TRUE;
      }
    }
    return FALSE;
  }

  function saveViewLinkContent($viewId, $viewModeId, $xmlString) {
    $data = array('viewlink_data' => $xmlString);
    $filter = array('view_id' => $viewId, 'viewmode_id' => $viewModeId);
    return (FALSE !== $this->databaseUpdateRecord($this->tableViewLinks, $data, $filter));
  }

  function loadModuleGroups() {
    unset($this->moduleGroups);
    $sql = "SELECT modulegroup_id, modulegroup_title
              FROM %s
             ORDER BY modulegroup_title";
    if ($res = $this->databaseQueryFmt($sql, $this->tableModuleGroups)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->moduleGroups[$row['modulegroup_id']] = $row;
      }
    }
  }

  function loadModulesList() {
    unset($this->modules);
    $sql = "SELECT module_guid, module_title, module_type, modulegroup_id
              FROM %s
             WHERE (module_type = 'page' OR module_type = 'box') AND module_active = 1
             ORDER BY module_title";
    if ($res = $this->databaseQueryFmt($sql, $this->tableModules)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->modules[$row['module_guid']] = $row;
      }
    }
  }

  function loadViewModesList() {
    unset($this->viewModes);
    $sql = "SELECT viewmode_id, viewmode_ext
              FROM %s
             ORDER BY viewmode_ext";
    if ($res = $this->databaseQueryFmt($sql, $this->tableViewModes)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->viewModes[$row['viewmode_id']] = $row;
      }
    }
  }

  function loadOutputModulesList() {
    unset($this->outputModules);
    $sql = "SELECT module_guid, module_title, modulegroup_id
              FROM %s
             WHERE module_type = 'output' AND module_active = 1
             ORDER BY module_title";
    if ($res = $this->databaseQueryFmt($sql, $this->tableModules)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->outputModules[$row['module_guid']] = $row;
      }
    }
  }

  function loadViewMode($id) {
    unset($this->viewMode);
    $sql = "SELECT viewmode_id, viewmode_ext, viewmode_charset, viewmode_contenttype, module_guid
              FROM %s
             WHERE viewmode_id = %d";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableViewModes, $id))) {
      if($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->viewMode = $row;
        return TRUE;
      }
    }
    return FALSE;
  }

  function createViewMode() {
    $data = array(
      'viewmode_ext' => $this->params['viewmode_ext'],
      'viewmode_charset' => $this->params['viewmode_charset'],
      'viewmode_contenttype' => $this->params['viewmode_contenttype'],
      'module_guid' => $this->params['module_guid'],
    );
    return $this->databaseInsertRecord($this->tableViewModes, 'viewmode_id', $data);

  }

  function saveViewMode() {
    $data = array(
      'viewmode_ext' => $this->params['viewmode_ext'],
      'viewmode_charset' => $this->params['viewmode_charset'],
      'viewmode_contenttype' => $this->params['viewmode_contenttype'],
      'module_guid' => $this->params['module_guid'],
    );
    return (FALSE !== $this->databaseUpdateRecord($this->tableViewModes, $data, 'viewmode_id', $this->viewMode['viewmode_id']));
  }

  function deleteViewMode() {
    return (
      FALSE !== $this->databaseDeleteRecord($this->tableViewLinks,'viewmode_id', $this->viewMode['viewmode_id']) &&
      FALSE !== $this->databaseDeleteRecord($this->tableViewModes, 'viewmode_id', $this->viewMode['viewmode_id'])
    );
  }

  /**
  * Load list
  *
  * @access public
  */
  function loadViewsList() {
    unset($this->brokenViews);
    unset($this->views);
    $sql = "SELECT v.view_id, v.view_title, v.module_guid, m.modulegroup_id, m.module_type
              FROM %s AS v
              LEFT OUTER JOIN %s AS m ON (m.module_guid = v.module_guid)
             ORDER BY v.view_title";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableViews, $this->tableModules))) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->views[$row["view_id"]] = $row;
        if (isset($this->moduleGroups[$row['modulegroup_id']])) {
          $this->moduleGroups[$row['modulegroup_id']]['VIEWS'][$row["view_id"]] = &$this->views[$row["view_id"]];
        } else {
          $this->brokenViews[$row["view_id"]] = &$this->views[$row["view_id"]];
        }
      }
    }
  }

  /**
  * Load view
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function loadView($id) {
    unset($this->view);
    if ($id > 0) {
      $sql = "SELECT v.view_id, v.view_title, v.view_file, v.view_printfile, v.view_editfile, v.view_shot,
                     m.module_guid, m.module_useoutputfilter
                FROM %s AS v
                LEFT OUTER JOIN %s AS m ON m.module_guid = v.module_guid
               WHERE v.view_id = %d";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableViews, $this->tableModules, $id))) {
        if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->view = $row;
          $this->loadViewLinkedModes();
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  function loadViewLinkedModes() {
    if (isset($this->view)) {
      unset($this->view['MODES']);
      $sql = "SELECT view_id, viewmode_id FROM %s WHERE view_id = %d";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableViewLinks, $this->view['view_id']))) {
        while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->view['MODES'][$row['viewmode_id']] = $row;
        }
      }
    }
  }

  /**
  * Save
  *
  * @access public
  * @return boolean
  */
  function saveView() {
    $data = array(
       'view_title' => $this->params['view_title'],
       'module_guid' => $this->params['module_guid'],
    );
    return (FALSE !== $this->databaseUpdateRecord($this->tableViews, $data, 'view_id', $this->view['view_id']));
  }

  /**
  * Add
  *
  * @access public
  * @return mixed
  */
  function createView() {
    $data = array(
       'view_title' => $this->params['view_title'],
       'module_guid' => $this->params['module_guid'],
    );
    return $this->databaseInsertRecord($this->tableViews, 'view_id', $data);
  }

  /**
  * Delete
  *
  * @access public
  * @return boolean
  */
  function deleteView() {
    return (
      FALSE !== $this->databaseDeleteRecord($this->tableViewLinks, 'view_id', $this->view['view_id']) &&
      FALSE !== $this->databaseDeleteRecord($this->tableViews, 'view_id', $this->view['view_id'])
    );
  }

  function addViewLink($viewId, $viewModeId) {
    $sql = "SELECT COUNT(*) FROM %s WHERE view_id = %d AND viewmode_id = %s";
    $params = array($this->tableViewLinks, $viewId, $viewModeId);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($res->fetchField() == 0) {
        $data = array('view_id'=>(int)$viewId, 'viewmode_id'=>(int)$viewModeId);
        return $this->databaseInsertRecord($this->tableViewLinks, NULL, $data);
      }
    }
  }

  function deleteViewLink($viewId, $viewModeId) {
    $filter = array('view_id'=>(int)$viewId, 'viewmode_id'=>(int)$viewModeId);
    return (FALSE !== $this->databaseDeleteRecord($this->tableViewLinks, $filter));
  }

  /**
  * Get XML buttons
  *
  * @access public
  * @return string XML
  */
  function getButtonsXML() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $toolbar = &new base_btnbuilder;
    $toolbar->images = &$this->images;

    $toolbar->addButton('Views', $this->getLink(array('cmd'=>'mode','mode'=>0)), 92, '', @$this->params['mode'] == 0);
    $toolbar->addButton('Filter', $this->getLink(array('cmd'=>'mode','mode'=>1)), 16, '', @$this->params['mode'] == 1);
    $toolbar->addSeperator();
    switch($this->params['mode']) {
      case 1 :
        $toolbar->addButton('New filter', $this->getLink(array('viewmode_id'=>0)), 70, '');
        if (isset($this->viewMode)) {
          $toolbar->addButton('Delete filter',
            $this->getLink(array('cmd'=>'viewmode_delete', 'viewmode_id'=>(int)$this->viewMode['viewmode_id'])), 10, '');
        }
        break;
      default :
        $toolbar->addButton('New view', $this->getLink(array('view_id'=>0)), 70, '');
        if (isset($this->view)) {
          $toolbar->addButton('Delete view',
            $this->getLink(array('cmd'=>'view_delete', 'view_id'=>(int)$this->view['view_id'])), 10, '');
        }
        break;
    }
    if ($str = $toolbar->getXML()) {
      $this->layout->add('<menu>'.$str.'</menu>', 'menus');
    }
  }

  function getXML() {
    switch($this->params['mode']) {
      case 1 :
        switch(@$this->params['cmd']) {
          case 'viewmode_delete' :
            $this->getViewModeDeleteDialog();
            break;
          default :
            $this->getViewModeDialog();
            break;
        }
        $this->getViewModesList();
        break;
      default :
        switch(@$this->params['cmd']) {
          case 'view_delete' :
            $this->getViewDeleteDialog();
            break;
          case 'view_unlink' :
            $this->getViewUnlinkDialog();
            break;
          case 'view_link' :
          case 'viewlink_edit' :
            break;
          default :
            $this->getViewDialog();
            break;
        }
        $this->getViewLinkInfos();
        $this->getViewsList();
        break;
    }
    $this->getButtonsXML();
  }

  function getViewModesList() {
    if (isset($this->viewModes) && is_array($this->viewModes) && count($this->viewModes) > 0) {
      $result = sprintf('<listview title="%s">', $this->_gt('Filter'));
      $result .= '<items>';
      foreach($this->viewModes as $viewMode) {
        $selected = (@$this->params['viewmode_id'] == $viewMode['viewmode_id']) ? ' selected="selected"' : '';
        $result .= sprintf('<listitem title="%s" href="%s" image="%s" %s/>',
          papaya_strings::escapeHTMLChars($viewMode['viewmode_ext']),
          $this->getLink(array('viewmode_id' => (int)$viewMode['viewmode_id'])),
          papaya_strings::escapeHTMLChars($this->images[16]), $selected);
      }
      $result .= '</items>';
      $result .= '</listview>';
      $this->layout->addLeft($result);
    }
  }

  function initializeViewModeDialog() {
    if (!(isset($this->dialogViewMode) && is_object($this->dialogViewMode))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      if (isset($this->viewMode) && is_array($this->viewMode)) {
        $data = $this->viewMode;
        $hidden = array(
          'viewmode_id' => (int)$this->viewMode['viewmode_id'],
          'cmd' => 'viewmode_edit'
        );
        $btnCaption = 'Edit';
      } else {
        $data = array();
        $hidden = array(
          'cmd' => 'viewmode_add'
        );
        $btnCaption = 'Add';
      }
      $viewModules = array();
      if (isset($this->outputModules) && is_array($this->outputModules) && count($this->outputModules) > 0) {
        foreach($this->outputModules as $module) {
          if (isset($this->moduleGroups[$module['modulegroup_id']])) {
            $viewModules[$module['module_guid']] = '['.$this->moduleGroups[$module['modulegroup_id']]['modulegroup_title'].'] '.
              $module['module_title'];
          } else {
            $viewModules[$module['module_guid']] = $module['module_title'];
          }
        }
        asort($viewModules);
      }
      $fields = array(
        'viewmode_ext' => array('Extension', '/^[a-z]{1,10}$/', TRUE, 'input', 10, ''),
        'viewmode_charset' => array('Charset', 'isAlphaNumChar', TRUE, 'combo', array('utf-8'=>'utf-8', 'iso-8859-1'=>'iso-8859-1'), '', 'utf-8'),
        'viewmode_contenttype' => array('Content-type', '~^(x-)?[a-z]+/[a-z_+-]+$~', TRUE, 'input', 50, '', 'text/html'),
        'module_guid' => array('Filter Module', 'isGuid', TRUE, 'combo', $viewModules, '')
      );
      $this->dialogViewMode = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->dialogViewMode->msgs = &$this->msgs;
      $this->dialogViewMode->loadParams();
      $this->dialogViewMode->dialogTitle = $this->_gt('Filter');
      $this->dialogViewMode->buttonTitle = $btnCaption;
      $this->dialogViewMode->dialogDoubleButtons = FALSE;
    }
  }

  function getViewModeDialog() {
    $this->initializeViewModeDialog();
    $this->layout->add($this->dialogViewMode->getDialogXML());
  }


  function getViewModeDeleteDialog() {
    if (isset($this->viewMode)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'viewmode_delete',
        'viewmode_id' => $this->viewMode['viewmode_id'],
        'confirm_delete'=>1,
      );
      $msg = sprintf($this->_gt('Delete filter "%s" (%s)?'),
         @papaya_strings::escapeHTMLChars($this->viewMode['viewmode_ext']), (int)$this->viewMode['viewmode_id']);
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->msgs = &$this->msgs;
      $dialog->dialogTitle = $this->_gt('Delete page');
      $dialog->buttonTitle = 'Delete';
      $this->layout->add($dialog->getMsgDialog());
    }
  }

  /**
  * Checks input params for view mode - extension has to be unique
  *
  * @access public
  * @return boolean;
  */
  function checkViewModeData() {
    if (in_array($this->params['viewmode_ext'], $this->reservedModes)) {
      $this->addMsg(MSG_ERROR, 'This filter extension is reserved. Please choose another.');
    } else {
      if (isset($this->viewMode) && is_array($this->viewMode)) {
        $sql = "SELECT COUNT(*) FROM %s WHERE viewmode_ext = '%s' AND (NOT(viewmode_id = %d))";
        $params = array($this->tableViewModes, $this->params['viewmode_ext'], $this->viewMode['viewmode_id']);
      } else {
        $sql = "SELECT COUNT(*) FROM %s WHERE viewmode_ext = '%s'";
        $params = array($this->tableViewModes, $this->params['viewmode_ext']);
      }
      if ($res = $this->databaseQueryFmt($sql, $params)) {
        list($count) = $res->fetchRow();
        if ($count > 0) {
          $this->addMsg(MSG_ERROR, 'The filter extension must be unique.');
        } else {
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  function getViewsList() {
    if (isset($this->moduleGroups) && is_array($this->moduleGroups) && (
        (isset($this->views) && is_array($this->views)) || (isset($this->brokenViews) && is_array($this->brokenViews))
        )) {
      $result = sprintf('<listview title="%s">', papaya_strings::escapeHTMLChars($this->_gt('Views')));
      $result .= '<items>';
      if (isset($this->brokenViews) && count($this->brokenViews) > 0) {
        $result .= sprintf('<listitem title="%s" image="%s"/>', papaya_strings::escapeHTMLChars($this->_gt('Unknown')),
          $this->images[57]);
        foreach($this->brokenViews as $view) {
          $selected = (@$this->params['view_id'] == $view['view_id']) ? ' selected="selected"' : '';
          $imageIdx = ($view['module_type'] == 'page') ? 92 : 5;
          $result .= sprintf('<listitem title="%s" node="empty" image="%s" href="%s" indent="1"%s/>',
            papaya_strings::escapeHTMLChars($view['view_title']), $this->images[$imageIdx],
            $this->getLink(array('cmd'=>'view_select', 'view_id' => (int)$view['view_id'])),
            $selected);
        }
      }
      foreach($this->moduleGroups as $moduleGroup) {
        if (isset($moduleGroup['VIEWS']) && count($moduleGroup['VIEWS']) > 0) {
          if (isset($this->openModuleGroups[$moduleGroup['modulegroup_id']])) {
            $node = 'open';
            $nodeHref = $this->getLink(array('cmd'=>'group_close', 'group_id' => (int)$moduleGroup['modulegroup_id']));
            $imageIdx = 57;
          } else {
            $node = 'close';
            $nodeHref = $this->getLink(array('cmd'=>'group_open', 'group_id' => (int)$moduleGroup['modulegroup_id']));
            $imageIdx = 56;
          }
          $result .= sprintf('<listitem title="%s" image="%s" href="%s" node="%s" nhref="%s"/>',
            papaya_strings::escapeHTMLChars($moduleGroup['modulegroup_title']), $this->images[$imageIdx],
             $nodeHref, $node, $nodeHref);
          if (isset($this->openModuleGroups[$moduleGroup['modulegroup_id']])) {
            foreach($moduleGroup['VIEWS'] as $view) {
              $selected = (@$this->params['view_id'] == $view['view_id']) ? ' selected="selected"' : '';
              $imageIdx = ($view['module_type'] == 'page') ? 92 : 5;
              $result .= sprintf('<listitem title="%s" node="empty" image="%s" href="%s" indent="1"%s/>',
                papaya_strings::escapeHTMLChars($view['view_title']), $this->images[$imageIdx],
                $this->getLink(array('cmd'=>'view_select', 'view_id' => (int)$view['view_id'])),
                $selected);
            }
          }
        }
      }
      $result .= '</items>';
      $result .= '</listview>';
      $this->layout->addLeft($result);
    }
  }

  function getViewLinkInfos() {
    if (isset($this->view) && isset($this->viewModes)) {
      $result = sprintf('<listview title="%s">', papaya_strings::escapeHTMLChars($this->_gt('Filter')));
      $result .= '<cols>';
      $result .= sprintf('<col>%s</col>', papaya_strings::escapeHTMLChars($this->_gt('Extension')));
      $result .= '<col/>';
      $result .= sprintf('<col align="center">%s</col>', papaya_strings::escapeHTMLChars($this->_gt('Linked')));
      $result .= '</cols>';
      $result .= '<items>';
      foreach($this->viewModes as $viewMode) {
        $href = $this->getLink(array('cmd'=>'viewlink_edit', 'view_id'=>$this->view['view_id'], 'viewmode_id'=>$viewMode['viewmode_id']));
        $selected = (@$this->params['viewmode_id'] == $viewMode['viewmode_id']) ? ' selected="selected"' : '';
        $result .= sprintf('<listitem title="%s" href="%s" image="%s"%s>',
          papaya_strings::escapeHTMLChars($viewMode['viewmode_ext']), $href, $this->images[16], $selected);
        if (isset($this->view['MODES'][$viewMode['viewmode_id']])) {
          $result .= sprintf('<subitem align="center"><a href="%s"><glyph src="%s" alt="%s"/></a></subitem>',
            $href, $this->images[14], papaya_strings::escapeHTMLChars($this->_gt('Edit')));
          $activeImage = 27;
          $activeString = 'Yes';
          $cmd = 'view_unlink';
        } else {
          $result .= '<subitem/>';
          $activeImage = 28;
          $activeString = 'No';
          $cmd = 'view_link';
        }
        $result .= sprintf('<subitem align="center"><a href="%s"><glyph src="%s" alt="%s"/></a></subitem>',
          $this->getLink(array('cmd'=>$cmd, 'view_id'=>$this->view['view_id'], 'viewmode_id'=>$viewMode['viewmode_id'])),
          $this->images[$activeImage], papaya_strings::escapeHTMLChars($this->_gt($activeString)));
        $result .= '</listitem>';
      }
      $result .= '</items>';
      $result .= '</listview>';
      $this->layout->addRight($result);
    }
  }

  function initializeViewDialog() {
    if (!(isset($this->dialogView) && is_object($this->dialogView))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      if (isset($this->view) && is_array($this->view)) {
        $data = $this->view;
        $hidden = array(
          'view_id' => (int)$this->view['view_id'],
          'cmd' => 'view_edit'
        );
        $btnCaption = 'Edit';
        $css = 'large';
      } else {
        $data = array();
        $hidden = array(
          'cmd' => 'view_add'
        );
        $btnCaption = 'Add';
        $css = 'x-large';
      }
      $viewModules = array();
      if (isset($this->modules) && is_array($this->modules) && count($this->modules) > 0) {
        foreach($this->modules as $module) {
          if (isset($this->moduleGroups[$module['modulegroup_id']])) {
            $viewModules[$module['module_guid']] = '['.$this->moduleGroups[$module['modulegroup_id']]['modulegroup_title'].
              ' - '.$module['module_type'].'] '.$module['module_title'];
          } else {
            $viewModules[$module['module_guid']] = $module['module_title'];
          }
        }
        asort($viewModules);
      }
      $fields = array(
        'view_title' => array('Title', 'isNoHTML', TRUE, 'input', 100, ''),
        'module_guid' => array('Module', 'isGuid', TRUE, 'combo', $viewModules, '')
      );
      $this->dialogView = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->dialogView->msgs = &$this->msgs;
      $this->dialogView->loadParams();
      $this->dialogView->dialogTitle = $this->_gt('View');
      $this->dialogView->buttonTitle = $btnCaption;
      $this->dialogView->inputFieldSize = $css;
      $this->dialogView->dialogDoubleButtons = FALSE;
    }
  }

  function getViewDialog() {
    $this->initializeViewDialog();
    $this->layout->add($this->dialogView->getDialogXML());
    $this->layout->add($this->getViewUseList());
  }
  
  function loadViewCount($tableName) {
    $result = array();
    $sql = "SELECT COUNT(*) AS counted, lng_id 
              FROM %s 
             WHERE view_id = '%d'
             GROUP BY lng_id";
    if ($res = $this->databaseQueryFmt($sql, array($tableName, $this->view['view_id']))) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $result[$row['lng_id']] = $row['counted'];
      }
    }
    return $result;
  }
  
  function getViewUseList() {
    if (isset($this->view) && $this->view['view_id'] > 0) {
      $result = sprintf('<listview title="%s" width="100%%">'.LF, $this->_gt('View Use'));
      $result .= '<cols>'.LF;
      $result .= sprintf('<col/>'.LF);
      $result .= sprintf('<col align="center">%s</col>'.LF, $this->_gt('Current'));
      $result .= sprintf('<col align="center">%s</col>'.LF, $this->_gt('Published'));
      $result .= sprintf('<col align="center">%s</col>'.LF, $this->_gt('Versions'));
      $result .= '</cols>'.LF;
      $result .= '<items>'.LF;
      $result .= sprintf('<listitem title="%s" image="%s">', $this->_gt('Pages'), $this->images[92]);
      $result .= '<subitem/><subitem/><subitem/></listitem>'.LF;
      $editUse = $this->loadViewCount(PAPAYA_DB_TBL_TOPICS_TRANS);
      $publicUse = $this->loadViewCount(PAPAYA_DB_TBL_TOPICS_PUBLIC_TRANS);
      $versionUse = $this->loadViewCount(PAPAYA_DB_TBL_TOPICS_VERSIONS_TRANS);
  
      if (isset($this->lngSelect->languages) && is_array($this->lngSelect->languages)) {
        foreach($this->lngSelect->languages as $lngId=>$lng) {
          $result .= sprintf('<listitem title="%s" indent="2">'.LF,
            papaya_strings::escapeHTMLChars($lng['lng_title'].' ('.$lng['lng_short'].')'));
          $result .= sprintf('<subitem align="center">%d</subitem>'.LF, @(int)$editUse[$lngId]);
          $result .= sprintf('<subitem align="center">%d</subitem>'.LF, @(int)$publicUse[$lngId]);
          $result .= sprintf('<subitem align="center">%d</subitem>'.LF, @(int)$versionUse[$lngId]);
          $result .= '</listitem>'.LF;
        }
      }
      $result .= sprintf('<listitem title="%s" image="%s">', $this->_gt('Boxes'), $this->images[5]);
      $result .= '<subitem/><subitem/><subitem/></listitem>'.LF;
      $editUse = $this->loadViewCount(PAPAYA_DB_TBL_BOX_TRANS);
      $publicUse = $this->loadViewCount(PAPAYA_DB_TBL_BOX_PUBLIC_TRANS);
      $versionUse = $this->loadViewCount(PAPAYA_DB_TBL_BOX_VERSIONS_TRANS);
  
      if (isset($this->lngSelect->languages) && is_array($this->lngSelect->languages)) {
        foreach($this->lngSelect->languages as $lngId=>$lng) {
          $result .= sprintf('<listitem title="%s" indent="2">'.LF,
            papaya_strings::escapeHTMLChars($lng['lng_title'].' ('.$lng['lng_short'].')'));
          $result .= sprintf('<subitem align="center">%d</subitem>'.LF, @(int)$editUse[$lngId]);
          $result .= sprintf('<subitem align="center">%d</subitem>'.LF, @(int)$publicUse[$lngId]);
          $result .= sprintf('<subitem align="center">%d</subitem>'.LF, @(int)$versionUse[$lngId]);
          $result .= '</listitem>'.LF;
        }
      }
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
      return $result;
    }
    return '';
  }

  /**
  * Get delete formular
  *
  * @access public
  * @return string
  */
  function getViewDeleteDialog() {
    if (isset($this->view) && is_array($this->view)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'view_delete',
        'view_id' => $this->view['view_id'],
        'confirm_delete' =>1,
      );
      $msg = sprintf($this->_gt('Delete view "%s" (%s)?'), papaya_strings::escapeHTMLChars($this->view['view_title']), (int)$this->view['view_id']);
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Delete';
      $this->layout->add($dialog->getMsgDialog());
    }
  }
  /**
  * Get delete formular
  *
  * @access public
  * @return string
  */
  function getViewUnlinkDialog() {
    if (isset($this->viewLink) && is_array($this->viewLink)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'view_unlink',
        'view_id' => $this->viewLink['view_id'],
        'viewmode_id' => $this->viewLink['viewmode_id'],
        'confirm_unlink' => 1,
      );
      $msg = sprintf($this->_gt('Unlink filter "%s" (%d) from view "%s" (%d)?'),
        papaya_strings::escapeHTMLChars($this->viewLink['viewmode_ext']), (int)$this->viewLink['viewmode_id'] ,
        papaya_strings::escapeHTMLChars($this->view['view_title']), (int)$this->viewLink['view_id']);
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Unlink';
      $this->layout->add($dialog->getMsgDialog());
    }
  }
}
?>