<?php
/**
* MySQL database access classes (ext/mysql)
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: mysql.php,v 1.16 2006/06/06 11:35:42 weinert Exp $
*/

/**
* basic database connection and result class
*/
require_once(dirname(__FILE__).'/base.php');

/**
* DB-abstraction layer - connection object MySQLDB
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class dbcon_mysql extends dbcon_base {

  var $mysqlVersion = 0;

  function extensionFound() {
    return (bool)extension_loaded('mysql');
  }
  
  /**
  * Establish connection to database
  *
  * @return resource $this->databaseConnection connection ID
  */
  function connect() {
    if (!(isset($this->databaseConnection) && is_resource($this->databaseConnection))) {
      if (isset($this->databaseConfiguration['protocol']) && ($this->databaseConfiguration['protocol'] == 'unix')) {
        $server = 'localhost:'.$this->databaseConfiguration['socket'];
      } else {
        $server = ((is_numeric($this->databaseConfiguration['port'])) ? ($this->databaseConfiguration['hostspec'].':'.$this->databaseConfiguration['port']) :
          $this->databaseConfiguration['hostspec']);
      }
      if ($this->databaseConnection = @mysql_connect($server, $this->databaseConfiguration['username'], $this->databaseConfiguration['password'])) {
        if (!@mysql_select_db($this->databaseConfiguration['database'], $this->databaseConnection)) {
          unset($this->databaseConnection);
        }
        if (preg_match('/^(\d+)\.(\d+)\.(\d+)/', @mysql_get_server_info($this->databaseConnection), $regs)) {
          $this->mysqlVersion = (int)$regs[1]*10000 + (int)$regs[2] * 100 + (int)$regs[3];
          if ($this->mysqlVersion >= 40100) {
            @mysql_query("SET NAMES utf8", $this->databaseConnection);
            if (defined('PAPAYA_DATABASE_COLLATION')) {
              @mysql_query("SET CHARACTER SET ".PAPAYA_DATABASE_COLLATION, $this->databaseConnection);
            }
          }
        }
      }
      return (isset($this->databaseConnection) && is_resource($this->databaseConnection));
    } else {
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Provides error messages of last database connection
  *
  * @return array error number, error message and last sql query
  */
  function getError() {
    if (isset($this->databaseConnection) && is_resource($this->databaseConnection)) {
      return array('no'=>mysql_errno($this->databaseConnection), 'str'=>mysql_error($this->databaseConnection), 'lastquery'=>$this->lastSQLQuery);
    } else {
      return array('no'=>NULL, 'str'=>NULL, 'lastquery'=>$this->lastSQLQuery);
    }
  }

  /**
  * Return an explanation of the last SELECT-SQL-query
  *
  * Explan the last query, number of lines etc.
  *
  * @return mixed Result or FALSE, if last request wasn't a SELECT query
  */
  function getExplain() {
    if (preg_match('/^SELECT\s/i', $this->lastSQLQuery)) {
      if ($res = @mysql_query('EXPLAIN  '.$this->lastSQLQuery, $this->databaseConnection)) {
        $result = '<table border="1" cellspacing="0" cellpadding="2" width="100%">';
        $result .= '<tr>';
        $result .= '<td>id</td>';
        $result .= '<td>select_type</td>';
        $result .= '<td>table</td>';
        $result .= '<td>type</td>';
        $result .= '<td>possible_keys</td>';
        $result .= '<td>key</td>';
        $result .= '<td>key_len</td>';
        $result .= '<td>ref</td>';
        $result .= '<td>rows</td>';
        $result .= '<td>Extra</td>';
        $result .= '</tr>';
        while($row = @mysql_fetch_assoc($res)) {
          $result .= '<tr>';
          $result .= (isset($row['id'])) ? '<td>'.$row['id'].'</td>' : '<td>&nbsp;</td>';
          $result .= (isset($row['select_type'])) ? '<td>'.$row['select_type'].'</td>' : '<td>&nbsp;</td>';
          $result .= (isset($row['table'])) ? '<td>'.$row['table'].'</td>' : '<td>&nbsp;</td>';
          $result .= (isset($row['type'])) ? '<td>'.$row['type'].'</td>' : '<td>&nbsp;</td>';
          $result .= (isset($row['possible_keys'])) ? '<td>'.$row['possible_keys'].'</td>' : '<td>&nbsp;</td>';
          $result .= (isset($row['key'])) ? '<td>'.$row['key'].'</td>' : '<td>&nbsp;</td>';
          $result .= (isset($row['key_len'])) ? '<td>'.$row['key_len'].'</td>' : '<td>&nbsp;</td>';
          $result .= (isset($row['ref'])) ? '<td>'.$row['ref'].'</td>' : '<td>&nbsp;</td>';
          $result .= (isset($row['rows'])) ? '<td>'.$row['rows'].'</td>' : '<td>&nbsp;</td>';
          $result .= (isset($row['Extra']) && $row['Extra'] != '') ? '<td>'.$row['Extra'].'</td>' : '<td>&nbsp;</td>';
          $result .= '</tr>';
        }
        $result .= '</table>';
        return $result;
      }
    }
    return FALSE;
  }


  /**
  * String ecsaping for MySQL use
  *
  * @param mixed $value Value to escape
  * @return string escaped value.
  */
  function escapeStr($value) {
    return mysql_escape_string((string)$value);
  }

  /**
  * Execute MySQL-query
  *
  * @param string $sql SQL-String with query
  * @param integer $max maximum number of returned records
  * @param integer $offset Offset
  * @param boolean $debug Debug Query?
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function &query($sql, $max=NULL, $offset=NULL, $debug = FALSE) {
    if (@get_class($this->lastResult) == 'dbresult_mysql') {
      $this->lastResult->free();
    }
    $absCount = 0;
    $queryRowCount = FALSE;
    if (isset($max) && strpos(trim(strtoupper($sql)), 'SELECT') === 0) {
      $countSql = '';
      if ($this->mysqlVersion >= 40100) {
        $sql = 'SELECT SQL_CALC_FOUND_ROWS '.substr(trim($sql), 6);
        $queryRowCount = TRUE;
      } else {
        if (preg_match('~^\s*(SELECT.*)(\bFROM\b.*?)(\bORDER\s+BY.*)?$~si', $sql, $regs)) {
          $countSql = 'SELECT COUNT(*) '.$regs[2];
        }
      }
      if ($countSql != '' && ($res = mysql_query($countSql, $this->databaseConnection))) {
        if ($row = mysql_fetch_row($res)) {
          $absCount = $row[0];
        }
        @mysql_free_result($res);
      }
      $sql .= (isset($offset)) ? ' LIMIT '.(int)$offset.','.(int)$max : ' LIMIT '.(int)$max;
    }
    $this->lastSQLQuery = $sql;
    $res = @mysql_query($sql, $this->databaseConnection);
    if ($res) {
      if (@is_resource($res)) {
        $this->lastResult = &new dbresult_mysql($res);
        $this->lastResult->setLimit($max, $offset);
        if ($queryRowCount) {
          if ($res2 = mysql_query("SELECT FOUND_ROWS()", $this->databaseConnection)) {
            if ($row = mysql_fetch_row($res2)) {
              $absCount = $row[0];
            }
            mysql_free_result($res2);
          }
        }
        $this->lastResult->_absCount = $absCount;
        return $this->lastResult;
      } else {
        $result = @mysql_affected_rows($this->databaseConnection);
        return $result;
      }
    } else {
      $result = FALSE;
      return $result;
    }
  }

  /**
  * Insert record into table
  *
  * @param string $table table
  * @param string $idField primary key value
  * @param array $values insert values
  * @return mixed FALSE or Id of new record
  */
  function insertRecord($table, $idField, $values = NULL) {
    if (isset($values) && is_array($values) && count($values) > 0) {
      $fieldString = '';
      $valueString = '';
      foreach($values as $field=>$value) {
        if (isset($idField) && $idField == $field) {
          continue;
        }
        $fieldString .= $this->escapeStr($field).', ';
        if ($value === NULL) {
          $valueString .= "NULL, ";
        } else {
          $valueString .= "'".$this->escapeStr($value)."', ";
        }
      }
      $fieldString = substr($fieldString, 0, -2);
      $valueString = substr($valueString, 0, -2);
      $sql = 'INSERT INTO '.$this->escapeStr($table).' ('.$fieldString.') VALUES ('.$valueString.')';
      if ($this->query($sql)) {
        if (isset($idField)) {
          if ($res = $this->query('SELECT LAST_INSERT_ID()')) {
            $new = $res->fetchField();
            return $new;
          }
        } else {
          return mysql_affected_rows($this->databaseConnection);
        }
      }
    }
    return FALSE;
  }

  function insertRecords($table, $values) {
    $baseSQL = 'INSERT INTO '.$this->escapeStr($table).' ';
    $valueString = '';
    $lastFields = NULL;
    if (isset($values) && is_array($values) && count($values) > 0) {
      foreach($values as $data) {
        if (is_array($data) && count($data) > 0) {
          $fields = array();
          $valueData = array();
          foreach($data as $key=>$val) {
            $fields[] = $this->escapeStr($key);
            $valueData[] = $this->escapeStr($val);
          }
          if (!isset($lastFields)) {
            $valueString = "('".implode("','", $valueData)."'), ";
            $lastFields = $fields;
          } elseif (count(array_diff($fields, $lastFields)) == 0) {
            $valueString .= "('".implode("','", $valueData)."'), ";
          } else {
            if (trim($valueString) != '') {
              $sql = $baseSQL."(".implode(",", $lastFields).") VALUES ".substr($valueString, 0, -2);
              if (FALSE === $this->query($sql)) {
                return FALSE;
              }
            }
            $valueString = '';
            $lastFields = $fields;
          }
        }
      }
      if (trim($valueString) != '') {
        $sql = $baseSQL."(".implode(",", $lastFields).") VALUES ".substr($valueString, 0, -2);
        if (FALSE !== $this->query($sql)) {
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  /**
  * Update records via filter
  *
  * @param string $table table name
  * @param array $values update values
  * @param mixed $filter Filter string without WHERE condition
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function updateRecord($table, $values, $filter) {
    if (isset($values) && is_array($values) && count($values) > 0) {
      $sql = '';
      foreach($values as $col=>$val) {
        $fieldName = trim($col);
        if (preg_match('/[^`]+/', $fieldName)) {
          $sql .= " ".$this->escapeStr($fieldName)." = '".$this->escapeStr($val)."', ";
        }
      }
      if (!empty($sql)) {
        $sql = "UPDATE ".$this->escapeStr($table)." SET ".substr($sql, 0, -2)." WHERE ".$this->getSQLCondition($filter);
        $this->lastSQLQuery = $sql;
        return $this->query($sql);
      } else {
        $this->lastSQLQuery = 'NO VALID DATA';
      }
    } else {
      $this->lastSQLQuery = 'NO DATA';
    }
    return FALSE;
  }

  /**
  * Delete records by filter
  *
  * @param string $table table name
  * @param string $filter Filter string without WHERE condition
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function deleteRecord($table, $filter) {
    $sql = "DELETE FROM $table WHERE ".$this->getSQLCondition($filter);
    return $this->query($sql);
  }

  /**
  * Get table names
  *
  * @access public
  * @return array
  */
  function queryTableNames() {
    $sql = "SHOW TABLES";
    $result = array();
    if ($res = $this->query($sql)) {
      while($row = $res->fetchRow()) {
        $result[] = $row[0];
      }
    }
    return $result;
  }

  /**
  * Parse MySQL Field type
  *
  * @param string $typeString
  * @access private
  * @return array
  */
  function parseMySQLFieldType($typeString) {
    $p = strpos($typeString, '(');
    if ($p !== FALSE) {
      $mysqlType = trim(substr($typeString, 0, $p));
      $size = trim(substr($typeString, $p+1, strpos($typeString, ')')-$p-1));
    } else {
      $mysqlType = trim($typeString);
      $size = 0;
    }
    switch (strtoupper($mysqlType)) {
      case 'TINYINT' :
      case 'SMALLINT' :
        $type = 'integer';
        $size = 2;
        break;
      case 'MEDIUMINT' :
      case 'INT' :
      case 'INTEGER' :
        $type = 'integer';
        $size = 4;
        break;
      case 'BIGINT' :
        $size = 8;
        $type = 'integer';
        break;
      case 'FLOAT' :
      case 'DOUBLE' :
      case 'REAL' :
      case 'DECIMAL' :
      case 'NUMERIC' :
        $type = 'float';
        break;
      case 'TINYTEXT' :
        $type = 'string';
        $size = '255';
        break;
      case 'TEXT' :
        $type = 'string';
        $size = '65535';
        break;
      case 'MEDIUMTEXT' :
        $type = 'string';
        $size = '16777215';
        break;
      case 'LONGTEXT' :
        $type = 'string';
        $size = '4294967295';
        break;
      case 'CHAR' :
      case 'VARCHAR' :
      default :
        $type = 'string';
        break;
    }
    return array($type, $size);
  }

  /**
  * Parse MySQL field data
  *
  * @param array &$row
  * @access private
  * @return array
  */
  function parseMySQLFieldData(&$row) {
    $type = $this->parseMySQLFieldType($row['Type']);
    if ($type[0] == 'integer') {
      $default = (int)$row['Default'];
    } elseif ($default = '' && strtolower($row['Null']) == 'yes') {
      $default = NULL;
    } else {
      $default = $row['Default'];
    }
    return array(
      'name' => $row['Field'],
      'type' => $type[0],
      'size' => $type[1],
      'null' => strtolower($row['Null'] == 'yes') ? 'yes' : 'no',
      'default' => $default,
      'autoinc' => ($row['Extra'] == 'auto_increment') ? 'yes' : 'no'
    );
  }

  /**
  * MySQL field type
  *
  * @param string $type
  * @param string $size
  * @access private
  * @return string
  */
  function getMySQLFieldType($type, $size) {
    switch (strtolower(trim($type))) {
      case 'integer' :
        $size = ($size > 0) ? (int)$size : 1;
        if ($size <= 2) {
          $result = "SMALLINT";
        } else if ($size <= 4) {
          $result = "INT";
        } else {
          $result = "BIGINT";
        }
        break;
      case 'float' :
        list($before,$after) = explode(',',$size);
        $before = ($before > 0) ? (int)$before : 1;
        $after = (int)$after;
        $result = "DECIMAL(".$before.','.$after.")";
        break;
      case 'string' :
      default :
        $size = ($size > 0) ? (int)$size : 1;
        if ($size <= 4) {
          $result = "CHAR(".$size.")";
        } else if ($size <= 255) {
          $result = "VARCHAR(".$size.")";
        } else if ($size <= 65535) {
          $result = "TEXT";
        } else if ($size <= 16777215) {
          $result = "MEDIUMTEXT";
        } else {
          $result = "LONGTEXT";
        }
        break;
    }
    return $result;
  }

  /**
  * Get MySQL field extras
  *
  * @param array $field
  * @access private
  * @return string
  */
  function getMySQLFieldExtras(&$field, $allowAutoInrement = FALSE) {
    if (isset($field['null']) && $field['null'] == 'yes') {
      $default = NULL;
      $notNullStr = '';
    } else {
      $default = '';
      $notNullStr = ' NOT NULL';
    }
    if (isset($field['default'])) {
      $default = $field['default'];
    }
    $defaultStr = '';
    if (isset($default)) {
      switch(strtolower($field['type'])) {
        case 'integer' :
          $defaultStr = " DEFAULT '".(int)$default."'";
          break;
        case 'float' :
          $defaultStr = " DEFAULT '".(double)$default."'";
          break;
        default :
          $defaultStr = " DEFAULT '".$this->escapeStr($default)."'";
          break;
      }
    }

    if (($allowAutoInrement) && isset($field['autoinc']) && $field['autoinc'] == 'yes') {
      $autoIncrementField = TRUE;
      $autoIncrementString = ' auto_increment';
      $defaultStr = '';
    } else {
      $autoIncrementString = '';
    }

    return $defaultStr.$notNullStr.$autoIncrementString;
  }

  /**
  * Query table structure
  *
  * @param string $tableName
  * @param string $tablePrefix optional, default value ''
  * @access public
  * @return array
  */
  function queryTableStructure($tableName, $tablePrefix = '') {
    $fields = array();
    if ($tablePrefix) {
      $table = $tablePrefix.'_'.$tableName;
    } else {
      $table = $tableName;
    }
    $sql = "SHOW FIELDS FROM $table";
    if ($res = $this->query($sql)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $fields[$row['Field']] = $this->parseMySQLFieldData($row);
      }
    }
    $keys = array();
    $sql = "SHOW KEYS FROM $table";
    if ($res = $this->query($sql)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $keyName = $row['Key_name'];
        $keys[$keyName]['name'] = $keyName;
        $keys[$keyName]['unique'] = ($row['Non_unique'] == 0) ? 'yes' : 'no';
        $keys[$keyName]['fields'][$row['Seq_in_index']] = $row['Column_name'];
        if (isset($row['Sub_part'])) {
          $keys[$keyName]['keysize'][$row['Column_name']] = (int)$row['Sub_part'];
        } elseif ($fields[$row['Column_name']] == 'string' && $fields[$row['Column_name']] >= 255) {
          $keys[$keyName]['keysize'][$row['Column_name']] = 255;
        }
        if (isset($row['Index_type']) && $row['Index_type'] == 'FULLTEXT') {
          $keys[$keyName]['fulltext'] = 'yes';
        } elseif ($row['Comment'] == 'FULLTEXT') {
          $keys[$keyName]['fulltext'] = 'yes';
        } else {
          $keys[$keyName]['fulltext'] = 'no';
        }
      }
    }
    return array(
      'name' => $tableName,
      'fields' => $fields,
      'keys' => $keys
      );
  }

  /**
  * Create given table
  *
  * @param string $tableData
  * @param string $tablePrefix
  * @access public
  * @return boolean
  */
  function createTable(&$tableData, $tablePrefix) {
    if (is_array($tableData['fields']) && trim($tableData['name']) != '') {
      if ($tablePrefix) {
        $table = $tablePrefix.'_'.trim($tableData['name']);
      } else {
        $table = trim($tableData['name']);
      }
      $sql = 'CREATE TABLE `'.$this->escapeStr(strtolower($table)).'` ('.LF;
      $autoIncrementField = FALSE;
      foreach ($tableData['fields'] as $field) {
        $sql .= '  `'.strtolower($field['name']).'` '.
          $this->getMySQLFieldType($field['type'], $field['size']).
          $this->getMySQLFieldExtras($field, !$autoIncrementField).",\n";
      }
      $fulltextIndex = FALSE;
      if (is_array($tableData['keys'])) {
        if (isset($tableData['keys']['PRIMARY'])) {
          $sql .= '  PRIMARY KEY (`'.implode('`,`', $tableData['keys']['PRIMARY']['fields'])."`),\n";
        }
        foreach($tableData['keys'] as $keyName=>$key) {
          if ($keyName != 'PRIMARY') {
            if (isset($key['unique']) && $key['unique'] == 'yes') {
              $sql .= '  UNIQUE ';
            } elseif (isset($key['fulltext']) && $key['fulltext'] == 'yes') {
              $sql .= '  FULLTEXT ';
              $fulltextIndex = TRUE;
            } else {
              $sql .= '  KEY ';
            }
            $fieldStr = '(';
            foreach ($key['fields'] as $fieldName) {
              if (isset($key['keysize'][$fieldName]) && $key['keysize'][$fieldName] > 0) {
                $fieldStr .= '`'.$this->escapeStr($fieldName).'` ('.(int)$key['keysize'][$fieldName].'), ';
              } else {
                $fieldStr .= '`'.$this->escapeStr($fieldName).'`, ';
              }
            }
            $sql .= '`'.$keyName.'` '.substr($fieldStr, 0, -2)."),\n";
          }
        }
      }
      $sql = substr($sql, 0, -2)."\n) ";
      if ($fulltextIndex) {
        if ($this->mysqlVersion >= 40102) {
          $sql .= 'ENGINE=MyISAM ';
        } else {
          $sql .= 'TYPE=MyISAM ';        
        }
      }
      if ($this->mysqlVersion >= 40100) {
        $sql .= 'DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci';
      }
      return ($this->query($sql) !== FALSE);
    }
    return FALSE;
  }

  /**
  * Add Field
  *
  * @param string $table
  * @param array &$fieldData
  * @access public
  * @return booblean
  */
  function addField($table, &$fieldData) {
    $sql = "ALTER TABLE `".$this->escapeStr($table)."` ADD COLUMN `".$this->escapeStr($fieldData['name'])."` ".
          $this->getMySQLFieldType($fieldData['type'], $fieldData['size']).
          $this->getMySQLFieldExtras($fieldData);
    return ($this->query($sql) !== FALSE);
  }

  /**
  * Change Field
  *
  * @param string $table
  * @param array &$fieldData
  * @access public
  * @return booblean
  */
  function changeField($table, &$fieldData) {
    $allowAutoInrement = FALSE;
    if (isset($fieldData['autoinc']) && $fieldData['autoinc'] == 'yes') {
      $sql = "SHOW COLUMNS FROM `".$this->escapeStr($table)."`";
      if ($res = $this->query($sql)) {
        $autoIncrementField = NULL;
        $fieldExists = FALSE;
        while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          if ($row['Field'] == $fieldData['name']) {
            $allowAutoInrement = (trim($row['Key']) != '');
            if (strtolower($row['Extra']) == 'auto_increment') {
              unset($autoIncrementField);
              break;
            }
            $fieldExists = TRUE;
          } elseif (strtolower($row['Extra']) == 'auto_increment') {
            $autoIncrementField = $this->parseMySQLFieldData($row);
          }
          if ($fieldExists && isset($autoIncrementField)) {
            break;
          }
        }
        $res->free();
        if (isset($autoIncrementField)) {
          $autoIncrementField['autoinc'] = 'no';
          $sql = "ALTER TABLE `".$this->escapeStr($table)."` MODIFY COLUMN `".$this->escapeStr($autoIncrementField['name'])."` ".
            $this->getMySQLFieldType($autoIncrementField['type'], $autoIncrementField['size']).
            $this->getMySQLFieldExtras($autoIncrementField);
          $this->query($sql);
        }
      }
    }
    $sql = "ALTER TABLE `".$this->escapeStr($table)."` MODIFY COLUMN `".$this->escapeStr($fieldData['name'])."` ".
          $this->getMySQLFieldType($fieldData['type'], $fieldData['size']).
          $this->getMySQLFieldExtras($fieldData, $allowAutoInrement);
    return ($this->query($sql) !== FALSE);
  }

  /**
  * Drop field
  *
  * @param string $table
  * @param string $field fieldname
  * @access public
  * @return booblean
  */
  function dropField($table, $field) {
    $sql = "ALTER TABLE `".$this->escapeStr($table)."` DROP COLUMN `".$this->escapeStr($field)."`";
    return ($this->query($sql) !== FALSE);
  }

  /**
  * Add index
  *
  * @param table $table
  * @param array &$index
  * @access public
  * @return boolean
  */
  function addIndex($table, &$index) {
    return $this->changeIndex($table, $index, FALSE);
  }

  /**
  * Change Index
  *
  * @param atring $table
  * @param array &$index
  * @param boolean $dropCurrent optional, default value TRUE
  * @access public
  * @return boolean
  */
  function changeIndex($table, &$index, $dropCurrent = TRUE) {
    if (isset($index['fields']) && is_array($index['fields'])) {
      $sql = "SHOW COLUMNS FROM `".$this->escapeStr($table)."`";
      if ($res = $this->query($sql)) {
        $needed = count($index['fields']);
        $indb = 0;
        while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          if (in_array($row['Field'], $index['fields'])) {
            if (++$indb >= $needed) break;
          }
        }
        $res->free();

        if ($indb >= $needed) {
          $fields = '(';
          foreach ($index['fields'] as $fieldName) {
            if (isset($index['keysize'][$fieldName]) && $index['keysize'][$fieldName] > 0) {
              $fields .= '`'.$this->escapeStr($fieldName).'` ('.(int)$index['keysize'][$fieldName].'), ';
            } else {
              $fields .= '`'.$this->escapeStr($fieldName).'`, ';
            }
          }
          $fields = substr($fields, 0, -2).")";
          $drop = ($dropCurrent) ? " DROP INDEX `".$this->escapeStr($index['name'])."`," : '';
          if ($index['name'] == 'PRIMARY') {
            $sql = "ALTER TABLE `".$this->escapeStr($table)."`".$drop." ADD PRIMARY KEY ".$fields;
          } elseif (isset($index['fulltext']) && $index['fulltext'] == 'yes') {
            $sql = "ALTER TABLE `".$this->escapeStr($table)."`".$drop." ADD FULLTEXT `".
              $this->escapeStr($index['name'])."` ".$fields;
          } elseif (isset($index['unique']) && $index['unique'] == 'yes') {
            $sql = "ALTER TABLE `".$this->escapeStr($table)."`".$drop." ADD UNIQUE `".
              $this->escapeStr($index['name'])."` ".$fields;
          } else {
            $sql = "ALTER TABLE `".$this->escapeStr($table)."`".$drop." ADD INDEX `".
              $this->escapeStr($index['name'])."` ".$fields;
          }
          return ($this->query($sql) !== FALSE);
        }
      }
    }
    return FALSE;
  }

  /**
  * Drop Index
  *
  * @param string $table
  * @param string $name
  * @access public
  * @return boolean
  */
  function dropIndex($table, $name) {
    if ($name == 'PRIMARY') {
      $sql = "ALTER TABLE `".$this->escapeStr($table)."` DROP PRIMARY KEY";
    } else {
      $sql = "ALTER TABLE `".$this->escapeStr($table)."` DROP INDEX `".$this->escapeStr($name)."`";
    }
    return ($this->query($sql) !== FALSE);
  }

  function getSQLFunctionParams($params) {
    $result = '';
    for($i=0;$i<count($params);$i+=2) {
      if (isset($params[$i+1]) && $params[$i+1] == FALSE) {
        $result .= $params[$i].',';
      } else {
        $result .= "'".$this->escapeStr($params[$i])."',";
      }
    }
    return substr($result, 0, -1);
  }

  /**
  * DBMS spezific SQL source
  *
  * @param $function sql function
  * @param $params params
  * @access public
  * @return mixed sql string or FALSE
  */
  function getSQLSource($function, $params) {
    switch (strtoupper($function)) {
      case 'CONCAT' :
        return ' CONCAT('.$this->getSQLFunctionParams($params).')';
        break;
      case 'SUBSTRING' :
        return ' SUBSTRING('.$this->getSQLFunctionParams($params).')';
      case 'LOWER' :
        return ' LOWER('.$this->getSQLFunctionParams($params).')';
      case 'UPPER' :
        return ' UPPER('.$this->getSQLFunctionParams($params).')';
      case 'LOCATE' :
        return ' LOCATE('.$this->getSQLFunctionParams($params).')';
    }
    return FALSE;
  }


  function compareFieldStructure(&$xmlField, &$databaseField) {
    if (@$xmlField['type'] != @$databaseField['type']) {
      return TRUE;
    } elseif (@$xmlField['size'] != @$databaseField['size']) {
      return TRUE;
    } elseif (@$xmlField['null'] != @$databaseField['null']) {
      return TRUE;
    } elseif (@$xmlField['autoinc'] == 'yes' && @$databaseField['autoinc'] != 'yes') {
      return TRUE;
    } elseif (@$xmlField['default'] != @$databaseField['default']) {
      return TRUE;
    }
    return FALSE;
  }

  function compareKeyStructure(&$xmlKey, &$databaseKey) {
    if ((@$xmlKey['unique'] == 'yes' || $xmlKey['name'] == "PRIMARY") !=
          (@$databaseKey['unique'] == 'yes' || $databaseKey['name'] == "PRIMARY")) {
      return TRUE;
    } elseif (@$xmlKey['fulltext'] == 'yes' && @$databaseKey['fulltext'] != 'yes') {
      return TRUE;
    } elseif (@count(array_diff_assoc($xmlKey['keysize'], $databaseKey['keysize'])) > 0) {
      return TRUE;    
    } elseif (@count(array_diff($xmlKey['fields'], $databaseKey['fields'])) > 0) {
      return TRUE;
    }
    return FALSE;
  }
}

/**
* DB-Abstractionslayer - Ergebnisobjekt MySQL
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class dbresult_mysql extends dbresult_base {

  /**
  * destructor
  *
  * Free memory, unset self and resultID
  */
  function free() {
    if (@is_resource($this->result)) {
      @mysql_free_result($this->result);
      unset($this->result);
      unset($this);
    }
  }

  /**
  * Fetch next row of result
  *
  * @param integer $mode line return modus
  * @return mixed FALSE or next line
  */
  function fetchRow($mode = DB_FETCHMODE_DEFAULT) {
    if (@is_resource($this->result)) {
      if ($mode == DB_FETCHMODE_ASSOC) {
        $result = @mysql_fetch_assoc($this->result);
      } else {
        $result = @mysql_fetch_row($this->result);
      }
      if (@is_array($result)) {
        $this->recNo++;
      }
      return $result;
    }
    return FALSE;
  }

  /** helpme
  * Return absolute data number of query
  *
  * @return mixed number or FALSE
  */
  function absCount() {
    if (($i = $this->count()) < $this->_absCount) {
      return $this->_absCount;
    } else {
      return $i;
    }
    return FALSE;
  }

  /**
  * Number rows affected by query
  * @return mixed number of rows or FALSE
  */
  function count() {
    if (is_resource($this->result)) {
      return @mysql_num_rows($this->result);
    }
    return FALSE;
  }

  /**
  * Search index
  *
  * Move record pointer to given index
  * next call of mysql_fetch_row() returns wanted value
  *
  * @param $index
  * @return boolean
  */
  function seek($index) {
    if (@is_resource($this->result)) {
      if (@mysql_data_seek($this->result, $index)) {
        $this->recNo = $index;
        return TRUE;
      }
    }
    return FALSE;
  }
}
?>