<?php 
/**
* User autentification secure functions - show data
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_auth_secure.php,v 1.3 2006/05/11 11:49:56 weinert Exp $
*/


/**
* secure base function
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_auth_secure.php');

/**
* 
* 
*/
class papaya_auth_secure extends base_auth_secure {

  var $listLimit = 20;
  var $loginTrys = array();
  var $paramName = 'asec';
  
  function initialize() {
    $this->initializeParams();
  }
  
  function execute() {
    if (isset($this->params['ip']) && !empty($this->params['ip'])) {
      switch(@$this->params['cmd']) {
      case 'allow_ip' :
        if (isset($this->params['confirm_allow']) && $this->params['confirm_allow']) {
          if ($this->addIpToWhitelist($this->params['ip'])) {
            $this->addMsg(MSG_INFO, 'IP added to whitelist.');
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error.')); 
          }
        }
        break;
      case 'deny_ip' :
        if (isset($this->params['confirm_deny']) && $this->params['confirm_deny']) {
          if ($this->params['ip'] != $_SERVER['REMOTE_ADDR']) {
            if ($this->addIpToBlacklist($this->params['ip'])) {
              $this->addMsg(MSG_INFO, 'IP added to blacklist.');
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error.')); 
            }
          } else {
            $this->addMsg(MSG_ERROR, 'You can not add your own IP to the blacklist.');
          }
        }
        break;
      case 'delete_ip' :
        if (isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
          if (defined('PAPAYA_LOGIN_RESTRICTION') && PAPAYA_LOGIN_RESTRICTION == 3 &&
              $this->params['ip'] == $_SERVER['REMOTE_ADDR']) {
            $this->addMsg(MSG_ERROR, $this->_gt('You can not delete your own IP from the list, because access is restricted to whitelist currently.'));
          } else {
            if ($this->deleteIpFromlist($this->params['ip'])) {
              $this->addMsg(MSG_INFO, $this->_gt('IP deleted.'));
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error.')); 
            }
          }
        }
        break;
      }
    }
    if (@$this->params['cmd'] == 'empty_try') {
      if (isset($this->params['confirm_empty']) && $this->params['confirm_empty']) {
        if ($this->emptyLoginTryList()) {
          $this->addMsg(MSG_INFO, $this->_gt('List deleted.'));
        } else {
          $this->addMsg(MSG_ERROR, $this->_gt('Database error.'));        
        }
      }
    }
    
    $this->loadLastTrys(@(int)$this->params['try_offset']);  
    $this->loadIpList(@(int)$this->params['ip_offset']);
    if (isset($this->params['ip']) && !empty($this->params['ip'])) {
      $this->loadIpDetails(trim($this->params['ip']));
    }
  }
  
  function getXML() {
    if (isset($this->params['ip']) && !empty($this->params['ip'])) {
      switch(@$this->params['cmd']) {
      case 'allow_ip' :
        if (!(isset($this->params['confirm_allow']) && $this->params['confirm_allow'])) {
          $this->layout->add($this->getWhitelistDialog($this->params['ip']));    
        }
        break;
      case 'deny_ip' :
        if (!(isset($this->params['confirm_deny']) && $this->params['confirm_deny'])) {
          if ($this->params['ip'] != $_SERVER['REMOTE_ADDR']) {
            $this->layout->add($this->getBlacklistDialog($this->params['ip']));      
          } else {
            $this->addMsg(MSG_ERROR, 'You can not add your own IP to the blacklist.');
          }
        }
        break;
      case 'delete_ip' :
        if (!(isset($this->params['confirm_delete']) && $this->params['confirm_delete'])) {
          if (defined('PAPAYA_LOGIN_RESTRICTION') && PAPAYA_LOGIN_RESTRICTION == 3 &&
              $this->params['ip'] == $_SERVER['REMOTE_ADDR']) {
            $this->addMsg(MSG_ERROR, 'You can not delete your own IP from the list, because access is restricted to whitelist currently.');
          } else {
            $this->layout->add($this->getDeleteIpDialog($this->params['ip']));   
          }
        }
        break; 
      }     
    }
    if (@$this->params['cmd'] == 'empty_try') {
      if (!(isset($this->params['confirm_empty']) && $this->params['confirm_empty'])) {
        $this->layout->add($this->getEmptyLoginTryDialog());
      }
    }
    
    $this->layout->addLeft($this->getIPListXML());
    $this->layout->addRight($this->getIpDetailXML());
    $this->layout->add($this->getLoginTryListXML());
    
    $this->getButtonsXML();
  }
  
  function getButtonsXML() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $menubar = &new base_btnbuilder;
    $menubar->images = &$this->images;

    if (isset($this->ipDetail) && is_array($this->ipDetail) && isset($this->ipDetail['ip'])) {
      if ($this->ipDetail['ip_status'] != 1) {
        $menubar->addButton('Whitelist', 
          $this->getLink(array('cmd' => 'allow_ip', 'ip' => $this->ipDetail['ip'])),
          125, 'Add to whitelist');
      }
      if ($this->ipDetail['ip_status'] != 2) {
        $menubar->addButton('Blacklist', 
          $this->getLink(array('cmd' => 'deny_ip', 'ip' => $this->ipDetail['ip'])),
          123, 'Add to blacklist');
      }
      if ($this->ipDetail['ip_status'] != 0) {
        $menubar->addButton('Delete', 
          $this->getLink(array('cmd' => 'delete_ip', 'ip' => $this->ipDetail['ip'])),
          10, 'Delete from list');
      }
    }
    
    $menubar->addSeperator();
    $offset = (isset($this->params['try_offset'])) ? (int)$this->params['try_offset'] : 0;
    $this->getPagesNav($menubar, (int)$offset,  $this->listLimit, $this->loginTryCount, 'Login Trys', 'try_offset');
    $menubar->addSeperator();
    $offset = (isset($this->params['ip_offset'])) ? (int)$this->params['ip_offset'] : 0;
    $this->getPagesNav($menubar, (int)$offset,  $this->listLimit, $this->ipCount, 'IPs', 'ip_offset');
    
    $menubar->addSeperator();
    $menubar->addButton('Delete login trys', 
      $this->getLink(array('cmd' => 'empty_try')),
      10, 'Delete login try list');
    
    if ($str = $menubar->getXML()) {
      $this->layout->add('<menu>'.$str.'</menu>', 'menus');
    }  
  }
  
  function addIPToWhitelist($ip) {
    $data = array(
      'auth_ip_status' => 1
    );
    if ($this->isIpInList($ip)) {
      $cond = array(
        'auth_ip' => $ip
      );
      return (FALSE !== $this->databaseUpdateRecord($this->tableAuthIp, $data, $cond));
    } else {
      $data['auth_ip'] = $ip;
      return (FALSE !== $this->databaseInsertRecord($this->tableAuthIp, NULL, $data));
    }
    return FALSE;
  }
  
  function addIPToBlacklist($ip) {
    $data = array(
      'auth_ip_status' => 2
    );
    if ($this->isIpInList($ip)) {
      $cond = array(
        'auth_ip' => $ip
      );
      return (FALSE !== $this->databaseUpdateRecord($this->tableAuthIp, $data, $cond));
    } else {
      $data['auth_ip'] = $ip;
      return (FALSE !== $this->databaseInsertRecord($this->tableAuthIp, NULL, $data));
    }  
    return FALSE;
  }
  
  function deleteIpFromList($ip) {
    if ($this->isIpInList($ip)) {
      $cond = array(
        'auth_ip' => $ip
      );
      return (FALSE !== $this->databaseDeleteRecord($this->tableAuthIp, $cond));
    }
    return FALSE;
  }
  
  function isIpInList($ip) {
    $sql = "SELECT COUNT(*)
              FROM %s
             WHERE auth_ip = '%s'";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableAuthIp, $ip))) {
      return (boolean)$res->fetchField();
    }
    return FALSE;
  }
  
  function loadLastTrys($offset = 0) {
    $this->loginTrys = array();
    $sql = "SELECT at.authtry_id, at.authtry_username, at.authtry_ip, 
                   at.authtry_time, at.authtry_group, ip.auth_ip_status
              FROM %s AS at
              LEFT OUTER JOIN %s AS ip ON (ip.auth_ip = at.authtry_ip)
             ORDER BY at.authtry_time DESC";
    $params = array($this->tableAuthTry, $this->tableAuthIp);
    if ($res = $this->databaseQueryFmt($sql, $params, 
        $this->listLimit, (int)$offset)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->loginTrys[$row['authtry_id']] = $row;
      }
      $this->loginTryCount = $res->absCount();
    }           
  }
  
  function emptyLoginTryList() {
    return $this->databaseEmptyTable($this->tableAuthTry);
  }
  
  function getLoginTryListXML() {
    $result = '';
    if (isset($this->loginTrys) && is_array($this->loginTrys) && 
        count($this->loginTrys) > 0) {
      $result .= sprintf('<listview title="%s">', $this->_gt('Login Requests'));
      $result .= '<cols>';
      $result .= '<col>'.papaya_strings::escapeHTMLChars($this->_gt('IP')).'</col>';
      $result .= '<col>'.papaya_strings::escapeHTMLChars($this->_gt('Time')).'</col>';
      $result .= '<col>'.papaya_strings::escapeHTMLChars($this->_gt('Username')).'</col>';
      $result .= '<col>'.papaya_strings::escapeHTMLChars($this->_gt('Login Type')).'</col>';
      $result .= '<col/>';
      $result .= '<col/>';
      $result .= '</cols>';
      $result .= '<items>';
      foreach($this->loginTrys as $try) {
        $result .= sprintf('<listitem href="%s" title="%s">', 
          $this->getLink(array('cmd' => 'show_ip', 'ip' => $try['authtry_ip'])),
          papaya_strings::escapeHTMLChars($try['authtry_ip']));
        $result .= sprintf('<subitem>%s</subitem>', 
          date('Y-m-d H:i:s', $try['authtry_time']));
        $result .= sprintf('<subitem>%s</subitem>', 
          papaya_strings::escapeHTMLChars($try['authtry_username']));
        $result .= sprintf('<subitem>%s</subitem>', 
          papaya_strings::escapeHTMLChars($try['authtry_group']));
        if ($try['auth_ip_status'] != 1) {
          $result .= sprintf('<subitem align="center"><a href="%s"><glyph src="%s"/></a></subitem>', 
            $this->getLink(array('cmd' => 'allow_ip', 'ip' => $try['authtry_ip'])), 
            papaya_strings::escapeHTMLChars($this->images[125]));
        } else {
          $result .= '<subitem/>';
        }
        if ($try['auth_ip_status'] != 2) {
          $result .= sprintf('<subitem align="center"><a href="%s"><glyph src="%s"/></a></subitem>', 
            $this->getLink(array('cmd' => 'deny_ip', 'ip' => $try['authtry_ip'])), 
            papaya_strings::escapeHTMLChars($this->images[123]));
        } else {
          $result .= '<subitem/>';
        }
        $result .= '</listitem>';
      }
      $result .= '</items>';
      $result .= '</listview>';
    }
    return $result;
  }
  
  function loadIpList($offset = 0) {
    $this->ips = array();
    $sql = "SELECT auth_ip, auth_ip_status
             FROM %s
            ORDER BY auth_ip";
    if ($res = $this->databaseQueryFmt($sql, $this->tableAuthIp, 
        $this->listLimit, (int)$offset)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->ips[$row['auth_ip']] = $row;
      }
      $this->ipCount = $res->absCount();
    }
  }
  
  function getIPListXML() {
    $result = '';
    if (isset($this->ips) && is_array($this->ips) && count($this->ips) > 0) {      
      $result .= sprintf('<listview title="%s">', $this->_gt('IP list'));
      $result .= '<items>';
      foreach($this->ips as $ip) {
        switch($ip['auth_ip_status']) {
        case '1' :
          $imageIdx = 125;
          break;
        case '2' :
        default :
          $imageIdx = 123;
          break;
        }
        $result .= sprintf('<listitem title="%s" href="%s" image="%s">', 
          papaya_strings::escapeHTMLChars($ip['auth_ip']),
          $this->getLink(array('cmd' => 'show_ip', 'ip' => $ip['auth_ip'])), 
          papaya_strings::escapeHTMLChars($this->images[$imageIdx]));
        $result .= '</listitem>';
      }
      $result .= '</items>';
      $result .= '</listview>';
    }
    return $result;
  }
  
  function loadIpDetails($ip) {
    $time = time();
    $this->ipDetail = array(
      'ip'        => $ip,
      'ip_status' => $this->getIPStatus($ip),
      'min10'     => $this->loadIpCount($ip, $time - 600),
      'min30'     => $this->loadIpCount($ip, $time - 1800),
      'hour'      => $this->loadIpCount($ip, $time - 3600),
      'day'       => $this->loadIpCount($ip, $time - 86400),
      'month'     => $this->loadIpCount($ip, $time - 2635200),
    );      
  }
  
  function loadIpCount($ip, $time) {
    $sql = "SELECT COUNT(*) 
              FROM %s
             WHERE authtry_ip = '%s'
               AND authtry_time >= '%d'";    
    $params = array($this->tableAuthTry, $ip, $time);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      return $res->fetchField();
    }
    return 0;
  }
  
  function getIpDetailXML() {
    $result = '';
    if (isset($this->ipDetail) && is_array($this->ipDetail) && 
        !empty($this->ipDetail['ip'])) {     
      $result .= sprintf('<listview title="%s">', $this->_gt('IP details'));
      $result .= '<items>';
      switch($this->ipDetail['ip_status']) {
      case '1' :
        $imageIdx = 125;
        break;
      case '2' :
        $imageIdx = 123;
        break;
      default :
        $imageIdx = 0;
        break;
      }
      if ($imageIdx > 0) {
        $result .= sprintf('<listitem title="%s" image="%s">', 
          papaya_strings::escapeHTMLChars($this->ipDetail['ip']), 
          papaya_strings::escapeHTMLChars($this->images[$imageIdx])
        );
      } else {
        $result .= sprintf('<listitem title="%s">', 
          papaya_strings::escapeHTMLChars($this->ipDetail['ip'])
        );
      }
      $hostName = getHostByAddr($this->ipDetail['ip']);
      if ($hostName != $this->ipDetail['ip']) {
        $result .= sprintf('<subitem>%s</subitem>', 
          papaya_strings::escapeHTMLChars($hostName)
        );
      } else {
        $result .= '<subitem/>';      
      }
      $result .= '</listitem>';
      
      $result .= sprintf('<listitem title="10 %s" indent="1"><subitem>%s</subitem></listitem>', 
        papaya_strings::escapeHTMLChars($this->_gt('Minutes')), 
        papaya_strings::escapeHTMLChars($this->ipDetail['min10'])
      );
      $result .= sprintf('<listitem title="30 %s" indent="1"><subitem>%s</subitem></listitem>', 
        papaya_strings::escapeHTMLChars($this->_gt('Minutes')), 
        papaya_strings::escapeHTMLChars($this->ipDetail['min30'])
      );
      $result .= sprintf('<listitem title="1 %s" indent="1"><subitem>%s</subitem></listitem>', 
        papaya_strings::escapeHTMLChars($this->_gt('Hour')), 
        papaya_strings::escapeHTMLChars($this->ipDetail['hour'])
      );
      $result .= sprintf('<listitem title="1 %s" indent="1"><subitem>%s</subitem></listitem>', 
        papaya_strings::escapeHTMLChars($this->_gt('Day')), 
        papaya_strings::escapeHTMLChars($this->ipDetail['day'])
      );
      $result .= sprintf('<listitem title="1 %s" indent="1"><subitem>%s</subitem></listitem>', 
        papaya_strings::escapeHTMLChars($this->_gt('Month')), 
        papaya_strings::escapeHTMLChars($this->ipDetail['month'])
      );
      
      $result .= '</items>';
      $result .= '</listview>';        
    }
    return $result;
  }
  
  function getWhitelistDialog($ip) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'allow_ip',
      'confirm_allow' => 1,
      'ip' => $ip
    );
    $msg = sprintf($this->_gt('Add IP "%s" to whitelist?'),
        papaya_strings::escapeHTMLChars($ip));
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Add';
    return $dialog->getMsgDialog();
  }
  
  function getBlacklistDialog($ip) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'deny_ip',
      'confirm_deny' => 1,
      'ip' => $ip
    );
    $msg = sprintf($this->_gt('Add IP "%s" to blacklist?'),
        papaya_strings::escapeHTMLChars($ip));
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Add';
    return $dialog->getMsgDialog();  
  }
  
  function getDeleteIpDialog($ip) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'delete_ip',
      'confirm_delete' => 1,
      'ip' => $ip
    );
    $msg = sprintf($this->_gt('Delete IP "%s" from list?'),
        papaya_strings::escapeHTMLChars($ip));
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Delete';
    return $dialog->getMsgDialog();  
  }
  
  function getEmptyLoginTryDialog() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'empty_try',
      'confirm_empty' => 1,
    );
    $msg = $this->_gt('Delete login try list?');
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Delete';
    return $dialog->getMsgDialog();  
  }
  
  /**
  * Get pages navigation
  *
  * @param object &$toolbar
  * @param integer $offset
  * @param integer $step
  * @param integer $max
  * @access public
  */
  function getPagesNav(&$toolbar, $offset, $step, $max, $title = '', $paramName = 'offset') {
    if (($max > $step) && is_object($toolbar)) {
      if ($offset > 0) {
        $i = (($offset - $step) > 0) ? ($offset - $step) : 0;
        $toolbar->addButton('Prior', $this->getLink(array('cmd'=>'show', $paramName=>$i)), 8, '');
      }
      for($i=0,$x=1;$i<$max;$i+=$step,$x++) {
        $pages[$i] = $x;
      }
      $toolbar->addCombo($title, $this->getLink(array('cmd'=>'show')),
        $this->paramName.'['.$paramName.']', $offset, $pages);
      if ($offset < ($max-$step)) {
        $i = (($offset + $step) < $max) ? ($offset + $step) : ($max-$step);
        $toolbar->addButton('Next', $this->getLink(array('cmd'=>'show', $paramName=>$i)), 9, '');
      }
    }
  }
}

?>