<?php
/**
* Administration class for change modules
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya_modules
* @version $Id: papaya_editmodules.php,v 1.3 2005/10/21 10:02:27 weinert Exp $
*/

/**
*  Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* Administration class for change modules
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya_modules
*/
class papaya_editmodules extends base_db {
  var $tableModules = PAPAYA_DB_TBL_MODULES;
  var $tableModulegroups = PAPAYA_DB_TBL_MODULEGROUPS;

  /**
  * Constructor
  *
  * @param string $moduleClass optional, default value ''
  * @access public
  */
  function __construct($moduleClass='') {
    $this->moduleClass = $moduleClass;
  }

  /**
  * Load modules list
  *
  * @access public
  */
  function loadModulesList() {
    unset($this->modules);
    $sql = "SELECT module_guid, module_title, module_class, module_file, module_path, module_glyph
            FROM %s
            WHERE module_type = 'admin' AND module_active = 1
            ORDER BY module_title";
    if ($res = $this->databaseQueryFmt($sql,array($this->tableModules))) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->modules[$row["module_guid"]] = $row;
      }
    }
  }

  /**
  * Get button array
  *
  * @access public
  * @return mixed
  */
  function getButtonArray() {
    $result = NULL;
    if (isset($this->modules) && is_array($this->modules)) {
      foreach($this->modules as $id=>$module) {
        if ($this->authUser->isAdmin() || $this->authUser->hasModulePerm(1, $id)) {
          if (trim($module['module_glyph']) != '') {
            $glyph = 'modglyph.php?module='.urlencode($module['module_guid']);
          } else {
            $glyph = '';
          }
          $result[] = array($module['module_title'], $module['module_title'], $glyph, 0,
            'module_'.$module['module_class'].'.php', '_self',
            $this->moduleClass == $module['module_class'], NULL, TRUE);
          }
      }
    }
    return $result;
  }

  /**
  * Load module
  *
  * @access public
  * @return boolean
  */
  function loadModule() {
    unset($this->module);
    $sql = "SELECT m.module_guid, m.module_title, m.module_class, m.module_file,
              m.module_path, m.module_glyph, m.modulegroup_id,
              mg.modulegroup_tables
            FROM %s AS m
            LEFT OUTER JOIN %s AS mg ON mg.modulegroup_id = m.modulegroup_id
            WHERE module_class = '%s' AND module_active = 1";
    $params = array($this->tableModules, $this->tableModulegroups, $this->moduleClass);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->module = $row;
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Initialize parameters
  *
  * @access public
  * @return boolean
  */
  function initialize() {
    if ($this->loadModule()) {
      if ($this->checkTables($this->module['modulegroup_tables'])) {
        $parent = NULL;
        $this->moduleObject = &$this->createPluginObject($this->module['module_path'].$this->module['module_file'],
          $this->module['module_class'], $this->module['module_guid'], $parent);
        if (isset($this->moduleObject) && is_object($this->moduleObject)) {
          $this->moduleObject->guid = $this->module['module_guid'];
          $this->moduleObject->images = &$this->images;
          $this->moduleObject->layout = &$this->layout;
          $this->moduleObject->msgs = &$this->msgs;
          $this->moduleObject->authUser = &$this->authUser;
          $this->layout->setParam('PAGE_TITLE', papaya_strings::escapeHTMLChars($this->module['module_title']));
          return TRUE;
        }
      }
    } else {
      //Modulliste
      $this->loadModulesList();
      if (isset($this->modules) && is_array($this->modules) && count($this->modules) > 0) {
        $this->addMsg(MSG_INFO, 'Please select a module!');
      } else {
        $this->addMsg(MSG_INFO, 'No modules installed');
      }
    }
    return FALSE;
  }

  /**
  * Execute
  *
  * @access public
  */
  function execute() {
    if (isset($this->moduleObject) && is_object($this->moduleObject)) {
      $this->moduleObject->execModule();
    }
  }

  function checkTables($tableString) {
    $result = TRUE;
    if(trim($tableString) != '') {
      $tables = explode(',',$tableString);
      if (isset($tables) && is_array($tables) && count($tables) > 0 &&
          @is_array($dbTables = $this->databaseQueryTableNames())) {
        $dbTables = array_flip($dbTables);
        foreach($tables as $tableName) {
          if (!isset($dbTables[PAPAYA_DB_TABLEPREFIX.'_'.$tableName])) {
            include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
            $hidden = array(
              'pkg_id' => (int)$this->module['modulegroup_id'],
              'module_id' => 0,
              'table'=>'',
            );
            $msg = $this->_gt('Missing tables - Go to module managment?');
            $dialog = &new base_msgdialog($this, 'mods', $hidden, $msg, 'warning');
            $dialog->msgs = &$this->msgs;
            $dialog->buttonTitle = 'Goto';
            $dialog->baseLink = 'modules.php';
            if ($str = $dialog->getMsgDialog()) {
              $this->layout->add($str);
            }
            return FALSE;
          }
        }
        $data = array('modulegroup_tables'=>'');
        $this->databaseUpdateRecord($this->tableModulegroups, $data, 'modulegroup_id', (int)$this->module['modulegroup_id']);
      }
    }
    return $result;
  }
}
?>