<?php
/**
* Install default database
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
* @version $Id: papaya_installer.php,v 1.6 2006/04/26 10:29:00 weinert Exp $
*/

/**
* Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');
require_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');

/**
* Install default database
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class papaya_installer extends base_db {
  var $paramName = 'inst';

  var $phraseTables = array(
    'lng', 'phrase', 'phrase_log', 'phrase_module', 'phrase_relmod', 'phrase_trans'
  );
  var $authTables = array(
    'auth_groups', 'auth_link', 'auth_modperm', 'auth_modperm_link', 'auth_perm', 'auth_user'
  );

  /**
  * tables in database
  * @var array $existingTables
  */
  var $existingTables = array();

  /**
  * module manager instance to create/sync tables
  * @var object papaya_modulemanager
  */
  var $moduleManager = NULL;

  /**
  * Debug mode for RPC (shows iframe and rpc messages)
  * @var string $debugRPC
  */
  var $debugRPC = FALSE;

  /**
  * Output as js?
  * @var string $outputMode
  */
  var $outputMode = 'js';

  /**
  * Installer wants to output a javascript for rpc
  * @var boolean $javascriptRPC
  */
  var $javascriptRPC = FALSE;
  
  /**
  * Path to Framework
  * @var string
  */
  var $pathFrameworkRoot = '';

  /**
  * Initialize parameters
  * @access public
  */
  function initialize() {
    $this->pathDocumentRoot = $_SERVER['DOCUMENT_ROOT'];
    $this->pathFrameworkRoot = dirname(dirname(__FILE__)).'/';
    $this->initializeParams();
    if (defined('PAPAYA_DBG_DEVMODE') && PAPAYA_DBG_DEVMODE) {
      $this->debugRPC = TRUE;
    }
  }

  /**
  * Execute
  *
  * @access public
  */
  function execute() {
    if ($this->checkDatabase()) {
      if (defined('PAPAYA_DB_TBL_OPTIONS') && checkit::isAlpha(PAPAYA_DB_TBL_OPTIONS, TRUE)) {
        $this->existingTables = @array_flip($this->databaseQueryTableNames());
        $this->initModuleManager();
        $optionFileName = $this->pathFrameworkRoot.'modules/_base/DATA/table_options.xml';
        if ((!$this->checkTableExists(PAPAYA_DB_TBL_OPTIONS)) &&
            isset($this->params['cmd']) && $this->params['cmd'] == 'table_create' &&
            isset($this->params['table']) && $this->params['table'] == PAPAYA_DB_TBL_OPTIONS) {
          if ($this->createTable($optionFileName, PAPAYA_DB_TBL_OPTIONS)) {
            $this->reloadInstaller();
          }
        }
        $this->initPhrases();
        if ($this->checkTableExists(PAPAYA_DB_TBL_OPTIONS)) {
          //interactive installer interface
          if ($this->checkAuthTables()) {
            if ($this->initAuthentication()) {
              $this->executeInstaller();
            } elseif (isset($this->params['cmd']) && trim($this->params['cmd']) != '') {
              $this->outputJavascriptCommand(-1, FALSE, $this->_gt('Authentication needed - Please log in and restart installation process.'), 'authNeeded');
            }
          } else {
            $this->addMsg(MSG_WARNING, 'No authentication yet.');
            $this->executeInstaller();
          }
        } elseif (file_exists($optionFileName)) {
          $this->dialogCreateTable(PAPAYA_DB_TBL_OPTIONS);
        } else {
          $this->addMsg(MSG_ERROR, sprintf($this->_gt('Can not find "%s".'), papaya_strings::escapeHTMLChars($optionFileName)));
        }
      } else {
        $this->addMsg(MSG_ERROR, $this->_gt('Invalid name for option table. Please check your "conf.inc.php".'));
      }
    } else {
      $this->addMsg(MSG_ERROR, $this->_gt('Can not connect to database. Please check your "conf.inc.php".'));
    }
  }

  function executeInstaller() {
    switch (@$this->params['cmd']) {
      case 'check_database' :
        if (isset($this->params['table']) && isset($this->params['table_idx'])) {
          $count = $this->getTableCount($this->params['table'], PAPAYA_DB_TABLEPREFIX);
          $this->outputJavascriptCommand((int)$this->params['table_idx'], TRUE, (int)$count, 'count');
          if ($this->checkTableStruct(trim($this->params['table']))) {
            $this->outputJavascriptCommand((int)$this->params['table_idx'], TRUE, $this->_gt('Table OK.'), 'check');
          } else {
            $this->outputJavascriptCommand((int)$this->params['table_idx'], FALSE, $this->_gt('Table does not match data file.'), 'check');
          }
        } else {
          $this->outputJavascriptCommand((int)$this->params['table_idx'], FALSE, $this->_gt('Table does not exists.'), 'check');
        }
        break;
      case 'install_database' :
        if (isset($this->params['table']) && isset($this->params['table_idx'])) {
          $this->installDatabaseTable(trim($this->params['table']), (int)$this->params['table_idx']);
        }
        break;
      case 'reset_data' :
        if (isset($this->params['table']) && isset($this->params['table_idx'])) {
          $this->outputJavascriptCommand((int)$this->params['table_idx'], TRUE, 'Reset table content', 'update');
          $this->resetDatabaseTableData(trim($this->params['table']), (int)$this->params['table_idx']);
        }
        break;
      case 'init_options' :
        if ($this->initOptionValues()) {
          $this->outputJavascriptCommand($this->params['table_idx'], TRUE, $this->_gt('Options checked.'), 'init');
        } else {
          $this->outputJavascriptCommand($this->params['table_idx'], FALSE, $this->_gt('Options not checked.'), 'init');
        }
        break;
      case 'init_modules' :
        if ($this->initModules()) {
          $this->outputJavascriptCommand($this->params['table_idx'], TRUE, $this->_gt('Scan completed.'), 'init');
        } else {
          $this->outputJavascriptCommand($this->params['table_idx'], FALSE, $this->_gt('Could not complete modules scan.'), 'init');
        }
        break;
      default :
        $this->getTablesListView();
        $this->getProgressBarPanel();
        break;
    }
    $this->outputJavaScriptFinish();
  }

  function initAuthentication() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_auth.php');
    $GLOBALS['PAPAYA_USER'] = &new base_auth();
    $this->authUser = &$GLOBALS['PAPAYA_USER'];
    $this->authUser->layout = &$this->layout;
    $this->authUser->images = &$this->images;
    $this->authUser->msgs = &$this->msgs;
    $this->authUser->initialize();
    if ($this->authUser->execLogin()) {
      if ($this->authUser->isAdmin()) {
        $this->layout->setParam('PAGE_USER', $this->authUser->user['fullname']);
        return TRUE;
      }
    }
    return FALSE;
  }

  function initAdminAccount() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_auth.php');
    $GLOBALS['PAPAYA_USER'] = &new base_auth();
    $this->authUser = &$GLOBALS['PAPAYA_USER'];
    $this->authUser->layout = &$this->layout;
    $this->authUser->images = &$this->images;
    $this->authUser->msgs = &$this->msgs;
    $this->authUser->initialize();

    $sql = "SELECT user_id, group_id FROM %s WHERE group_id = -1 OR username = 'admin' ORDER BY group_id ASC";
    if ($res = $this->databaseQueryFmt($sql, PAPAYA_DB_TBL_AUTHUSER)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        if ($row['group_id'] != -1) {
          $data = array('group_id' => -1, 'active'=>1);
          $this->databaseUpdateRecord(PAPAYA_DB_TBL_AUTHUSER, $data, 'user_id', $row['user_id']);
        }
        $this->authUser->load($row['user_id'], TRUE);
        $this->authUser->setSessionToken($row['user_id']);
        return $this->authUser->isAdmin();
      } elseif ($this->authUser->loadLogin('admin','')) {
        return TRUE;
      }
    }
    return FALSE;
  }

  function initPhrases() {
    if (defined('PAPAYA_GLOBAL_PHRASEOBJECT') && (trim(PAPAYA_GLOBAL_PHRASEOBJECT) != '')) {
      include_once(PAPAYA_INCLUDE_PATH.'system/sys_phrases.php');
      $GLOBALS[PAPAYA_GLOBAL_PHRASEOBJECT] = &new base_phrases();
      if (isset($this->authUser)) {
        $GLOBALS[PAPAYA_GLOBAL_PHRASEOBJECT]->getLngId($this->authUser->options['PAPAYA_UI_LANGUAGE']);
      } else {
        $GLOBALS[PAPAYA_GLOBAL_PHRASEOBJECT]->getLngId(PAPAYA_UI_LANGUAGE);
      }
    }
  }

  function getTablesListView() {
    $result = sprintf('<listview title="%s">', papaya_strings::escapeHTMLChars($this->_gt('Tables')));
    $result .= '<cols>';
    $result .= sprintf('<col>%s</col>', papaya_strings::escapeHTMLChars($this->_gt('Name')));
    $result .= sprintf('<col align="center">%s</col>', papaya_strings::escapeHTMLChars($this->_gt('Found')));
    $result .= sprintf('<col align="center">%s</col>', papaya_strings::escapeHTMLChars($this->_gt('Structure')));
    $result .= sprintf('<col align="center">%s</col>', papaya_strings::escapeHTMLChars($this->_gt('Record count')));
    $result .= sprintf('<col align="center">%s</col>', papaya_strings::escapeHTMLChars($this->_gt('Default data')));
    $result .= '</cols>';
    $result .= '<items>';
    $counter = 0;
    $max = count($this->options->tables);
    foreach($this->options->tables as $table) {
      $tableName = PAPAYA_DB_TABLEPREFIX.'_'.$table;
      if ($tableName != PAPAYA_DB_TBL_OPTIONS) {
        $result .= sprintf('<listitem title="%s">',
          papaya_strings::escapeHTMLChars($tableName));
        $imageIndex = ($this->checkTableExists($tableName)) ? 29 : 30;
        $result .= sprintf('<subitem align="center"><glyph src="%s" id="sign_exists_%s"/></subitem>',
          $this->images[$imageIndex], papaya_strings::escapeHTMLChars($table));
        $imageIndex = ($imageIndex != 30) ? 58 : 30;
        $result .= sprintf('<subitem align="center"><glyph src="%s" id="sign_struct_%s"/></subitem>',
          $this->images[$imageIndex], papaya_strings::escapeHTMLChars($table));
        $count = ($this->checkTableExists($tableName)) ? '?' : '0';
        $result .= sprintf('<subitem align="center"><div id="records_%s">%s</div></subitem>',
          papaya_strings::escapeHTMLChars($table), $count);
        if (file_exists($this->pathFrameworkRoot.'modules/_base/DATA/table_'.$table.'.csv')) {
          $result .= sprintf('<subitem align="center"><a href="javascript:dataSelection(\'%s\',\'%d\');"><glyph src="%s" id="box_data_%s"/></a></subitem>',
            papaya_strings::escapeHTMLChars($table), $counter, $this->images[30], papaya_strings::escapeHTMLChars($table));
        } else {
          $result .= '<subitem />';
        }
        $result .= '</listitem>';
        $counter++;
      }
    }
    $result .= '</items>';
    $result .= '</listview>';
    $this->layout->add($result);
  }

  function getProgressJavascript(&$tableCounts) {
    $result = '<script type="text/javascript" src="script/installer.js"></script>'.LF;
    $result .= '<script type="text/javascript">'.LF;
    $result .= 'var paramname = "'.$this->paramName.'"; '.LF;
    $result .= 'var img_checked = "'.$this->images[29].'"; '.LF;
    $result .= 'var img_unchecked = "'.$this->images[30].'"; '.LF;
    $result .= 'var box_checked = "'.$this->images[27].'"; '.LF;
    $result .= 'var box_unchecked = "'.$this->images[28].'"; '.LF;
    $result .= 'var tabledata = new Array(); '.LF;
    foreach($this->options->tables as $table) {
      $tableName = PAPAYA_DB_TABLEPREFIX.'_'.$table;
      if ($tableName != PAPAYA_DB_TBL_OPTIONS) {

        $result .= 'tabledata['.$tableCounts['all'].'] = new Array(); '.LF;
        $result .= 'tabledata['.$tableCounts['all'].'].name = "'.$table.'"; '.LF;
        $result .= 'tabledata['.$tableCounts['all'].'].synced = false; '.LF;
        $result .= 'tabledata['.$tableCounts['all'].'].insert = false; '.LF;
        $result .= 'tabledata['.$tableCounts['all'].'].recordCount = 0; '.LF;

        if ($this->checkTableExists($tableName)) {
          $exists = 'true';
          $tableCounts['exists']++;
        } else {
          $exists = 'false';
        }
        $result .= 'tabledata['.$tableCounts['all'].'].exists = '.$exists.'; '.LF;

        if (file_exists($this->pathFrameworkRoot.'modules/_base/DATA/table_'.$table.'.csv')) {
          $hasData = 'true';
          $tableCounts['hasdata']++;
        } else {
          $hasData = 'false';
        }
        $result .= 'tabledata['.$tableCounts['all'].'].csvfile = '.$hasData.'; '.LF;

        $tableCounts['all']++;
      }
    }
    $result .= '</script>'.LF;
    return $result;
  }

  function getInstallerTableLine($no, $id, $title, $text, $enabled = TRUE) {
    if ($enabled) {
      $styleDefaultLayer = '';
      $styleActiveLayer = ' style="display: none;"';
    } else {
      $styleDefaultLayer = ' style="display: none;"';
      $styleActiveLayer = '';
    }
    $result = '<tr id="header'.$id.'"'.$styleDefaultLayer.'>';
    $result .= '<td class="bullet"><a href="#" onclick="initInstall'.$id.'();"><img src="pics/steps/step'.$no.'.gif" alt="Step '.$no.'" /></a></td>';
    $result .= '<th><a href="#" onclick="initInstall'.$id.'();">'.$title.'</a></th>';
    $result .= '</tr>';
    $result .= '<tr id="headerDisabled'.$id.'"'.$styleActiveLayer.'>';
    $result .= '<td class="bullet"><img src="pics/steps/step'.$no.'_disabled.gif" alt="Step '.$no.'" /></td>';
    $result .= '<th>'.$title.'</th>';
    $result .= '</tr>';
    if (isset($text)) {
      $result .= '<tr>';
      $result .= '<td></td>';
      $result .= '<td><p>'.$text.'</p>';

      $result .= '<div class="progress" id="installerProgress'.$id.'" style="display: none;">';
      $result .= '<div class="progressLabel" id="installerProgressLabel'.$id.'"></div>';
      $result .= '<div class="progressBar"><img src="pics/tpoint.gif"  id="installerProgressBar'.$id.'"/></div>';
      $result .= '</div>';

      $result .= '</td>';

      $result .= '</tr>';
    }
    return $result;
  }

  function getProgressBarPanel() {
    $tableCounts = array('all' => 0, 'exists' => 0, 'hasdata' => 0);

    $result = sprintf('<panel title="%s">'.LF, $this->_gt('Installation / Update'));
    $result .= '<data>'.LF;
    $result .= $this->getProgressJavascript($tableCounts);
    $result .= '<div class="installer">';
    $result .= '<table>';

    $result .= $this->getInstallerTableLine(
      1,
      'Analyze',
      papaya_strings::escapeHTMLChars('Analyze Database'),
      papaya_strings::escapeHTMLChars('Checks the structure of all existing tables'),
      ($tableCounts['exists'] > 0));
    $result .= $this->getInstallerTableLine(
      2,
      'Create',
      papaya_strings::escapeHTMLChars('Update Database'),
      papaya_strings::escapeHTMLChars('Creates missing tables and updates existing tables'),
      ($tableCounts['exists'] == 0));
    $result .= $this->getInstallerTableLine(
      3,
      'Insert',
      papaya_strings::escapeHTMLChars('Insert default data'),
      papaya_strings::escapeHTMLChars('Deletes current records in selected tables and inserts default data. You can select the tables by clicking on the checkboxes in the right column of the large listview.'),
      FALSE);
    $result .= $this->getInstallerTableLine(
      4,
      'Init',
      papaya_strings::escapeHTMLChars('Check options and modules'),
      papaya_strings::escapeHTMLChars('Sets default values for undefined options and searches for modules'),
      FALSE);
    $result .= $this->getInstallerTableLine(
      5,
      'GoTo',
      papaya_strings::escapeHTMLChars('Go to admin interface'),
      NULL,
      TRUE);

    $result .= '</table>';
    $result .= '</div>';
    $showIframe = ($this->debugRPC) ? 'block' : 'none';
    $result .= '<div style="width: 300px; text-align: left; display: '.$showIframe.';"><iframe name="rpcListener" style="display: block; width: 294px; height: 150px; background-color: #FFF;"></iframe></div>';
    $result .= '</data>'.LF;
    $result .= '</panel>'.LF;
    $this->layout->addRight($result);
  }

  function reloadInstaller() {
    $url = $this->getAbsoluteURL('install.php');
    header('Location: '.$url);
    exit;
  }

  function checkAuthTables() {
    //tables missed, check table structure
    $missed = 0;
    $old = 0;
    foreach($this->authTables as $table) {
      if (!$this->checkTableExists(PAPAYA_DB_TABLEPREFIX.'_'.$table)) {
        $missed++;
      } elseif (!$this->checkTableStruct($table)) {
        $old++;
      }
    }
    if ($missed + $old == 0) {
      $sql = "SELECT COUNT(*) FROM %s_%s";
      $params = array(PAPAYA_DB_TABLEPREFIX, $this->options->tables['PAPAYA_DB_TBL_AUTHUSER']);
      if ($res = $this->databaseQueryFmt($sql, $params)) {
        list($count) = $res->fetchRow();
        return ($count > 0) ? TRUE : FALSE;
      }
    }
    return FALSE;
  }

  /**
  * Check database
  *
  * @access public
  * @return boolean
  */
  function checkDatabase() {
    if ($this->databaseConnect()) {
      return TRUE;
    }
    return FALSE;
  }

  function checkTableExists($tableName) {
    return (isset($this->existingTables[$tableName]));
  }

  function checkTableStruct($table) {
    $tableFileName = $this->pathFrameworkRoot.'modules/_base/DATA/table_'.$table.'.xml';
    $result = TRUE;
    if ($struct = $this->moduleManager->loadTableStructure($tableFileName)) {
      if (@$struct['actions'] > 0) {
        $result = FALSE;
      }
      unset($struct);
    }
    return $result;
  }

  function getTableCount($table, $prefix = NULL) {
    $tableName = (isset($prefix)) ? $prefix.'_'.$table : $table;
    $sql = "SELECT COUNT(*) FROM %s";
    if ($res = $this->databaseQueryFmt($sql, $tableName)) {
      list($count) = $res->fetchRow();
      return $count;
    }
    return 0;
  }

  function initModuleManager() {
    if (!(isset($this->moduleManager) && is_object($this->moduleManager))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/papaya_modulemanager.php');
      $this->moduleManager = &new papaya_modulemanager();
    }
  }

  function dialogCreateTable($tableName, $prefix = NULL) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'table_create',
      'table' => $tableName,
      'table_create_confirm'=>1,
    );
    $table = (isset($prefix)) ? $prefix.'_'.$tableName : $tableName;
    $msg = sprintf($this->_gt('Create table "%s"?'), papaya_strings::escapeHTMLChars($table));
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Create';
    if ($str = $dialog->getMsgDialog()) {
      $this->layout->add($str);
    }
  }

  function createTable($xmlFileName, $tableName) {
    $result = FALSE;
    if ($struct = $this->moduleManager->loadTableStructure($xmlFileName)) {
      $struct['name'] = $tableName;
      if ($this->databaseCreateTable($struct, NULL)) {
        unset($this->params['cmd']);
        $this->addMsg(MSG_INFO, $this->_gt('Table created.'));
        $result = TRUE;
      }
      unset($struct);
    }
    return $result;
  }

  function syncTable($xmlFileName, $tableName) {
    $result = FALSE;
    if ($struct = $this->moduleManager->loadTableStructure($xmlFileName)) {
      $struct['name'] = $tableName;
      if ($this->moduleManager->syncTableStructure($struct)) {
        $result = TRUE;
      }
      unset($struct);
    }
    return $result;
  }

  function installDatabaseTable($table, $idx) {
    if (preg_match('/^\w+$/', $table)) {
      $tableName = PAPAYA_DB_TABLEPREFIX.'_'.$table;
      $tableFileName = $this->pathFrameworkRoot.'modules/_base/DATA/table_'.$table.'.xml';
      if (file_exists($tableFileName)) {
        if ($this->checkTableExists($tableName)) {
          if ($this->syncTable($tableFileName, $table)) {
            $this->outputJavascriptCommand($idx, TRUE, $this->_gt('Table structure updated.'), 'sync');
          } else {
            $this->outputJavascriptCommand($idx, FALSE, $this->_gt('Database error.'), 'sync');
          }
        } else {
          if ($this->createTable($tableFileName, $tableName)) {
            $this->outputJavascriptCommand($idx, TRUE, $this->_gt('Table created.'), 'sync');
          } else {
            $this->outputJavascriptCommand($idx, FALSE, $this->_gt('Database error.'), 'sync');
          }
        }
      } else {
        $this->outputJavascriptCommand($idx, FALSE, $this->_gt('File not found.'), 'sync');
      }
    } else {
      $this->outputJavascriptCommand($idx, FALSE, $this->_gt('Invalid table name.'), 'sync');
    }
  }

  function resetDataBaseTableData($table, $idx) {
    if (preg_match('/^\w+$/', $table)) {
      $tableName = PAPAYA_DB_TABLEPREFIX.'_'.$table;
      $dataFileName = $this->pathFrameworkRoot.'modules/_base/DATA/table_'.$table.'.csv';
      if (file_exists($dataFileName)) {
        if ($this->checkTableExists($tableName)) {
          if (FALSE !== $this->databaseEmptyTable($tableName)) {
            if (@is_array($done = $this->insertCSV($tableName, $dataFileName))) {
              $count = $this->getTableCount($this->params['table'], PAPAYA_DB_TABLEPREFIX);
              if ($tableName == PAPAYA_DB_TBL_AUTHUSER) {
                $this->initAdminAccount();
              }
              $this->outputJavascriptCommand((int)$this->params['table_idx'], TRUE, (int)$count, 'count');
              $this->outputJavascriptCommand($idx, TRUE, $this->_gtf('Done. %d queries for %d bytes data',
                array($done['queries'], $done['data'])), 'reset');
            } else {
              $this->outputJavascriptCommand((int)$this->params['table_idx'], TRUE, 0, 'count');
              $this->outputJavascriptCommand($idx, FALSE, $this->_gt('Database error.'), 'reset');
            }
          } else {
            $this->outputJavascriptCommand($idx, FALSE, $this->_gt('Database error.'), 'reset');
          }
        } else {
          $this->outputJavascriptCommand($idx, FALSE, $this->_gt('Table not found.'), 'reset');
        }
      } else {
        $this->outputJavascriptCommand($idx, FALSE, $this->_gt('File not found.'), 'reset');
      }
    } else {
      $this->outputJavascriptCommand($idx, FALSE, $this->_gt('Invalid table name.'), 'reset');
    }
  }

  function outputJavaScriptCommand($idx, $success, $msg, $cmd) {
    if ($this->outputMode = 'js') {
      if (!$this->javascriptRPC) {
        $this->javascriptRPC = TRUE;
        echo '<html><body onload="parent.onRpcStopped();">',"\n";
      }
      $jsCmd = sprintf('parent.onReturn_%s(%d, %s, "%s");',
        $cmd, (int)$idx, (($success) ? 'true' : 'false'), papaya_strings::escapeHTMLChars($msg));
      echo '<script type="text/javascript">',"\n";
      echo $jsCmd;
      echo '</script>',"\n";
      if ($this->debugRPC) {
        echo $jsCmd,'<br>';
      }
      flush();
    }
  }

  function outputJavaScriptFinish() {
    if ($this->outputMode = 'js' && $this->javascriptRPC) {
      print('<script type="text/javascript">'.LF);
      print('window.onload = null;'.LF);
      print('</script>'.LF);
      print('</body></html>'.LF);
      flush();
    }
  }

  /**
  * Insert CSV
  *
  * @param string $table
  * @param string $file
  * @access public
  * @return array
  */
  function insertCSV($table, $file) {
    $dataSize = 0;
    $sqlQueryCount = 0;
    include_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');
    if ($fh = fopen($file, 'r')) {
      $buffer = array();
      $bufferSize = 0;
      $maximumSize = filesize($file);
      if($dataFields = fgetcsv($fh, $maximumSize, ',')) {
        foreach($dataFields as $key=>$val) {
          $fields[$key] = papaya_strings::ensureUTF8(strtr($val, array('\\\\n'=>'\\n','\\n'=>"\n")));
        }
      }
      $outputJS = ((!ini_get('session.use_trans_sid')) && $this->javascriptRPC);
      if ($outputJS) {
        $this->outputJavascriptCommand((int)$this->params['table_idx'], TRUE, 'Reading ', 'update');
      }
      while($dataLine = fgetcsv($fh, $maximumSize, ',')) {
        foreach($dataLine as $key=>$val) {
          $dataRow[$fields[$key]] = papaya_strings::ensureUTF8(strtr($val, array('\\\\n'=>'\\n','\\n'=>"\n")));
          $bufferSize += strlen($dataRow[$fields[$key]]);
        }
        $buffer[] = $dataRow;
        if ($bufferSize > 10000 && count($buffer) > 0) {
          if ($outputJS) {
            $this->outputJavascriptCommand((int)$this->params['table_idx'], TRUE, 'Insert data', 'update');
          }
          $this->databaseInsertRecords($table, $buffer);
          $buffer = array();
          $dataSize += $bufferSize;
          $sqlQueryCount++;
          $bufferSize = 0;
          if ($outputJS) {
            $this->outputJavascriptCommand((int)$this->params['table_idx'], TRUE, 'Reading', 'update');
          }
        }
      }
      if ($bufferSize > 0 && count($buffer) > 0) {
        if ($outputJS) {
          $this->outputJavascriptCommand((int)$this->params['table_idx'], TRUE, 'Insert data', 'update');
        }
        $this->databaseInsertRecords($table, $buffer);
        $dataSize += $bufferSize;
        $sqlQueryCount++;
      }
      return array('queries'=>$sqlQueryCount,'data'=>$dataSize);
    }
    return FALSE;
  }

  function initOptionValues() {
    if (isset($GLOBALS['PAPAYA_OPTIONS'])) {
      return $GLOBALS['PAPAYA_OPTIONS']->loadOptions();
    }
    return FALSE;
  }

  function initModules() {
    $modules = &new papaya_modulemanager();
    $path = dirname(dirname(__FILE__));
    $modules->loadTables = TRUE;
    $modules->searchModules($path.'/'.$modules->modulesPath);
    if (isset($modules->packages) && is_array($modules->packages) && count($modules->packages) > 0 &&
        isset($modules->modules) && is_array($modules->modules) && count($modules->modules) > 0) {
      $modules->updataPackageTable();
      $this->outputJavascriptCommand(PAPAYA_DB_TBL_MODULEGROUPS, TRUE,
        (int)$this->getTableCount(PAPAYA_DB_TBL_MODULEGROUPS, PAPAYA_DB_TABLEPREFIX), 'count');
      $modules->updataModuleTable();
      $this->outputJavascriptCommand(PAPAYA_DB_TBL_MODULES, TRUE,
        (int)$this->getTableCount(PAPAYA_DB_TBL_MODULES, PAPAYA_DB_TABLEPREFIX), 'count');
      return TRUE;
    }

    return FALSE;
  }
}

?>