<?php
/**
* Administration Papaya-Module
*
* @package papaya_modules
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_modulemanager.php,v 1.17 2006/05/03 10:57:22 weinert Exp $
*/

/**
* Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');
/**
* XML-Handling
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_simple_xmltree.php');

/**
* Administration Papaya-Module
*
* @package papaya_modules
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class papaya_modulemanager extends base_db {
  var $modulesFileName = 'modules.xml';
  var $modulesPath = 'modules/';
  var $modulesParamNames = array('type','guid','name','class','file');
  var $moduleTypes = array('page', 'box', 'admin', 'cronjob', 'time', 'event', 'output', 'image', 'parser');
  var $tableModules = PAPAYA_DB_TBL_MODULES;
  var $tableModulegroups = PAPAYA_DB_TBL_MODULEGROUPS;

  var $packages = array();
  var $modules = array();
  var $tables = array();
  var $baseDirectory = '';

  var $loadTables = FALSE;

  var $moduleDialog = NULL;


  /**
  * Initialize parameters
  *
  * @param string $paramName optional, default value 'mods'
  * @access public
  */
  function initialize($paramName = 'mods') {
    $this->paramName = $paramName;
    $this->sessionParamName = 'PAPAYA_SESS_'.$this->paramName;
    $this->initializeParams();

    $this->sessionParams = $this->getSessionValue($this->sessionParamName);

    $this->initializeSessionParam('module_id', array('cmd', 'table', 'field'));
    $this->initializeSessionParam('table', array('cmd', 'module_id', 'field'));
    $this->initializeSessionParam('pkg_id', array('module_id', 'cmd', 'table', 'field'));
  }
  
  function getModulesPath() {
    $path = realpath(PAPAYA_INCLUDE_PATH);
    return $this->checkTrailingSlash(str_replace('\\', '/', $path.'/'.$this->modulesPath));
  }

  /**
  * Execute - basic function for handling parameters
  *
  * @access public
  */
  function execute() {
    switch (@$this->params['cmd']) {
      case 'scan' :
        // Hier wird ein absoluter Pfad gebraucht - also wandern wir ein wenig durch den Verzeichnisbaum
        
        $this->loadTables = TRUE;
        $this->searchModules($this->getModulesPath());
        $this->updataPackageTable();
        $this->updataModuleTable();
        unset($this->params);
        break;
      case 'module_save' :
        if (isset($this->params['module_id']) && $this->loadModule($this->params['module_id'])) {
          if ($this->initModuleDialog()) {
            if ($this->moduleDialog->modified()) {
              if ($this->saveModule()) {
                $this->addMsg(MSG_INFO, $this->_gtf('%s modified.', $this->_gt('Module')));
              }
            }
          }
        }
        break;
      case 'pkg_table_create' :
        if (isset($this->params['pkg_table_create_confirm']) && isset($this->params['pkg_id'])) {
          $this->loadPackages();
          if (isset($this->packages[$this->params['pkg_id']])) {
            $relPath = $this->packages[$this->params['pkg_id']]['modulegroup_path'];
            $path = $this->getModulesPath().$relPath;
            $this->loadPackageData($this->params['pkg_id']);
            if (isset($this->tables) && is_array($this->tables) && count($this->tables) > 0) {
              $tableExists = 0;
              $tableCreated = 0;
              foreach($this->tables as $tableName=>$inDatabase) {
                if ($inDatabase === TRUE) {
                  @$tableExits++;
                } else {
                  $xmlFileName = $path.'DATA/table_'.$tableName.'.xml';
                  if ($struct = $this->loadTableStructure($xmlFileName)) {
                    if ($this->databaseCreateTable($struct, PAPAYA_DB_TABLEPREFIX)) {
                      $tableCreated++;
                      $this->addMsg(MSG_INFO, $this->_gtf('Table "%s" created.', $tableName));
                    } else {
                      $this->addMsg(MSG_ERROR, $this->_gtf('Could not create table "%s".', $tableName));
                    }
                    unset($struct);
                  } else {
                    $this->addMsg(MSG_ERROR, $this->_gtf('Could not find xml file for table "%s".', $tableName));
                  }
                }
              }
              if ($tableExists + $tableCreated = count($this->tables)) {
                unset($this->params['cmd']);
              }
            }
          }
        }
      case 'table_create' :
        if (isset($this->params['table']) && isset($this->params['table_create_confirm'])) {
          $this->loadPackages();
          if (isset($this->packages[$this->params['pkg_id']])) {
            $relPath = $this->packages[$this->params['pkg_id']]['modulegroup_path'];
          }
          $path = $this->getModulesPath().$relPath;
          $xmlFileName = $path.'DATA/table_'.$this->params['table'].'.xml';
          if ($struct = $this->loadTableStructure($xmlFileName)) {
            if ($this->databaseCreateTable($struct, PAPAYA_DB_TABLEPREFIX)) {
              unset($this->params['cmd']);
              $this->addMsg(MSG_INFO, $this->_gt('Table created.'));
            }
            unset($struct);
          }
        }
        break;
      case 'field_add' :
      case 'field_update' :
        if (isset($this->params['field_action_confirm']) && $this->params['field_action_confirm'] &&
            isset($this->params['table']) && checkit::isAlphaNumChar($this->params['table'])) {
          $this->loadPackages();
          if (isset($this->packages[$this->params['pkg_id']])) {
            $relPath = $this->packages[$this->params['pkg_id']]['modulegroup_path'];
          }
          $path = $this->getModulesPath().$relPath;
          $xmlFileName = $path.'DATA/table_'.$this->params['table'].'.xml';
          if ($struct = $this->loadTableStructure($xmlFileName)) {
            //ok Struktur geladen - jetzt die eigentliche aktion
            if (isset($struct['fields'][$this->params['field']])) {
              $field = &$struct['fields'][$this->params['field']];
              switch($this->params['cmd']) {
                case 'field_add' :
                  if ($this->databaseAddField(PAPAYA_DB_TABLEPREFIX.'_'.$this->params['table'], $field)) {
                    unset($this->params['cmd']);
                    $this->addMsg(MSG_INFO, $this->_gtf('%s added.', $this->_gt('Field')));
                  }
                  break;
                case 'field_update' :
                  if ($this->databaseChangeField(PAPAYA_DB_TABLEPREFIX.'_'.$this->params['table'], $field)) {
                    unset($this->params['cmd']);
                    $this->addMsg(MSG_INFO, $this->_gtf('%s modified.', $this->_gt('Field')));
                  }
                  break;
              }
            }
            unset($struct);
          }
        }
        break;
      case 'field_delete' :
        if (isset($this->params['field_action_confirm']) && $this->params['field_action_confirm'] &&
            isset($this->params['table']) && checkit::isAlphaNumChar($this->params['table']) &&
            isset($this->params['field']) && checkit::isAlphaNumChar($this->params['field'])) {
          // hier ist xml laden nicht ntig - da das feld dort ehh nicht drin ist
          if ($this->databaseDropField(PAPAYA_DB_TABLEPREFIX.'_'.$this->params['table'], $this->params['field'])) {
            unset($this->params['cmd']);
            $this->addMsg(MSG_INFO, $this->_gtf('%s deleted.', $this->_gt('Field')));
          }
        }
        break;
      case 'index_add' :
      case 'index_update' :
        if (isset($this->params['index_action_confirm']) && $this->params['index_action_confirm'] &&
            isset($this->params['table']) && checkit::isAlphaNumChar($this->params['table'])) {
          $this->loadPackages();
          if (isset($this->packages[$this->params['pkg_id']])) {
            $relPath = $this->packages[$this->params['pkg_id']]['modulegroup_path'];
          }
          $path = $this->getModulesPath().$relPath;
          $xmlFileName = $path.'DATA/table_'.$this->params['table'].'.xml';
          if ($struct = $this->loadTableStructure($xmlFileName)) {
            //ok Struktur geladen - jetzt die eigentliche aktion
            if (isset($struct['keys'][$this->params['index']])) {
              $index = &$struct['keys'][$this->params['index']];
              switch($this->params['cmd']) {
                case 'index_add' :
                  if ($this->databaseAddIndex(PAPAYA_DB_TABLEPREFIX.'_'.$this->params['table'], $index)) {
                    unset($this->params['cmd']);
                    $this->addMsg(MSG_INFO, $this->_gtf('%s added.', $this->_gt('Index')));
                  }
                  break;
                case 'index_update' :
                  if ($this->databaseChangeIndex(PAPAYA_DB_TABLEPREFIX.'_'.$this->params['table'], $index)) {
                    unset($this->params['cmd']);
                    $this->addMsg(MSG_INFO, $this->_gtf('%s modified.', $this->_gt('Index')));
                  }
                  break;
              }
            }
            unset($struct);
          }
        }
        break;
      case 'index_delete' :
        if (isset($this->params['index_action_confirm']) && $this->params['index_action_confirm'] &&
            isset($this->params['table']) && checkit::isAlphaNumChar($this->params['table']) &&
            isset($this->params['index']) && checkit::isAlphaNumChar($this->params['index'])) {
          // hier ist xml laden nicht ntig - da der index dort ehh nicht drin ist
          if ($this->databaseDropIndex(PAPAYA_DB_TABLEPREFIX.'_'.$this->params['table'], $this->params['index'])) {
            unset($this->params['cmd']);
            $this->addMsg(MSG_INFO, $this->_gtf('%s deleted.', $this->_gt('Index')));
          }
        }
        break;
      case 'sync' :
        if (isset($this->params['table_action_confirm']) && $this->params['table_action_confirm'] &&
            isset($this->params['table']) && checkit::isAlphaNumChar($this->params['table'])) {
          $this->loadPackages();
          if (isset($this->packages[$this->params['pkg_id']])) {
            $relPath = $this->packages[$this->params['pkg_id']]['modulegroup_path'];
          }
          $path = $this->getModulesPath().$relPath;
          $xmlFileName = $path.'DATA/table_'.$this->params['table'].'.xml';
          if ($struct = $this->loadTableStructure($xmlFileName)) {
            $struct['name'] = $this->params['table'];
            if ($this->syncTableStructure($struct)) {
              $this->addMsg(MSG_INFO, $this->_gt('Table structure updated!'));
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error!'));
            }
            unset($struct);
            unset($this->params['cmd']);
          }
        }

    }
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);

    $this->loadPackages();
    $this->loadPackageData(@(int)$this->params['pkg_id']);
    $this->loadModule(@$this->params['module_id']);
    $this->loadTable(@(int)$this->params['pkg_id'], @$this->params['table']);
  }

  /**
  * Get XML
  *
  * @access public
  */
  function getXML() {
    $this->getButtonsXML();
    $this->getPackageListView();
    $this->getModuleListView();
    $this->getTablesListView();
    if (!$this->getPackageDialog()) {
      $this->getIndexDialog();
      $this->getFieldDialog();
      $this->getTableDialog();
      $this->getModuleDialog();
    }
  }

  /**
  * Get Buttons XML
  *
  * @access public
  */
  function getButtonsXML() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $menubar = &new base_btnbuilder;
    $menubar->images = &$this->images;
    $menubar->addButton('Scan',
        $this->baseLink.'?'.$this->paramName.'[cmd]=scan',
        58, 'Search modules');
    if (isset($this->tableStruct) && is_array($this->tableStruct) &&
        isset($this->tableStruct['actions']) && $this->tableStruct['actions'] > 0) {
      $menubar->addSeperator();
      $menubar->addButton('Syncronize',
          $this->baseLink.'?'.$this->paramName.'[cmd]=sync',
          95, 'Syncronize table structure');
    }
    if ($str = $menubar->getXML()) {
      $this->layout->add('<menu ident="edit">'.$str.'</menu>', 'menus');
    }
  }

  /**
  * Get glyph
  *
  * @access public
  * @return string
  */
  function getGlyph() {
    if (isset($_GET['module']) && preg_match('/^[a-fA-F\d]{32}$/D', $_GET['module'])) {
      $sql = "SELECT module_path, module_glyph FROM %s WHERE module_guid = '%s'";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableModules, $_GET['module']))) {
        if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $imageFileName = $this->getModulesPath().$row['module_path'].$row['module_glyph'];
          list(,,$type) = getImageSize($imageFileName);
          if ($type > 0 && $type < 4) {
            if ($fh = @fopen($imageFileName, 'r')) {
              switch($type) {
                case 1 : header('Content-type: image/gif'); break;
                case 2 : header('Content-type: image/jpg'); break;
                case 3 : header('Content-type: image/png'); break;
              }
              fpassthru($fh);
              fclose($fh);
            }
          }
        }
      }
    }
    return '';
  }

  /**
  * Update data module table
  *
  * @access public
  */
  function updataModuleTable() {
    if (isset($this->modules)) {
      $this->addMsg(MSG_INFO, $this->_gt('Updating modules table.'));
      $this->loadPackages();
      $packages = array();
      if (isset($this->packages) && is_array($this->packages)) {
        foreach($this->packages as $pkgIdx=>$package) {
          $packages[$package['modulegroup_path']] = $pkgIdx;
        }
      }
      $modUpdate = array();
      $modDelete = array();
      $sql = "SELECT module_guid, module_type, module_useoutputfilter,
                      module_title, module_title_org, module_description, module_class,
                      module_file, module_path, module_glyph, modulegroup_id
              FROM %s
              ORDER BY module_guid";
      if ($res = $this->databaseQueryFmt($sql, $this->tableModules)) {
        $modInDatabase = array();
        while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          if (isset($this->modules[$row['module_guid']])) {
            $module = $this->modules[$row['module_guid']];
            if ($module['type'] !== $row['module_type'] ||
                ($module['outputfilter'] == 'no') == $row['module_useoutputfilter'] ||
                $module['name'] !== $row['module_title_org'] ||
                $module['description'] !== $row['module_description'] ||
                $module['class'] !== $row['module_class'] ||
                $module['file'] !== $row['module_file'] ||
                $module['path'] !== $row['module_path'] ||
                $module['glyph'] !== $row['module_glyph'] ||
                $packages[$module['path']] !== $row['modulegroup_id']) {
              $modUpdate[] = $row['module_guid'];
            } else {
              $modInDatabase[ $row['module_guid']] = TRUE;
            }
          } else {
            $modDelete[] = $row['module_guid'];
          }
        }
        $res->free();
        // nicht mehr vorhandene lschen
        if (is_array($modDelete) && count($modDelete) > 0) {
          $this->databaseDeleteRecord($this->tableModules, 'module_guid', $modDelete);
          $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modules deleted.'), count($modDelete)));
        }
        // genderte updaten
        if (is_array($modUpdate) && count($modUpdate) > 0) {
          foreach($modUpdate as $guid) {
            $module = $this->modules[$guid];
            $modInDatabase[$guid] = TRUE;
            $data = array(
              'module_type' => $module['type'],
              'module_useoutputfilter' => ($module['outputfilter'] == 'no') ? 0 : 1,
              'module_title_org' => $module['name'],
              'module_title_org' => $module['name'],
              'module_description' => $module['description'],
              'module_class' => $module['class'],
              'module_file' => $module['file'],
              'module_path' => $module['path'],
              'module_glyph' => $module['glyph'],
              'modulegroup_id' => $packages[$module['path']]
            );
            $this->databaseUpdateRecord($this->tableModules, $data, 'module_guid', $guid);
          }
          $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modules updated.'), count($modUpdate)));
        }
        // neue einfgen
        if (is_array($this->modules)) {
          $countNew = 0;
          foreach($this->modules as $guid=>$module) {
            if (!isset($modInDatabase[$guid])) {
              $data = array(
                'module_guid' => $module['guid'],
                'module_type' => $module['type'],
                'module_useoutputfilter' => ($module['outputfilter'] == 'no') ? 0 : 1,
                'module_title' => $module['name'],
                'module_title_org' => $module['name'],
                'module_description' => $module['description'],
                'module_class' => $module['class'],
                'module_file' => $module['file'],
                'module_path' => $module['path'],
                'module_glyph' => $module['glyph'],
                'module_active' => 1,
                'modulegroup_id' => $packages[$module['path']]
              );
              $this->databaseInsertRecord($this->tableModules, NULL, $data);
              $countNew++;
            }
          }
          if ($countNew > 0) {
            $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modules added.'), $countNew));
          }
        }
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Update data package table
  *
  * @access public
  */
  function updataPackageTable() {
    if (isset($this->packages) && is_array($this->packages)) {
      if ($this->databaseEmptyTable($this->tableModulegroups) !== FALSE) {
        $idx = 1;
        $data = array();
        foreach($this->packages as $package) {
          if (isset($package['tables']) && is_array($package['tables']) && count($package['tables'])) {
            $tables = implode(',', $package['tables']);
          } else {
            $tables = '';
          }
          $data[] = array(
            'modulegroup_id' =>  $idx++,
            'modulegroup_title' =>  $package['name'],
            'modulegroup_path' => $package['path'],
            'modulegroup_tables' => $tables
          );
        }
        if (isset($data) && is_array($data) && count($data) > 0) {
          $this->databaseInsertRecords($this->tableModulegroups, $data);
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  /**
  * Search modules
  *
  * @param string $path
  * @access public
  */
  function searchModules($path) {
    unset($this->packages);
    unset($this->modules);
    if (file_exists($path)) {
      $this->baseDirectory = $path;
      $this->scanDirectory($path);
      if (is_array($this->packages)) {
        uasort($this->packages, array(&$this, 'comparePackageArrays'));
      }
      if (is_array($this->modules)) {
        uasort($this->modules, array(&$this, 'compareModuleArrays'));
      }
    } else {
      $this->addMsg(MSG_ERROR, $this->_gt('No modules directory found.'));
    }
  }

  /**
  * Check trailing slash
  *
  * @param string $path
  * @access public
  * @return string path ends with /
  */
  function checkTrailingSlash($path) {
    if (substr($path, -1) != '/') {
      return $path.'/';
    }
    return $path;
  }

  /**
  * Scan directory
  *
  * @param string $path
  * @access public
  */
  function scanDirectory($path) {
    $path = $this->checkTrailingSlash($path);
    $packageFileName =  $path.$this->modulesFileName;
    if (file_exists($packageFileName)) {
      $relPath = substr($path, strlen($this->baseDirectory));
      $this->packages[$relPath] = $this->loadPackageFile($packageFileName, $relPath);
      $this->packages[$relPath]['path'] = $relPath;
    }
    if ($dh = opendir($path)) {
      while($file = readdir($dh)) {
        if (substr($file, 0, 1) != '.') {
          if (is_dir($path.$file) && $file != 'CVS') {
            $this->scanDirectory($path.$file);
          }
        }
      }
      closedir($dh);
    }
  }

  /**
  * Load package file
  *
  * @param $packageFileName
  * @param $relPath
  * @access public
  * @return array
  */
  function loadPackageFile($packageFileName, $relPath) {
    if (file_exists($packageFileName)) {
      $packageData = array();
      $xmlTree = $this->createXMLTree(file_get_contents($packageFileName));
      if (is_object($xmlTree) && is_array($xmlTree->children)) {
        foreach($xmlTree->children as $node) {
          if (is_object($node)) {
            switch ($node->name) {
              case 'name' :
                $s = '';
                $node->printText($s);
                $packageData['name'] = $s;
                break;
              case 'description' :
                $s = '';
                $node->printText($s);
                $packageData['description'] = $s;
                break;
              case 'modules' :
                if (is_array($node->children)) {
                  foreach($node->children as $moduleNode) {
                    if (is_object($moduleNode) && $moduleNode->name == 'module') {
                      $allParams = TRUE;
                      $moduleData = array();
                      foreach($this->modulesParamNames as $moduleParam) {
                        if (isset($moduleNode->attributes[$moduleParam])) {
                          $moduleData[$moduleParam] = $moduleNode->attributes[$moduleParam];
                        } else {
                          $allParams = FALSE;
                        }
                      }
                      $moduleData['glyph'] = (isset( $moduleNode->attributes['glyph'])) ?
                          $moduleNode->attributes['glyph'] : '';
                      $moduleData['outputfilter'] = (isset( $moduleNode->attributes['outputfilter'])) ?
                          $moduleNode->attributes['outputfilter'] : 'yes';
                      if ($allParams && in_array($moduleData['type'], $this->moduleTypes)) {
                        $s = '';
                        $moduleNode->printText($s);
                        $moduleData['description'] = $s;
                        $moduleData['path'] = $relPath;
                        $this->modules[$moduleData['guid']] = $moduleData;
                        $packageData['modules'][] = $moduleData['guid'];
                      }
                    }
                    if (@is_array($packageData['modules'])) {
                      $packageData['modules'] = array_unique($packageData['modules']);
                    }
                  }
                }
                break;
              case 'tables' :
                if ($this->loadTables && is_array($node->children)) {
                  foreach($node->children as $tableNode) {
                    if (is_object($tableNode) && $tableNode->name == 'table' &&
                        isset($tableNode->attributes['name'])) {
                      $packageData['tables'][] = strtolower($tableNode->attributes['name']);
                    }
                  }
                  if (isset($packageData['tables']) && is_array($packageData['tables'])) {
                    sort($packageData['tables']);
                  }
                }
                break;
            }
          }
        }
        unset($xmlTree);
        return $packageData;
      }
    }
    return FALSE;
  }

  /**
  * Create XML tree
  * @param string $xmlData XML-string
  * @return mixed object simple_xmlnode $xmlTree XML-tree
  */
  function createXMLTree($xmlData) {
    if (trim($xmlData) != '') {
      $xmlTree = simple_xmltree::create();
      if (is_object($xmlTree)) {
        if (0 === $xmlTree->loadFromString($xmlData)) {
          if (is_object($xmlTree->root)) {
            return $xmlTree->root;
          }
        } elseif (defined('PAPAYA_DBG_XML_USERINPUT') && PAPAYA_DBG_XML_USERINPUT === '1') {
          $this->lastXMLError = @$xmlTree->lastError;
          unset($xmlTree);
        }
      }
    }
    return FALSE;
  }

  /**
  * compare module arrays
  *
  * @param array $a
  * @param array $b
  * @access public
  * @return integer
  */
  function compareModuleArrays($a,$b) {
    if ($a['name'] != $b['name']) {
      return ($a['name'] > $b['name']) ? 1 : -1;
    } elseif ($a['class'] != $b['class']) {
      return ($a['class'] > $b['class']) ? 1 : -1;
    } else {
      return ($a['guid'] > $b['guid']) ? 1 : -1;
    }
    return 0;
  }

  /**
  * Compare package arrays
  *
  * @param array $a
  * @param array $b
  * @access public
  * @return integer
  */
  function comparePackageArrays($a,$b) {
    if ($a['path'] != $b['path']) {
      return ($a['path'] > $b['path']) ? 1 : -1;
    }elseif ($a['name'] != $b['name']) {
      return ($a['name'] > $b['name']) ? 1 : -1;
    } elseif ($a['description'] != $b['description']) {
      return ($a['description'] > $b['description']) ? 1 : -1;
    }
    return 0;

  }

  /**
  * Load packages
  *
  * @access public
  */
  function loadPackages() {
    unset($this->packages);
    $sql = "SELECT modulegroup_id, modulegroup_title, modulegroup_path FROM %s ORDER BY modulegroup_title";
    if ($res = $this->databaseQueryFmt($sql, $this->tableModulegroups)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->packages[(int)$row['modulegroup_id']] = $row;
      }
    }
  }

  /**
  * Load package data
  *
  * @param integer $pkgId
  * @access public
  */
  function loadPackageData($pkgId) {
    unset($this->modules);
    if (isset($this->packages[$pkgId])) {
      $relPath = $this->packages[$pkgId]['modulegroup_path'];
      $packageFileName = $this->getModulesPath().$relPath.$this->modulesFileName;
      $this->loadTables = TRUE;
      $package = $this->loadPackageFile($packageFileName, $relPath);
      $this->loadModuleDBData();
      $this->loadTableDBData($package['tables']);
    }
  }

  /**
  * Load module database data
  *
  * @access public
  */
  function loadModuleDBData() {
    if (isset($this->modules) && is_array($this->modules)) {
      $moduleGuids = array_keys($this->modules);
      if (count($moduleGuids) == 1) {
        $filter = "WHERE module_guid = '".addslashes($moduleGuids[0])."' ";
      } else {
        $filter = '';
        foreach ($moduleGuids as $moduleGuid) {
          $filter .= ", '".addslashes($moduleGuid)."'";
        }
        $filter = 'WHERE module_guid IN ('.substr($filter, 2).') ';
      }
      $sql = "SELECT module_guid, module_type, module_useoutputfilter,
                      module_title, module_title_org, module_description, module_class,
                      module_file, module_path, module_glyph,
                      module_active
              FROM %s
              $filter
              ORDER BY module_guid";
      if ($res = $this->databaseQueryFmt($sql, $this->tableModules)) {
        while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          if (isset($this->modules[$row['module_guid']])) {
            $module = $this->modules[$row['module_guid']];
            if ($module['type'] !== $row['module_type'] ||
                $module['name'] !== $row['module_title_org'] ||
                $module['description'] !== $row['module_description'] ||
                $module['class'] !== $row['module_class'] ||
                $module['file'] !== $row['module_file'] ||
                $module['path'] !== $row['module_path'] ||
                $module['glyph'] !== $row['module_glyph']) {
              $this->modules[$row['module_guid']]['error'] = TRUE;
            } else {
              $this->modules[$row['module_guid']]['error'] = FALSE;
            }
            $this->modules[$row['module_guid']]['active'] = (boolean)$row['module_active'];
          }
        }
        $res->free();
      }
    }
  }
  
  function loadModulesList($moduleType) {
    $modules = array();
    $sql = "SELECT module_guid, module_type, module_title
              FROM %s
             WHERE module_type = '%s' AND module_active = 1";
    $params = array($this->tableModules, $moduleType);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $modules[] = $row;
      }
      $res->free();
    }
    return $modules;    
  }

  /**
  * Load module
  *
  * @param integer $moduleId
  * @access public
  * @return boolean
  */
  function loadModule($moduleId) {
    unset($this->module);
    $sql = "SELECT module_guid, module_type, module_useoutputfilter,
                    module_title, module_title_org, module_description, module_class,
                    module_file, module_path, module_glyph,
                    module_active
            FROM %s
            WHERE module_guid = '%s'";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableModules, $moduleId))) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->module = $row;
      }
      $res->free();
    }
    return isset($this->module);
  }

  /**
  * Load table
  *
  * @param integer $pkgId
  * @param string $table
  * @access public
  */
  function loadTable($pkgId, $table) {
    if (isset($table) && trim($table) != '' && isset($this->packages[$pkgId]) && is_array($this->packages[$pkgId])) {
      $package = &$this->packages[$pkgId];
      $relPath = $package['modulegroup_path'];
      $absPath = $this->getModulesPath().$relPath;
      $tableFile = $absPath.'DATA/table_'.$table.'.xml';
      $this->tableStruct = $this->loadTableStructure($tableFile);
    }
  }

  /**
  * Compare field definition in XML and database
  *
  * @param array &$xmlField
  * @param array &$databaseField
  * @access public
  * @return boolean
  */
  function compareFieldStructure(&$xmlField, &$databaseField) {
    if ($this->databaseConnect() && is_object($this->databaseObject) && method_exists($this->databaseObject, 'compareFieldStructure')) {
      return $this->databaseObject->compareFieldStructure($this, $xmlField, $databaseField);
    }
    return FALSE;
  }

  /**
  * Compare key structure
  *
  * @param array $xmlKey
  * @param array $databaseKey
  * @access public
  * @return boolean
  */
  function compareKeyStructure($xmlKey, $databaseKey) {
    if ($this->databaseConnect() && is_object($this->databaseObject) && method_exists($this->databaseObject, 'compareKeyStructure')) {
      return $this->databaseObject->compareKeyStructure($this, $xmlField, $databaseField);
    }
    return FALSE;
  }

  /**
  * Load table structure
  *
  * @param string $xmlFileName
  * @access public
  * @return mixed array or FALSE
  */
  function loadTableStructure($xmlFileName) {
    if (file_exists($xmlFileName)) {
      $tableData = array();
      $xmlTree = $this->createXMLTree(file_get_contents($xmlFileName));
      if (is_object($xmlTree) && is_array($xmlTree->children)) {
        $tableData['name'] = strtolower($xmlTree->attributes['name']);
        $tableDatabaseStruct = $this->databaseQueryTableStructure($tableData['name'], PAPAYA_DB_TABLEPREFIX);
        foreach($xmlTree->children as $node) {
          if (is_object($node)) {
            switch ($node->name) {
              case 'fields' :
                if (is_array($node->children)) {
                  foreach($node->children as $fieldNode) {
                    if (is_object($fieldNode) && $fieldNode->name == 'field' &&
                        isset($fieldNode->attributes['name'])) {
                      $fieldName = strtolower($fieldNode->attributes['name']);
                      $tableData['fields'][$fieldName] = $fieldNode->attributes;
                      $tableData['fields'][$fieldName]['name'] = $fieldName;
                      if (isset($tableDatabaseStruct['fields'][$fieldName])) {
                        if ($this->compareFieldStructure($tableData['fields'][$fieldName],
                              $tableDatabaseStruct['fields'][$fieldName])) {
                          $tableData['fields'][$fieldName]['action'] = 'update';
                          @$tableData['actions']++;
                        } else {
                          $tableData['fields'][$fieldName]['action'] = FALSE;
                        }
                      } else {
                        $GLOBALS['debug'] = TRUE;
                        $tableData['fields'][$fieldName]['action'] = 'add';
                        @$tableData['actions']++;
                      }
                    }
                  }
                }
                if (isset($tableDatabaseStruct['fields']) && is_array($tableDatabaseStruct['fields'])) {
                  foreach($tableDatabaseStruct['fields'] as $field) {
                    $fieldName = strtolower($field['name']);
                    if (!isset($tableData['fields'][$fieldName])) {
                      $tableData['fields'][$fieldName] = $field;
                      $tableData['fields'][$fieldName]['action'] = 'delete';
                      @$tableData['actions']++;
                    }
                  }
                }
                break;
              case 'keys' :
                if (is_array($node->children)) {
                  foreach($node->children as $keyNode) {
                    if (is_object($keyNode) && is_array($keyNode->children)) {
                      $keyName = strtolower(@$keyNode->attributes['name']);
                      switch ($keyNode->name) {
                        case 'primary-key' :
                          $keyName = 'PRIMARY';
                        case 'key' :
                          $tableData['keys'][$keyName] = $keyNode->attributes;
                          $tableData['keys'][$keyName]['name'] = $keyName;
                          foreach($keyNode->children as $fieldNode) {
                            if (is_object($fieldNode) && $fieldNode->name == 'field') {
                              $s = '';
                              $fieldNode->printText($s);
                              $tableData['keys'][$keyName]['fields'][] = strtolower($s);
                              if (isset($fieldNode->attributes['size']) && $fieldNode->attributes['size'] > 0) {
                                $tableData['keys'][$keyName]['keysize'][strtolower($s)] =
                                  (int)$fieldNode->attributes['size'];
                              }
                            }
                          }
                          if (isset($tableDatabaseStruct['keys'][$keyName])) {
                            if ($this->compareKeyStructure($tableData['keys'][$keyName],
                                  $tableDatabaseStruct['keys'][$keyName])) {
                              $tableData['keys'][$keyName]['action'] = 'update';
                              @$tableData['actions']++;
                            } else {
                              $tableData['keys'][$keyName]['action'] = FALSE;
                            }
                          } else {
                            $tableData['keys'][$keyName]['action'] = 'add';
                            @$tableData['actions']++;
                          }
                          break;
                      }
                    }
                  }
                }
                if (isset($tableDatabaseStruct['keys']) && is_array($tableDatabaseStruct['keys'])) {
                  foreach($tableDatabaseStruct['keys'] as $key) {
                    $keyName = $key['name'];
                    if (!isset($tableData['keys'][$keyName])) {
                      $tableData['keys'][$keyName] = $key;
                      $tableData['keys'][$keyName]['action'] = 'delete';
                      @$tableData['actions']++;
                    }
                  }
                }
                break;
            }
          }
        }
        unset($tableDatabaseStruct);
      }
      unset($xmlTree);
      return $tableData;
    }
    return FALSE;
  }

  /**
  * Syncronize a table structure
  *
  * @param array $tableStruct Table stucture
  * @access public
  * @return boolean;
  */
  function syncTableStructure(&$tableStruct) {
    foreach($tableStruct['fields'] as $fieldName=>$field) {
      switch($field['action']) {
        case 'add' :
          if ($this->databaseAddField(PAPAYA_DB_TABLEPREFIX.'_'.$tableStruct['name'], $field)) {
            $this->addMsg(MSG_INFO, $this->_gtf('%s added.', $this->_gt('Field').' '.$fieldName));
          } else {
            $this->addMsg(MSG_INFO, $this->_gtf('Can not add %s.', $this->_gt('Field').' '.$fieldName));
            return FALSE;
          }
          break;
        case 'delete' :
          if ($this->databaseDropField(PAPAYA_DB_TABLEPREFIX.'_'.$tableStruct['name'], $fieldName)) {
            $this->addMsg(MSG_INFO, $this->_gtf('%s deleted.', $this->_gt('Field').' '.$fieldName));
          } else {
            $this->addMsg(MSG_INFO, $this->_gtf('Can not delete %s.', $this->_gt('Field').' '.$fieldName));
            return FALSE;
          }
          break;
        case 'update' :
          if ($this->databaseChangeField(PAPAYA_DB_TABLEPREFIX.'_'.$tableStruct['name'], $field)) {
            $this->addMsg(MSG_INFO, $this->_gtf('%s modified.', $this->_gt('Field').' '.$fieldName));
          } else {
            $this->addMsg(MSG_INFO, $this->_gtf('Can not change %s.', $this->_gt('Field').' '.$fieldName));
            return FALSE;
          }
          break;
      }
    }
    foreach($tableStruct['keys'] as $keyName=>$key) {
      switch($key['action']) {
        case 'add' :
          if ($this->databaseAddIndex(PAPAYA_DB_TABLEPREFIX.'_'.$tableStruct['name'], $key)) {
            $this->addMsg(MSG_INFO, $this->_gtf('%s added.', $this->_gt('Index').' '.$keyName));
          } else {
            $this->addMsg(MSG_INFO, $this->_gtf('Can not add %s.', $this->_gt('Index').' '.$keyName));
            return FALSE;
          }
          break;
        case 'delete' :
          if ($this->databaseDropIndex(PAPAYA_DB_TABLEPREFIX.'_'.$tableStruct['name'], $keyName)) {
            $this->addMsg(MSG_INFO, $this->_gtf('Can not delete %s.', $this->_gt('Index').' '.$keyName));
          } else {
            return FALSE;
          }
          break;
        case 'update' :
          if ($this->databaseChangeIndex(PAPAYA_DB_TABLEPREFIX.'_'.$tableStruct['name'], $key)) {
            $this->addMsg(MSG_INFO, $this->_gtf('%s modified.', $this->_gt('Index').' '.$keyName));
          } else {
            $this->addMsg(MSG_INFO, $this->_gtf('Can not change %s.', $this->_gt('Index').' '.$keyName));
            return FALSE;
          }
          break;
      }
    }
    return TRUE;
  }

  /**
  * Save module
  *
  * @access public
  * @return mixed
  */
  function saveModule() {
    $data = array(
      'module_active' => (int)$this->params['module_active'],
      'module_title' => $this->params['module_title']
    );
    return $this->databaseUpdateRecord($this->tableModules, $data, 'module_guid', $this->params['module_id']);
  }

  /**
  * Load table database data
  *
  * @param &$tables
  * @access public
  */
  function loadTableDBData(&$tables) {
    unset($this->tables);
    if (isset($tables) && is_array($tables)) {
      $this->tables = array_flip($tables);
      if (@is_array($dbTables = $this->databaseQueryTableNames())) {
        foreach($dbTables as $tableName) {
          if (strpos($tableName, PAPAYA_DB_TABLEPREFIX.'_') !== FALSE) {
            $tableName = substr($tableName, strlen(PAPAYA_DB_TABLEPREFIX)+1);
            if (isset($this->tables[$tableName])) {
              $this->tables[$tableName] = TRUE;
            }
          }
        }
      }
    }
  }

  /**
  * Get package list view
  *
  * @access public
  */
  function getPackageListView() {
    $str = sprintf('<listview title="%s">', $this->_gt('Groups'));
    if (isset($this->packages) && is_array($this->packages)) {
      $str .= '<items>';
      foreach($this->packages as $package) {
        $selected = ($package['modulegroup_id'] == @$this->params['pkg_id']) ? ' selected="selected"' : '';
        $str .= sprintf('<listitem href="%s" image="%s" title="%s"%s/>',
          $this->baseLink.'?'.$this->paramName.'[pkg_id]='.(int)$package['modulegroup_id']
          .'&'.$this->paramName.'[module_id]=0&'.$this->paramName.'[table]=',
          $this->images[67], papaya_strings::escapeHTMLChars($package['modulegroup_title']), $selected);
      }
      $str .= '</items>';
    }
    $str .= '</listview>';
    $this->layout->addLeft($str);
  }

  /**
  * Get module list view
  *
  * @access public
  */
  function getModuleListView() {
    if (isset($this->packages) && is_array($this->packages) &&
        isset($this->modules) && is_array($this->modules)) {
      $str = sprintf('<listview title="%s">', $this->_gt('Modules'));
      $str .= '<cols>';
      $str .= '<col>'.papaya_strings::escapeHTMLChars($this->_gt('Module')).'</col>';
      $str .= '<col align="center">'.papaya_strings::escapeHTMLChars($this->_gt('Type')).'</col>';
      $str .= '<col align="center">'.papaya_strings::escapeHTMLChars($this->_gt('Active')).'</col>';
      $str .= '</cols>';
      $str .= '<items>';
      foreach($this->modules as $module) {
        if (isset($module) && is_array($module)) {
          $selected = ($module['guid'] == @$this->params['module_id']) ? ' selected="selected"' : '';

          if (isset($module['error']) && $module['error']) {
            $glyph = $this->images[65];
          } elseif (trim($module['glyph']) != '') {
            $glyph = './modglyph.php?module='.urlencode($module['guid']);
          } else {
            switch ($module['type']) {
              case 'page' :
                $glyph = $this->images[3];
                break;
              case 'box' :
                $glyph = $this->images[22];
                break;
              case 'admin' :
                $glyph = $this->images[15];
                break;
              case 'cronjob' :
                $glyph = $this->images[22];
                break;
              case 'time' :
                $glyph = $this->images[68];
                break;
              case 'event' :
                $glyph = $this->images[34];
                break;
              case 'output' :
                $glyph = $this->images[92];
                break;
              case 'image' :
                $glyph = $this->images[140];
                break;
              case 'parser' :
                $glyph = $this->images[15];
                break;
              default :
                $glyph = $this->images[3];
            }
          }

          $str .= sprintf('<listitem href="%s" image="%s" title="%s"%s>'.LF,
            $this->baseLink.'?'.$this->paramName.'[module_id]='.urlencode($module['guid']),
            $glyph, papaya_strings::escapeHTMLChars($module['name']), $selected);
          $str .= '<subitem align="center">'.papaya_strings::escapeHTMLChars($module['type']).'</subitem>';
          $activeGlyph = papaya_strings::escapeHTMLChars((@$module['active']) ? $this->images[29] : $this->images[30]);
          $str .= '<subitem align="center"><glyph src="'.$activeGlyph.'"/></subitem>';
          $str .= '</listitem>';
        }
      }
      $str .= '</items>';
      $str .= '</listview>';
      $this->layout->addRight($str);
    }
  }

  /**
  * Get tables list view
  *
  * @access public
  */
  function getTablesListView() {
    if (isset($this->tables) && is_array($this->tables) && count($this->tables) > 0) {
      $str = sprintf('<listview title="%s">', $this->_gt('Tables'));
      $str .= '<cols>';
      $str .= '<col>'.papaya_strings::escapeHTMLChars($this->_gt('Table')).'</col>';
      $str .= '<col align="center">'.papaya_strings::escapeHTMLChars($this->_gt('Exists')).'</col>';
      $str .= '</cols>';
      $str .= '<items>';
      foreach($this->tables as $tableName=>$inDatabase) {
        $selected = ($tableName == @$this->params['table']) ? ' selected="selected"' : '';
        $str .= sprintf('<listitem href="%s" image="%s" title="%s"%s>',
          $this->baseLink.'?'.$this->paramName.'[table]='.urlencode($tableName),
          $this->images[95], papaya_strings::escapeHTMLChars(PAPAYA_DB_TABLEPREFIX.'_'.$tableName), $selected);
        $activeGlyph = papaya_strings::escapeHTMLChars(($inDatabase === TRUE) ? $this->images[29] : $this->images[30]);
        $str .= '<subitem align="center"><glyph src="'.$activeGlyph.'"/></subitem>';
        $str .= '</listitem>';
      }
      $str .= '</items>';
      $str .= '</listview>';
      $this->layout->addRight($str);
    }
  }

  /**
  * Initialize module dialog
  *
  * @access public
  * @return boolean
  */
  function initModuleDialog() {
    if (!(isset($this->moduleDialog) && is_object($this->moduleDialog))) {
      if (isset($this->module) && is_array($this->module)) {
        include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
        $hidden = array(
          'save'=> 1,
          'cmd'=> 'module_save',
          'module_id' => $this->module['module_guid']
        );
        $data = array(
          'module_active' => $this->module['module_active'],
          'module_title' => $this->module['module_title'],
          'module_title_org' => $this->module['module_title_org'],
          'module_useoutputfilter' => $this->module['module_useoutputfilter'],
        );
        $yesnoArray = array(1=> $this->_gt('Yes'), 0=> $this->_gt('No'));
        $fields = array(
          'module_title' => array('Title', 'isAlphaNumChar', TRUE, 'input', 100, ''),
          'module_title_org' => array('Title default', 'isAlphaNumChar', TRUE, 'disabled_input', 100, ''),
          'module_active' => array('Active', 'isNum', TRUE, 'combo', $yesnoArray, '', 2),
          'module_useoutputfilter' => array('Output filter', 'isNum', TRUE, 'disabled_combo', $yesnoArray, '', 2),
        );
        $this->moduleDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
        $this->moduleDialog->dialogTitle = $this->_gt('Properties');
        $this->moduleDialog->baseLink = $this->baseLink;
        $this->moduleDialog->msgs = &$this->msgs;
        $this->moduleDialog->loadParams();
      } else {
        return FALSE;
      }
    }
    return TRUE;
  }

  /**
  * Get module dialog
  *
  * @access public
  */
  function getModuleDialog() {
    if ($this->initModuleDialog()) {
      if ($str = $this->moduleDialog->getDialogXML()) {
        $this->layout->add($str);
      }
    }
    if (isset($this->module) && is_array($this->module)) {
      if ($counts = $this->loadModuleUseCount($this->module['module_guid'])) {
        $result = sprintf('<listview title="%s">', $this->_gt('Used'));
        $result .= '<cols>';
        $result .= '<col>'.papaya_strings::escapeHTMLChars($this->_gt('Table')).'</col>';
        $result .= '<col align="center">'.papaya_strings::escapeHTMLChars($this->_gt('Count')).'</col>';
        $result .= '</cols>';
        $result .= '<items>';
        foreach($counts as $table=>$count) {
          $result .= sprintf('<listitem title="%s">', papaya_strings::escapeHTMLChars($table));
          $result .= '<subitem align="center">'.(int)@$count.'</subitem>';
          $result .= '</listitem>';
        }
        $result .= '</items>';
        $result .= '</listview>';
      }
      $this->layout->add($result);
    }
  }

  function getPackageDialog() {
    if ((isset($this->module) && is_array($this->module)) ||
        (isset($this->tableStruct) && is_array($this->tableStruct))) {
      return FALSE;
    } elseif (isset($this->params['pkg_id']) && isset($this->packages[$this->params['pkg_id']])
        && is_array($this->packages[$this->params['pkg_id']])) {
      $missing = FALSE;
      if (isset($this->tables)) {
        foreach($this->tables as $table=>$inDatabase) {
          if ($inDatabase !== TRUE) {
            $missing = TRUE;
            break;
          }
        }
      }
      if ($missing) {
        include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
        $hidden = array(
          'cmd' => 'pkg_table_create',
          'pkg_id' => $this->params['pkg_id'],
          'pkg_table_create_confirm'=>1,
        );
        $pkgTitle = papaya_strings::escapeHTMLChars($this->packages[$this->params['pkg_id']]['modulegroup_title']);
        $msg = $this->_gtf('Create missing tables for package "%s"?' , $pkgTitle);
        $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
        $dialog->msgs = &$this->msgs;
        $dialog->buttonTitle = 'Create';
        if ($str = $dialog->getMsgDialog()) {
          $this->layout->add($str);
        }
      }
    }
    return FALSE;
  }

  function loadModuleUseCount($modId) {
    $elements = array(
      PAPAYA_DB_TBL_VIEWS => 'module_guid',
      PAPAYA_DB_TBL_VIEWMODES => 'module_guid',
      PAPAYA_DB_TBL_CRONJOBS => 'cron_module_guid',
      PAPAYA_DB_TBL_CRONJOBS => 'job_module_guid'
    );
    $sql = "SELECT COUNT(*) FROM %s WHERE %s = '%s'";
    $result = FALSE;
    foreach($elements as $table=>$field) {
      $params = array($table, $field, $modId);
      if ($res = $this->databaseQueryFmt($sql, $params)) {
        list($count) = $res->fetchRow();
        $res->free();
        if (isset($result[$table])) {
          $result[$table] += $count;
        } else {
          $result[$table] = (int)$count;
        }
      }
    }
    return $result;
  }

  /**
  * Get table dialog
  *
  * @access public
  */
  function getTableDialog() {
    if (isset($this->tableStruct) && is_array($this->tableStruct)) {
      if (isset($this->tables[$this->tableStruct['name']]) && $this->tables[$this->tableStruct['name']] === TRUE) {
        if (isset($this->tableStruct['actions']) && $this->tableStruct['actions'] > 0 &&
            isset($this->params['cmd']) && $this->params['cmd'] == 'sync') {
          $this->getTableSyncDialog($this->tableStruct['name']);
        }
        $this->getFieldsListView($this->tableStruct);
        $this->getKeysListView($this->tableStruct);
      } else {
        //Nachfrage Tabelle anlegen
        $this->getCreateTableDialog($this->tableStruct['name']);
      }
    } elseif (isset($this->params['table']) && !empty($this->params['table'])) {
      $this->addMsg(MSG_ERROR, $this->_gt('Could not load table structure file!'));
    }
  }

  /**
  * Get fields list view
  *
  * @param array &$struct
  * @access public
  */
  function getFieldsListView(&$struct) {
    if (isset($struct['fields']) && is_array($struct['fields'])) {
      $str = sprintf('<listview title="%s">', $this->_gt('Fields'));
      $str .= '<cols>';
      $str .= '<col>'.papaya_strings::escapeHTMLChars($this->_gt('Name')).'</col>';
      $str .= '<col align="center">'.papaya_strings::escapeHTMLChars($this->_gt('Type')).'</col>';
      $str .= '<col align="center">'.papaya_strings::escapeHTMLChars($this->_gt('Size')).'</col>';
      $str .= '<col align="center">'.papaya_strings::escapeHTMLChars($this->_gt('Null')).'</col>';
      $str .= '<col align="center">'.papaya_strings::escapeHTMLChars($this->_gt('Auto Increment')).'</col>';
      $str .= '<col align="center">'.papaya_strings::escapeHTMLChars($this->_gt('Action')).'</col>';
      $str .= '</cols>';
      $str .= '<items>';
      foreach($struct['fields'] as $field) {
        $str .= sprintf('<listitem title="%s">', papaya_strings::escapeHTMLChars($field['name']));
        $str .= '<subitem align="center">'.papaya_strings::escapeHTMLChars($field['type']).'</subitem>';
        $str .= '<subitem align="center">'.papaya_strings::escapeHTMLChars($field['size']).'</subitem>';
        $activeGlyph = papaya_strings::escapeHTMLChars((@$field['null'] == 'yes') ? $this->images[29] : $this->images[30]);
        $str .= '<subitem align="center"><glyph src="'.$activeGlyph.'"/></subitem>';
        $activeGlyph = papaya_strings::escapeHTMLChars((@$field['autoinc'] == 'yes') ? $this->images[29] : $this->images[30]);
        $str .= '<subitem align="center"><glyph src="'.$activeGlyph.'"/></subitem>';
        if (isset($field['action']) && $field['action']) {
          switch ($field['action']) {
            case 'add' : $glyph = $this->images[70]; break;
            case 'delete' : $glyph = $this->images[10]; break;
            case 'update' : $glyph = $this->images[45]; break;
            default : $glyph = $this->images[0];
          }
          $href = $this->baseLink.'?'.$this->paramName.'[cmd]=field_'.$field['action'].'&amp;'.
                  $this->paramName.'[field]='.urlencode($field['name']);
          $str .= '<subitem align="center"><a href="'.$href.'"><glyph src="'.$glyph.'"/></a></subitem>';
        } else {
          $str .= '<subitem/>';
        }
        $str .= '</listitem>';
      }
      $str .= '</items>';
      $str .= '</listview>';
      $this->layout->add($str);
    }
  }

  /**
  * Get Keys list view
  *
  * @param array &$struct
  * @access public
  */
  function getKeysListView(&$struct) {
    if (isset($struct['keys']) && is_array($struct['keys'])) {
      $str = sprintf('<listview title="%s">', $this->_gt('Index'));
      $str .= '<cols>';
      $str .= '<col>'.papaya_strings::escapeHTMLChars($this->_gt('Name')).'</col>';
      $str .= '<col align="center">'.papaya_strings::escapeHTMLChars($this->_gt('Fields')).'</col>';
      $str .= '<col align="center">'.papaya_strings::escapeHTMLChars($this->_gt('Unique')).'</col>';
      $str .= '<col align="center">'.papaya_strings::escapeHTMLChars($this->_gt('Fulltext')).'</col>';
      $str .= '<col align="center">'.papaya_strings::escapeHTMLChars($this->_gt('Action')).'</col>';
      $str .= '</cols>';
      $str .= '<items>';
      foreach($struct['keys'] as $key) {
        $str .= sprintf('<listitem title="%s">', papaya_strings::escapeHTMLChars($key['name']));
        $str .= '<subitem>'.papaya_strings::escapeHTMLChars(implode(', ', $key['fields'])).'</subitem>';
        $activeGlyph = papaya_strings::escapeHTMLChars((@$key['unique'] == 'yes' || $key['name'] == 'PRIMARY') ?
          $this->images[29] : $this->images[30]);
        $str .= '<subitem align="center"><glyph src="'.$activeGlyph.'"/></subitem>';
        $activeGlyph = papaya_strings::escapeHTMLChars((@$key['fulltext'] == 'yes') ? $this->images[29] : $this->images[30]);
        $str .= '<subitem align="center"><glyph src="'.$activeGlyph.'"/></subitem>';
        if (isset($key['action']) && $key['action']) {
          switch ($key['action']) {
            case 'add' : $glyph = $this->images[70]; break;
            case 'delete' : $glyph = $this->images[10]; break;
            case 'update' : $glyph = $this->images[45]; break;
            default : $glyph = $this->images[0];
          }
          $href = $this->baseLink.'?'.$this->paramName.'[cmd]=index_'.$key['action'].'&amp;'.
                  $this->paramName.'[index]='.urlencode($key['name']);
          $str .= '<subitem align="center"><a href="'.$href.'"><glyph src="'.$glyph.'"/></a></subitem>';
        } else {
          $str .= '<subitem/>';
        }
        $str .= '</listitem>';
      }
      $str .= '</items>';
      $str .= '</listview>';
      $this->layout->add($str);
    }
  }

  /**
  * Get create table dialog
  *
  * @param string $tableName
  * @access public
  */
  function getCreateTableDialog($tableName) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'table_create',
      'table' => $tableName,
      'table_create_confirm'=>1,
    );
    $msg = $this->_gtf('Create table "%s_%s"?' , array(PAPAYA_DB_TABLEPREFIX, papaya_strings::escapeHTMLChars($tableName)));
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Create';
    if ($str = $dialog->getMsgDialog()) {
      $this->layout->add($str);
    }
  }

  function getTableSyncDialog($table) {
    if (isset($table) && trim($table != '') && isset($this->tableStruct) && is_array($this->tableStruct)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'sync',
        'table' => $table,
        'table_action_confirm'=>1,
      );
      $msg = $this->_gtf('Syncronize table "%s_%s"?',
        array(PAPAYA_DB_TABLEPREFIX, papaya_strings::escapeHTMLChars($this->params['table'])));
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Syncronize';
      if ($str = $dialog->getMsgDialog()) {
        $this->layout->add($str);
      }
    }
  }

  /**
  * Get field dialog
  *
  * @access public
  */
  function getFieldDialog() {
    if (isset($this->params['cmd']) &&
        in_array($this->params['cmd'], array('field_add', 'field_delete', 'field_update')) &&
        isset($this->params['field']) && isset($this->tableStruct) && is_array($this->tableStruct) &&
        isset($this->tableStruct['fields'][$this->params['field']])) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => $this->params['cmd'],
        'table' => $this->params['table'],
        'field' => $this->params['field'],
        'field_action_confirm'=>1,
      );
      switch ($this->params['cmd']) {
        case 'field_add' :
          $msgPattern = 'Add field "%s" to table "%s_%s"?';
          $btnCaption = 'Add';
          break;
        case 'field_delete' :
          $msgPattern = 'Delete field "%s" from table "%s_%s"?';
          $btnCaption = 'Delete';
          break;
        case 'field_update' :
          $msgPattern = 'Modify field "%s" in table "%s_%s"?';
          $btnCaption = 'Modify';
          break;
      }
      $msg = $this->_gtf($msgPattern,
        array($this->params['field'], PAPAYA_DB_TABLEPREFIX, papaya_strings::escapeHTMLChars($this->params['table'])));
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = $btnCaption;
      if ($str = $dialog->getMsgDialog()) {
        $this->layout->add($str);
      }
    }
  }

  /**
  * Get index dialog
  *
  * @access public
  */
  function getIndexDialog() {
    if (isset($this->params['cmd']) &&
        in_array($this->params['cmd'], array('index_add', 'index_delete', 'index_update')) &&
        isset($this->params['index']) && isset($this->tableStruct) && is_array($this->tableStruct) &&
        isset($this->tableStruct['keys'][$this->params['index']])) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => $this->params['cmd'],
        'table' => $this->params['table'],
        'index' => $this->params['index'],
        'index_action_confirm'=>1,
      );
      switch ($this->params['cmd']) {
        case 'index_add' :
          $msgPattern = 'Add index "%s" to table "%s_%s"?';
          $btnCaption = 'Add';
          break;
        case 'index_delete' :
          $msgPattern = 'Delete index "%s" from table "%s_%s"?';
          $btnCaption = 'Delete';
          break;
        case 'index_update' :
          $msgPattern = 'Modify index "%s" in table "%s_%s"?';
          $btnCaption = 'Modify';
          break;
      }
      $msg = $this->_gtf($msgPattern,
        array($this->params['index'], PAPAYA_DB_TABLEPREFIX, papaya_strings::escapeHTMLChars($this->params['table'])));
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = $btnCaption;
      if ($str = $dialog->getMsgDialog()) {
        $this->layout->add($str);
      }
    }
  }
}

?>