<?php
/**
* papaya_options variable
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
* @version $Id: papaya_options.php,v 1.11 2006/06/06 11:50:06 weinert Exp $
*/

/**
* base class options
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_options.php');

/**
* papaya_options variable
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class papaya_options extends base_options {
  var $tableOptions = PAPAYA_DB_TBL_OPTIONS;

  var $inputFieldSize = 'large';

  var $advancedOptions = array(1,6);

  /**
  * get XML Buttons
  *
  * @access public
  */
  function getXMLButtons() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $toolbar = &new base_btnbuilder;
    $toolbar->images = &$this->images;
    $toolbar->addButton('Install/Upgrade', 'install.php', 49, '', FALSE);
    $toolbar->addButton('Check paths', $this->baseLink.'?'.$this->paramName.'[cmd]=opt_paths', 58, 'Check and create data paths', FALSE);
    if (isset($this->optLinks[0]) && is_array($this->optLinks[0])) {
      $toolbar->addSeperator();
      $toolbar->addButton('Remove unknown', $this->baseLink.'?'.$this->paramName.'[cmd]=opt_clear', 10, 'Remove unknown options', FALSE);
    }
    $toolbar->addSeperator();
    $toolbar->addButton('View Glyphs', 'glyphview.php', 31, '', FALSE);
    if ($result = $toolbar->getXML()) {
      $this->layout->add('<menu>'.$result.'</menu>', 'menus');
    }
  }

  /**
  * Initialize parameters
  *
  * @access public
  */
  function initialize() {
    $this->paramName = 'opt';
    $this->sessionParamName = 'PAPAYA_SESS_user_'.$this->paramName;
    $this->initializeParams($this->sessionParamName);
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);

    $this->loadOptions();
    $this->loadOptGroups();
  }

  /**
  * Execute - basic function fot handling parameters
  *
  * @access public
  */
  function execute() {
    switch (@$this->params['cmd']) {
      case 'open' :
        $this->sessionParams['opened'][$this->params['gid']] = TRUE;
        break;
      case 'close' :
        unset($this->sessionParams['opened'][$this->params['gid']]);
        break;
      case 'edit' :
        if ($this->checkDialogInput()) {
          if ($this->save()) {
            $this->addMsg(MSG_INFO, $this->_gtf('%s modified.', $this->_gt('Option')));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Can not change option.'));
          }
        }
        break;
      case 'opt_clear' :
        $this->removeUnknown();
        break;
      case 'opt_paths' :
        $this->checkDirectoriesWriteable();
        break;
    }
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  /**
  * Get XML
  *
  * @access public
  */
  function getXML() {
    $this->getXMLButtons();
    $this->getList();
    $this->getForm();
  }

  /**
  * Load options
  *
  * @access public
  */
  function loadOptions() {
    unset($this->options);
    unset($this->optLinks);
    $sql = "SELECT opt_name, opt_value
            FROM %s
            ORDER BY opt_name";
    if ($res = $this->databaseQueryFmt($sql,array($this->tableOptions))) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->options[$row['opt_name']] = $row;
        if (isset($this->optFields[$row['opt_name']])) {
          $row['opt_group'] = (int)$this->optFields[$row['opt_name']][0];
        } else {
          $row['opt_group'] = 0;
        }
        $this->optLinks[$row['opt_group']][] = $row['opt_name'];
      }
      return $this->checkOptions();
    }
    return FALSE;
  }

  /**
  * Remove Unknown
  *
  * @access public
  */
  function removeUnknown() {
    if (isset($this->optLinks[0]) && is_array($this->optLinks[0])) {
      foreach($this->optLinks[0] as $key=>$optName) {
        if (FALSE !== $this->databaseDeleteRecord($this->tableOptions, 'opt_name', $optName)) {
          unset($this->optLinks[0][$key]);
        } else {
          break;
        }
      }
      if (count($this->optLinks[0]) < 1) {
        unset($this->optLinks[0]);
      }
    }
  }

  /**
  * Check options
  *
  * @access public
  */
  function checkOptions() {
    foreach($this->optFields as $key=>$val) {
      if (!isset($this->options[$key])) {
        switch ($key) {
          case 'PAPAYA_DEFAULT_HOST' :
            $optValue = strtolower($_SERVER['HTTP_HOST']);
            break;
          case 'PAPAYA_PATH_DATA' :
            //confixx
            $optValue = dirname($_SERVER['DOCUMENT_ROOT']).'/files/papaya-data/';
            if (!file_exists($optValue.'.')) {
              //next to document root
              $optValue = dirname($_SERVER['DOCUMENT_ROOT']).'/papaya-data/';
              if (!file_exists($optValue.'.')) {
                //document root subdirectory
                $optValue = $_SERVER['DOCUMENT_ROOT'].'/papaya-data/';
              }
            }
            break;
          case 'PAPAYA_PATH_WEB' :
            $optValue = dirname(dirname($this->getBaseLink())).'/';
            if (substr($optValue, 0, 1) == '.') {
              $optValue = substr($optValue, 1);
            }
            $optValue = strtr($optValue, array('//'=>'/', '\\/'=>'/', '\\\\/'=>'/'));
            break;
          default :
            $optValue = $val[4];
            break;
        }
        $data = array('opt_name' => $key,'opt_value' =>$optValue);
        if (FALSE !== $this->databaseInsertRecord($this->tableOptions, NULL, $data)) {
          $this->options[$key] = array('opt_name'=>$key, 'opt_value'=>$optValue);
          $this->optLinks[$val[0]][] = $key;
        } else {
          return FALSE;
        }
      }
    }
    return TRUE;
  }

  /**
  * Load option groups
  *
  * @access public
  * @return boolean TRUE
  */
  function loadOptGroups() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_statictables.php');
    foreach(base_statictables::getTableOptGroups() as $key=>$val) {
      $this->optionGroups[$key] = $this->_gt($val);
    }
    return TRUE;
  }

  /**
  * activate theme
  *
  * @param string $baseDirectory
  * @param string $theme
  * @access public
  */
  function activateTheme($baseDirectory, $theme) {
    $themeFileName = $baseDirectory.'/themes/'.$theme.'/theme.php';
    if (file_exists($themeFileName)) {
      include_once($themeFileName);
    }
    if (!defined('PAPAYA_BGCOLOR')) define('PAPAYA_BGCOLOR', 'threedface');
    if (!defined('PAPAYA_BGCOLOR_LIGHT')) define('PAPAYA_BGCOLOR_LIGHT', 'threedlightshadow');
    if (!defined('PAPAYA_BGCOLOR_DARKER')) define('PAPAYA_BGCOLOR_DARKER', 'threedshadow');
    if (!defined('PAPAYA_BGCOLOR_DARK')) define('PAPAYA_BGCOLOR_DARK', 'appworkspace');
    if (!defined('PAPAYA_FGCOLOR')) define('PAPAYA_FGCOLOR', 'windowtext');
    if (!defined('PAPAYA_WINDOWCOLOR')) define('PAPAYA_WINDOWCOLOR', 'window');
    if (!defined('PAPAYA_HIGHLIGHT_COLOR')) define('PAPAYA_HIGHLIGHT_COLOR', 'highlight');
    if (!defined('PAPAYA_HIGHLIGHT_TEXTCOLOR')) define('PAPAYA_HIGHLIGHT_TEXTCOLOR', 'highlighttext');
    if (!defined('PAPAYA_SYSTEMPICS_PATH')) define('PAPAYA_SYSTEMPICS_PATH', './themes/system/');
  }

  /**
  * Save option to database table
  *
  * @access public
  * @return boolean
  */
  function save() {
    $option = $this->options[$this->params['id']];
    $value = $this->params[$this->params['id']];
    if (isset($option) && is_array($option)) {
      if (FALSE !== $this->databaseUpdateRecord($this->tableOptions, array('opt_value' => $value), 'opt_name', $option['opt_name'])) {
        $this->options[$this->params['id']]['opt_value'] = $value;
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Get list of options
  *
  * @param boolean $all optional, default value FALSE
  * @access public
  */
  function getList($all = FALSE) {
    if (isset($this->optionGroups) && is_array($this->optionGroups)) {
      $result = sprintf('<listview title="%s" width="500">', $this->_gt('Options'));
      $result .= '<cols>';
      $result .= '<col>'.$this->_gt('Option').'</col>';
      $result .= '<col>'.$this->_gt('Active').'</col>';
      $result .= '</cols>';
      $result .= '<items>';
      foreach($this->optionGroups as $groupId=>$optionGroup) {
        if ($all || ((!in_array($groupId, $this->advancedOptions)) && isset($this->optLinks[$groupId]))) {
          if (isset($this->optLinks[$groupId]) && is_array($this->optLinks[$groupId]) &&
              isset($this->sessionParams['opened'][$groupId]) && $this->sessionParams['opened'][$groupId]) {
            $nodeHref = sprintf('?%s[cmd]=close&amp;%s[gid]=%s',
              $this->paramName, $this->paramName, $groupId);
            $node = sprintf(' node="open" nhref="%s"', $nodeHref);
          } elseif (isset($this->optLinks[$groupId]) && is_array($this->optLinks[$groupId])) {
            $nodeHref = sprintf('?%s[cmd]=open&amp;%s[gid]=%s',
              $this->paramName, $this->paramName, $groupId);
            $node = sprintf(' node="close" nhref="%s"', $nodeHref);
          } else {
            $node = ' indent="1"';
          }
          if (isset($this->group['group_id']) && $groupId == $this->group['group_id']) {
            $selected = ' selected="selected"';
          } else {
            $selected = '';
          }
          $result .= sprintf('<listitem title="%s" image="%s"%s>'.LF,
            $optionGroup, $this->images[49], $node.$selected);
          $result .= '<subitem/>';
          $result .= '</listitem>';
          if (isset($this->optLinks[$groupId]) && is_array($this->optLinks[$groupId]) &&
              isset($this->sessionParams['opened'][$groupId]) && ($this->sessionParams['opened'][$groupId])) {
            foreach ($this->optLinks[$groupId] as $optId) {
              if (isset($this->options[$optId]) && is_array($this->options[$optId])) {
                $option = $this->options[$optId];
                if (isset($this->params['id']) && $this->params['id'] == $optId) {
                  $selected = ' selected="selected"';
                } else {
                  $selected = '';
                }
                $result .= sprintf('<listitem title="%s" indent="2" href="%s?%s[id]=%s" image="%s"%s>'.LF,
                  $option['opt_name'], $this->baseLink, $this->paramName, $optId, $this->images[50], $selected);
                if (strlen(constant($optId)) > 30) {
                  $value = substr(constant($optId), 0, 30).'<b>...</b>';
                } else {
                  $value = constant($optId);
                }
                $result .= '<subitem>'.$value.'</subitem>';
                $result .= '</listitem>';
              }
            }
          }
        }
      }
      $result .= '</items>';
      $result .= '</listview>';
      $this->layout->addLeft($result);
    }
  }

  /**
  * initilaize option dialog
  *
  * @access public
  */
  function initOptionDialog() {
    if (!(isset($this->optionDialog) && is_object($this->optionDialog))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');

      $option = $this->options[$this->params['id']];
      $hidden = array(
        'save'=> 1,
        'cmd'=> 'edit',
        'id' => @$option['opt_name']
      );
      $data = array(
        'opt_name' => $option['opt_name'],
        'opt_value' => $option['opt_value'],
      );

      $fields = array(
        'opt_name' => array('Name', '', FALSE, 'info', 0, '', $option['opt_name'], 'left')
      );
      if (@is_array($optionField = $this->optFields[$option['opt_name']])) {
        $fields[$option['opt_name']] =
          array('Value', $optionField[1], TRUE, $optionField[2], $optionField[3], '', $option['opt_value']);
      } else {
        $fields[$option['opt_name']] =
          array('Value', '', TRUE, 'info', '', '', $option['opt_value']);
      }
      $this->optionDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->optionDialog->dialogTitle = $this->_gt('Option');
      $this->optionDialog->baseLink = $this->baseLink;
      $this->optionDialog->msgs = &$this->msgs;
      $this->optionDialog->loadParams();
    }
  }

  /**
  * Get form
  *
  * @access public
  */
  function getForm() {
    if (isset($this->options[@$this->params['id']])) {
      $this->initOptionDialog();
      $this->layout->add($this->optionDialog->getDialogXML());
    }
  }

  /**
  * Get XSLT extensions combo
  *
  * @param string $name
  * @param array $element
  * @param mixed $data
  * @access public
  * @return string
  */
  function getXSLTExtensionsCombo($name, $element, $data) {
    unset($extensions);
    if (extension_loaded('xslt')) $extensions[] = 'xslt';
    if (extension_loaded('domxml')) $extensions[] = 'domxml';
    if (extension_loaded('xsl')) $extensions[] = 'xsl';
    $result = '';
    if (is_array($extensions)) {
      $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">'.LF,
        $this->paramName, $name);
      foreach($extensions as $ext) {
        if ($ext == $data) {
          $selected = ' selected="selected"';
        } else {
          $selected = '';
        }
        $result .= sprintf('<option value="%s"%s><![CDATA[%s]]></option>'.LF,
          papaya_strings::escapeHTMLChars($ext), $selected, $ext);
      }
      $result .= '</select>'.LF;
    } else {
      $result = sprintf('<input type="text" disabled="disabled" value="%s"/>', 'No XSLT Extension found');
    }
    return $result;
  }

  /**
  * Get admin themes combo
  *
  * @param string $name
  * @param array $element
  * @param mixed $data
  * @access public
  * @return string XML
  */
  function getAdminThemesCombo($name, $element, $data) {
    $path = $this->getBaseLink(TRUE).'/theme/';
    $result = '';
    if ($dh = opendir($path)) {
      $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">'.LF,
        $this->paramName, $name);
      while($directoryName = readdir($dh)) {
        if (is_dir($path.$directoryName) && substr($directoryName, 0, 1) == '.')  {
          $selected = ($directoryName == $data) ? ' selected="selected"' : '';
          $result .= sprintf('<option value="%s"%s><![CDATA[%s]]></option>'.LF,
            papaya_strings::escapeHTMLChars($directoryName), $selected, $directoryName);
        }
      }
      $result .= '</select>'.LF;
      closedir($dh);
    } else {
      $result = sprintf('<input type="text" disabled="disabled" value="%s"/>', 'No themes found');
    }
    return $result;
  }

  /**
  * Get language combo
  *
  * @param string $name
  * @param array $element
  * @param mixed $data
  * @access public
  * @return string XML
  */
  function getInterfaceLanguageCombo($name, $element, $data) {
    $sql = "SELECT lng_short, lng_title FROM %s WHERE is_interface_lng = 1 ORDER BY lng_title";
    $result = '';
    if ($res = $this->databaseQueryFmt($sql, PAPAYA_DB_TBL_LNG)) {
      $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">'.LF,
        $this->paramName, $name);
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $selected = ($row['lng_short'] == $data) ? ' selected="selected"' : '';
        $result .= sprintf('<option value="%s"%s><![CDATA[%s (%s)]]></option>'.LF,
          papaya_strings::escapeHTMLChars($row['lng_short']), $selected, $row['lng_title'], $row['lng_short']);
      }
      $result .= '</select>'.LF;
      $res->free();
    } else {
      $result = sprintf('<input type="text" disabled="disabled" value="%s"/>', 'No language found');
    }
    return $result;
  }

  /**
  * Get language combo
  *
  * @param string $name
  * @param array $element
  * @param mixed $data
  * @access public
  * @return string XML
  */
  function getContentLanguageCombo($name, $element, $data) {
    $sql = "SELECT lng_id, lng_short, lng_title FROM %s WHERE is_content_lng = 1 ORDER BY lng_title";
    $result = '';
    if ($res = $this->databaseQueryFmt($sql, PAPAYA_DB_TBL_LNG)) {
      $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">'.LF,
        $this->paramName, $name);
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $selected = ($row['lng_short'] == $data) ? ' selected="selected"' : '';
        $result .= sprintf('<option value="%d"%s><![CDATA[%s (%s)]]></option>'.LF,
          papaya_strings::escapeHTMLChars($row['lng_id']), $selected, $row['lng_title'], $row['lng_short']);
      }
      $result .= '</select>'.LF;
      $res->free();
    } else {
      $result = sprintf('<input type="text" disabled="disabled" value="%s"/>', 'No language found');
    }
    return $result;
  }

  /**
  * Check dialog input
  *
  * @access public
  * @return booelan
  */
  function checkDialogInput() {
    $checkFunctions = get_class_methods('checkit');
    foreach($checkFunctions as $idx=>$functionName) {
      $checkFunctions[$idx] = strtolower($functionName);
    }
    $checkObj = &new checkit;
    $result = FALSE;
    $name = $this->params['id'];
    $optField = $this->optFields[$name];
    if ((isset($checkFunctions) && is_array($checkFunctions)) && (isset($optField) && is_array($optField))) {
      $checkFunction = strtolower($optField[1]);
      if (in_array($checkFunction, $checkFunctions)) {
        if ($checkObj->$checkFunction($this->params[$name], TRUE)) {
          if ($this->checkOptionSpecial($name, $this->params[$name])) {
            $result = TRUE;
          }
        } else  {
          $this->addMsg(MSG_ERROR, $this->_gt('Invalid value for option.'));
        }
      } else {
        $this->addMsg(MSG_ERROR, $this->_gtf('Invalid check function %s.', $checkFunction));
      }
    }
    return $result;
  }
  
  function checkOptionSpecial($option, $value) {
    if ($option == 'PAPAYA_LOGIN_RESTRICTION') {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_auth_secure');
      $authsec = &new base_auth_secure();
      $ipStatus = $authsec->getIpStatus($_SERVER['REMOTE_ADDR']);
      if ($value == 3 && $ipStatus != 1) {
        $this->addMsg(MSG_INFO, $this->_gt('Your IP is not in whitelist.'));
        return FALSE;
      }
    }
    return TRUE;
  }

  function checkDirectoryWriteable($path, $name) {
    $title = papaya_strings::escapeHTMLChars($this->_gt($name));
    if (!file_exists($path)) {
      umask(011);
      if (chdir(dirname($path))&& mkdir(basename($path), 0777)) {
        $this->addMsg(MSG_INFO, $this->_gtf('%s created.', $title));
      }
    }
    if (is_dir($path)) {
      umask(011);
      if (@chmod($path, 0777)) {
        $this->addMsg(MSG_INFO, $this->_gtf('%s is writeable.', $title));
        return TRUE;
      } elseif (is_writeable($path)) {
        $this->addMsg(MSG_INFO, $this->_gtf('%s is writeable.', $title));
        return TRUE;
      } else {
        $this->addMsg(MSG_INFO, $this->_gtf('Can not modify %s.', $title));
      }
    } else {
      $this->addMsg(MSG_INFO, $this->_gtf('Can not create %s.', $title));
    }
    return FALSE;
  }

  function checkDirectoriesWriteable() {
    if (trim(PAPAYA_PATH_DATA) != '') {
      $this->checkDirectoryWriteable(PAPAYA_PATH_MEDIAFILES, 'Media database path');
      $this->checkDirectoryWriteable(PAPAYA_PATH_THUMBFILES, 'Thumbnail path');
      $this->checkDirectoryWriteable(PAPAYA_PATH_CACHE, 'Cache path');
    }
  }
}

?>