<?php
/**
* Create topic list
*
* @author Raphael Kalla <kalla@papaya-cms.com>
* @package papaya
*/


/**
* Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* Create topic list
*
* @author  Raphael Kalla <kalla@papaya-cms.com>
* @package papaya
*/
class papaya_overview extends base_db {

  var $tableTopicsPublic = PAPAYA_DB_TBL_TOPICS_PUBLIC;
  var $tableTopicsPublicTrans = PAPAYA_DB_TBL_TOPICS_PUBLIC_TRANS;
  var $tableAuthUser = PAPAYA_DB_TBL_AUTHUSER;
  var $tableTopicsVersion = PAPAYA_DB_TBL_TOPICS_VERSIONS;
  var $tableTopicsTrans = PAPAYA_DB_TBL_TOPICS_TRANS;
  var $tableTopics = PAPAYA_DB_TBL_TOPICS;
  var $tableMessages = PAPAYA_DB_TBL_MESSAGES;
  var $tableTodos = PAPAYA_DB_TBL_TODOS;
  var $topics = NULL;
  var $messageList = NULL;
  var $users = Null;
  var $todos = Null;

  var $paramName = 'overview';

  function initialize() {
    $this->initializeParams();
  }

  function execute() {
    switch (@$this->params['cmd']) {
      case 'search_do' :
        if (isset($this->params['confirm_search']) && $this->params['confirm_search']) {
          $this->searchFor();
          $this->params['cmd'] = 'search_exec';
          if (!isset($this->topics) && !is_array($this->topics)) {
            $this->addMsg(MSG_WARNING, $this->_gt('No Page Found'));
          }
        }
        break;
      case 'refreshpages' :
        if ($this->authUser->hasPerm(37)) {
          $this->refreshPages();
        }
        break;
     }
  }
  /**
  * Add Content search dialog / search results / main page (topic/messages/todos)
  *
  * @access public
  */
  function getXML() {
    switch(@$this->params['cmd']) {
      case 'search' :
        $this->layout->addLeft($this->getSearchDialog());
        $this->layout->add('<sheet width="100%" align="center"><text><div style="padding: 0px 5px 0px 5px; "><p>'._gt('You can search for all kind of pages. Please use the Options on the left.').'</p></div></text></sheet>');
        break;
      case 'search_exec' :
        $this->layout->addLeft($this->getSearchDialog());
        $this->layout->add($this->getTopicsList('Search Results', '100%'));
        break;
      default :
        $this->loadNoPublishedTopicList(2, true, $this->authUser->options['PAPAYA_OVERVIEW_ITEMS_UNPUBLISHED']);
        $this->layout->addLeft($this->getTopicsList("Not published"));
        $this->loadTopicList(2, true, $this->authUser->options['PAPAYA_OVERVIEW_ITEMS_PUBLISHED']);
        $this->layout->addLeft($this->getTopicsList("Latest publications"));
        $this->loadUsers();
        $this->loadTodoList($this->authUser->options['PAPAYA_OVERVIEW_ITEMS_TASKS']);
        $this->layout->add($this->getTodoList('100%'));
        $this->loadMessageList(true, $this->authUser->options['PAPAYA_OVERVIEW_ITEMS_MESSAGES']);
        $this->layout->add($this->getMessageList('100%'));
        break;
    }
  }
  
  /**
  * Get buttons
  *
  * @access public
  * @return mixed
  */
  function getButtons() {
    $result = NULL;
    $result[] = array(
      'Messages', 'Messaging between the users', $this->images[83], 34,
      'msgbox.php',
      '_self'
    );
    $result[] = array('Todo', 'Todo management', $this->images[130], 34,
      'todo.php',
      '_self'
    );
    $result[] = '-';
    $result[] = array(
      'Search Page', 'Search for a Page', $this->images[4], 16,
      $this->baseLink.'?'.$this->paramName.'[cmd]=search',
      '_self'
    );
    $result[] = '-';
    if ($this->authUser->hasPerm(37)) {
      $result[] = array('Empty cache', 'Empty output cache', $this->images[10], 33,
        $this->baseLink.'?'.$this->paramName.'[cmd]=refreshpages',
        '_self', FALSE);
    }

    return $result;
  }

  /**
  * Load all published Topics
  *
  * @access public
  * @param integer $lngId
  * @param boolean $desc
  * @param integer $max
  */
  function loadTopicList($lngId, $desc = TRUE, $max = 8) {
    unset($this->topics);
    
    $order = ($desc) ? "DESC":  "ASC";
    $sql = "SELECT tp.topic_modified AS topic_published, tt.topic_title, 
                   t.topic_modified, t.topic_id, t.author_id, t.topic_created, 
                   u.givenname, u.surname, u.user_id
             FROM %s t
             LEFT JOIN %s tt ON (t.topic_id = tt.topic_id AND tt.lng_id = %d)
             LEFT JOIN %s tp ON (t.topic_id = tp.topic_id)
             LEFT JOIN papaya_auth_user u ON (t.author_id = u.user_id)
            WHERE (tp.topic_modified >= t.topic_modified)
            ORDER BY topic_published %s";
    $params = array(
      $this->tableTopics, $this->tableTopicsTrans, 
      $lngId, $this->tableTopicsPublic, $order);

    if ($res = $this->databaseQueryFmt($sql, $params, $max)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->topics[$row["topic_id"]] = $row;
      }
    }
  }

  /**
  * Load all created but NOT published Topics
  *
  * @access public
  * @param integer $langId
  * @param boolean $desc
  * @param integer $max
  */
  function loadNoPublishedTopicList($langId, $desc=TRUE, $max = 8) {
    unset($this->topics);
    
    $order = ($desc) ? "DESC":  "ASC";
    $sql = "SELECT tp.topic_modified AS topic_published, tt.topic_title, 
                   t.topic_modified, t.topic_id, t.author_id, t.topic_created, 
                   u.givenname, u.surname, u.user_id
             FROM %s t
             LEFT JOIN %s tt ON (t.topic_id = tt.topic_id AND tt.lng_id = %d)
             LEFT JOIN %s tp ON (t.topic_id = tp.topic_id)
             LEFT JOIN papaya_auth_user u ON (t.author_id = u.user_id)
            WHERE (tp.topic_id IS NULL OR (tp.topic_modified < t.topic_modified))
            ORDER BY t.topic_modified %s";
    $params = array(
      $this->tableTopics, $this->tableTopicsTrans, 
      $langId, $this->tableTopicsPublic, $order);

    if ($res = $this->databaseQueryFmt($sql, $params, $max)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->topics[$row["topic_id"]] = $row;
      }
    }
  }

  /**
  * Build a list of all Topics
  *
  * @access public
  * @param string $title
  */
  function getTopicsList($title, $width = 490) {
    $result = '';
    if (isset($this->topics) && is_array($this->topics) && count($this->topics) > 0) {
      $result .= sprintf('<listview width="%s" title="%s">'.LF, 
        $width, $this->_gt($title));
      $result .= '<items>'.LF;
      $result .= $this->getTopicElements();
      $result .=  '</items></listview>'.LF;
    }
    return $result;
  }
  /**
  * Build listitems
  *
  * @access public
  * @param boolean $published
  */
  function getTopicElements() {
    $result = '';
      if (isset($this->topics) && is_array($this->topics)) {
        foreach ($this->topics as $id=>$values) {
          if (isset($values) && is_array($values)) {
             if ($values['topic_title']=="") {
              $title = 'No Title!';
            } else {
              $title = $values['topic_title'];
            }
            if (isset($values['topic_published']) && $values['topic_published'] < $values['topic_modified']) {
              $topicGlyph = 48;
            } elseif (isset($values['topic_published']) && $values['topic_published'] >= $values['topic_modified']) {
              $topicGlyph = 16;
            } else {
              $topicGlyph = 15;
            }
            if (isset($values['topic_modified']) && $values['topic_modified'] > 0) {
              $dateString = $this->_gt('Modified'). 
                date(': Y-m-d ', $values['topic_modified']);
            } else {
              $dateString = $this->_gt('Created'). 
                date(': Y-m-d ', $values['topic_created']);
            }
            if (isset($values['topic_published']) && $values['topic_published'] > 0) {
              $dateString .= $this->_gt('Published'). 
                date(': Y-m-d ', $values['topic_published']);
            }
            
            $result .= sprintf('<listitem image="%s" href="%s">',
              $this->images[$topicGlyph],
              $this->getLink(array('page_id'=>$values['topic_id']), 'tt', 'topic.php'));
            $result .= '<subitem>';
            if ($values['user_id'] == $this->authUser->userId) {
              $result .= sprintf('<a href="%s"><b>%s</b>',
                $this->getLink(array('page_id'=>$values['topic_id']), 'tt', 'topic.php'),
                papaya_strings::escapeHTMLChars($title));
            } else {
              $result .= sprintf('<a href="%s">%s',
                $this->getLink(array('page_id'=>$values['topic_id']), 'tt', 'topic.php'),
                papaya_strings::escapeHTMLChars($title));
            }
            $result .= sprintf('<br/><i>%s (%s)</i></a></subitem>'.LF,
              papaya_strings::escapeHTMLChars($values['givenname']." ".$values['surname']),
              papaya_strings::escapeHTMLChars(substr($dateString, 0, -1)));
            $result .= '</listitem>'."\r\n";
          }
        }
      }
    return $result;
  }

  /**
  * Load all NEW messages
  *
  * @access public
  * @param boolean $desc
  * @param integer $max
  */
  function loadMessageList($desc=TRUE, $max = 8) {
    if ($desc) {
      $order = "DESC";
    } else {
      $order = "ASC";
    }
    unset($this->messageList);
    
    $sql = "SELECT msg_id, msg_owner_id, msg_folder_id, msg_from, msg_datetime, msg_subject, msg_priority, msg_type
              FROM %s
             WHERE msg_owner_id = '%s' AND msg_folder_id = '0' AND msg_new = '1'
             ORDER BY msg_datetime %s";
    $params = array($this->tableMessages,$this->authUser->user['user_id'], $order);

    if ($res = $this->databaseQueryFmt($sql,$params,$max)) {
      while($messages = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->messageList[$messages['msg_id']] = $messages;
      }
    }
  }

  /**
  * Build a list of all NEW Messages
  *
  * @access public
  */
  function getMessageList($width = '490') {
    $result = '';
    if (isset($this->messageList) && is_array($this->messageList) && count($this->messageList) > 0) {
      $result .= sprintf('<listview width="%s" title="%s">'.LF, $width, $this->_gt('New messages'));
      $result .= '<cols>'.LF;
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Subject'));
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Priority'));
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('From'));
      $result .= '</cols>'.LF;
      $result .= '<items>'.LF;
      $counter = '0';
      $result .= $this->getMessageElements();
      $result .=  '</items></listview>'.LF;
    } else {
      $this->addMsg(MSG_INFO, $this->_gt('No new messages.'));
    }
    return $result;
  }
  
  /**
  * Build listitems
  *
  * @access public
  * @return string $result
  */
  function getMessageElements() {
    $result = '';
      if (isset($this->messageList) && is_array($this->messageList)) {
        foreach ($this->messageList as $id=>$values) {
          if (isset($values) && is_array($values)) {

            $result .= sprintf('<listitem title="%s" image="%s" href="%s">',
              papaya_strings::escapeHTMLChars($values['msg_subject']),
              $this->images[82],
              $this->getLink(array('msg_id'=>$values['msg_id']), 'msg', 'msgbox.php'));
            $result .= sprintf('<subitem align="center"><glyph src="%s"/></subitem>'.LF,
              $this->images[$this->getPriorityGlyphIndex($values['msg_priority'])]);
            $result .= sprintf('<subitem>%s</subitem>'.LF,
              papaya_strings::escapeHTMLChars(@$this->getUserEmailName($this->users[$values['msg_from']])));
            $result .= '</listitem>'.LF;
          }
        }
      }
    return $result;
  }

  /**
  * Get priority glyph index
  *
  * @param integer $priority
  * @access public
  * @return integer
  */
  function getPriorityGlyphIndex($priority) {
    switch($priority) {
      case 2 : return 85;
      case 1 : return 84;
      default : return 0;
    }
  }
  
  /**
  * Get the full name of a user
  *
  * @param varchar $userData
  * @access public
  * @return integer
  */
   function getUserEmailName(&$userData) {
    if (isset($userData)) {
      return strtr($userData['givenname'].' '.$userData['surname'], ',;', '');
    }
    return '';
  }

  /**
  * load all User
  *
  * @access public
  */
  function loadUsers() {
    unset($this->users);
    $sql = "SELECT user_id, surname, givenname
              FROM %s
             ORDER BY surname, givenname";
    if ($res = $this->databaseQueryFmt($sql,array($this->tableAuthUser))) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->users[$row['user_id']] = $row;
      }
    }
  }

  /**
  * Load all todos
  *
  * @access public
  * @param boolean $desc
  * @param integer $max
  */
  function loadTodoList($max = 8) {
    $sql = "SELECT td.todo_id, td.title, td.priority, td.status, td.date_to, 
                   td.date_to, td.user_id_from, td.topic_id
              FROM %s AS td
             WHERE td.user_id_to = '%s'
             ORDER BY td.date_to ASC";
    $params = array($this->tableTodos, $this->authUser->user["user_id"]);
    if ($res = $this->databaseQueryFmt($sql, $params, $max)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->todos[$row["todo_id"]] = $row;
      }
    }
  }

  /**
  * Build a list of all NEW Messages
  *
  * @access public
  */
  function getTodoList($width = '490') {
    $result = '';
    if (isset($this->todos) && is_array($this->todos) && count($this->todos) > 0) {
      $result .= sprintf('<listview width="%s" title="%s">'.LF, $width, $this->_gt('Open todos'));
      $result .= '<cols>'.LF;
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Title'));
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Priority'));
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Date to'));
      $result .= sprintf('<col/>'.LF);
      $result .= '</cols>'.LF;
      $result .= '<items>'.LF;
      $result .= $this->getTodoElements();
      $result .=  '</items></listview>'.LF;
    } else {
      $this->addMsg(MSG_INFO, $this->_gt('No todos.'));
    }
    return $result;
  }

  /**
  * Build listitems
  *
  * @access public
  * @return string $result
  */
  function getTodoElements() {
    $result = '';
      if (isset($this->todos) && is_array($this->todos)) {
        foreach ($this->todos as $id=>$values) {
          if (isset($values) && is_array($values)) {
            if($values['date_to'] > 0 ) {
              $dateTo = date("Y-m-d H:i",$values['date_to']);
            } else {
              $dateTo = '';
            }
            $result .= sprintf('<listitem title="%s" href="%s" image="%s">',
              papaya_strings::escapeHTMLChars($values['title']),
              $this->getLink(array('todo_id'=>$values['todo_id']), 'todo', 'todo.php'),
              $this->images[$this->getStatusGlyphIndex($values['status'])]);
            $result .= sprintf('<subitem align="center"><glyph src="%s"/></subitem>'.LF,
              $this->images[$this->getPriorityGlyphIndex($values['priority'])]);
            $result .= sprintf('<subitem>%s</subitem>'.LF, $dateTo);
            $result .= sprintf('<subitem align="center"><a href="%s"><glyph src="%s"/></a></subitem>'.LF,
              $this->getLink(array('page_id'=>$values['topic_id']), 'tt', 'topic.php'), $this->images[4]);
            $result .= '</listitem>'.LF;
          }
        }
      }
    return $result;
  }

  function getStatusGlyphIndex($status) {
    switch($status) {
      case 2 : return 123;
      case 1 : return 124;
      default : return 125;
    }
  }

  function getSearchDialog() {
    $this->initializeSearchDialog();
    return $this->dialogSearch->getDialogXML();
  }

  /**
  * Build Page Search dialog
  *
  * @access public
  */
  function initializeSearchDialog() {
    if (!(isset($this->dialogSearch) && is_object($this->dialogSearch))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      
      $hidden = array (
        'cmd' => 'search_do',
        'confirm_search' => 1
      );

      if(isset($this->params['filter_date_from']) && $this->params['filter_date_from'] != "") {
        $timeFrom = $this->params['filter_date_from'];
      } else {
        $timeFrom  = time();
      }
      if(isset($this->params['filter_date_to']) && $this->params['filter_date_to'] != "") {
        $timeTo = $this->params['filter_date_to'];
      } else {
        $timeTo  = time();
      }

      $timeFromStr = date("Y-m-d", $timeFrom);
      $timeToStr = date("Y-m-d", $timeTo);
      
      $statusCombo = array(
        'all'       => $this->_gt('Ignore'), 
        'created'   => $this->_gt('Created'),
        'modified'  => $this->_gt('modified'),
        'published' => $this->_gt('published')
      );

      $fields = array(
        'filter_title' => array('Title', 'isSometext', FALSE, 'input', 255, '', ''),
        'filter_user' => array('Username', 'isSometext', FALSE, 'input', 255, '', ''),
        'Date',
        'filter_date_option' => array('Search for', 'isNum', TRUE, 'combo', 
          $statusCombo, '', 1),
        'filter_date_from' => array('From', 'isISODate', TRUE, 'input', 255, 
          'ISO Date YYYY-MM-DD', $timeTromStr),
        'filter_date_to' => array('To', 'isISODate', TRUE, 'input', 255, 
          'ISO Date YYYY-MM-DD', $timeToStr)
      );
      $this->dialogSearch = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->dialogSearch->msgs = &$this->msgs;
      $this->dialogSearch->loadParams();
      $this->dialogSearch->dialogTitle = $this->_gt('Search Page');
      $this->dialogSearch->buttonTitle = $this->_gt('Search');
      $this->dialogSearch->dialogDoubleButtons = FALSE;
    }
  }
  
  /** Fill array with all searchresults
   *
   * @access public
   * 
   */
  function searchFor() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_searchstringparser.php');
    include_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');
    
    $sql = "SELECT t.topic_id, t.topic_created, t.topic_modified, 
                   tp.topic_modified AS topic_published,
                   au.givenname, au.surname, au.user_id, 
                   tt.topic_title
              FROM %s AS t, %s AS au
              LEFT OUTER JOIN %s AS tt ON (tt.topic_id = t.topic_id)
              LEFT OUTER JOIN %s AS tp ON (tp.topic_id = t.topic_id)
             WHERE t.author_id = au.user_id";
                 
    $dateFrom = checkit::convertISODateTimeToUnix(@$this->params['filter_date_from']);
    $dateTo = checkit::convertISODateTimeToUnix(@$this->params['filter_date_to']) + 86400;
    $sqlParams = array(
      $this->tableTopics, $this->tableAuthUser, 
      $this->tableTopicsTrans, $this->tableTopicsPublic, 
      $dateFrom, $dateTo);
                  
    $sqlString = &new searchstringparser;
    if (isset($this->params['filter_title']) && $this->params['filter_title'] != "") {
      if (strlen($this->params['filter_title']) > 2) {
        $filterTitle = " AND ".$sqlString->getSQL($this->params['filter_title'],
          array('tt.topic_title'));
        $sql .= str_replace('%', '%%', $filterTitle);
      } else {
        $this->addMsg(MSG_INFO, $this->_gt('Searchstring "Title" to short'));
      }
    }
    if (isset($this->params['filter_user']) && $this->params['filter_user'] != "") {
      if (strlen($this->params['filter_user']) > 2) {
        $filterUser = " AND ".$sqlString->getSQL($this->params['filter_user'],
          array('au.givenname', 'au.surname'));
        $sql .= str_replace('%', '%%', $filterUser);
      } else {
        $this->addMsg(MSG_INFO, $this->_gt('Searchstring "Username" to short'));
      }
    }
                  
    
    switch($this->params['filter_date_option']) {
    case 'created':
      $sql .= " AND t.topic_created >= '%d' AND t.topic_created <= '%d'";
      break;
    case 'modified':
      $sql .= " AND t.topic_modified >= '%d' AND t.topic_modified <= '%d'";      
      break;
    case 'published':
      $sql .= " AND tp.topic_modified >= '%d' AND tp.topic_modified <= '%d'";      
      break;
    }
    
    if ($res = $this->databaseQueryFmt($sql, $sqlParams)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->topics[$row["topic_id"]."-".@$row["lng_id"]] = $row;
      }
    }
  }
      
  /**
  * Refresh pages
  *
  * @access public
  */
  function refreshPages() {
    if (file_exists(PAPAYA_PATH_CACHE) && ($dh = opendir(PAPAYA_PATH_CACHE))) {
      $counter = 0;
      while(is_string($file = readdir($dh))) {
        if (preg_match('#^\.\d+_\d+_[a-z]{0,4}_[a-f\d]{32}\.([a-z]+)(\.gz)?$#i', $file)) {
          unlink(PAPAYA_PATH_CACHE.$file);
          $counter++;
        } elseif (preg_match('#^\.box_\d+_\d+_\d+_[a-f\d]{32}_[a-f\d]{32}\.\d+$#i', $file)) {
          unlink(PAPAYA_PATH_CACHE.$file);
          $counter++;        
        }
      }
      if ($counter > 0) {
        $this->addMsg(MSG_INFO, sprintf($this->_gt('%s files deleted.'), $counter));
      } else {
        $this->addMsg(MSG_INFO, $this->_gt('Cache was empty - no files deleted.'));
      }
      closedir($dh);
    } else {
      $this->addMsg(MSG_WARNING, $this->_gt('Can not find cache directory.'));
    }
  }
}
?>