<?php
/**
* Control particular activities (example: for flash)
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
* @version $Id: papaya_rpc.php,v 1.15 2006/04/26 10:29:00 weinert Exp $
*/

/**
* Basic class base_topic_edit
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_topic_edit.php');
/**
* Control particular activities (example: for flash)
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class papaya_rpc extends base_object {

  var $paramName = "rpc";
  var $params;

  var $topicTree = NULL;
  var $xmlBuffer = NULL;

  /**
  * papaya tag pattern
  * @var string $papayaTagPattern
  */
  var $papayaTagPattern = '/<(papaya|ndim):([a-z]\w+)\s?([^>]*)\/?>(<\/(\1):(\2)>)?/ims';

  /**
  * papaya parameters pattern
  * @var string $papayaParamsPattern
  */
  var $papayaParamsPattern = '/(?:^|\s)([a-z]\w+)(?:=)(?:(?:\'([^\']+)\')|(?:"([^"]*)")|([^\s,]+))/i';

  /**
  * Initialize parameters
  *
  * @param mixed $paramName optional, default value NULL
  * @access public
  */
  function initialize($paramName = NULL) {
    if (isset($paramName)) $this->paramName = $paramName;
    $this->initializeParams();
  }

  /**
  * Execute - basic function for handling parameters
  *
  * @access public
  */
  function execute() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_language_select.php');
    $this->lngSelect = &base_language_select::getInstance();
    include_once(PAPAYA_INCLUDE_PATH.'system/papaya_topic_tree.php');
    $this->topicTree = &new papaya_topic_tree;
    $this->topicTree->images = $this->images;
    $this->topicTree->msgs = &$this->msgs;
    $this->topicTree->authUser = &$this->authUser;
    switch(@$this->params['cmd']) {
    case 'topic_detail' :
      $this->sessionParamName = 'PAPAYA_SESS_tt';
      $this->sessionParams = $this->getSessionValue($this->sessionParamName);
      $this->xmlBuffer = $this->getTopicDetails();
      break;
    case 'publish' :
      $this->sessionParamName = 'PAPAYA_SESS_tt';
      $this->sessionParams = $this->getSessionValue($this->sessionParamName);
      $this->publishTopic();
      $this->setSessionValue($this->sessionParamName, $this->sessionParams);
      break;
    case 'mv' :
    case 'cp' :
      $this->topicTree->initialize(@(int)$_REQUEST['p_id']);
      $this->topicTree->params = $this->params;
      $this->topicTree->execute();
      break;
    case 'strings' :
      switch ($this->params['module']) {
        case 'sitemap' :
          $this->xmlBuffer = $this->getSitemapTranslations();
          break;
      }
      break;
    case 'strings_navigation' :
      switch ($this->params['module']) {
        case 'sitemap' :
          $this->xmlBuffer = $this->getNavigationTranslations();
          break;
      }
      break;
    case 'select' :
      $this->sessionParamName = 'PAPAYA_SESS_tt';
      $this->sessionParams = $this->getSessionValue($this->sessionParamName);
      $this->sessionParams['page_id'] = (int)$this->params['page_id'];
      $this->setSessionValue($this->sessionParamName, $this->sessionParams);
      $this->xmlBuffer = '<dummy>'.$this->params['page_id'].'</dummy>';
      break;    
    case 'imageconf_form' : 
      $this->xmlBuffer = '<response>';
      $this->xmlBuffer .= '<method>showImageConfDialog</method>';
      $data = $this->getImageConfDialog($this->params['image_ident']);
      $this->xmlBuffer .= '<data><![CDATA['.papaya_strings::escapeHTMLChars($data).']]></data>';
      $this->xmlBuffer .= '</response>';
      break;
    case 'addon_form' :
      $this->xmlBuffer = '<response>';
      $this->xmlBuffer .= '<method>showAddOnDialog</method>';
      $data = $this->getAddOnDialog($this->params['addon_guid']);
      $this->xmlBuffer .= '<data><![CDATA['.papaya_strings::escapeHTMLChars($data).']]></data>';
      $this->xmlBuffer .= '</response>';
      break;
    case 'addon_data' :
      $this->xmlBuffer = '<response>';
      $this->xmlBuffer .= '<method>setAddOnData</method>';
      $xmlData = $this->getAddOnData($this->params['addon_guid']);
      $this->xmlBuffer .= '<data>'.$xmlData.'</data>';
      $this->xmlBuffer .= '</response>';
      break;
    case 'image_data' :
      $this->xmlBuffer = '<response>';
      $this->xmlBuffer .= '<method>rpcSetImageData</method>';
      $xmlData = $this->getImageData($this->params['image_conf']);
      $this->xmlBuffer .= '<data>'.$xmlData.'</data>';
      $this->xmlBuffer .= '</response>';
      break;
    }
  }

  /**
  * Get XML
  *
  * @access public
  * @return string
  */
  function getXML() {
    if (isset($this->msgs->errorList) && is_array($this->msgs->errorList)) {
      return $this->msgs->get();
    } elseif (isset($this->xmlBuffer)) {
      return $this->xmlBuffer;
    } else {
      $this->topicTree->loadSimplyAll($this->lngSelect->currentLanguageId);
      return $this->topicTree->getXML();
    }
  }

  /**
  * Publish topic
  *
  * @access public
  */
  function publishTopic() {
    include_once(PAPAYA_INCLUDE_PATH.'system/papaya_topic.php');
    $topic = &new papaya_topic();
    $topic->params = $this->params;
    //$topic->msgs = &$this->msgs;
    $topic->authUser = &$this->authUser;
    $topic->load($this->params['page_id'], @(int)$this->lngSelect->currentLanguageId);
    if (isset($topic->topic) && is_array($topic->topic)) {
      if (($topic->hasPermUser(PERM_WRITE, $this->authUser) && 
           $topic->editable($this->authUser)) &&
          ($this->authUser->hasPerm(24) && $this->authUser->hasPerm(26)) && isset($this->params['commit_message'])) {
        $this->sessionParams['last_publish_message'] = $this->params['commit_message'];
        if ($topic->publishTopic()) {
          $this->addMsg(MSG_INFO, $this->_gt('Page published.'));
           $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_PAGES,
             sprintf($this->_gt('Page "%s (%d)" published.'),
               papaya_strings::escapeHTMLChars($topic->topic['TRANSLATION']['topic_title']),
               $topic->topicId));
        } else {
          $this->addMsg(MSG_ERROR, $this->_gt('Couldn\'t publish this page.'));
        }
      } else {
        $this->addMsg(MSG_ERROR, $this->_gt('Couldn\'t publish this page.'));
      }
    } else {
      $this->addMsg(MSG_ERROR, $this->_gt('Couldn\'t load this page.'));
    }
  }
  
  function getTranslations($strs) {
    $result = '<phrases>';
    foreach ($strs as $id=>$str) {
      $result .= sprintf('<phrase id="%s">%s</phrase>', papaya_strings::escapeHTMLChars($id), papaya_strings::escapeHTMLChars($this->_gt($str, 'sitemap')));
    }
    $result .= '</phrases>';  
    return $result;
  }

  /**
  * Get sitemap translation
  *
  * @access public
  */
  function getSitemapTranslations() {
    $strs = array(
      'str_publish'=>'Publish',
      'str_publishquestion'=>'Publish page "%s" (%s)?',
      'str_copy'=>'Copy',
      'str_copyquestion'=>'Copy page "%s" (%d) to "%s" (%d)?',
      'str_move'=>'Move',
      'str_movequestion'=>'Move page "%s" (%d) to "%s" (%d)?',
      'str_info'=>'Information',
      'str_warning'=>'Warning',
      'str_error'=>'Error',
      'str_ok'=>'OK',
      'str_cancel'=>'Cancel',
      'str_close'=>'Close',
      'str_loading'=>'Loading'
    );
    return $this->getTranslations($strs);
  }
  
  function getNavigationTranslations() {
    $strs = array(
      'str_base'=>'Base',
      'str_current'=>'Current',
      'str_offset'=>'Offset',
      'str_depth'=>'Depth',
      'str_focus'=>'Focus'
    );
    return $this->getTranslations($strs);  
  }
  
  function getTopicDetails() {
    $result = NULL;
    include_once(PAPAYA_INCLUDE_PATH.'system/papaya_topic.php');
    $topic = &new papaya_topic();
    $topic->params = $this->params;
    $topic->msgs = &$this->msgs;
    $topic->authUser = &$this->authUser;
    $topic->load($this->params['page_id'], $this->lngSelect->currentLanguageId);
    if (isset($topic->topic) && is_array($topic->topic)) {
      if ($topic->hasPermUser(PERM_WRITE, $this->authUser) && 
          $topic->editable($this->authUser) &&
          $this->authUser->hasPerm(24) && 
          $this->authUser->hasPerm(26)) {
        $topic->loadTranslationsInfo();
        if (isset($this->sessionParams['last_publish_message']) && trim($this->sessionParams['last_publish_message']) != '') {
          $publishVersionMessage = $this->sessionParams['last_publish_message'];
        } else {
          $publishVersionMessage = '';
        }
        $result .= sprintf('<item id="%d" commitmsg="%s">', $topic->topicId, 
          papaya_strings::escapeHTMLChars($publishVersionMessage));
        if (isset($this->lngSelect->languages) && is_array($this->lngSelect->languages)) {
          foreach($this->lngSelect->languages as $lngId=>$lng) {
            $selected = '';
            if ($this->lngSelect->currentLanguageId == $lngId) {
              $selected = ' checked="checked"';
            }
            if (isset($this->topic['TRANSLATIONINFOS'][$lngId]['topic_trans_published']) &&
                  $this->topic['TRANSLATIONINFOS'][$lngId]['topic_trans_published'] >=
                  $this->topic['TRANSLATIONINFOS'][$lngId]['topic_trans_modified']) {
              break;
            }
            $lngTitle = $this->lngSelect->languages[$lngId]['lng_title'].' ('.
              $this->lngSelect->languages[$lngId]['lng_short'].')';
            $result .= sprintf('<translation lng_title="%s" lng_id="%s" title="%s"%s/>',
              papaya_strings::escapeHTMLChars($lngTitle), $lngId, $topic->topicId, $selected);
          }
        }
        $result .= '</item>';
        return $result;
      } else {
        $this->addMsg(MSG_ERROR, $this->_gt('Couldn\'t publish this page.'));
      }
    } else {
      $this->addMsg(MSG_ERROR, $this->_gt('Couldn\'t load this page.'));
    }  
  }
  
  function getImageConfDialog($imageIdent) {
    $result = '';
    include_once(PAPAYA_INCLUDE_PATH.'/system/papaya_imagegenerator.php');
    $imgGenerator = &new papaya_imagegenerator();
    $imgGenerator->loadByIdent($imageIdent);
    if (isset($imgGenerator->imageConf) && isset($imgGenerator->imageConf['module_guid'])) {
      $parent = NULL;
      $moduleObj = &$this->createPluginObject($imgGenerator->imageConf['module_path'].$imgGenerator->imageConf['module_file'],
        $imgGenerator->imageConf['module_class'], $imgGenerator->imageConf['module_guid'], $parent);
      if (isset($moduleObj) && is_object($moduleObj)) {
        $moduleObj->images = &$imgGenerator->images;
        $moduleObj->setData($imgGenerator->imageConf['image_data']);
        
        $fields = $data = $hidden = NULL;
        include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
        $dialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
        $dialog->msgs = &$this->msgs;
        $dialog->paramName = 'img';        
        if (isset($moduleObj->attributeFields) && is_array($moduleObj->attributeFields) && count($moduleObj->attributeFields) > 0) {
          foreach($moduleObj->attributeFields as $fieldName=>$field) {
            $result .= '<label>'.papaya_strings::escapeHTMLChars($this->_gt(@$field[0])).'</label>';
            $result .= $dialog->getDlgElement($fieldName, $field, @$field[6]);
          }
        }
      }
    }
    return $result;
  }
  
  function getAddOnDialog($moduleGuid) {
    $result = '';
    include_once(PAPAYA_INCLUDE_PATH.'/system/papaya_modulemanager.php');
    $moduleMng = &new papaya_modulemanager();
    if ($moduleMng->loadModule($moduleGuid) && isset($moduleMng->module) && isset($moduleMng->module['module_guid'])) {
      $parent = NULL;
      $moduleObj = &$this->createPluginObject($moduleMng->module['module_path'].$moduleMng->module['module_file'],
        $moduleMng->module['module_class'], $moduleMng->module['module_guid'], $parent);
      if (isset($moduleObj) && is_object($moduleObj)) {      
        $fields = $data = $hidden = NULL;
        include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
        $dialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
        $dialog->msgs = &$this->msgs;
        $dialog->paramName = 'mod';
        if (isset($moduleObj->attributeFields) && is_array($moduleObj->attributeFields) &&
            count($moduleObj->attributeFields) > 0) {
          foreach($moduleObj->attributeFields as $fieldName=>$field) {
            $result .= '<label>'.papaya_strings::escapeHTMLChars($this->_gt(@$field[0])).'</label>';
            $result .= $dialog->getDlgElement($fieldName, $field, @$field[6]);
          }
        }
      }
    }
    return $result;
  }
  
  function getAddOnData($moduleGuid) {
    $result = '';
    include_once(PAPAYA_INCLUDE_PATH.'/system/papaya_modulemanager.php');
    $moduleMng = &new papaya_modulemanager();
    if ($moduleMng->loadModule($moduleGuid) && isset($moduleMng->module) && isset($moduleMng->module['module_guid'])) {
      $parent = NULL;
      $moduleObj = &$this->createPluginObject($moduleMng->module['module_path'].$moduleMng->module['module_file'],
        $moduleMng->module['module_class'], $moduleMng->module['module_guid'], $parent);
      if (isset($moduleObj) && is_object($moduleObj)) {
        $attr = $moduleObj->getPapayaTagData($this->params);
        if (isset($attr) && is_array($attr)) {
          foreach($attr as $key=>$value) {
            $result .= sprintf('<param name="%s" value="%s" />',
              papaya_strings::escapeHTMLChars($key), papaya_strings::escapeHTMLChars($value)
            );
          }
        }
      }
    }
    return $result;
  }
  
  function getImageData($imageConf) {
    $params = array();
    if (preg_match($this->papayaTagPattern, $imageConf, $regs)) {
      if (preg_match_all($this->papayaParamsPattern, $regs[3], $subRegs, PREG_SET_ORDER)) {
        foreach($subRegs as $match) {
          $paramName = $match[1];
          $paramValue = @($match[2].$match[3].$match[4]);
          $params[$paramName] = $paramValue;
        }
      }
      return $regs[0];
    } elseif (preg_match('~^([^.,]+\.\w+)(,(\d+)(,(\d+)(,(\w+))?)?)?$~i', $imageConf, $regs)) {
      $params = array(
        'src' => $regs[1],
        'width' => @(int)$regs[3],
        'height' => @(int)$regs[5], 
      );
      if (in_array(@$regs[7], array('max', 'min', 'mincrop', 'abs'))) {
        $params['resize'] = $regs[7];
      } else {
        $params['resize'] = 'max';      
      }
    }
    if ((!empty($params['src'])) && file_exists(PAPAYA_PATH_MEDIAFILES.$params['src'])) {
      list($width, $height, $type) = @getimagesize(PAPAYA_PATH_MEDIAFILES.$params['src']);
      $params['org_width'] = @(int)$width;
      $params['org_height'] = @(int)$height;
      $params['file_type'] = @(int)$type;
    }
    $result = '';
    foreach($params as $key=>$value) {
      $result .= sprintf('<param name="%s" value="%s" />',
        papaya_strings::escapeHTMLChars($key), papaya_strings::escapeHTMLChars($value)
      );
    }
    return $result;
  }
}
?>