<?php
/**
* Object to show sitemap in edit area (n-dimensional)
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
* @version $Id: papaya_topic_tree.php,v 1.10 2006/05/17 15:28:21 kelm Exp $
*/


/**
* Basic classe base_topictree
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_topictree.php');
/**
* Classe papaya_topic.inc
*/
require_once(PAPAYA_INCLUDE_PATH.'system/papaya_topic.php');
/**
* object to show sitemap in edit area (n-dimensional)
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class papaya_topic_tree extends base_topic_tree {
  var $tableAuthUser = PAPAYA_DB_TBL_AUTHUSER;
  var $tableTopics = PAPAYA_DB_TBL_TOPICS;
  var $tableTopicsPublic = PAPAYA_DB_TBL_TOPICS_PUBLIC;
  var $tableTopicsTrans = PAPAYA_DB_TBL_TOPICS_TRANS;
  var $tableTopicsPublicTrans = PAPAYA_DB_TBL_TOPICS_PUBLIC_TRANS;
  var $tableModules = PAPAYA_DB_TBL_MODULES;
  var $tableViews = PAPAYA_DB_TBL_VIEWS;

  /**
  * Temporres Array fr zu kopierende Daten
  * @var array $copyTopics
  * @access private
  */
  var $copyTopics = NULL;
  /**
  * Temporres Array fr zu kopierende Seitenids
  * @var array $copyTopicsTree
  * @access private
  */
  var $copyTopicsTree = NULL;

  /**
  * Execute - basic function for handling parameters
  *
  * @access public
  * @return string
  */
  function execute(){
    include_once(PAPAYA_INCLUDE_PATH.'system/base_language_select.php');
    $this->lngSelect = &base_language_select::getInstance();
    
    base_topic_tree::execute();
    $result = '';

    $pageId = (isset($this->params['page_id']) && $this->params['page_id'] > 0) ? (int)$this->params['page_id'] : 0;
    $targetId = (isset($this->params['tgt']) && $this->params['tgt'] > 0) ? (int)$this->params['tgt'] : 0;

    switch (@$this->params['cmd']) {
      case 'publish' :
        if ($pageId > 0) {
          $topic = &new papaya_topic;
          $topic->authUser = &$this->authUser;
          $topic->lngSelect = &$this->lngSelect;
          $topic->params = $this->params;
          if ($topic->load($pageId, $this->lngSelect->currentLanguageId)) {
            $topic->loadTranslationsInfo();
            $topic->initializePublishDialog();
            $topic->dialogPublish->loadParams();
            if (isset($this->params['publish_confirm']) && $this->params['publish_confirm'] && $topic->dialogPublish->checkDialogInput() ) {
              if ($topic->publishTopic()) {
                $this->addMsg(MSG_INFO, $this->_gt('Page published.'));
              } else {
                $this->addMsg(MSG_ERROR, $this->_gt('Can not publish this page.'));
              }
            } else {
              $topic->paramName = $this->paramName;
              $result .= $topic->getPublishForm();
            }
          } else {
            $this->addMsg(MSG_WARNING, $this->_gt('No page selected.'));
          }
        }
        break;
      case 'mv' :
        $success = FALSE;
        $this->load($this->authUser->startNode, $this->lngSelect->currentLanguageId);
        $errorString = $this->_gt('Can not move this page.');
        $topic = (isset($this->topics[$pageId])) ? $this->topics[$pageId] : NULL;

        if (isset($topic) && is_array($topic)) {
          if ($this->topicEditable($pageId) &&
              $this->topicEditable($targetId) &&
              $this->authUser->hasPerm(21)) {
            $this->movePage($topic, $targetId, $errorString);
          } else {
            $this->addMsg(MSG_WARNING, $errorString.' '.$this->_gt('Permission error!'));
          }
        } else {
          $this->addMsg(MSG_WARNING, $errorString.' '.$this->_gt('Invalid page id!'));
        }
        break;
      case 'cp':
        $success = FALSE;
        $this->load($this->authUser->startNode, $this->lngSelect->currentLanguageId);
        $errorString = $this->_gt('Can not copy this page.');
        $topic = (isset($this->topics[$pageId])) ? $this->topics[$pageId] : NULL;

        /**
        * @todo implement multi language page copy
        */
        if (isset($topic) && is_array($topic)) {
          if ($this->topicEditable($targetId) &&
              $this->authUser->hasPerm(22)) {
            $this->copyPage($topic, $targetId, $errorString);
          } else {
            $this->addMsg(MSG_WARNING, $errorString.' '.$this->_gt('Permission error!'));
          }
        } else {
          $this->addMsg(MSG_WARNING, $errorString.' '.$this->_gt('Invalid page id!'));
        }
        break;
    }
    $this->layout->add($result);
  }

  function movePage(&$topic, $targetId, $errorString) {
    if ($targetId < 1) {
      $target = array("topic_id" => 0,
                        "prev_path" => '',
                        "prev" => ';',
                        "topic_title" => '**root**',
                        "ALLPREV" => array(0 => 0));
    } else {
      $target = $this->topics[$this->params["tgt"]];
    }
    $family = array_merge($target['ALLPREV'], array(0));
    if (!($target['topic_id'] != $topic['topic_id'] &&
        !in_array($topic['topic_id'], $family) &&
        $target['topic_id'] != $topic['prev'] )){
      $this->addMsg(MSG_WARNING, $errorString.' '.$this->_gt('Input error!'));
    } else {
      $newPath = $target["prev_path"].$target["prev"].";";
      $newPath = str_replace(';;', ';',$newPath);
      $values = array('prev' => $target["topic_id"], 'prev_path' => $newPath);
      if ((FALSE !== $this->databaseUpdateRecord($this->tableTopics, $values, 'topic_id', $topic["topic_id"])) &&
          (FALSE !== $this->databaseUpdateRecord($this->tableTopicsPublic, $values, 'topic_id', $topic["topic_id"]))){
        $this->addMsg(MSG_INFO, $this->_gt('Page moved!'));
        $oldPath = $topic["prev_path"].$topic["prev"].";";
        $strLength = strlen($oldPath)+1;
        $newPath = $target["prev_path"].$target["prev"].";".$target["topic_id"].";";
        $newPath = str_replace(';;', ';',$newPath);
        //change all subnode paths (n+x)|x>0
        $sqlReplace = $this->databaseGetSQLSource('CONCAT', $newPath, TRUE,
          $this->databaseGetSQLSource('SUBSTRING', 'prev_path', FALSE, $strLength, TRUE), FALSE);
        $sql = "UPDATE %s
                    SET prev_path = ".$sqlReplace."
                  WHERE prev = '%d'
                    OR prev_path LIKE '%s%d;%%'";
        $params = array(
          $this->tableTopics, $topic["topic_id"], $oldPath, $topic["topic_id"]
        );
        $paramsPub = array(
          $this->tableTopicsPublic, $topic["topic_id"], $oldPath, $topic["topic_id"]
        );
        if (!(FALSE !== $this->databaseQueryFmt($sql,$params) && FALSE !== $this->databaseQueryFmt($sql,$paramsPub))){
          $this->addMsg(MSG_WARNING, $this->_gt('Could not move subpages!'));
        } else {
          $success = TRUE;
          if (!isset($this->opened[$target['topic_id']])) {
            $this->sessionParams = $this->getSessionValue($this->sessionParamName);
            $this->opened[$target['topic_id']] = TRUE;
            $this->sessionParams['opened'] = $this->opened;
            $this->setSessionValue($this->sessionParamName, $this->sessionParams);
          }
          $this->load($this->authUser->startNode, $this->lngSelect->currentLanguageId);
        }
      } else {
        $this->addMsg(MSG_WARNING, $errorString.' '.$this->_gt('Database error!'));
      }
    }
  }

  function copyPage(&$topic, $targetId, $errorString) {
    if ($targetId < 1) {
      $target = array("topic_id" => 0,
                        "prev_path" => '',
                        "prev" => '',
                        "ALLPREV" => array(0 => 0));
    } else {
      $target = $this->topics[$this->params["tgt"]];
    }
    if ($target['topic_id'] == $topic['topic_id'] || $topic['topic_id'] == $target['prev'] ||
        in_array($topic['topic_id'], $target['ALLPREV'])){
      $this->addMsg(MSG_INFO, $errorString.' '.$this->_gt('Input error!'));
    } else {
      unset($this->copyTopics);
      unset($this->copyTopicsTree);
      $sql = "SELECT *
                FROM %s
               WHERE topic_id = %d OR prev = %d";
      $params = array($this->tableTopics, $topic['topic_id'], $topic['topic_id']);   
      if ($res = $this->databaseQueryFmt($sql, $params)) {  
        while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $row['topic_created'] = time();
          $row['topic_modified'] = time();
          $row['author_id'] = $this->authUser->userId;
          $this->copyTopics[(int)$row['topic_id']] = $row;
          $this->copyTopicsTree[(int)$row['prev']][] = $row['topic_id'];    
        }
        $sql = "SELECT *
                  FROM %s
                 WHERE prev_path LIKE '%s%d;%d;%%'
              ORDER BY prev_path ASC";
        $params = array($this->tableTopics, $topic['prev_path'], $topic['prev'], $topic['topic_id']); 
        if ($res = $this->databaseQueryFmt($sql, $params, 200)) {
          while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
            $row['topic_created'] = time();
            $row['topic_modified'] = time();
            $row['author_id'] = $this->authUser->userId;
            $this->copyTopics[(int)$row['topic_id']] = $row;
            $this->copyTopicsTree[(int)$row['prev']][] = $row['topic_id'];
          }
        }  
      }
      if(isset($this->copyTopics) && is_array($this->copyTopics)) {
        $this->copyTopicsTrans = array();
        $filter = $this->databaseGetSQLCondition('topic_id', array_keys($this->copyTopics));
        $sql = "SELECT * FROM %s WHERE $filter";
        if ($res = $this->databaseQueryFmt($sql, $this->tableTopicsTrans)) {
          while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
            $this->copyTopicsTrans[(int)$row['topic_id']][(int)$row['lng_id']] = $row;
          }
        }
        $success = TRUE;
        $current = &$this->copyTopics[$topic['topic_id']];
        $current['prev'] = $target['topic_id'];
        $current['prev_path'] = $target['prev_path'].';'.$target['prev'].';';
        unset($current['topic_id']);
        if ($newId = $this->databaseInsertRecord($this->tableTopics, 'topic_id', $current)) {
          if (isset($this->copyTopicsTree[$topic['topic_id']]) &&
              is_array($this->copyTopicsTree[$topic['topic_id']])) {
            $newPath = $current['prev_path'].$current['prev'].';';
            if ($success = $this->copyPageTranslations($topic['topic_id'], $newId, $newPath)) {
              $success = $this->copySubPages($this->copyTopicsTree[$topic['topic_id']], $newId, $newPath);
            }
          }
        } else {
          $success = FALSE;
        }
        if($success){
          $this->addMsg(MSG_INFO, $this->_gt('Page copied!'));
        } else {
          $this->addMsg(MSG_ERROR, $errorString.' '.$this->_gt('Database error!'));
        } 
      }
    }
  }
  
  function copyPageTranslations($oldId, $newId, $newPath) {
    if (isset($this->copyTopicsTrans[$oldId]) && is_array($this->copyTopicsTrans[$oldId])) {
      $translations = $this->copyTopicsTrans[$oldId];
      foreach($translations as $id=>$translation) {
        $translations[$id]['topic_id'] = $newId;
        $translations[$id]['topic_title'] = $this->_gt('Copy of').' '.$translation['topic_title'];
      }
      return (FALSE != $this->databaseInsertRecords($this->tableTopicsTrans, $translations));
    }
    return TRUE;
  }

  function copySubPages($ids, $prev, $prevPath) {
    $success = TRUE;
    if (isset($ids) && is_array($ids)) {
      foreach($ids as $id) {
        if (isset($this->copyTopics[$id]) && is_array($this->copyTopics[$id])) {
          $current = &$this->copyTopics[$id];
          $current['prev'] = $prev;
          $current['prev_path'] = $prevPath;
          unset($current['topic_id']);
          if ($newId = $this->databaseInsertRecord($this->tableTopics, 'topic_id', $current)) {
            $newPath = $current['prev_path'].$current['prev'].';';
            if ($success = $this->copyPageTranslations($id, $newId, $newPath)) {
              if (isset($this->copyTopicsTree[$id]) && is_array($this->copyTopicsTree[$id])) {
                $success = $this->copySubPages($this->copyTopicsTree[$id], $newId, $newPath);
              }
            }
          } else {
            $success = FALSE;
          }
          if (!$success) return FALSE;
        }
      }
    }
    return $success;
  }

  /**
  * Get part tree
  *
  * @param object base_topic &$topic
  * @access public
  * @return string
  */
  function getPartTree(&$topic) {
    $result = '';
    $this->topicId = $topic->topicId;
    $this->topic = &$topic;
    $this->opened[$topic->topic['prev']] = TRUE;
    $this->opened[$this->topicId] = TRUE;
    $base = 0;
    if (preg_match_all('/\d+/', $topic->topic['prev_path'], $regs, PREG_PATTERN_ORDER)) {
      $base = end($regs[0]);
    }

    include_once(PAPAYA_INCLUDE_PATH.'system/base_language_select.php');
    $lngSelect = &base_language_select::getInstance();
    $base = (int)$base;
    $this->load($base, $lngSelect->currentLanguageId);
    if(isset($this->topicLinks) && is_array($this->topicLinks))
      $nodes = $this->topicLinks[$base]['childs'];
    if (isset($nodes)&& !is_array($nodes)) {
      $nodes = $this->rootTopics;
    }
    if (isset($this->topics) && is_array($this->topics)) {
      $result .= '<listview title="Seiten" width="246"><items>'.LF;
      $this->params['page_id'] = @(int)$selected;
      $counter = 0;
      if ($base > 0) {
        $result .= sprintf('<listitem title="xxx" image="%s" href="%s"><subitem></subitem></listitem>',
          $this->images[62], $this->getLink(array('page_id'=>$base)));
        $indent = 1;
      } else {
        $indent = 0;
      }
      $result .= $this->getStaticSubTree($nodes, $indent);
      $result .=  '</items></listview>'."\r\n";;
    }
    return $result;
  }

  /**
  * Get stratic sub tree
  *
  * @param array $nodes
  * @param integer $indent optional, default value 0
  * @access public
  * @return string
  */
  function getStaticSubTree($nodes, $indent=0) {
    $result = '';
    if (isset($nodes) && is_array($nodes)) {
      foreach($nodes as $id) {
        $val = $this->topics[$id];
        switch ($val['topic_status']) {
          case 4 : $imageIdx = 10; break; // deleted
          case 3 : $imageIdx = 48; break; // published and modified
          case 2 : $imageIdx = 16; break; // published and up to date
          default : $imageIdx = 15; break; // created - no public version
        }
        if ($id == $this->topicId) {
          $selected = ' selected="selected"';
        } else {
          $selected = '';
        }
        if (isset($val['topic_title']) && trim($val['topic_title']) != '') {
          $title = papaya_strings::escapeHTMLChars($val['topic_title']);
        } else {
          $title = papaya_strings::escapeHTMLChars($this->_gt('No title'));
        }
        $result .= sprintf('<listitem title="%s" href="%s" indent="%d" image="%s"%s>',
          $title, $this->getLink(array('page_id'=>$id)),
          $indent, $this->images[$imageIdx], $selected);
        $result .= sprintf('<subitem align="right"><glyph src="%s"/></subitem>',
          ($val['is_navigation']) ? $this->images[43] : $this->images[0]);
        $result .= '</listitem>';
        if (isset($this->topicLinks[$id]['childs']) && is_array($this->topicLinks[$id]['childs'])) {
          $result .= $this->getStaticSubTree($this->topicLinks[$id]['childs'],$indent+1);
        }
      }
    }
    return $result;
  }

  /**
  * Get
  *
  * @access public
  * @return string
  */
  function get() {
    $this->load($this->authUser->startNode, $this->lngSelect->currentLanguageId);
    $result = '';
    if (isset($this->topics) && is_array($this->topics)) {
      $result .= sprintf('<listview title="%s" width="100%%"><items>'.LF, $this->_gt('Sitemap'));
      $counter = '0';
      $result .= $this->getSubTree($this->topicLinks[0]['childs'],$counter);
      if ($this->params['page_id'] > 0 &&
          $this->topicEditable($this->params['page_id']) &&
          $this->params['page_id'] != $this->authUser->startNode){
        //$result .= $this->getEditTopic($this->topics[$this->params['page_id']]);
      }
      $result .=  '</items></listview>'."\r\n";;
    }
    $this->layout->add($result);
    
    $topic = &new papaya_topic;
    if ($topic->load($this->params['page_id'], $this->lngSelect->currentLanguageId)) {
      $this->layout->addRight($topic->getTopicInformation());
    }
  }

  /**
  * Get sub tree
  *
  * @param array $idArr
  * @param integer &$counter
  * @access public
  * @return string
  */
  function getSubTree($idArr,&$counter) {
    $result = '';
    if (isset($idArr) && is_array($idArr)) {
      foreach($idArr as $id) {
        $val = $this->topics[$id];
        if (isset($val) && is_array($val)) {
          $counter++;
          if (isset($val['ALLPREV']) && is_array($val['ALLPREV'])) {
            $indent = count($val['ALLPREV'])-1;
          } else {
            $indent = 0;
          }
          if (@$this->topics[$id]["prev"] == 0) {
            $node = ' node="open"';          
          } elseif (isset($this->subtopicCount[$id])) {
            if (isset($this->opened[$id])) {
              $nodeHref = sprintf('%s#node%s',
                $this->getLink(array('cmd' => 'close', 'page_id' => (int)$id)), (int)$id);
              $node = sprintf(' node="open" nhref="%s"',$nodeHref);
            } else {
              $nodeHref = sprintf('%s#node%s',
                $this->getLink(array('cmd' => 'open', 'page_id' => (int)$id)), (int)$id);
              $node = sprintf(' node="close" nhref="%s"',$nodeHref);
            }
          } else {
            $node = ' node="empty"';
          }
          if ($val['topic_published']) {
            if ($val['topic_published'] >= $val['topic_modified']) {
              $imageIdx = 16;
              $showPublish = FALSE;
            } else {
              $imageIdx = 48;
              $showPublish = TRUE;
            }
          } else {
            $imageIdx = 15;
            $showPublish = TRUE;
          }
          if ($id == $this->topicId) {
            $selected = ' selected="selected"';
          } else {
            $selected = '';
          }
          $result .= sprintf('<listitem title="%s (%s)" href="%s#node%d" image="%s" indent="%s" %s>'.LF,
            papaya_strings::escapeHTMLChars($val['topic_title']), $val['topic_id'],
            $this->getLink(array('page_id'=>$id)), $id, $this->images[$imageIdx], $indent,
            $node.$selected);
          if (preg_match_all('/\d+/', $val['prev_path'].';'.$val['prev'].';'.$val['topic_id'],
              $match, PREG_PATTERN_ORDER)) {
            $family = $match[0];
          } else {
            $family = FALSE;
          }
          $mvRadio = FALSE;
          $cpRadio = FALSE;
          if (!@in_array($this->topicId, $family)){
            $cpRadio = TRUE;
            if ($val['topic_id'] != @$this->topics[$this->topicId]["prev"] &&
                $this->topicId != $val['prev']){
              $mvRadio = TRUE;
            }
          }
          $result .= '<subitem>'.date('d.m.Y H:i',$val['topic_modified']).'</subitem>';
          if ($showPublish && $this->authUser->hasPerm(26) && $this->topicEditable($val['topic_id'])) {
            $result .= '<subitem align="center">';
            $result .= sprintf('<a href="%s"><glyph src="%s" hint="%s (%s) verffentlichen"/></a>',
              $this->getLink(array('cmd' => 'publish', 'page_id' => (int)$id)), 
              $this->images[16], papaya_strings::escapeHTMLChars($val['topic_title']), $id);
            $result .= '</subitem>';
          } else {
            $result .= '<subitem/>';
          }
          $result .= '<subitem><glyph src="tpoint.gif"/></subitem>';
          if ($this->topicId > 0) {
            if ($cpRadio) {
              $result .= '<subitem align="right">';
              $result .= sprintf('<a href="%s"><glyph src="%s" hint="Kopieren nach %s (%s)"/></a>',
                $this->getLink(array('cmd' => 'cp', 'page_id' => (int)$this->topicId, 'tgt' => (int)$id)), 
                $this->images[18], papaya_strings::escapeHTMLChars($val['topic_title']), $id);
              $result .= '</subitem>';
            } else {
              $result .= '<subitem/>';
            }
            $result .= '<subitem><glyph src="tpoint.gif"/></subitem>';
            if ($mvRadio) {
              $result .= '<subitem>';
              $result .= sprintf('<a href="%s"><glyph src="%s" hint="Verschieben nach %s (%s)"/></a>',
                $this->getLink(array('cmd' => 'mv', 'page_id' => (int)$this->topicId, 'tgt' => (int)$id)), 
                $this->images[19], papaya_strings::escapeHTMLChars($val['topic_title']), $id);
              $result .= '</subitem>';
            } else {
              $result .= '<subitem/>';
            }
          }
          $result .= '<subitem><glyph src="tpoint.gif"/></subitem>';
          $result .= '<subitem align="right">';
          $result .= sprintf('<a href="%s"><glyph src="%s" hint="Seite bearbeiten"/></a>',
            $this->getLink(array('page_id' => (int)$id), NULL, 'topic.php'), 
            $this->images[14]);
          $result .= '</subitem>';
          $result .= '</listitem>'.LF;
          if (isset($this->topicLinks[$id]) && is_array($this->topicLinks[$id])) {
            $result .= $this->getSubTree($this->topicLinks[$id]['childs'],$counter);
          }
        }
      }
    }
    return $result;
  }
}
?>