<?php
/**
* Seassion object
*
* @package papaya_base
* @author Thomas Weinert
* @version $Id: sys_session.php,v 1.13 2006/05/31 13:53:32 weinert Exp $
*/

/**
* Basisklasse
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_object.php');

/**
* Automatic url-decoding for seassion processing
*
* @package papaya_base
* @author Thomas Weinert
* @version 2.0
*/
class rewrite_session extends base_object {
  /**
  * Session ID
  * @var string
  */
  var $sessionId = "";
  /**
  * Session name starting with "sid"
  * @var string
  */
  var $sessionName = "sid";

  /**
  * Session-ID mode
  * @var string
  */
  var $sessionMode = "";

  /**
  * Robots, User-Agents von Suchmaschinen, Crawlern, Robots, ...
  * @var string $robots
  */
  var $robots = array('mnogosearch-dimensional', 'Bot', 'bot', 'Crawl', 'crawl', 'Spider', 'spider', 'appie',
    'Ask Jeeves', 'ASPSeek', 'ASPseek', 'AvantGo', 'bigfoot.com', 'BlitzBOT', 'blogchecker', 'BSDSeek',
    'bumblebee@relevare.com', 'Charybdis', 'cosmos', 'curl', 'db/0.2; spc', 'DiaGem', 'DISCo Pump', 'EmailSiphon',
    'FairAd', 'Firefly', 'flunky', 'fmII URL validator', 'gazz', 'GetRight', 'Getweb', 'Google', 'Gozilla',
    'Gulliver','htdig', 'HTTrack', 'ia_archiver', 'Ideare', 'Indy Library', 'InternetLinkAgent', 'Java', 'Jigsaw',
    'larbin', 'libcurl', 'libwww', 'LinkLint', 'LinkMan', 'LinkWalker', 'LiSEn', 'lwp', 'Mercator',
    'Microsoft URL Control', 'mnogo', 'moget', 'msnbot', 'MS Search', 'Netcraft', 'NG/1.0', 'Netprospector',
    'Offline Explorer', 'Openfind', 'OPen sourfce retriver', 'Perl', 'PHP', 'Plumtree', 'Pompos', 
    'Powermarks', 'reifier',
    'RPT-HTTPClient', 'Robozilla', 'sun4u', 'Scooter', 'ScoutAbout', 'SearchTone', 'SignSite',
    'sitecheck.internetseer.com', 'Slurp', 'SlySearch', 'Snoopy', 'Spinne', 'SurferX', 'Steeler', 'Teradex Mapper',
    'Tkensaku', 'Teleport', 'vias.ncsa.uiuc.edu', 'Vagabondo', 'Vayala', 'Viking', 'w3mir', 'W3C_Validator',
    'Watchfire WebXM', 'www.walhello.com', 'webcollage', 'WebCapture', 'WebCopier', 'Web Downloader', 'WebReaper',
    'Webshuttle', 'WebStripper', 'Wget', 'WhizBang', 'Whizbang', 'WWWC', 'WWWOFFLE', 'Xenu Link Sleuth',
    'yahoo.com', 'ZyBorg');

  /**
  * Session-variable - depend of PHP-version
  * @var integer $sessionVarMode
  */
  var $sessionVarMode;

  /**
  * Session desired ?
  * @var boolean $wantSession
  */
  var $wantSession;

  /**
  * Konstruktor
  * @param string $name Session name
  * @param boolean $start
  */
  function __construct($name="", $start = TRUE, $fallbackMode = 'rewrite') {
    $this->wantSession = $start;
    $this->sessionStartFromRewrite($name, $fallbackMode);
  }

  /**
  * Check mode for session variables
  * @return integer $this->sessionVarMode
  */
  function sessionVarMode() {
    if (isset($this->sessionVarMode)) {
      return $this->sessionVarMode;
    } else {
      if (ini_get('register_globals') && @is_array($_SERVER)) {
        $this->sessionVarMode = 3; // new php, register globals on ... using $_SESSION and global variables
      } elseif (ini_get('register_globals')) {
        $this->sessionVarMode = 0;  // old php, register globals on ... using global variables
      } elseif (@is_array($_SERVER)) {
        $this->sessionVarMode = 1;  // register globals off ... using $_SESSION only
      } else {
        $this->sessionVarMode = 2;  // old php, register globals off ... using old session arrray
      }
      return $this->sessionVarMode;
    }
  }

  /**
  * Determine Session-ID and URL, evt. start seassion
  * @param string $name optional, default value ""
  * @param string $fallbackMode optional, default value 'rewrite'
  * @access public
  */
  function sessionStartFromRewrite($name="", $fallbackMode = 'rewrite') {
    $this->started = FALSE;
    $this->fallbackMode = $fallbackMode;

    if ($this->sessionAllowed()) {
      $this->sessionName = "sid".$name;

      $this->checkSID();
      $this->start();
    }
  }
  
  function resetData() {
    session_unset();
    $_SESSION = array();
  }

  function regenerateSID() {
    if ($this->started) {
      $oldSession = $_SESSION;
      if (function_exists('session_regenerate_id')) {
        session_regenerate_id();
        $this->sessionId = session_id();
      }
      if ($this->checkSIDString($this->sessionId)) {
        $_SESSION = $oldSession;
        $this->reloadPage(TRUE);
      }
    }
  }

  /**
  * Check seassion id
  *
  * @access public
  * @return boolean
  */
  function checkSID() {
    if (!$this->wantSession) return FALSE;
    $sid = FALSE;
    if ($sid = $this->checkSIDCookie()) {
      //SID in Cookie
      if ($this->checkSIDPath() || ((!$this->disableTransSID()) && $this->checkSIDGETParam())) {
        //SID aus Pfad/Parameter entfernen
        $this->reloadPage(FALSE);
      } else {
        $this->sessionId = $sid;
      }
    } elseif ($this->fallbackMode == 'rewrite') {
      if ($sid = $this->checkSIDPath()) {
        //Pfadmodus
        if ((!$this->disableTransSID()) && $this->checkSIDGETParam()) {
          //SID aus Pfad/Parameter entfernen
          $this->reloadPage(TRUE);
        } else {
          $this->sessionId = $sid;
        }
      } else {
        $this->start();
        $this->reloadPage(TRUE);
      }
    } elseif ($this->fallbackMode == 'get') {
      //GET-Modus (Trans-SID) - Achtung! inkompatibel mit Caching
      if (!($sid = $this->checkSIDParam())) {
        $this->start();
        $this->reloadPage(TRUE);
      } else {
        $this->sessionId = $sid;
      }
    } else {
      $this->disableTransSID();
      return FALSE;
    }
    return TRUE;
  }

  /**
  * Get SID out of Cookie
  * @return mixed seassion id or boolean
  */
  function checkSIDCookie() {
    if (!$this->wantSession) return FALSE;
    if (isset($_COOKIE[$this->sessionName])) {
      $sessionId = $_COOKIE[$this->sessionName];
      if ($this->checkSIDString($sessionId)) {
        return $sessionId;
      }
    }
    return FALSE;
  }

  function checkSIDString($sessionId) {
    $pattern = '~^([a-zA-Z\d,-]{32}|[a-zA-Z\d,-]{40})$~';
    if (preg_match($pattern, $sessionId)) {
      return TRUE;
    } else {
      return FALSE;
    }
  }

  /**
  * Get SID out of Parmeter
  * @return mixed Seassion id or FALSE
  */
  function checkSIDParam() {
    if (!$this->wantSession) return FALSE;
    if ($sessionId = $this->checkSIDGETParam()) {
      return $sessionId;
    } elseif ($sessionId = $this->checkSIDPOSTParam()) {
      return $sessionId;
    }
    return FALSE;
  }

  function checkSIDGETParam() {
    if (!$this->wantSession) return FALSE;
    if (isset($_GET[$this->sessionName])) {
      $sessionId = $_GET[$this->sessionName];
      if ($this->checkSIDString($sessionId)) {
        return $sessionId;
      }
    }
    return FALSE;
  }

  function checkSIDPOSTParam() {
    if (!$this->wantSession) return FALSE;
    if (isset($_POST[$this->sessionName])) {
      $sessionId = $_POST[$this->sessionName];
      if ($this->checkSIDString($sessionId)) {
        return $sessionId;
      }
    }
    return FALSE;
  }

  /**
  * Get SID out of Pfad
  * @return mixed integer or FALSE
  */
  function checkSIDPath() {
    if (!$this->wantSession) return FALSE;
    if (preg_match("#^/".preg_quote($this->sessionName)."([0-9a-z]{32})#i",$_SERVER['REQUEST_URI'], $regs)) {
      return $regs[1];
    }
    return FALSE;
  }

  function disableTransSID() {
    @ini_set('session.use_trans_sid', '0');
    if ($transSid = @ini_get('session.use_trans_sid')) {
      $transSid = (!@ini_set('url_rewriter.tags', ''));
    }
    return $transSid;
  }

  /**
  * Reload page
  *
  * @param $sid
  */
  function reloadPage($sid) {
    $protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ? 'https' : 'http';
    $host = $protocol."://".$_SERVER['HTTP_HOST'];
    $path = $this->getBasePath();
    $file = $path.$this->getBaseLink();

    if ($sid) {
      $this->start();
      if (isset($this->sessionId) && $this->checkSIDString($this->sessionId)) {
        if ($this->fallbackMode == 'rewrite') {
          $destination = $host.'/'.$this->sessionName.$this->sessionId.$file;
          unset($_GET[$this->sessionName]);
        } else  {
          $_GET[$this->sessionName] = $this->sessionId;
          $destination = $host.$file;
        }
      } else {
        return FALSE;
      }
    } else {
      unset($_GET[$this->sessionName]);
      $destination = $host.$file;
    }

    if (isset($GLOBALS['PAPAYA_PAGE']) && is_object($GLOBALS['PAPAYA_PAGE'])) {
      $GLOBALS['PAPAYA_PAGE']->logRequest();
    }
    
    unset($_GET['preview']);
    if (php_sapi_name() == 'cgi' || php_sapi_name() == 'fast-cgi') {
      @header('Status: 302 Found');
    } else {
      @header('HTTP/1.1 302 Found');
    }
    @header('Location: '.$destination.$this->encodeQueryString($_GET));
    exit();
  }

  /**
  * Start session
  */
  function start() {
    if (!$this->started) {
      @session_name($this->sessionName);
      if (isset($this->sessionId) && $this->checkSIDString($this->sessionId)) {
        @session_id($this->sessionId);
      }
      @header('P3P: CP="NOI NID ADMa OUR IND UNI COM NAV"');
      if (defined('PAPAYA_ADMIN_PAGE') && PAPAYA_ADMIN_PAGE) {
        @session_cache_limiter('nocache');
      } elseif (defined('PAPAYA_SESSION_CACHE') && PAPAYA_SESSION_CACHE) {
        @session_cache_limiter(PAPAYA_SESSION_CACHE);
      } else {
        @session_cache_limiter('private');      
      }
      @session_start();
      $this->sessionId = @session_id();
      $this->started = TRUE;
    }
  }

  /**
  * Session permitted? - no sessions for robots
  */
  function sessionAllowed() {
    if ((!$this->wantSession) || $this->checkRobot()) {
      if (preg_match("#^/".preg_quote($this->sessionName)."([0-9a-z]{32})#i",$_SERVER['REQUEST_URI'], $regs)) {
        //session not permitted but sid in url - remove it
        $this->reloadPage(FALSE);
      }
      return FALSE;
    }
    return TRUE;
  }


  /**
  * Check browser is valid
  *
  * @access public
  * @return boolean
  */
  function checkRobot() {
    $agent = @$_SERVER["HTTP_USER_AGENT"];
    if ((!empty($agent)) && isset($this->robots) && is_array($this->robots)) {
      foreach($this->robots as $pattern) {
        if (strpos($agent, $pattern) !== FALSE) {
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  /**
  * Destroy seassion
  * @access public
  * @param string $url calling URL
  */
  function destroy() {
    if (isset($_SESSION) && is_array($_SESSION) && count($_SESSION) > 0) {
      foreach($_SESSION as $key=>$val) {
        unset($_SESSION[$key]);
      }
    }
    // kaputtmachen
    @session_unset();
    @session_destroy();

    // Where do you want to go now?
    @header("Status: 302 Found");
    @header("Location: ".$this->getAbsoluteURL($this->getBaselink(), FALSE));
  }

  /**
  * Variablen werden in der Session registriert!
  *
  * @access public
  */
  function register() {
    if ($this->started) {
      $vars = func_get_args();
      if (isset($vars) && is_array($vars)) {
        foreach($vars as $var) {
          if ($this->sessionVarMode() == 3 || $this->sessionVarMode() == 1) {
            if (!isset($_SESSION[$var])) $_SESSION[$var] = NULL;
          } else {
            @session_register($var);
          }
        }
      }
    }
  }


  /**
  * Store variable either in globals or in session. Decide by Variable sessionVarMode.
  *
  * @param string $globalVariableName variable name
  * @param string $value value variable
  */
  function setValue($globalVariableName, $value) {
    if (isset($globalVariableName) && is_string($globalVariableName)) {
      switch ($this->sessionVarMode()) {
        case 0 : $GLOBALS[$globalVariableName] = $value; break;
        case 3 : $GLOBALS[$globalVariableName] = $value;
        case 1 : $_SESSION[$globalVariableName] = $value; break;
        case 2 : $GLOBALS['HTTP_SESSION_VARS'][$globalVariableName] = $value; break;
      }
    }
  }

  /**
  * Get variable either out of globals or out of session. Decide by Variable sessionVarMode.
  *
  * @param string $globalVariableName variable name
  */
  function getValue($globalVariableName) {
    if (isset($globalVariableName) && is_string($globalVariableName)) {
      switch ($this->sessionVarMode()) {
        case 0 : if (isset($GLOBALS[$globalVariableName])) return $GLOBALS[$globalVariableName];
        case 3 :
        case 1 : if (isset($_SESSION[$globalVariableName])) return $_SESSION[$globalVariableName];
        case 2 : if (isset($GLOBALS['HTTP_SESSION_VARS'][$globalVariableName])) return $GLOBALS['HTTP_SESSION_VARS'][$globalVariableName];
      }
    }
    return NULL;
  }

}
?>